@@ -16,7 +16,8 @@
 #undef USE_TRANS
 #undef USE_HEBREW
 #undef USE_OPENSSL
-#undef USE_LINGER
+#undef USE_IPV6
+#undef USE_JCODE
 #undef SOCKS
 #undef PREFIX
 #undef USING_LINUX
