@@ -26,30 +26,41 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+
 #include "xchat.h"
 #include "cfgfiles.h"
 #include "util.h"
-#include "popup.h"
 #include "fe.h"
+#include "text.h"
+#include "xchatc.h"
+#ifdef USE_JCODE
+#include <locale.h>
+#endif
 
-
-extern struct xchatprefs prefs;
-
-extern int buf_get_line (char *, char **, int *, int len);
-extern void PrintText (struct session *sess, unsigned char *text);
-
+#define DEF_FONT "-b&h-lucidatypewriter-medium-r-normal-*-*-120-*-*-m-*-*-*"
+#define DEF_FONT_JCODE "-*-fixed-medium-r-normal-*-14-*-*-*-c-*-*-*"
+#define DEF_FONT_WIN32 "-*-Fixedsys-normal-r-normal-*-15-*-*-*-m-*-iso8859-1"
 
 void
 list_addentry (GSList ** list, char *cmd, char *name)
 {
-	struct popup *pop = malloc (sizeof (struct popup));
-	if (!pop)
-		return;
-	safe_strcpy (pop->name, name, sizeof (pop->name));
-	if (!cmd)
-		pop->cmd[0] = 0;
+	struct popup *pop;
+	int cmd_len = 1, name_len;
+
+	if (cmd)
+		cmd_len = strlen (cmd) + 1;
+	name_len = strlen (name) + 1;
+
+	pop = malloc (sizeof (struct popup) + cmd_len + name_len);
+	pop->name = (char *) pop + sizeof (struct popup);
+	pop->cmd = pop->name + name_len;
+
+	memcpy (pop->name, name, name_len);
+	if (cmd)
+		memcpy (pop->cmd, cmd, cmd_len);
 	else
-		safe_strcpy (pop->cmd, cmd, sizeof (pop->cmd));
+		pop->cmd[0] = 0;
+
 	*list = g_slist_append (*list, pop);
 }
 
@@ -76,8 +87,13 @@ list_loadconf (char *file, GSList ** list, char *defaultconf)
 		}
 		return;
 	}
+	if (fstat (fh, &st) != 0)
+	{
+		perror ("fstat");
+		abort ();
+	}
 
-	fstat (fh, &st);
+	/*fstat (fh, &st);*/
 	ibuf = malloc (st.st_size);
 	read (fh, ibuf, st.st_size);
 	close (fh);
@@ -132,6 +148,7 @@ list_delentry (GSList ** list, char *name)
 		if (!strcasecmp (name, pop->name))
 		{
 			*list = g_slist_remove (*list, pop);
+			free (pop);
 			return 1;
 		}
 		alist = alist->next;
@@ -227,12 +244,16 @@ char *xdir = 0;
 char *
 get_xdir (void)
 {
+#ifndef WIN32
 	if (!xdir)
 	{
 		xdir = malloc (strlen (g_get_home_dir ()) + 8);
 		sprintf (xdir, "%s/.xchat", g_get_home_dir ());
 	}
 	return xdir;
+#else
+	return "./data"; /* FIXME */
+#endif
 }
 
 void
@@ -241,8 +262,12 @@ check_prefs_dir (void)
 	char *xdir = get_xdir ();
 	if (access (xdir, F_OK) != 0)
 	{
+#ifdef WIN32
+		if (mkdir (xdir) != 0)
+#else
 		if (mkdir (xdir, S_IRUSR | S_IWUSR | S_IXUSR) != 0)
-			fe_message ("Cannot create ~/.xchat", FALSE);
+#endif
+			fe_message (_("Cannot create ~/.xchat"), FALSE);
 	}
 }
 
@@ -285,6 +310,7 @@ static struct prefs vars[] = {
 	{"background_pic", PREFS_OFFSET (background), TYPE_STR},
 	{"ban_type", PREFS_OFFINT (bantype), TYPE_INT},
 	{"beep_msg", PREFS_OFFINT (beepmsg), TYPE_BOOL},
+	{"beep_chans", PREFS_OFFINT (beepchans), TYPE_BOOL},
 	{"bluestring", PREFS_OFFSET (bluestring), TYPE_STR},
 
 	{"chanmodebuttons", PREFS_OFFINT (chanmodebuttons), TYPE_BOOL},
@@ -295,11 +321,14 @@ static struct prefs vars[] = {
 	{"ctcp_time_limit", PREFS_OFFINT (ctcp_time_limit), TYPE_INT},
 
 	{"dcc_blocksize", PREFS_OFFINT (dcc_blocksize), TYPE_INT},
+	{"dcc_ip", PREFS_OFFSET (dcc_ip_str), TYPE_STR},
 	{"dcc_permissions", PREFS_OFFINT (dccpermissions), TYPE_INT},
+	{"dcc_send_fillspaces", PREFS_OFFINT (dcc_send_fillspaces), TYPE_BOOL},
 	{"dcc_stall_timeout", PREFS_OFFINT (dccstalltimeout), TYPE_INT},
 	{"dcc_timeout", PREFS_OFFINT (dcctimeout), TYPE_INT},
 	{"dccdir", PREFS_OFFSET (dccdir), TYPE_STR},
 	{"dccwithnick", PREFS_OFFINT (dccwithnick), TYPE_BOOL},
+	{"dialog_height", PREFS_OFFINT (dialog_height), TYPE_INT},
 	{"dialog_indent_nicks", PREFS_OFFINT (dialog_indent_nicks), TYPE_BOOL},
 	{"dialog_indent_pixels", PREFS_OFFINT (dialog_indent_pixels), TYPE_INT},
 	{"dialog_show_separator", PREFS_OFFINT (dialog_show_separator), TYPE_BOOL},
@@ -308,6 +337,7 @@ static struct prefs vars[] = {
 	{"dialog_tint_green", PREFS_OFFINT (dialog_tint_green), TYPE_INT},
 	{"dialog_tint_red", PREFS_OFFINT (dialog_tint_red), TYPE_INT},
 	{"dialog_transparent", PREFS_OFFINT (dialog_transparent), TYPE_BOOL},
+	{"dialog_width", PREFS_OFFINT (dialog_width), TYPE_INT},
 	{"dialog_wordwrap", PREFS_OFFINT (dialog_wordwrap), TYPE_BOOL},
 	{"dnsprogram", PREFS_OFFSET (dnsprogram), TYPE_STR},
 	{"doubleclickuser", PREFS_OFFSET (doubleclickuser), TYPE_STR},
@@ -317,6 +347,9 @@ static struct prefs vars[] = {
 	{"first_dcc_send_port", PREFS_OFFINT (first_dcc_send_port), TYPE_INT},
 	{"font_dialog_normal", PREFS_OFFSET (dialog_font_normal), TYPE_STR},
 	{"font_normal", PREFS_OFFSET (font_normal), TYPE_STR},
+#ifdef USE_ZVT
+	{"font_shell", PREFS_OFFSET (font_shell), TYPE_STR},
+#endif
 	{"fudgeservernotice", PREFS_OFFINT (fudgeservernotice), TYPE_BOOL},
 
 #ifdef USE_HEBREW
@@ -336,7 +369,7 @@ static struct prefs vars[] = {
 	{"invisible", PREFS_OFFINT (invisible), TYPE_BOOL},
 	{"ip_from_server", PREFS_OFFINT (ip_from_server), TYPE_BOOL},
 
-	{"lagometer", PREFS_OFFINT (lagometer), TYPE_BOOL},
+	{"lagometer", PREFS_OFFINT (lagometer), TYPE_INT},
 	{"last_dcc_send_port", PREFS_OFFINT (last_dcc_send_port), TYPE_INT},
 	{"limitedtabhighlight", PREFS_OFFINT (limitedtabhighlight), TYPE_BOOL},
 	{"logging", PREFS_OFFINT (logging), TYPE_BOOL},
@@ -359,6 +392,7 @@ static struct prefs vars[] = {
 	{"nickname1", PREFS_OFFSET (nick1), TYPE_STR},
 	{"nickname2", PREFS_OFFSET (nick2), TYPE_STR},
 	{"nickname3", PREFS_OFFSET (nick3), TYPE_STR},
+	{"notices_tabs", PREFS_OFFINT (notices_tabs), TYPE_BOOL},
 	{"notify_timeout", PREFS_OFFINT (notify_timeout), TYPE_INT},
 	{"nu_color", PREFS_OFFINT (nu_color), TYPE_INT},
 
@@ -371,9 +405,11 @@ static struct prefs vars[] = {
 #endif
 	{"partreason", PREFS_OFFSET (partreason), TYPE_STR},
 	{"percascii", PREFS_OFFINT (perc_ascii), TYPE_BOOL},
+	{"perccolor", PREFS_OFFINT (perc_color), TYPE_BOOL},
 #ifdef USE_PERL
 	{"perlwarnings", PREFS_OFFINT (perlwarnings), TYPE_BOOL},
 #endif
+	{"persist_chans", PREFS_OFFINT (persist_chans), TYPE_BOOL},
 	{"pingtimeout", PREFS_OFFINT (pingtimeout), TYPE_INT},
 	{"priv_msg_tabs", PREFS_OFFINT (privmsgtab), TYPE_BOOL},
 	{"proxy_host", PREFS_OFFSET (proxy_host), TYPE_STR},
@@ -389,7 +425,6 @@ static struct prefs vars[] = {
 	{"servernotice", PREFS_OFFINT (servernotice), TYPE_BOOL},
 	{"show_away_message", PREFS_OFFINT (show_away_message), TYPE_BOOL},
 	{"show_away_once", PREFS_OFFINT (show_away_once), TYPE_BOOL},
-
 	{"show_invite_in_front_session",
 	 PREFS_OFFINT (show_invite_in_front_session), TYPE_BOOL},
 	{"show_notify_in_front_session",
@@ -399,6 +434,7 @@ static struct prefs vars[] = {
 	{"skipserverlist", PREFS_OFFINT (skipserverlist), TYPE_BOOL},
 	{"soundcmd", PREFS_OFFSET (soundcmd), TYPE_STR},
 	{"sounddir", PREFS_OFFSET (sounddir), TYPE_STR},
+	{"stamp_format", PREFS_OFFSET (stamp_format), TYPE_STR},
 	{"stripcolor", PREFS_OFFINT (stripcolor), TYPE_BOOL},
 	{"style_inputbox", PREFS_OFFINT (style_inputbox), TYPE_BOOL},
 	{"style_namelistgad", PREFS_OFFINT (style_namelistgad), TYPE_BOOL},
@@ -407,6 +443,7 @@ static struct prefs vars[] = {
 	{"tabs_position", PREFS_OFFINT (tabs_position), TYPE_INT},
 	{"thin_separator", PREFS_OFFINT (thin_separator), TYPE_BOOL},
 	{"throttle", PREFS_OFFINT (throttle), TYPE_BOOL},
+	{"throttlemeter", PREFS_OFFINT (throttlemeter), TYPE_INT},
 	{"timestamp", PREFS_OFFINT (timestamp), TYPE_BOOL},
 	{"timestamp_logs", PREFS_OFFINT (timestamp_logs), TYPE_BOOL},
 	{"tint", PREFS_OFFINT (tint), TYPE_BOOL},
@@ -427,6 +464,7 @@ static struct prefs vars[] = {
 #endif
 	{"userhost", PREFS_OFFINT (userhost), TYPE_BOOL},
 	{"userlist_sort", PREFS_OFFINT (userlist_sort), TYPE_INT},
+	{"userlist_icons", PREFS_OFFINT (userlist_icons), TYPE_BOOL},
 	{"userlistbuttons", PREFS_OFFINT (userlistbuttons), TYPE_BOOL},
 	{"username", PREFS_OFFSET (username), TYPE_STR},
 
@@ -444,6 +482,9 @@ load_config (void)
 	struct stat st;
 	char *cfg, *username;
 	int res, val, i, fh;
+#ifdef	USE_JCODE
+	gchar *locale;
+#endif
 
 	memset (&prefs, 0, sizeof (struct xchatprefs));
 	/* Just for ppl upgrading --AGL */
@@ -463,6 +504,7 @@ load_config (void)
 	prefs.ctcp_number_limit = 5;
 	prefs.topicbar = 1;
 	prefs.lagometer = 1;
+	prefs.throttlemeter = 1;
 	prefs.nickgad = 1;
 	prefs.autoopendccrecvwindow = 1;
 	prefs.autoopendccsendwindow = 1;
@@ -471,11 +513,26 @@ load_config (void)
 	prefs.userhost = 1;
 	prefs.userlistbuttons = 1;
 	prefs.hilightnick = 1;
+	prefs.persist_chans = 1;
+	prefs.userlist_icons = 1;
+	prefs.perc_color = 1;
+	prefs.dialog_width = 300;
+	prefs.dialog_height = 100;
+	prefs.dcc_send_fillspaces = 1;
+	strcpy (prefs.stamp_format, "[%H:%M:%S] ");
+
+#ifdef	USE_JCODE
+	locale = setlocale (LC_CTYPE, "");
+	if (locale != NULL && !g_strncasecmp (locale, "ja", 2))
+		prefs.kanji_conv = 1;
+#endif
 
 	check_prefs_dir ();
 	username = g_get_user_name ();
+	if (!username)
+		username = "root";
 
-	fh = open (default_file (), O_RDONLY);
+	fh = open (default_file (), OFLAGS | O_RDONLY);
 	if (fh != -1)
 	{
 		fstat (fh, &st);
@@ -508,22 +565,27 @@ load_config (void)
 
 	} else
 	{
+#ifndef WIN32
+
 #ifdef __EMX__
 		/* OS/2 uses UID 0 all the time */
-		fe_message ("The default DCC receive directory is your\n"
-						"home dir, you should change this at some stage.", TRUE);
+		fe_message (_("The default download directory is your\n"
+						"home dir, you should change this at some stage."), TRUE);
 #else
 		if (getuid () == 0)
-			fe_message ("* Running IRC as root is stupid! You should\n"
+			fe_message (_("* Running IRC as root is stupid! You should\n"
 							"  create a User Account and use that to login.\n\n"
-							"* The default DCC receive directory is your\n"
-							"  home dir, you should change this at some stage.",
+							"* The default download directory is your\n"
+							"  home dir, you should change this at some stage."),
 							TRUE);
 		else
-			fe_message ("The default DCC receive directory is your\n"
+			fe_message ("The default download directory is your\n"
 							"home dir, you should change this at some stage.", TRUE);
 #endif
+
+#endif /* !WIN32 */
 		/* put in default values, anything left out is automatically zero */
+
 		prefs.autoresume = 1;
 		prefs.show_away_once = 1;
 		prefs.show_away_message = 1;
@@ -542,12 +604,10 @@ load_config (void)
 		prefs.tabchannels = 1;
 		prefs.newtabstofront = 1;
 		prefs.use_server_tab = 1;
+		prefs.windows_as_tabs = 1;
 		prefs.privmsgtab = 1;
 		prefs.nickcompletion = 1;
 		prefs.style_inputbox = 1;
-		prefs.show_invite_in_front_session = 1;
-		prefs.show_notify_in_front_session = 1;
-		prefs.whois_on_notifyonline = 1;
 		prefs.nu_color = 4;
 		prefs.dccpermissions = 0600;
 		prefs.max_lines = 300;
@@ -568,15 +628,28 @@ load_config (void)
 		strcat (prefs.nick3, "__");
 		strcpy (prefs.realname, username);
 		strcpy (prefs.username, username);
+#ifdef WIN32
+		strcpy (prefs.sounddir, "./sound");
+		strcpy (prefs.dccdir, "./dcc");
+#else
 		strcpy (prefs.dccdir, g_get_home_dir ());
+		strcpy (prefs.sounddir, prefs.dccdir);
+#endif
 		strcpy (prefs.doubleclickuser, "/QUOTE WHOIS %s");
-		strcpy (prefs.awayreason, "I'm busy");
-		strcpy (prefs.quitreason, "Client Exiting");
+		strcpy (prefs.awayreason, _("I'm busy"));
+		strcpy (prefs.quitreason, _("Client Exiting"));
 		strcpy (prefs.partreason, prefs.quitreason);
-		strcpy (prefs.font_normal,
-				  "-b&h-lucidatypewriter-medium-r-normal-*-*-120-*-*-m-*-*-*");
+#ifdef USE_JCODE
+		prefs.use_fontset = 1;
+		strcpy (prefs.font_normal, DEF_FONT_JCODE);
+#else
+#ifdef WIN32
+		strcpy (prefs.font_normal, DEF_FONT_WIN32);
+#else
+		strcpy (prefs.font_normal, DEF_FONT);
+#endif
+#endif
 		strcpy (prefs.dialog_font_normal, prefs.font_normal);
-		strcpy (prefs.sounddir, g_get_home_dir ());
 		strcpy (prefs.soundcmd, "esdplay");
 		strcpy (prefs.dnsprogram, "host");
 	}
@@ -593,7 +666,7 @@ save_config (void)
 
 	check_prefs_dir ();
 
-	fh = open (default_file (), O_TRUNC | O_WRONLY | O_CREAT, 0600);
+	fh = open (default_file (), OFLAGS | O_TRUNC | O_WRONLY | O_CREAT, 0600);
 	if (fh == -1)
 		return 0;
 
