@@ -1,16 +1,21 @@
 /* cfgfiles.h */
 
-#ifndef cfgfiles_h
-#define cfgfiles_h
-
+extern char *xdir;
 
 char *cfg_get_str (char *cfg, char *var, char *dest);
+int cfg_get_bool (char *var);
+int cfg_get_int_with_result (char *cfg, char *var, int *result);
+int cfg_get_int (char *cfg, char *var);
+void cfg_put_int (int fh, int value, char *var);
 char *get_xdir (void);
+char *default_file (void);
 void check_prefs_dir (void);
 void load_config (void);
 int save_config (void);
 void list_free (GSList ** list);
 void list_loadconf (char *file, GSList ** list, char *defaultconf);
+int list_delentry (GSList ** list, char *name);
+void list_addentry (GSList ** list, char *cmd, char *name);
 
 #define STRUCT_OFFSET_STR(type,field) \
 ( (unsigned int) (((char *) (&(((type *) NULL)->field)))- ((char *) NULL)) )
@@ -31,5 +36,3 @@ struct prefs
 #define TYPE_STR 1
 #define TYPE_INT 2
 #define TYPE_BOOL 3
-
-#endif
