@@ -18,33 +18,19 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <stdlib.h>
-#include <fcntl.h>
 #include <unistd.h>
-#include <time.h>
+
 #include "xchat.h"
 #include "cfgfiles.h"
 #include "util.h"
-#include "popup.h"
 #include "plugin.h"
-#include "../common/util.h"
-
-
-extern void xchat_exec (char *cmd);
-extern int handle_command (char *cmd, void *sess, int history, int nocommand);
-extern int tcp_send (struct server *serv, char *buf);
-extern void channel_action (struct session *sess, char *tbuf, char *chan,
-									 char *from, char *text, int fromme);
-extern void handle_dcc (struct session *sess, char *outbuf, char *nick,
-								char *word[], char *word_eol[]);
-extern struct session *find_session_from_channel (char *chan,
-																  struct server *serv);
-extern int auto_insert (char *dest, char *src, char *word[], char *word_eol[],
-								char *a, char *c, char *d, char *h, char *n, char *s);
-
-
-extern GSList *ctcp_list;
-extern struct xchatprefs prefs;
+#include "util.h"
+#include "modes.h"
+#include "outbound.h"
+#include "inbound.h"
+#include "dcc.h"
+#include "ctcp.h"
+#include "xchatc.h"
 
 
 static void
@@ -97,7 +83,7 @@ handle_ctcp (struct session *sess, char *outbuf, char *to, char *nick,
 	{
 		sprintf (outbuf,
 					"NOTICE %s :\001VERSION xchat "VERSION
-					" %s\001\r\n", nick, get_cpu_str (0));
+					" %s\001\r\n", nick, get_cpu_str ());
 		tcp_send (sess->server, outbuf);
 	}
 
@@ -137,7 +123,7 @@ handle_ctcp (struct session *sess, char *outbuf, char *to, char *nick,
 	po = strchr (msg, '\001');
 	if (po)
 		po[0] = 0;
-	if (!is_channel (to))
+	if (!is_channel (sess->server, to))
 	{
 		EMIT_SIGNAL (XP_TE_CTCPGEN, sess->server->front_session, msg, nick,
 						 NULL, NULL, 0);
