@@ -17,58 +17,48 @@
  *
  * Wayne Conrad, 3 Apr 1999: Color-coded DCC file transfer status windows
  * Bernhard Valenti <bernhard.valenti@gmx.net> 2000-11-21: Fixed DCC send behind nat
+ *
+ * 2001-03-08 Added support for getting "dcc_ip" config parameter.
+ * Jim Seymour (jseymour@LinxNet.com)
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include "xchat.h"
-#include "util.h"
-#include <netdb.h>
 #include <time.h>
 #include <errno.h>
 #include <sys/stat.h>
-#include <fcntl.h>
 #include <unistd.h>
-#include <time.h>
+#include <fcntl.h>
+
+#define WANTSOCKET
+#define WANTARPA
+#include "inet.h"
+
+#include "xchat.h"
+#include "perlc.h"
+#include "util.h"
 #include "plugin.h"
 #include "fe.h"
+#include "outbound.h"
+#include "inbound.h"
+#include "network.h"
+#include "text.h"
+#include "xchatc.h"
+
 
 char *dcctypes[] = { "SEND", "RECV", "CHAT", "CHAT" };
 
 struct dccstat_info dccstat[] = {
-	{"Queued", 1 /*black */ },
-	{"Active", 12 /*cyan */ },
-	{"Failed", 4 /*red */ },
-	{"Done", 3 /*green */ },
-	{"Connect", 1 /*black */ },
-	{"Aborted", 4 /*red */ },
+	{N_("Waiting"), 1 /*black */ },
+	{N_("Active"), 12 /*cyan */ },
+	{N_("Failed"), 4 /*red */ },
+	{N_("Done"), 3 /*green */ },
+	{N_("Connect"), 1 /*black */ },
+	{N_("Aborted"), 4 /*red */ },
 };
 
-extern GSList *sess_list;
-extern GSList *serv_list;
-extern GSList *dcc_list;
-extern struct xchatprefs prefs;
-
-extern int tcp_send_len (struct server *serv, char *buf, int len);
-extern int tcp_send (struct server *serv, char *buf);
-extern char *errorstring (int err);
-extern void private_msg (struct server *serv, char *tbuf, char *from,
-								 char *ip, char *text);
-extern void PrintText (struct session *sess, char *text);
-extern void channel_action (struct session *sess, char *tbuf, char *chan,
-									 char *from, char *text, int fromme);
-extern struct session *find_session_from_channel (char *chan,
-																  struct server *serv);
-/* trans */
-extern void serv2user (unsigned char *s);
-extern void user2serv (unsigned char *s);
-/* ~trans */
-
-#ifdef USE_PERL
-/* perl.c */
-int perl_dcc_chat (struct session *sess, struct server *serv, char *buf);
-#endif
+static struct DCC *new_dcc (void);
 
 
 static void
@@ -155,18 +145,19 @@ static int
 dcc_connect_sok (struct DCC *dcc)
 {
 	int sok;
+	struct sockaddr_in addr;
 
 	sok = socket (AF_INET, SOCK_STREAM, 0);
 	if (sok == -1)
 		return -1;
 
-	dcc->SAddr.sin_port = htons (dcc->port);
-	dcc->SAddr.sin_family = AF_INET;
-	dcc->SAddr.sin_addr.s_addr = dcc->addr;
+	memset (&addr, 0, sizeof (addr));
+	addr.sin_port = htons (dcc->port);
+	addr.sin_family = AF_INET;
+	addr.sin_addr.s_addr = htonl (dcc->addr);
 
-	fcntl (sok, F_SETFL, O_NONBLOCK);
-	connect (sok, (struct sockaddr *) &dcc->SAddr,
-				sizeof (struct sockaddr_in));
+	set_nonblocking (sok);
+	connect (sok, (struct sockaddr *) &addr, sizeof (addr));
 
 	return sok;
 }
@@ -193,26 +184,38 @@ void
 dcc_close (struct DCC *dcc, int dccstat, int destroy)
 {
 	char type = dcc->type;
+
+	if (dcc->wiotag)
+	{
+		fe_input_remove (dcc->wiotag);
+		dcc->wiotag = 0;
+	}
+
+	if (dcc->iotag)
+	{
+		fe_input_remove (dcc->iotag);
+		dcc->iotag = 0;
+	}
+
 	if (dcc->sok != -1)
 	{
-		if (dcc->wiotag != -1)
-			fe_input_remove (dcc->wiotag);
-		fe_input_remove (dcc->iotag);
-		close (dcc->sok);
+		closesocket (dcc->sok);
 		dcc->sok = -1;
-		dcc->wiotag = -1;
 	}
+
 	if (dcc->fp != -1)
 	{
 		close (dcc->fp);
 		dcc->fp = -1;
 	}
+
 	dcc->dccstat = dccstat;
 	if (dcc->dccchat)
 	{
 		free (dcc->dccchat);
-		dcc->dccchat = 0;
+		dcc->dccchat = NULL;
 	}
+
 	if (destroy)
 	{
 		dcc_list = g_slist_remove (dcc_list, dcc);
@@ -225,6 +228,7 @@ dcc_close (struct DCC *dcc, int dccstat, int destroy)
 		update_dcc_window (type);
 		return;
 	}
+
 	switch (type)
 	{
 	case TYPE_SEND:
@@ -255,7 +259,7 @@ dcc_notify_kill (struct server *serv)
 	}
 }
 
-struct sockaddr_in *
+struct DCC *
 dcc_write_chat (char *nick, char *text)
 {
 #ifdef USE_TRANS
@@ -300,33 +304,90 @@ dcc_write_chat (char *nick, char *text)
 #endif
 		send (dcc->sok, "\n", 1, 0);
 		fe_dcc_update_chat_win ();
-		return &dcc->SAddr;
+		return dcc;
 	}
 	return 0;
 }
 
-static void
-dcc_read_chat (struct DCC *dcc, gint sok)
+/* returns: 0 - ok
+				1 - the dcc is closed! */
+
+static int
+dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 {
-	int i, skip, len;
+#ifdef USE_PERL
+	int skip;
+	char *host_n_nick_n_message;
+	GSList *list = dcc_list;
+
+	host_n_nick_n_message = malloc (strlen (dcc->nick) + strlen (line) + 26);
+
+	sprintf (host_n_nick_n_message, "%s %d %s: %s",
+				net_ip (dcc->addr), dcc->port, dcc->nick, line);
+
+	skip = perl_dcc_chat (find_session_from_channel (dcc->nick, dcc->serv),
+								 dcc->serv, host_n_nick_n_message);
+	free (host_n_nick_n_message);
+
+	/* Perl code may have closed the chat. */
+	while (list) 
+	{
+		if (((struct DCC *)list->data) == dcc) break;
+		list = list->next;
+	}
+	if (list == 0)
+		return 1;
+
+	if (skip)
+		return 0;
+#endif
+	fe_checkurl (line);
+
+	if (line[0] == 1 && !strncasecmp (line + 1, "ACTION", 6))
+	{
+		session *sess;
+		char *po = strchr (line + 8, '\001');
+		if (po)
+			po[0] = 0;
+		sess = find_session_from_channel (dcc->nick, dcc->serv);
+		if (!sess)
+			sess = dcc->serv->front_session;
+		channel_action (sess, tbuf, dcc->serv->nick, dcc->nick,
+							 line + 8, FALSE);
+	} else
+	{
+#ifdef USE_TRANS
+		if (prefs.use_trans)
+			serv2user (line);
+#endif
+		private_msg (dcc->serv, tbuf, dcc->nick, "", line);
+	}
+	return 0;
+}
+
+static gboolean
+dcc_read_chat (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
+{
+	int i, len, dead;
 	char tbuf[1226];
 	char lbuf[1026];
+	unsigned char *temp;
 
 	while (1)
 	{
-		len = recv (sok, lbuf, sizeof (lbuf) - 2, 0);
+		len = recv (dcc->sok, lbuf, sizeof (lbuf) - 2, 0);
 		if (len < 1)
 		{
 			if (len < 0)
 			{
-				if (errno == EAGAIN || errno == EWOULDBLOCK)
-					return;
+				if (would_block_again ())
+					return TRUE;
 			}
 			sprintf (tbuf, "%d", dcc->port);
 			EMIT_SIGNAL (XP_TE_DCCCHATF, dcc->serv->front_session, dcc->nick,
-							 inet_ntoa (dcc->SAddr.sin_addr), tbuf, NULL, 0);
+							 net_ip (dcc->addr), tbuf, NULL, 0);
 			dcc_close (dcc, STAT_FAILED, FALSE);
-			return;
+			return TRUE;
 		}
 		i = 0;
 		lbuf[len] = 0;
@@ -338,60 +399,20 @@ dcc_read_chat (struct DCC *dcc, gint sok)
 				break;
 			case '\n':
 				dcc->dccchat->linebuf[dcc->dccchat->pos] = 0;
-#ifdef USE_PERL
+
+				if (prefs.stripcolor)
 				{
-					char *host_n_nick_n_message;
-					GSList *list = dcc_list;
-
-					host_n_nick_n_message =
-						malloc (strlen (dcc->nick) +
-								  strlen (dcc->dccchat->linebuf) + 26);
-					sprintf (host_n_nick_n_message, "%s %d %s: %s",
-								inet_ntoa (dcc->SAddr.sin_addr), dcc->port, dcc->nick,
-								dcc->dccchat->linebuf);
-					skip =
-						perl_dcc_chat (find_session_from_channel
-											(dcc->nick, dcc->serv), dcc->serv,
-											host_n_nick_n_message);
-					free (host_n_nick_n_message);
-
-					/* Perl code may have closed the chat. */
-					while (list) 
-					{
-						if (((struct DCC *)list->data) == dcc) break;
-						list = list->next;
-					}
-					if (list == 0) return;
-				}
-#else
-				skip = 0;
-#endif
-				fe_checkurl (dcc->dccchat->linebuf);
-
-				if (!skip)
+					temp = strip_color (dcc->dccchat->linebuf);
+					dead = dcc_chat_line (dcc, temp, tbuf);
+					free (temp);
+				} else
 				{
-					if (dcc->dccchat->linebuf[0] == 1
-						 && !strncasecmp (dcc->dccchat->linebuf + 1, "ACTION", 6))
-					{
-						session *sess;
-						char *po = strchr (dcc->dccchat->linebuf + 8, '\001');
-						if (po)
-							po[0] = 0;
-						sess = find_session_from_channel (dcc->nick, dcc->serv);
-						if (!sess)
-							sess = dcc->serv->front_session;
-						channel_action (sess, tbuf, dcc->serv->nick, dcc->nick,
-											 dcc->dccchat->linebuf + 8, FALSE);
-					} else
-					{
-#ifdef USE_TRANS
-						if (prefs.use_trans)
-							serv2user (dcc->dccchat->linebuf);
-#endif
-						private_msg (dcc->serv, tbuf, dcc->nick, "",
-										 dcc->dccchat->linebuf);
-					}
+					dead = dcc_chat_line (dcc, dcc->dccchat->linebuf, tbuf);
 				}
+
+				if (dead) /* the dcc has been closed, don't use (DCC *)! */
+					return TRUE;
+
 				dcc->pos += dcc->dccchat->pos;
 				dcc->dccchat->pos = 0;
 				fe_dcc_update_chat_win ();
@@ -406,8 +427,8 @@ dcc_read_chat (struct DCC *dcc, gint sok)
 	}
 }
 
-static void
-dcc_read (struct DCC *dcc, gint sok)
+static gboolean
+dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 {
 	char buf[4096];
 	guint32 pos;
@@ -446,7 +467,7 @@ dcc_read (struct DCC *dcc, gint sok)
 		EMIT_SIGNAL (XP_TE_DCCFILEERR, dcc->serv->front_session, dcc->destfile,
 						 NULL, NULL, NULL, 0);
 		dcc_close (dcc, STAT_FAILED, FALSE);
-		return;
+		return TRUE;
 	}
 	while (1)
 	{
@@ -455,13 +476,13 @@ dcc_read (struct DCC *dcc, gint sok)
 		{
 			if (n < 0)
 			{
-				if (errno == EAGAIN || errno == EWOULDBLOCK)
-					return;
+				if (would_block_again ())
+					return TRUE;
 			}
 			EMIT_SIGNAL (XP_TE_DCCRECVERR, dcc->serv->front_session, dcc->file,
 							 dcc->destfile, dcc->nick, NULL, 0);
 			dcc_close (dcc, STAT_FAILED, FALSE);
-			return;
+			return TRUE;
 		}
 
 		write (dcc->fp, buf, n);
@@ -479,35 +500,45 @@ dcc_read (struct DCC *dcc, gint sok)
 			dcc_close (dcc, STAT_DONE, FALSE);
 			EMIT_SIGNAL (XP_TE_DCCRECVCOMP, dcc->serv->front_session,
 							 dcc->file, dcc->destfile, dcc->nick, buf, 0);
-			return;
+			return TRUE;
 		}
 	}
 }
 
-static void
-dcc_connect_finished (struct DCC *dcc, gint sok)
+static gboolean
+dcc_connect_finished (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 {
-	int er;
+	int er, sok = dcc->sok;
 	char host[128];
+	struct sockaddr_in addr;
 
 	fe_input_remove (dcc->iotag);
+	dcc->iotag = 0;
 
-	if (connect
-		 (sok, (struct sockaddr *) &dcc->SAddr,
-		  sizeof (struct sockaddr_in)) < 0)
+	memset (&addr, 0, sizeof (addr));
+	addr.sin_port = htons (dcc->port);
+	addr.sin_family = AF_INET;
+	addr.sin_addr.s_addr = htonl (dcc->addr);
+
+	/* check if it's already connected */
+	if (connect (sok, (struct sockaddr *) &addr, sizeof (addr)) != 0)
 	{
-		er = errno;
+		er = sock_error ();
+#ifndef WIN32
 		if (er != EISCONN)
+#else
+		if (er != WSAEISCONN)
+#endif
 		{
 			EMIT_SIGNAL (XP_TE_DCCCONFAIL, dcc->serv->front_session,
 							 dcctypes[(int) dcc->type], dcc->nick, errorstring (er),
 							 NULL, 0);
 			dcc->dccstat = STAT_FAILED;
 			update_dcc_window (dcc->type);
-			return;
+			return TRUE;
 		}
 	}
-	fcntl (sok, F_SETFL, O_NONBLOCK);
+
 	dcc->dccstat = STAT_ACTIVE;
 	switch (dcc->type)
 	{
@@ -525,10 +556,11 @@ dcc_connect_finished (struct DCC *dcc, gint sok)
 	dcc->starttime = time (0);
 	dcc->lasttime = dcc->starttime;
 
-	snprintf (host, sizeof host, "%s:%d", inet_ntoa (dcc->SAddr.sin_addr),
-				 dcc->port);
+	snprintf (host, sizeof host, "%s:%d", net_ip (dcc->addr), dcc->port);
 	EMIT_SIGNAL (XP_TE_DCCCON, dcc->serv->front_session,
 					 dcctypes[(int) dcc->type], dcc->nick, host, "to", 0);
+
+	return TRUE;
 }
 
 static void
@@ -551,11 +583,11 @@ dcc_connect (struct DCC *dcc)
 		fe_dcc_update_chat_win ();
 }
 
-static void
-dcc_send_data (struct DCC *dcc, gint sok)
+static gboolean
+dcc_send_data (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 {
 	char *buf;
-	int len, sent;
+	int len, sent, sok = dcc->sok;
 
 	if (prefs.dcc_blocksize < 1) /* this is too little! */
 		prefs.dcc_blocksize = 1024;
@@ -565,22 +597,22 @@ dcc_send_data (struct DCC *dcc, gint sok)
 
 	buf = malloc (prefs.dcc_blocksize);
 	if (!buf)
-		return;
+		return TRUE;
 
 	lseek (dcc->fp, dcc->pos, SEEK_SET);
 	len = read (dcc->fp, buf, prefs.dcc_blocksize);
-	if (len < 0)
+	if (len < 1)
 		goto abortit;
 	sent = send (sok, buf, len, 0);
 
-	if (sent < 0 && errno != EWOULDBLOCK)
+	if (send < 0 && !(would_block ()))
 	{
 abortit:
 		EMIT_SIGNAL (XP_TE_DCCSENDFAIL, dcc->serv->front_session,
 						 file_part (dcc->file), dcc->nick, NULL, NULL, 0);
 		dcc_close (dcc, STAT_FAILED, FALSE);
 		free (buf);
-		return;
+		return TRUE;
 	}
 	if (sent > 0)
 	{
@@ -593,38 +625,41 @@ abortit:
 	if (dcc->pos >= dcc->size)
 	{
 		/* it's all sent now, so remove the WRITE/SEND handler */
-		if (dcc->wiotag != -1)
+		if (dcc->wiotag)
 		{
 			fe_input_remove (dcc->wiotag);
-			dcc->wiotag = -1;
+			dcc->wiotag = 0;
 		}
 	}
 
 	free (buf);
+
+	return TRUE;
 }
 
-static void
-dcc_read_ack (struct DCC *dcc, gint sok)
+static gboolean
+dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 {
 	int len;
 	guint32 ack;
 	char buf[16];
+	int sok = dcc->sok;
 
 	len = recv (sok, (char *) &ack, 4, MSG_PEEK);
 	if (len < 1)
 	{
 		if (len < 0)
 		{
-			if (errno == EWOULDBLOCK)
-				return;
+			if (would_block_again ())
+				return TRUE;
 		}
 		EMIT_SIGNAL (XP_TE_DCCSENDFAIL, dcc->serv->front_session,
 						 file_part (dcc->file), dcc->nick, NULL, NULL, 0);
 		dcc_close (dcc, STAT_FAILED, FALSE);
-		return;
+		return TRUE;
 	}
 	if (len < 4)
-		return;
+		return TRUE;
 	recv (sok, (char *) &ack, 4, 0);
 	dcc->ack = ntohl (ack);
 
@@ -637,8 +672,8 @@ dcc_read_ack (struct DCC *dcc, gint sok)
 	if (!dcc->fastsend)
 	{
 		if (dcc->ack < dcc->pos)
-			return;
-		dcc_send_data (dcc, sok);
+			return TRUE;
+		dcc_send_data (NULL, 0, (gpointer)dcc);
 	}
 
 	if (dcc->pos >= dcc->size && dcc->ack >= dcc->size)
@@ -648,10 +683,12 @@ dcc_read_ack (struct DCC *dcc, gint sok)
 		EMIT_SIGNAL (XP_TE_DCCSENDCOMP, dcc->serv->front_session,
 						 file_part (dcc->file), dcc->nick, buf, NULL, 0);
 	}
+
+	return TRUE;
 }
 
-static void
-dcc_accept (struct DCC *dcc, gint sokk)
+static gboolean
+dcc_accept (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 {
 	char host[128];
 	struct sockaddr_in CAddr;
@@ -661,19 +698,17 @@ dcc_accept (struct DCC *dcc, gint sokk)
 	len = sizeof (CAddr);
 	sok = accept (dcc->sok, (struct sockaddr *) &CAddr, &len);
 	fe_input_remove (dcc->iotag);
-	close (dcc->sok);
+	dcc->iotag = 0;
+	closesocket (dcc->sok);
 	if (sok < 0)
 	{
-		dcc->iotag = -1;
 		dcc->sok = -1;
 		dcc_close (dcc, STAT_FAILED, FALSE);
-		return;
+		return TRUE;
 	}
-	fcntl (sok, F_SETFL, O_NONBLOCK);
+	set_nonblocking (sok);
 	dcc->sok = sok;
 	dcc->addr = ntohl (CAddr.sin_addr.s_addr);
-	memcpy ((char *) &dcc->SAddr.sin_addr, (char *) &CAddr.sin_addr,
-			  sizeof (struct in_addr));
 	dcc->dccstat = STAT_ACTIVE;
 	dcc->lasttime = dcc->starttime = time (0);
 	dcc->fastsend = prefs.fastdccsend;
@@ -683,7 +718,7 @@ dcc_accept (struct DCC *dcc, gint sokk)
 		if (dcc->fastsend)
 			dcc->wiotag = fe_input_add (sok, 0, 1, 0, dcc_send_data, dcc);
 		dcc->iotag = fe_input_add (sok, 1, 0, 1, dcc_read_ack, dcc);
-		dcc_send_data (dcc, dcc->sok);
+		dcc_send_data (NULL, 0, (gpointer)dcc);
 		break;
 
 	case TYPE_CHATSEND:
@@ -695,10 +730,11 @@ dcc_accept (struct DCC *dcc, gint sokk)
 
 	update_dcc_window (dcc->type);
 
-	snprintf (host, sizeof host, "%s:%d", inet_ntoa (dcc->SAddr.sin_addr),
-				 dcc->port);
+	snprintf (host, sizeof host, "%s:%d", net_ip (dcc->addr), dcc->port);
 	EMIT_SIGNAL (XP_TE_DCCCON, dcc->serv->front_session,
 					 dcctypes[(int) dcc->type], dcc->nick, host, "from", 0);
+
+	return TRUE;
 }
 
 static int
@@ -738,7 +774,8 @@ dcc_listen_init (struct DCC *dcc, session *sess)
 		while ((prefs.last_dcc_send_port > ntohs(SAddr.sin_port)) &&
 				(bindretval == -1))
 		{
-			SAddr.sin_port = htons (prefs.first_dcc_send_port+i++);
+			SAddr.sin_port = htons (prefs.first_dcc_send_port + i);
+			i++;
 			/*printf("Trying to bind against port: %d\n",ntohs(SAddr.sin_port));*/
 			bindretval = bind (dcc->sok, (struct sockaddr *) &SAddr, sizeof (SAddr));
 		}
@@ -761,20 +798,23 @@ dcc_listen_init (struct DCC *dcc, session *sess)
 	len = sizeof (SAddr);
 	getsockname (dcc->sok, (struct sockaddr *) &SAddr, &len);
 
-	memcpy ((char *) &dcc->SAddr, (char *) &SAddr, sizeof (struct sockaddr_in));
 	dcc->port = ntohs (SAddr.sin_port);
 
 	/*if we have a dcc_ip, we use that, so the remote client can connect*/
 	/*we would tell the client to connect to our LAN ip otherwise*/
-	if (prefs.dcc_ip == 0)
-		dcc->addr = ntohl (my_addr);
-	/*else we use the one we bound to*/
+	if (prefs.ip_from_server != 0 && prefs.dcc_ip != 0)
+		dcc->addr = prefs.dcc_ip;
+	else if (prefs.dcc_ip_str[0])
+		dcc->addr = inet_addr ((const char *) prefs.dcc_ip_str);
 	else
-		dcc->addr = ntohl (prefs.dcc_ip);
-	
-	fcntl (dcc->sok, F_SETFL, O_NONBLOCK);
+	/*else we use the one we bound to*/
+		dcc->addr = my_addr;
+
+	dcc->addr = ntohl (dcc->addr);
+
+	set_nonblocking (dcc->sok);
 	listen (dcc->sok, 1);
-	fcntl (dcc->sok, F_SETFL, 0);
+	set_blocking (dcc->sok);
 
 	dcc->iotag = fe_input_add (dcc->sok, 1, 0, 1, dcc_accept, dcc);
 
@@ -834,22 +874,39 @@ dcc_send (struct session *sess, char *tbuf, char *to, char *file)
 				{
 					if (dcc_listen_init (dcc, sess))
 					{
+						char havespaces = 0;
 						file = dcc->file;
 						while (*file)
 						{
 							if (*file == ' ')
-								*file = '_';
+							{
+								if (prefs.dcc_send_fillspaces)
+						    		*file = '_';
+							  	else
+							   	havespaces = 1;
+							}
 							file++;
 						}
 						dcc->nick = strdup (to);
 						if (prefs.autoopendccsendwindow)
-							fe_dcc_open_send_win ();
+							fe_dcc_open_send_win (TRUE);
 						else
 							fe_dcc_update_send_win ();
-						snprintf (tbuf, 400,
+						
+						if (havespaces)
+						{
+							snprintf (tbuf, 400,
+									 "PRIVMSG %s :\001DCC SEND \"%s\" %lu %d %d\001\r\n",
+									 to, file_part (dcc->file), dcc->addr, dcc->port,
+									 dcc->size);
+						}
+						else
+						{
+							snprintf (tbuf, 400,
 									 "PRIVMSG %s :\001DCC SEND %s %lu %d %d\001\r\n",
 									 to, file_part (dcc->file), dcc->addr, dcc->port,
 									 dcc->size);
+						}
 						tcp_send (sess->server, tbuf);
 						EMIT_SIGNAL (XP_TE_DCCOFFER, sess, file_part (dcc->file),
 										 to, NULL, NULL, 0);
@@ -862,7 +919,7 @@ dcc_send (struct session *sess, char *tbuf, char *to, char *file)
 			}
 		}
 	}
-	snprintf (tbuf, 400, "Cannot access %s\n", dcc->file);
+	snprintf (tbuf, 400, _("Cannot access %s\n"), dcc->file);
 	PrintText (sess, tbuf);
 	dcc_close (dcc, 0, TRUE);
 }
@@ -891,7 +948,7 @@ find_dcc (char *nick, char *file, int type)
 	while (list)
 	{
 		dcc = (struct DCC *) list->data;
-		if (!strcasecmp (nick, dcc->nick))
+		if (nick == NULL || !strcasecmp (nick, dcc->nick))
 		{
 			if (type == -1 || dcc->type == type)
 			{
@@ -978,7 +1035,7 @@ dcc_get_nick (struct session *sess, char *nick)
 		EMIT_SIGNAL (XP_TE_DCCIVAL, sess, NULL, NULL, NULL, NULL, 0);
 }
 
-struct DCC *
+static struct DCC *
 new_dcc (void)
 {
 	struct DCC *dcc = malloc (sizeof (struct DCC));
@@ -987,7 +1044,6 @@ new_dcc (void)
 	memset (dcc, 0, sizeof (struct DCC));
 	dcc->sok = -1;
 	dcc->fp = -1;
-	dcc->wiotag = -1;
 	dcc_list = g_slist_prepend (dcc_list, dcc);
 	return (dcc);
 }
@@ -1032,16 +1088,15 @@ dcc_chat (struct session *sess, char *nick)
 	dcc = new_dcc ();
 	if (!dcc)
 		return;
-	dcc->offertime = time (0);
+	dcc->starttime = dcc->offertime = time (0);
 	dcc->serv = sess->server;
 	dcc->dccstat = STAT_QUEUED;
 	dcc->type = TYPE_CHATSEND;
-	time (&dcc->starttime);
 	dcc->nick = strdup (nick);
 	if (dcc_listen_init (dcc, sess))
 	{
 		if (prefs.autoopendccchatwindow)
-			fe_dcc_open_chat_win ();
+			fe_dcc_open_chat_win (TRUE);
 		else
 			fe_dcc_update_chat_win ();
 		snprintf (outbuf, sizeof (outbuf),
@@ -1069,9 +1124,18 @@ dcc_resume (struct DCC *dcc)
 
 	if (dcc->dccstat == STAT_QUEUED && dcc->resumable)
 	{
-		snprintf (tbuf, sizeof (tbuf),
-					 "PRIVMSG %s :\001DCC RESUME %s %d %d\001\r\n", dcc->nick,
-					 dcc->file, dcc->port, dcc->resumable);
+		if (strchr (dcc->file, ' ') != NULL)
+		{	/*filename contains spaces*/
+			snprintf (tbuf, sizeof (tbuf),
+						 "PRIVMSG %s :\001DCC RESUME \"%s\" %d %d\001\r\n", dcc->nick,
+						 dcc->file, dcc->port, dcc->resumable);
+		}
+		else
+		{
+			snprintf (tbuf, sizeof (tbuf),
+						 "PRIVMSG %s :\001DCC RESUME %s %d %d\001\r\n", dcc->nick,
+						 dcc->file, dcc->port, dcc->resumable);
+		}
 		tcp_send (dcc->serv, tbuf);
 	}
 }
@@ -1083,13 +1147,12 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 	struct DCC *dcc;
 	char *type = word[5];
 	int port, size;
-	unsigned long addr;			  /* FIXME for ipv6 */
+	unsigned long addr;
 
 	if (!strcasecmp (type, "CHAT"))
 	{
 		port = atoi (word[8]);
 		sscanf (word[7], "%lu", &addr);
-		addr = ntohl (addr);
 
 		if (!addr || port < 1024)
 		{
@@ -1118,13 +1181,13 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 			EMIT_SIGNAL (XP_TE_DCCCHATOFFER, sess->server->front_session, nick,
 							 NULL, NULL, NULL, 0);
 			if (prefs.autoopendccchatwindow)
-				fe_dcc_open_chat_win ();
+				fe_dcc_open_chat_win (TRUE);
 			else
 				fe_dcc_update_chat_win ();
 			if (prefs.autodccchat)
 				dcc_connect (dcc);
-			return;
 		}
+		return;
 	}
 	if (!strcasecmp (type, "RESUME"))
 	{
@@ -1135,19 +1198,30 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 		if (dcc)
 		{
 			dcc->resumable = atoi (word[8]);
-			dcc->pos = dcc->resumable;
-			dcc->ack = dcc->resumable;
-			lseek (dcc->fp, dcc->pos, SEEK_SET);
-			snprintf (outbuf, 400,
-						 "PRIVMSG %s :\001DCC ACCEPT %s %d %d\001\r\n", dcc->nick,
-						 file_part (dcc->file), port, dcc->resumable);
-			tcp_send (dcc->serv, outbuf);
+			if (dcc->resumable < dcc->size)
+			{
+				dcc->pos = dcc->resumable;
+				dcc->ack = dcc->resumable;
+				lseek (dcc->fp, dcc->pos, SEEK_SET);
+				if (strchr (file_part (dcc->file), ' ') != NULL)
+				{	/*filename contains spaces*/
+					snprintf (outbuf, 400,
+								 "PRIVMSG %s :\001DCC ACCEPT \"%s\" %d %d\001\r\n", dcc->nick,
+								 file_part (dcc->file), port, dcc->resumable);
+				}
+				else
+				{
+					snprintf (outbuf, 400,
+								 "PRIVMSG %s :\001DCC ACCEPT %s %d %d\001\r\n", dcc->nick,
+								 file_part (dcc->file), port, dcc->resumable);
+				}
+				tcp_send (dcc->serv, outbuf);
+			}
 			sprintf (outbuf, "%d", dcc->pos);
-
 			EMIT_SIGNAL (XP_TE_DCCRESUMEREQUEST, sess, nick,
 							 file_part (dcc->file), outbuf, NULL, 0);
-			return;
 		}
+		return;
 	}
 	if (!strcasecmp (type, "ACCEPT"))
 	{
@@ -1166,7 +1240,6 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 		size = atoi (word[9]);
 
 		sscanf (word[7], "%lu", &addr);
-		addr = ntohl (addr);
 
 		if (!addr || !size || port < 1024)
 		{
@@ -1194,24 +1267,26 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 			}
 			strcat (dcc->destfile, file);
 
+			dcc->resumable = 0;
 			if (stat (dcc->destfile, &st) != -1)
-				dcc->resumable = st.st_size;
-			else
-				dcc->resumable = 0;
-			if (st.st_size == size)
-				dcc->resumable = 0;
+			{
+				if (st.st_size < size)
+					dcc->resumable = st.st_size;
+			}
+
 			dcc->pos = dcc->resumable;
 			dcc->serv = sess->server;
 			dcc->type = TYPE_RECV;
 			dcc->dccstat = STAT_QUEUED;
 			dcc->addr = addr;
-			dcc->SAddr.sin_addr.s_addr = addr;
 			dcc->port = port;
 			dcc->size = size;
 			dcc->nick = strdup (nick);
 			if (prefs.autodccsend)
 			{
-				if (prefs.autoresume && dcc->resumable)
+				/* don't resume the same file from two people! */
+				if (prefs.autoresume && dcc->resumable &&
+					 !find_dcc (NULL, dcc->file, TYPE_RECV))
 				{
 					dcc_resume (dcc);
 				} else
@@ -1222,7 +1297,7 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 				}
 			}
 			if (prefs.autoopendccrecvwindow)
-				fe_dcc_open_recv_win ();
+				fe_dcc_open_recv_win (TRUE);
 			else
 				fe_dcc_update_recv_win ();
 		}
@@ -1241,20 +1316,18 @@ dcc_show_list (struct session *sess, char *outbuf)
 	struct DCC *dcc;
 	GSList *list = dcc_list;
 
-	PrintText (sess,
-				  "\0038,2 Type  To/From    Status  Size    Pos     File    \017\n"
-				  "\002\0039--------------------------------------------------\017\n");
+	EMIT_SIGNAL (XP_TE_DCCHEAD, sess, NULL, NULL, NULL, NULL, 0);
 	while (list)
 	{
 		dcc = (struct DCC *) list->data;
 		i++;
 		snprintf (outbuf, 255, " %-5.5s %-10.10s %-7.7s %-7d %-7d %s\n",
 					 dcctypes[(int) dcc->type], dcc->nick,
-					 dccstat[(int) dcc->dccstat].name, dcc->size, dcc->pos,
+					 _(dccstat[(int) dcc->dccstat].name), dcc->size, dcc->pos,
 					 file_part (dcc->file));
 		PrintText (sess, outbuf);
 		list = list->next;
 	}
 	if (!i)
-		PrintText (sess, "No active DCCs\n");
+		PrintText (sess, _("No active DCCs\n"));
 }
