@@ -24,6 +24,9 @@ int fe_is_chanwindow (struct server *serv);
 void fe_add_chan_list (struct server *serv, char *chan, char *users,
 							  char *topic);
 void fe_chan_list_end (struct server *serv);
+int fe_is_banwindow (struct session *sess);
+void fe_add_ban_list (struct session *sess, char *mask, char *who, char *when);
+void fe_ban_list_end (struct session *sess);
 void fe_notify_update (char *name);
 void fe_text_clear (struct session *sess);
 void fe_close_window (struct session *sess);
@@ -35,15 +38,14 @@ void fe_userlist_remove (struct session *sess, struct User *user);
 void fe_userlist_move (struct session *sess, struct User *user, int new_row);
 void fe_userlist_numbers (struct session *sess);
 void fe_userlist_clear (struct session *sess);
-void fe_userlist_hide (struct session *sess);
 void fe_dcc_update_recv_win (void);
 void fe_dcc_update_send_win (void);
 void fe_dcc_update_chat_win (void);
 void fe_dcc_update_send (struct DCC *dcc);
 void fe_dcc_update_recv (struct DCC *dcc);
-void fe_dcc_open_recv_win (void);
-void fe_dcc_open_send_win (void);
-void fe_dcc_open_chat_win (void);
+void fe_dcc_open_recv_win (int passive);
+void fe_dcc_open_send_win (int passive);
+void fe_dcc_open_chat_win (int passive);
 void fe_clear_channel (struct session *sess);
 void fe_session_callback (struct session *sess);
 void fe_server_callback (struct server *serv);
@@ -59,6 +61,9 @@ void fe_change_nick (struct server *serv, char *nick, char *newnick);
 void fe_ignore_update (int level);
 void fe_beep (void);
 int fe_is_confmode (struct session *);
+int fe_is_beep (struct session *);
 void fe_lastlog (session *sess, session *lastlog_sess, char *sstr);
 void fe_set_lag (server *serv, int lag);
+void fe_set_throttle (server *serv);
 void fe_set_away (server *serv);
+int fe_open_serverlist (session *sess, int auto_connect, int dont_show);
