@@ -21,88 +21,26 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <sys/types.h>
-#include <sys/time.h>
 #include <unistd.h>
-#include <netdb.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
 #include <time.h>
+
+#define WANTARPA
+#define WANTDNS
+#include "inet.h"
+
 #include "xchat.h"
 #include "util.h"
 #include "ignore.h"
 #include "plugin.h"
+#include "perlc.h"
 #include "fe.h"
-
-/* xchat.c */
-
-extern session *menu_sess;
-
-extern int tcp_send_len (struct server *serv, char *buf, int len);
-extern int tcp_send (struct server *serv, char *buf);
-extern struct session *find_session_from_nick (char *nick,
-															  struct server *serv);
-extern struct session *find_session_from_channel (char *chan,
-																  struct server *serv);
-extern struct session *new_session (struct server *serv, char *from);
-struct away_msg *find_away_message (struct server *serv, char *nick);
-void save_away_message (struct server *serv, char *nick, char *msg);
-
-/* ignore.c */
-
-extern int flood_check (char *nick, char *ip, struct server *serv, struct session *sess, int what);
-
-/* text.c */
-
-extern unsigned char *strip_color (unsigned char *text);
-extern void PrintText (struct session *sess, char *text);
-extern void setup_logging (struct session *sess);
-
-/* dcc.c */
-
-extern void dcc_change_nick (struct server *serv, char *oldnick,
-									  char *newnick);
-
-/* userlist.c */
-
-extern void change_nick (struct session *sess, char *oldnick, char *newnick);
-
-/* outbound.c */
-
-extern int handle_command (char *cmd, struct session *sess, int history,
-									int nocommand);
-extern void process_data_init (unsigned char *buf, char *cmd, char *word[],
-										 char *word_eol[], int handle_quotes);
-
-/* notify.c */
-
-extern void notify_markonline (struct session *sess, char *outbuf,
-										 char *word[]);
-
-/* perl.c */
-
-int perl_inbound (struct session *sess, struct server *serv, char *buf, char *msg_type);
-
-/* text.c */
-
-extern void end_logging (int fd);
-
-/* ctcp.c */
-
-extern void handle_ctcp (struct session *sess, char *outbuf, char *to,
-								 char *nick, char *msg, char *word[],
-								 char *word_eol[]);
-
-
-extern GSList *sess_list;
-extern struct xchatprefs prefs;
-extern struct session *current_tab;
-
-#define find_word_to_end(a, b) word_eol[b]
-#define find_word(a, b) word[b]
-
-
-void channel_msg (struct server *serv, char *outbuf, char *chan, char *from,
-						char *text, char fromme);
+#include "modes.h"
+#include "notify.h"
+#include "outbound.h"
+#include "inbound.h"
+#include "text.h"
+#include "ctcp.h"
+#include "xchatc.h"
 
 
 /* black n white(0/1) are bad colors for nicks, and we'll use color 2 for us */
@@ -125,16 +63,75 @@ color_of (char *name)
 void
 clear_channel (struct session *sess)
 {
+	if (prefs.persist_chans)
+		safe_strcpy (sess->waitchannel, sess->channel, CHANLEN);
 	sess->channel[0] = 0;
+
 	if (sess->logfd != -1)
 	{
 		end_logging (sess->logfd);
 		sess->logfd = -1;
 	}
+
+	free (sess->current_modes);
+	sess->current_modes = strdup ("");
+
+	if (sess->mode_timeout_tag)
+	{
+		fe_timeout_remove (sess->mode_timeout_tag);
+		sess->mode_timeout_tag = 0;
+	}
+
 	fe_clear_channel (sess);
+	clear_user_list (sess);
+	fe_set_nonchannel (sess, FALSE);
 	fe_set_title (sess);
 }
 
+void
+set_topic(struct session *sess, char *topic)
+{
+	if (sess->topic)
+		free(sess->topic);
+	sess->topic = strdup(topic);
+	fe_set_topic(sess, topic);
+}
+
+static session *
+find_session_from_nick (char *nick, server *serv)
+{
+	session *sess;
+	GSList *list = sess_list;
+
+	sess = find_session_from_channel (nick, serv);
+	if (sess)
+		return sess;
+
+	if (serv->front_session)
+	{
+		if (find_name (serv->front_session, nick))
+			return serv->front_session;
+	}
+
+	if (menu_sess && menu_sess->server == serv)
+	{
+		if (find_name (menu_sess, nick))
+			return menu_sess;
+	}
+
+	while (list)
+	{
+		sess = list->data;
+		if (sess->server == serv)
+		{
+			if (find_name (sess, nick))
+				return sess;
+		}
+		list = list->next;
+	}
+	return 0;
+}
+
 void
 private_msg (struct server *serv, char *tbuf, char *from, char *ip,
 				 char *text)
@@ -144,22 +141,34 @@ private_msg (struct server *serv, char *tbuf, char *from, char *ip,
 	if (EMIT_SIGNAL (XP_PRIVMSG, serv, from, ip, text, NULL, 0) == 1)
 		return;
 
-	if (prefs.beepmsg)
-		fe_beep ();
 	sess = find_session_from_channel (from, serv);
+
+	if (((sess) && fe_is_beep (sess)) || prefs.beepmsg)
+		fe_beep ();
+
 	if (sess || prefs.autodialog)
 	{
 		/*0=ctcp  1=priv will set autodialog=0 here is flud detected */
 		if (!sess)
 		{
 			if (flood_check (from, ip, serv, menu_sess, 1))
-				sess = new_session (serv, from);	/* Create a dialog session */
+				sess = new_ircwindow (serv, from, SESS_DIALOG);	/* Create a dialog session */
 			else
 				sess = serv->front_session;
 		}
+		if (prefs.logging
+			&& sess->logfd != -1
+			&& ip
+			&& *ip
+			&& (!sess->topic || strcmp(sess->topic, ip)))
+		{
+			char temp[512];
+			snprintf(temp, sizeof(temp), "[%s has address %s]\n", from, ip);
+			write(sess->logfd, temp, strlen(temp));
+		}
 		channel_msg (serv, tbuf, from, from, text, FALSE);
 		if (ip && ip[0])
-			fe_set_topic (sess, ip);
+			set_topic (sess, ip);
 		return;
 	}
 	sess = find_session_from_nick (from, serv);
@@ -179,6 +188,9 @@ SearchNick (char *text, char *nicks)
 
 	if (nicks == NULL)
 		return 0;
+
+	text = strip_color (text);
+
 	safe_strcpy (S, nicks, NICKLEN);
 	n = strtok (S, ",");
 	while (n != NULL)
@@ -188,13 +200,17 @@ SearchNick (char *text, char *nicks)
 		while ((p = nocasestrstr (t, n)))
 		{
 			if ((p == text || !isalnum (*(p - 1))) && !isalnum (*(p + ns)))
+			{
+				free (text);
 				return 1;
+			}
 
 			t = p + 1;
 		}
 
 		n = strtok (NULL, ",");
 	}
+	free (text);
 	return 0;
 }
 
@@ -218,34 +234,40 @@ is_hilight (char *from, char *text, char *chan, session *sess, server *serv)
 }
 
 void
-channel_action (struct session *sess, char *tbuf, char *chan, char *from,
+channel_action (session *sess, char *tbuf, char *chan, char *from,
 					 char *text, int fromme)
 {
 	int hilight = FALSE;
-	struct session *def = sess;
+	session *def = sess;
+	server *serv = sess->server;
 
 	if (EMIT_SIGNAL (XP_CHANACTION, sess, chan, from, text, NULL, fromme) == 1)
 		return;
 
-	if (is_channel (chan) || fromme)
-		sess = find_session_from_channel (chan, sess->server);
+	if (is_channel (serv, chan) || fromme)
+	{
+		sess = find_session_from_channel (chan, serv);
+		if (!fromme && sess)
+			if (fe_is_beep (sess) || prefs.beepchans)
+				fe_beep ();
+	}
 	else
 	{
         	/* it's a private action! */
-		sess = find_session_from_channel (from, sess->server);
-		if (prefs.beepmsg)
+		sess = find_session_from_channel (from, serv);
+		if (((sess) && fe_is_beep (sess)) || prefs.beepmsg)
 			fe_beep ();
 	}
 
-	if (!sess && !is_channel (chan) && prefs.autodialog)
-		sess = new_session (def->server, from);
+	if (!sess && !is_channel (serv, chan) && prefs.autodialog)
+		sess = new_ircwindow (serv, from, SESS_DIALOG);
 
 	if (!sess)
 		sess = def;
 
 	if (!fromme)
 	{
-		switch (is_hilight (from, text, chan, sess, sess->server))
+		switch (is_hilight (from, text, chan, sess, serv))
 		{
 		case 2:
 			return;	/* plugin ate the event */
@@ -276,6 +298,7 @@ channel_msg (struct server *serv, char *outbuf, char *chan, char *from,
 	struct User *user;
 	struct session *sess;
 	int hilight = FALSE;
+	char nickchar[2] = "\000";
 
 	sess = find_session_from_channel (chan, serv);
 	if (!sess)
@@ -285,7 +308,12 @@ channel_msg (struct server *serv, char *outbuf, char *chan, char *from,
 
 	user = find_name (sess, from);
 	if (user)
+	{
+		nickchar[0] = user->prefix;
+		if (nickchar[0] == ' ')
+			nickchar[0] = 0;
 		user->lasttalk = time (0);
+	}
 
 	if (EMIT_SIGNAL (XP_CHANMSG, serv, chan, from, text, NULL, fromme) == 1)
 		return;
@@ -302,23 +330,26 @@ channel_msg (struct server *serv, char *outbuf, char *chan, char *from,
 	}
 
 	if (fromme)
-		EMIT_SIGNAL (XP_TE_UCHANMSG, sess, from, text, NULL, NULL, 0);
-	else if (sess->is_dialog)
-		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, NULL, NULL, 0);
+		EMIT_SIGNAL (XP_TE_UCHANMSG, sess, from, text, nickchar, NULL, 0);
+	else if (sess->type == SESS_DIALOG)
+		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, nickchar, NULL, 0);
 	else if (hilight)
-		EMIT_SIGNAL (XP_TE_HCHANMSG, sess, from, text, NULL, NULL, 0);
+		EMIT_SIGNAL (XP_TE_HCHANMSG, sess, from, text, nickchar, NULL, 0);
 	else if (prefs.colorednicks)
 	{
 		sprintf (outbuf, "\003%d%s", color_of (from), from);
-		EMIT_SIGNAL (XP_TE_CHANMSG, sess, outbuf, text, NULL, NULL, 0);
+		EMIT_SIGNAL (XP_TE_CHANMSG, sess, outbuf, text, nickchar, NULL, 0);
 	}
 	else
-		EMIT_SIGNAL (XP_TE_CHANMSG, sess, from, text, NULL, NULL, 0);
+		EMIT_SIGNAL (XP_TE_CHANMSG, sess, from, text, nickchar, NULL, 0);
+
+	if (!fromme && sess->type != SESS_DIALOG)
+		if (fe_is_beep (sess) || prefs.beepchans)
+			fe_beep ();
 }
 
 void
-user_new_nick (struct server *serv, char *outbuf, char *nick, char *newnick,
-					int quiet)
+user_new_nick (struct server *serv, char *nick, char *newnick, int quiet)
 {
 	int me;
 	struct session *sess;
@@ -358,8 +389,8 @@ user_new_nick (struct server *serv, char *outbuf, char *nick, char *newnick,
 			{
 				safe_strcpy (sess->channel, newnick, CHANLEN);
 				fe_set_channel (sess);
-				fe_set_title (sess);
 			}
+			fe_set_title (sess);
 		}
 		list = list->next;
 	}
@@ -380,8 +411,12 @@ find_unused_session (server *serv)
 	while (list)
 	{
 		sess = (session *) list->data;
-		if (!sess->is_server && sess->channel[0] == 0 && sess->server == serv)
-			return sess;
+		if (sess->type == SESS_CHANNEL && sess->channel[0] == 0 &&
+			 sess->server == serv)
+		{
+			if (!prefs.persist_chans || sess->waitchannel[0] == 0)
+				return sess;
+		}
 		list = list->next;
 	}
 	return 0;
@@ -395,7 +430,7 @@ find_session_from_waitchannel (char *chan, struct server *serv)
 	while (list)
 	{
 		sess = (struct session *) list->data;
-		if (sess->server == serv && sess->channel[0] == 0 && !sess->is_shell)
+		if (sess->server == serv && sess->channel[0] == 0 && sess->type == SESS_CHANNEL)
 		{
 			if (!strcasecmp (chan, sess->waitchannel))
 				return sess;
@@ -406,20 +441,24 @@ find_session_from_waitchannel (char *chan, struct server *serv)
 }
 
 static void
-you_joined (struct server *serv, char *outbuf, char *chan, char *nick,
-				char *ip)
+you_joined (server *serv, char *outbuf, char *chan, char *nick, char *ip)
 {
-	struct session *sess;
+	session *sess;
 
-	/* see if a window is waiting to join this channel */
-	sess = find_session_from_waitchannel (chan, serv);
+	/* already joined? probably a bnc */
+	sess = find_session_from_channel (chan, serv);
 	if (!sess)
 	{
-		/* find a "<none>" tab and use that */
-		sess = find_unused_session (serv);
+		/* see if a window is waiting to join this channel */
+		sess = find_session_from_waitchannel (chan, serv);
 		if (!sess)
-			/* last resort, open a new tab/window */
-			sess = new_session (serv, 0);
+		{
+			/* find a "<none>" tab and use that */
+			sess = find_unused_session (serv);
+			if (!sess)
+				/* last resort, open a new tab/window */
+				sess = new_ircwindow (serv, NULL, SESS_CHANNEL);
+		}
 	}
 
 	safe_strcpy (sess->channel, chan, CHANLEN);
@@ -458,7 +497,6 @@ you_kicked (struct server *serv, char *tbuf, char *chan, char *kicker,
 	struct session *sess = find_session_from_channel (chan, serv);
 	if (sess)
 	{
-		fe_set_nonchannel (sess, FALSE);
 		EMIT_SIGNAL (XP_TE_UKICK, sess, serv->nick, chan, kicker, reason, 0);
 		clear_channel (sess);
 		if (prefs.autorejoin)
@@ -474,13 +512,11 @@ you_kicked (struct server *serv, char *tbuf, char *chan, char *kicker,
 }
 
 static void
-you_parted (struct server *serv, char *tbuf, char *chan, char *ip,
-				char *reason)
+you_parted (struct server *serv, char *chan, char *ip, char *reason)
 {
 	struct session *sess = find_session_from_channel (chan, serv);
 	if (sess)
 	{
-		fe_set_nonchannel (sess, FALSE);
 		if (*reason)
 			EMIT_SIGNAL (XP_TE_UPARTREASON, sess, serv->nick, ip, chan, reason,
 							 0);
@@ -491,7 +527,7 @@ you_parted (struct server *serv, char *tbuf, char *chan, char *ip,
 }
 
 static void
-names_list (struct server *serv, char *tbuf, char *chan, char *names)
+names_list (server *serv, char *chan, char *names)
 {
 	struct session *sess;
 	char name[NICKLEN];
@@ -537,7 +573,7 @@ names_list (struct server *serv, char *tbuf, char *chan, char *names)
 }
 
 static void
-topic (struct server *serv, char *tbuf, char *buf)
+topic (server *serv, char *buf)
 {
 	session *sess;
 	char *po, *new_topic;
@@ -550,7 +586,7 @@ topic (struct server *serv, char *tbuf, char *buf)
 		if (sess)
 		{
 			new_topic = strip_color (po + 2);
-			fe_set_topic (sess, new_topic);
+			set_topic (sess, new_topic);
 			free (new_topic);
 			EMIT_SIGNAL (XP_TE_TOPIC, sess, buf, po + 2, NULL, NULL, 0);
 		}
@@ -558,20 +594,18 @@ topic (struct server *serv, char *tbuf, char *buf)
 }
 
 static void
-new_topic (struct server *serv, char *tbuf, char *nick, char *chan,
-			  char *topic)
+new_topic (server *serv, char *nick, char *chan, char *topic)
 {
 	struct session *sess = find_session_from_channel (chan, serv);
 	if (sess)
 	{
-		fe_set_topic (sess, topic);
+		set_topic (sess, topic);
 		EMIT_SIGNAL (XP_TE_NEWTOPIC, sess, nick, topic, chan, NULL, 0);
 	}
 }
 
 static void
-user_joined (struct server *serv, char *outbuf, char *chan, char *user,
-				 char *ip)
+user_joined (server *serv, char *chan, char *user, char *ip)
 {
 	struct session *sess = find_session_from_channel (chan, serv);
 
@@ -587,8 +621,7 @@ user_joined (struct server *serv, char *outbuf, char *chan, char *user,
 }
 
 static void
-user_kicked (struct server *serv, char *outbuf, char *chan, char *user,
-				 char *kicker, char *reason)
+user_kicked (server *serv, char *chan, char *user, char *kicker, char *reason)
 {
 	struct session *sess = find_session_from_channel (chan, serv);
 	if (sess)
@@ -617,7 +650,7 @@ user_parted (struct server *serv, char *chan, char *user, char *ip,
 }
 
 static void
-channel_date (struct session *sess, char *tbuf, char *chan, char *timestr)
+channel_date (session *sess, char *chan, char *timestr)
 {
 	long n = atol (timestr);
 	char *tim = ctime (&n);
@@ -626,8 +659,7 @@ channel_date (struct session *sess, char *tbuf, char *chan, char *timestr)
 }
 
 static void
-topic_nametime (struct server *serv, char *tbuf, char *chan, char *nick,
-					 char *date)
+topic_nametime (server *serv, char *chan, char *nick, char *date)
 {
 	long n = atol (date);
 	char *tim = ctime (&n);
@@ -653,7 +685,7 @@ set_server_name (struct server *serv, char *name)
 			fe_set_title (sess);
 		list = list->next;
 	}
-	if (serv->front_session->is_server)
+	if (serv->front_session->type == SESS_SERVER)
 	{
 		safe_strcpy (serv->front_session->channel, name, CHANLEN);
 		fe_set_channel (serv->front_session);
@@ -661,34 +693,38 @@ set_server_name (struct server *serv, char *name)
 }
 
 static void
-user_quit (struct server *serv, char *outbuf, char *nick, char *reason)
+user_quit (server *serv, char *nick, char *ip, char *reason)
 {
 	GSList *list = sess_list;
 	struct session *sess;
+	int was_on_front_session = FALSE;
 
 	while (list)
 	{
 		sess = (struct session *) list->data;
 		if (sess->server == serv)
 		{
+ 			if (sess == menu_sess)
+ 				was_on_front_session = TRUE;
 			if (sub_name (sess, nick))
 			{
 				if (!fe_is_confmode (sess))
-					EMIT_SIGNAL (XP_TE_QUIT, sess, nick, reason, NULL, NULL, 0);
-			} else if (sess->is_dialog && !strcasecmp (sess->channel, nick))
+					EMIT_SIGNAL (XP_TE_QUIT, sess, nick, reason, ip, NULL, 0);
+			} else if (sess->type == SESS_DIALOG && !strcasecmp (sess->channel, nick))
 			{
-				EMIT_SIGNAL (XP_TE_QUIT, sess, nick, reason, NULL, NULL, 0);
+				EMIT_SIGNAL (XP_TE_QUIT, sess, nick, reason, ip, NULL, 0);
 			}
 		}
 		list = list->next;
 	}
+
+	notify_set_offline (serv, nick, was_on_front_session);
 }
 
 static void
 got_ping_reply (struct session *sess, char *outbuf,
 					 char *timestring, char *from)
 {
-	struct timeval timev;
 	unsigned long tim, nowtim, dif;
 	int lag = 0;
 
@@ -698,9 +734,8 @@ got_ping_reply (struct session *sess, char *outbuf,
 		lag = 1;
 	}
 
-	gettimeofday (&timev, 0);
 	sscanf (timestring, "%lu", &tim);
-	nowtim = (timev.tv_sec - 50000) * 1000000 + timev.tv_usec;
+	nowtim = make_ping_time ();
 	dif = nowtim - tim;
 
 	sess->server->ping_recv = time (0);
@@ -726,19 +761,66 @@ got_ping_reply (struct session *sess, char *outbuf,
 	}
 }
 
+static session *
+find_session_from_type (int type, server *serv)
+{
+	session *sess;
+	GSList *list = sess_list;
+	while (list)
+	{
+		sess = list->data;
+		if (sess->type == type && serv == sess->server)
+			return sess;
+		list = list->next;
+	}
+	return 0;
+}
+
 static void
 notice (struct server *serv, char *outbuf, char *to, char *nick, char *msg,
 		  char *ip)
 {
-	char *po;
+	char *po,*ptr=to;
 	struct session *sess = 0;
 
-	if (is_channel (to))
-		sess = find_session_from_channel (to, serv);
+	if (is_channel (serv, ptr))
+		sess = find_session_from_channel (ptr, serv);
 
+	if (!sess && ptr[0] == '@')
+	{
+		ptr++;
+		sess = find_session_from_channel (ptr, serv);
+	}
+	
 	if (!sess)
 	{
-		sess = find_session_from_nick (nick, serv);
+		ptr = 0;
+		if (prefs.notices_tabs) {
+			int stype = strcmp(nick, ip) ? SESS_NOTICES : SESS_SNOTICES;
+			sess = find_session_from_type (stype, serv);
+			if (!sess)
+			{
+				register unsigned int oldh = prefs.hideuserlist;
+				prefs.hideuserlist = 1;
+				if (stype == SESS_NOTICES)
+					sess = new_ircwindow (serv, "(notices)", SESS_NOTICES);
+				else
+					sess = new_ircwindow (serv, "(snotices)", SESS_SNOTICES);
+				prefs.hideuserlist = oldh;
+				fe_set_channel (sess);
+				fe_set_title (sess);
+				fe_set_nonchannel (sess, FALSE);
+				clear_user_list (sess);
+				if (prefs.logging)
+					setup_logging (sess);
+			}
+			/* Avoid redundancy with some Undernet notices */
+			if (!strncmp (msg, "*** Notice -- ", 14))
+				msg += 14;
+		} else
+		{
+			sess = find_session_from_nick (nick, serv);
+		}
 		if (!sess)
 			sess = serv->front_session;
 	}
@@ -755,11 +837,14 @@ notice (struct server *serv, char *outbuf, char *to, char *nick, char *msg,
 	po = strchr (msg, '\001');
 	if (po)
 		po[0] = 0;
-	EMIT_SIGNAL (XP_TE_NOTICE, sess, nick, msg, NULL, NULL, 0);
+	if (ptr)
+		EMIT_SIGNAL (XP_TE_CHANNOTICE, sess, nick, to, msg, NULL, 0);
+	else
+		EMIT_SIGNAL (XP_TE_NOTICE, sess, nick, msg, NULL, NULL, 0);
 }
 
 static void
-handle_away (struct server *serv, char *outbuf, char *nick, char *msg)
+handle_away (server *serv, char *nick, char *msg)
 {
 	struct away_msg *away = find_away_message (serv, nick);
 	struct session *sess = NULL;
@@ -781,280 +866,8 @@ handle_away (struct server *serv, char *outbuf, char *nick, char *msg)
 	EMIT_SIGNAL (XP_TE_WHOIS5, sess, nick, msg, NULL, NULL, 0);
 }
 
-static void
-channel_mode (struct server *serv, char *outbuf, char *chan, char *nick,
-				  char sign, char mode, char *extra, int quiet)
-{
-	struct session *sess = find_session_from_channel (chan, serv);
-	char tbuf[2][2];
-
-	if (sess)
-	{
-		switch (sign)
-		{
-		case '+':
-			switch (mode)
-			{
-			case 'k':
-				if (EMIT_SIGNAL (XP_CHANSETKEY, sess, chan, nick, extra, NULL, 0)
-					 == 1)
-					return;
-				strncpy (sess->channelkey, extra, 60);
-				fe_update_channel_key (sess);
-				fe_update_mode_buttons (sess, mode, sign);
-				if (!quiet)
-					EMIT_SIGNAL (XP_TE_CHANSETKEY, sess, nick, extra, NULL, NULL,
-									 0);
-				return;
-			case 'l':
-				if (EMIT_SIGNAL
-					 (XP_CHANSETLIMIT, sess, chan, nick, extra, NULL, 0) == 1)
-					return;
-				sess->limit = atoi (extra);
-				fe_update_channel_limit (sess);
-				fe_update_mode_buttons (sess, mode, sign);
-				if (!quiet)
-					EMIT_SIGNAL (XP_TE_CHANSETLIMIT, sess, nick, extra, NULL,
-									 NULL, 0);
-				return;
-			case 'o':
-				if (EMIT_SIGNAL (XP_CHANOP, sess, chan, nick, extra, NULL, 0) ==
-					 1)
-					return;
-				ul_op_name (sess, extra);
-				if (!quiet)
-					EMIT_SIGNAL (XP_TE_CHANOP, sess, nick, extra, NULL, NULL, 0);
-				return;
-			case 'h':
-				if (EMIT_SIGNAL (XP_CHANHOP, sess, chan, nick, extra, NULL, 0) ==
-					 1)
-					return;
-				ul_hop_name (sess, extra);
-				if (!quiet)
-					EMIT_SIGNAL (XP_TE_CHANHOP, sess, nick, extra, NULL, NULL, 0);
-				return;
-			case 'v':
-				if (EMIT_SIGNAL (XP_CHANVOICE, sess, chan, nick, extra, NULL, 0)
-					 == 1)
-					return;
-				voice_name (sess, extra);
-				if (!quiet)
-					EMIT_SIGNAL (XP_TE_CHANVOICE, sess, nick, extra, NULL, NULL,
-									 0);
-				return;
-			case 'b':
-				if (EMIT_SIGNAL (XP_CHANBAN, sess, chan, nick, extra, NULL, 0) ==
-					 1)
-					return;
-				if (!quiet)
-					EMIT_SIGNAL (XP_TE_CHANBAN, sess, nick, extra, NULL, NULL, 0);
-				return;
-			case 'e':
-				if (EMIT_SIGNAL (XP_CHANEXEMPT, sess, chan, nick, extra, NULL, 0)
-					 == 1)
-					return;
-				if (!quiet)
-					EMIT_SIGNAL (XP_TE_CHANEXEMPT, sess, nick, extra, NULL, NULL,
-									 0);
-				return;
-			case 'I':
-				if (EMIT_SIGNAL (XP_CHANINVITE, sess, chan, nick, extra, NULL, 0)
-					 == 1)
-					return;
-				if (!quiet)
-					EMIT_SIGNAL (XP_TE_CHANINVITE, sess, nick, extra, NULL, NULL,
-									 0);
-				return;
-			}
-			break;
-		case '-':
-			switch (mode)
-			{
-			case 'k':
-				if (EMIT_SIGNAL (XP_CHANRMKEY, sess, chan, nick, NULL, NULL, 0) ==
-					 1)
-					return;
-				sess->channelkey[0] = 0;
-				fe_update_channel_key (sess);
-				fe_update_mode_buttons (sess, mode, sign);
-				if (!quiet)
-					EMIT_SIGNAL (XP_TE_CHANRMKEY, sess, nick, NULL, NULL, NULL, 0);
-				return;
-			case 'l':
-				if (EMIT_SIGNAL (XP_CHANRMLIMIT, sess, chan, nick, NULL, NULL, 0)
-					 == 1)
-					return;
-				sess->limit = 0;
-				fe_update_channel_limit (sess);
-				fe_update_mode_buttons (sess, mode, sign);
-				if (!quiet)
-					EMIT_SIGNAL (XP_TE_CHANRMLIMIT, sess, nick, NULL, NULL, NULL,
-									 0);
-				return;
-			case 'o':
-				if (EMIT_SIGNAL (XP_CHANDEOP, sess, chan, nick, extra, NULL, 0) ==
-					 1)
-					return;
-				deop_name (sess, extra);
-				if (!quiet)
-					EMIT_SIGNAL (XP_TE_CHANDEOP, sess, nick, extra, NULL, NULL, 0);
-				return;
-			case 'h':
-				if (EMIT_SIGNAL (XP_CHANDEHOP, sess, chan, nick, extra, NULL, 0)
-					 == 1)
-					return;
-				dehop_name (sess, extra);
-				if (!quiet)
-					EMIT_SIGNAL (XP_TE_CHANDEHOP, sess, nick, extra, NULL, NULL,
-									 0);
-				return;
-			case 'v':
-				if (EMIT_SIGNAL (XP_CHANDEVOICE, sess, chan, nick, extra, NULL, 0)
-					 == 1)
-					return;
-				devoice_name (sess, extra);
-				if (!quiet)
-					EMIT_SIGNAL (XP_TE_CHANDEVOICE, sess, nick, extra, NULL,
-									 NULL, 0);
-				return;
-			case 'b':
-				if (EMIT_SIGNAL (XP_CHANUNBAN, sess, chan, nick, extra, NULL, 0)
-					 == 1)
-					return;
-				if (!quiet)
-					EMIT_SIGNAL (XP_TE_CHANUNBAN, sess, nick, extra, NULL, NULL,
-									 0);
-				return;
-			case 'e':
-				if (EMIT_SIGNAL
-					 (XP_CHANRMEXEMPT, sess, chan, nick, extra, NULL, 0) == 1)
-					return;
-				if (!quiet)
-					EMIT_SIGNAL (XP_TE_CHANRMEXEMPT, sess, nick, extra, NULL, NULL,
-									 0);
-				return;
-			case 'I':
-				if (EMIT_SIGNAL
-					 (XP_CHANRMINVITE, sess, chan, nick, extra, NULL, 0) == 1)
-					return;
-				if (!quiet)
-					EMIT_SIGNAL (XP_TE_CHANRMINVITE, sess, nick, extra, NULL, NULL,
-									 0);
-				return;
-			}
-			break;
-		}
-
-		if (!quiet)
-		{
-			tbuf[0][0] = sign;
-			tbuf[0][1] = 0;
-			tbuf[1][0] = mode;
-			tbuf[1][1] = 0;
-			if (extra && *extra)
-				EMIT_SIGNAL (XP_TE_CHANMODEGEN, sess, nick, tbuf[0], tbuf[1],
-								 extra, 0);
-			else
-				EMIT_SIGNAL (XP_TE_CHANMODEGEN, sess, nick, tbuf[0], tbuf[1],
-								 extra, 0);
-		}
-		fe_update_mode_buttons (sess, mode, sign);
-	} else
-	{
-		if (!quiet)
-		{
-			tbuf[0][0] = sign;
-			tbuf[0][1] = 0;
-			tbuf[1][0] = mode;
-			tbuf[1][1] = 0;
-
-			EMIT_SIGNAL (XP_TE_CHANMODEGEN, serv->front_session, nick, tbuf[0],
-							 tbuf[1], chan, 0);
-		}
-	}
-}
-
-static void
-channel_modes (struct server *serv, char *outbuf, char *word[], char *nick,
-					int displacement)
-{
-	char *chan = find_word (pdibuf, 3 + displacement);
-	if (*chan)
-	{
-		char *modes = find_word (pdibuf, 4 + displacement);
-		if (*modes)
-		{
-			int i = 5 + displacement;
-			char sign;
-			if (*modes == ':')
-				modes++;				  /* not a channel mode, eg +i */
-			sign = modes[0];
-			modes++;
-			if (prefs.raw_modes && !displacement)
-				displacement = 1;
-			while (1)
-			{
-				if (sign == '+')
-				{
-				 stupidcode:
-					switch (modes[0])
-					{
-					case '-':
-						modes++;
-						goto stupidcode;
-					case 'h':
-					case 'o':
-					case 'v':
-					case 'k':
-					case 'l':
-					case 'b':
-					case 'e':
-					case 'I':
-						channel_mode (serv, outbuf, chan, nick, sign, modes[0],
-										  word[i], displacement);
-						i++;
-						break;
-					default:
-						channel_mode (serv, outbuf, chan, nick, sign, modes[0], "",
-										  displacement);
-					}
-				} else
-				{
-					switch (modes[0])
-					{
-					case 'h':
-					case 'o':
-					case 'v':
-					case 'k':
-					case 'b':
-					case 'e':
-					case 'I':
-						channel_mode (serv, outbuf, chan, nick, sign, modes[0],
-										  word[i], displacement);
-						i++;
-						break;
-					default:
-						channel_mode (serv, outbuf, chan, nick, sign, modes[0], "",
-										  displacement);
-					}
-				}
-				modes++;
-				if (modes[0] == 0 || modes[0] == ' ')
-					return;
-				if (modes[0] == '-' || modes[0] == '+')
-				{
-					sign = modes[0];
-					modes++;
-					if (modes[0] == 0 || modes[0] == ' ')
-						return;
-				}
-			}
-		}
-	}
-}
-
 static int
-end_of_names (struct server *serv, char *outbuf, char *chan, char *text)
+end_of_names (server *serv, char *chan)
 {
 	struct session *sess;
 	GSList *list;
@@ -1087,6 +900,7 @@ end_of_names (struct server *serv, char *outbuf, char *chan, char *text)
 static void
 check_willjoin_channels (struct server *serv, char *tbuf)
 {
+	char *po;
 	struct session *sess;
 	GSList *list = sess_list;
 	while (list)
@@ -1096,7 +910,7 @@ check_willjoin_channels (struct server *serv, char *tbuf)
 		{
 			if (sess->willjoinchannel[0] != 0)
 			{
-				safe_strcpy (sess->waitchannel, sess->willjoinchannel, CHANLEN);
+				strcpy (sess->waitchannel, sess->willjoinchannel);
 				sess->willjoinchannel[0] = 0;
 				if (sess->channelkey[0] == '\0')
 					sprintf (tbuf, "JOIN %s\r\n", sess->waitchannel);
@@ -1104,6 +918,9 @@ check_willjoin_channels (struct server *serv, char *tbuf)
 					sprintf (tbuf, "JOIN %s %s\r\n", sess->waitchannel,
 								sess->channelkey);
 				tcp_send (serv, tbuf);
+				po = strchr (sess->waitchannel, ',');
+				if (po)
+					*po = 0;
 			}
 		}
 		list = list->next;
@@ -1195,9 +1012,9 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 	switch (n)
 	{
 	case 1:
-		user_new_nick (serv, outbuf, serv->nick, word[3], TRUE);
+		user_new_nick (serv, serv->nick, word[3], TRUE);
 		set_server_name (serv, word_eol[1]);
-		if (sess->is_server && prefs.logging)
+		if (sess->type == SESS_SERVER && prefs.logging)
 			setup_logging (sess);
 		goto def;
 	case 4:	/* check the ircd type */
@@ -1212,21 +1029,25 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 			serv->six_modes = TRUE;			/* allow 6 modes per line */
 		}
 		goto def;
+	case 5:
+		handle_005 (serv, word);
+		goto def;
 	case 301:
-		handle_away (serv, outbuf, find_word (pdibuf, 4),
-						 find_word_to_end (buf, 5) + 1);
+		handle_away (serv, word[4],
+				(word_eol[5][0] == ':') ? word_eol[5] + 1 : word_eol[5]);
 		break;
 	case 303:
 		word[4]++;
-		notify_markonline (sess, outbuf, word);
+		notify_markonline (serv, word);
 		break;
 	case 305:
 		serv->is_away = FALSE;
+		serv->reconnect_away = FALSE;
 		fe_set_away (serv);
 		goto def;
 	case 306:
 		serv->is_away = TRUE;
-		serv->away_time = time (0);
+		serv->away_time = time (NULL);
 		fe_set_away (serv);
 		goto def;
 	case 312:
@@ -1237,25 +1058,25 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 		/* FALL THROUGH */
 	case 314:
 		EMIT_SIGNAL (XP_TE_WHOIS1, sess,
-						 find_word (pdibuf, 4), find_word (pdibuf, 5),
-						 find_word (pdibuf, 6), find_word_to_end (buf, 8) + 1, 0);
+						 word[4], word[5],
+						 word[6], word_eol[8] + 1, 0);
 		break;
 	case 317:
 		{
-			long n = atol (find_word (pdibuf, 6));
-			long idle = atol (find_word (pdibuf, 5));
+			long n = atol (word[6]);
+			long idle = atol (word[5]);
 			char *tim;
 			sprintf (outbuf, "%02ld:%02ld:%02ld", idle / 3600, (idle / 60) % 60,
 						idle % 60);
 			if (n == 0)
 				EMIT_SIGNAL (XP_TE_WHOIS4, serv->front_session,
-								 find_word (pdibuf, 4), outbuf, NULL, NULL, 0);
+								 word[4], outbuf, NULL, NULL, 0);
 			else
 			{
 				tim = ctime (&n);
 				tim[19] = 0; 	/* get rid of the \n */
 				EMIT_SIGNAL (XP_TE_WHOIS4T, serv->front_session,
-								 find_word (pdibuf, 4), outbuf, tim, NULL, 0);
+								 word[4], outbuf, tim, NULL, 0);
 			}
 		}
 		break;
@@ -1271,20 +1092,20 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 		break;
 	case 321:
 		if (!fe_is_chanwindow (sess->server))
-			PrintText (sess, "\037Channel          Users   Topic\017\n");
+			EMIT_SIGNAL(XP_TE_CHANLISTHEAD, sess, NULL, NULL, NULL, NULL, 0);
 		break;
 	case 322:
 		if (fe_is_chanwindow (sess->server))
 		{
-			fe_add_chan_list (sess->server, find_word (pdibuf, 4),
-									find_word (pdibuf, 5),
-									find_word_to_end (buf, 6) + 1);
+			fe_add_chan_list (sess->server, word[4],
+									word[5],
+									word_eol[6] + 1);
 		} else
 		{
 			sprintf (outbuf, "%-16.16s %-7d %s\017\n",
-						find_word (pdibuf, 4),
-						atoi (find_word (pdibuf, 5)),
-						find_word_to_end (buf, 6) + 1);
+						word[4],
+						atoi (word[5]),
+						word_eol[6] + 1);
 			PrintText (sess, outbuf);
 		}
 		break;
@@ -1313,7 +1134,7 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 		fe_update_mode_buttons (sess, 'm', '-');
 		fe_update_mode_buttons (sess, 'l', '-');
 		fe_update_mode_buttons (sess, 'k', '-');
-		channel_modes (serv, outbuf, word, word[3], 1);
+		handle_mode (serv, outbuf, word, word_eol, "", TRUE);
 		break;
 	case 329:
 		sess = find_session_from_channel (word[4], serv);
@@ -1322,20 +1143,19 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 			if (sess->ignore_date)
 				sess->ignore_date = FALSE;
 			else
-				channel_date (sess, outbuf, word[4], word[5]);
+				channel_date (sess, word[4], word[5]);
 		}
 		break;
 	case 332:
-		topic (serv, outbuf, text);
+		topic (serv, text);
 		break;
 	case 333:
-		topic_nametime (serv, outbuf, find_word (pdibuf, 4),
-							 find_word (pdibuf, 5), find_word (pdibuf, 6));
+		topic_nametime (serv, word[4], word[5], word[6]);
 		break;
 	case 341:						  /* INVITE ACK */
 		if (prefs.show_invite_in_front_session)
-			if (current_tab)
-				sess = current_tab;
+			if (menu_sess)
+				sess = menu_sess;
 
 		EMIT_SIGNAL (XP_TE_UINVITE, sess, word[4], word[5], serv->servername,
 						 NULL, 0);
@@ -1411,11 +1231,11 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 			names = word_eol[6];
 			if (*names == ':')
 				names++;
-			names_list (serv, outbuf, chan, names);
+			names_list (serv, chan, names);
 		}
 		break;
 	case 366:
-		if (!end_of_names (serv, outbuf, find_word (pdibuf, 4), text))
+		if (!end_of_names (serv, word[4]))
 			goto def;
 		break;
 	case 367: /* banlist entry */
@@ -1428,29 +1248,44 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 			sess = find_session_from_channel (word[4], serv);
 			if (!sess)
 				sess = serv->front_session;
-			EMIT_SIGNAL (XP_TE_BANLIST, sess, word[4], word[5], word[6], time_str, 0);
+		   if (!fe_is_banwindow (sess))
+				EMIT_SIGNAL (XP_TE_BANLIST, sess, word[4], word[5], word[6], time_str, 0);
+			else
+				fe_add_ban_list (sess, word[5], word[6], time_str);
 		}
 		break;
+	 case 368:
+		sess = find_session_from_channel (word[4], serv);
+		if (!sess)
+			sess = serv->front_session;
+		if (!fe_is_banwindow (sess))
+			goto def;
+		fe_ban_list_end (sess);
+		break;
 	case 376:
 	case 422:						  /* end of motd */
-		if (!serv->end_of_motd && prefs.ip_from_server)
+		if (!serv->end_of_motd)
 		{
-			serv->skip_next_who = TRUE;
-			sprintf (outbuf, "WHO %s\r\n", serv->nick);
-			tcp_send (serv, outbuf);
+			if (prefs.ip_from_server)
+			{
+				serv->skip_next_who = TRUE;
+				sprintf (outbuf, "WHO %s\r\n", serv->nick);
+				tcp_send (serv, outbuf);
+			}
+			set_default_modes (serv, outbuf);
+			check_willjoin_channels (serv, outbuf);
+			if (serv->supports_watch)
+				notify_send_watches (serv);
+			serv->end_of_motd = TRUE;
 		}
-		serv->end_of_motd = TRUE;
-		set_default_modes (serv, outbuf);
-		check_willjoin_channels (serv, outbuf);
 		goto def;
 	case 433:
 		if (serv->end_of_motd)
 			goto def;
-		else
-			next_nick (sess, outbuf, word[4]);
+		next_nick (sess, outbuf, word[4]);
 		break;
 	case 437:
-		if (!is_channel (word[4]) && !serv->end_of_motd)
+		if (!is_channel (serv, word[4]) && !serv->end_of_motd)
 			next_nick (sess, outbuf, word[4]);
 		else
 			goto def;
@@ -1467,6 +1302,16 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 	case 475:
 		EMIT_SIGNAL (XP_TE_KEYWORD, sess, word[4], NULL, NULL, NULL, 0);
 		break;
+	case 601:
+		notify_set_offline (serv, word[4], FALSE);
+		break;
+	case 605:
+		notify_set_offline (serv, word[4], TRUE);
+		break;
+	case 600:
+	case 604:
+		notify_set_online (serv, word[4]);
+		break;
 
 	default:
 	 def:
@@ -1488,9 +1333,9 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 			return;
 		}
 
-		if (is_channel (text))
+		if (is_channel (serv, text))
 		{
-			char *chan = find_word (pdibuf, 3);
+			char *chan = word[3];
 			if (!strncasecmp (serv->nick, chan, strlen (serv->nick)))
 				chan += strlen (serv->nick) + 1;
 			if (*chan == ':')
@@ -1512,16 +1357,17 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 void
 process_line (struct server *serv, char *buf)
 {
-	session *sess, *tmp_sess;
+	session *sess;
+#ifdef USE_PERL
+	session *tmp_sess;
+	char *type;
+#endif
 	char pdibuf[522];				  /* 1 line can't exceed 512 bytes!! */
 	char outbuf[4096];
 	char ip[128], nick[NICKLEN];
-	char *word[32];
-	char *word_eol[32];
+	char *word[PDIWORDS];
+	char *word_eol[PDIWORDS];
 	char *text, *ex, *cmd;
-#ifdef USE_PERL
-	char *type;
-#endif
 	int n;
 
 	sess = serv->front_session;
@@ -1545,7 +1391,7 @@ process_line (struct server *serv, char *buf)
 
 #ifdef USE_PERL
 	/* tell perl that we got a line of data */
-	if (*buf == ':' && is_channel (word[3]))
+	if (*buf == ':' && is_channel (serv, word[3]))
 		tmp_sess = find_session_from_channel (word[3], serv);
 	else
 		tmp_sess = sess;
@@ -1592,7 +1438,7 @@ process_line (struct server *serv, char *buf)
 	n = atoi (word[2]);
 	if (n)
 	{
-		text = find_word_to_end (buf, 3);
+		text = word_eol[3];
 		if (*text)
 		{
 			if (!strncasecmp (serv->nick, text, strlen (serv->nick)))
@@ -1627,8 +1473,8 @@ process_line (struct server *serv, char *buf)
 			return;
 
 		if (prefs.show_invite_in_front_session)
-			if (current_tab)
-				sess = current_tab;
+			if (menu_sess)
+				sess = menu_sess;
 
 		EMIT_SIGNAL (XP_TE_INVITED, sess, word[4] + 1, nick, serv->servername,
 						 NULL, 0);
@@ -1636,51 +1482,34 @@ process_line (struct server *serv, char *buf)
 	}
 	if (!strcmp ("JOIN", cmd))
 	{
-		char *chan = find_word (buf, 3);
+		char *chan = word[3];
 
 		if (*chan == ':')
 			chan++;
 		if (!strcasecmp (nick, serv->nick))
 			you_joined (serv, outbuf, chan, nick, ip);
 		else
-			user_joined (serv, outbuf, chan, nick, ip);
+			user_joined (serv, chan, nick, ip);
 		return;
 	}
 	if (!strcmp ("MODE", cmd))
 	{
-		if (prefs.raw_modes)
-		{
-			int l;
-
-			/* Strip all spaces at the end off --AGL */
-			l = strlen (word_eol[3]);
-			memcpy (outbuf, word_eol[3], l + 1);
-			for (l--; l > 0; l--)
-			{
-				if (outbuf[l] == ' ')
-					outbuf[l] = 0;
-				else
-					break;
-			}
-			tmp_sess = find_session_from_channel (word[3], serv);
-			if (!tmp_sess)
-				tmp_sess = sess;
-			EMIT_SIGNAL (XP_TE_RAWMODES, tmp_sess, nick, outbuf, NULL, NULL, 0);
-		}
-		channel_modes (serv, outbuf, word, nick, 0);
+		handle_mode (serv, outbuf, word, word_eol, nick, FALSE);	/* modes.c */
 		return;
 	}
 	if (!strcmp ("NICK", cmd))
 	{
-		user_new_nick (serv, outbuf, nick, find_word_to_end (buf, 3), FALSE);
+		user_new_nick (serv, nick, word_eol[3], FALSE);
 		return;
 	}
 	if (!strcmp ("NOTICE", cmd))
 	{
-		char *to = find_word (pdibuf, 3);
+		char *to = word[3];
 		if (*to)
 		{
-			char *msg = find_word_to_end (buf, 4) + 1;
+			char *msg = word_eol[4];
+			if (*msg == ':')
+				msg++;
 			if (prefs.fudgeservernotice && (strcmp (nick, serv->servername) == 0 || strchr (nick, '.')))
 			{
 				EMIT_SIGNAL (XP_TE_SERVERGENMESSAGE, sess, msg, NULL,
@@ -1704,17 +1533,19 @@ process_line (struct server *serv, char *buf)
 		if (*reason == ':')
 			reason++;
 		if (!strcmp (nick, serv->nick))
-			you_parted (serv, outbuf, chan, ip, reason);
+			you_parted (serv, chan, ip, reason);
 		else
 			user_parted (serv, chan, nick, ip, reason);
 		return;
 	}
 	if (!strcmp ("PRIVMSG", cmd))
 	{
-		char *to = find_word (pdibuf, 3);
+		char *to = word[3];
 		if (*to)
 		{
-			char *msg = find_word_to_end (buf, 4) + 1;
+			char *msg = word_eol[4];
+			if (*msg == ':')
+				msg++;
 			if (msg[0] == 1 && msg[strlen (msg) - 1] == 1)	/* ctcp */
 			{
 				if (strncasecmp (msg + 1, "ACTION", 6) != 0)
@@ -1727,7 +1558,7 @@ process_line (struct server *serv, char *buf)
 				handle_ctcp (sess, outbuf, to, nick, msg + 1, word, word_eol);
 			} else
 			{
-				if (is_channel (to))
+				if (is_channel (serv, to))
 				{
 					if (ignore_check (pdibuf, 0, 0, 1, 0, 0))
 						return;
@@ -1744,35 +1575,37 @@ process_line (struct server *serv, char *buf)
 	}
 	if (!strcmp ("PONG", cmd))
 	{
-		char *timestring = find_word (pdibuf, 4);
+		char *timestring = word[4];
 		if (*timestring == ':')
 			timestring++;
 		got_ping_reply (serv->front_session, outbuf,
-								timestring, find_word (pdibuf, 3));
+								timestring, word[3]);
 		return;
 	}
 	if (!strcmp ("QUIT", cmd))
 	{
-		user_quit (serv, outbuf, nick, find_word_to_end (buf, 3) + 1);
+		user_quit (serv, nick, ip,
+			(word_eol[3][0] == ':') ? word_eol[3] + 1 : word_eol[3]);
 		return;
 	}
 	if (!strcmp ("TOPIC", cmd))
 	{
-		new_topic (serv, outbuf, nick, find_word (pdibuf, 3),
-					  find_word_to_end (buf, 4) + 1);
+		new_topic (serv, nick, word[3],
+			(word_eol[4][0] == ':') ? word_eol[4] + 1 : word_eol[4]);
 		return;
 	}
 	if (!strcmp ("KICK", cmd))
 	{
-		char *kicked = find_word (pdibuf, 4);
+		char *kicked = word[4];
+		char *reason = word_eol[5];
 		if (*kicked)
 		{
+			if (*reason == ':')
+				reason++;
 			if (!strcmp (kicked, serv->nick))
-				you_kicked (serv, outbuf, find_word (pdibuf, 3), nick,
-								find_word_to_end (buf, 5) + 1);
+ 				you_kicked (serv, outbuf, word[3], nick, reason);
 			else
-				user_kicked (serv, outbuf, find_word (pdibuf, 3),
-								 kicked, nick, find_word_to_end (buf, 5) + 1);
+				user_kicked (serv, word[3], kicked, nick, reason);
 			return;
 		}
 	}
@@ -1789,6 +1622,6 @@ process_line (struct server *serv, char *buf)
 		EMIT_SIGNAL (XP_TE_WALLOPS, sess, nick, text, NULL, NULL, 0);
 		return;
 	}
-	sprintf (outbuf, "(%s/%s) %s\n", nick, ip, find_word_to_end (buf, 2));
+	sprintf (outbuf, "(%s/%s) %s\n", nick, ip, word_eol[2]);
 	PrintText (sess, outbuf);
 }
