@@ -21,49 +21,31 @@
 #include "../../config.h"
 #undef PACKAGE
 
-#ifdef USE_PERL
-
 #include <EXTERN.h>
 #ifndef _SEM_SEMUN_UNDEFINED
 #define HAS_UNION_SEMUN
 #endif
+#define __G_WIN32_H__
+#define WIN32IOP_H
 #include <perl.h>
 #include <XSUB.h>
-#include <sys/mman.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #undef PACKAGE
+#include <stdlib.h>
 #include <stdio.h>
 #include "xchat.h"
 #include "cfgfiles.h"
 #include "util.h"
 #include "ignore.h"
+#include "notify.h"
 #include "fe.h"
-#include "cfgfiles.h"
+#include "text.h"
+#include "outbound.h"
+#include "xchatc.h"
+#include "perlc.h"
 
-extern GSList *sess_list;
-extern GSList *serv_list;
-extern GSList *dcc_list;
-extern GSList *ignore_list;
-
-extern int is_session (session * sess);
-extern int tcp_send_len (struct server *serv, char *buf, int len);
-extern int tcp_send (struct server *serv, char *buf);
-extern void PrintText (struct session *sess, char *text);
-extern struct session *find_session_from_channel (char *chan,
-																  struct server *serv);
-extern int handle_command (char *cmd, struct session *sess, int history, int);
-
-int perl_load_file (char *script_name);
-void perl_init (struct session *default_sess, int autoload);
-void perl_end (void);
-
-
-static PerlInterpreter *my_perl = NULL;
-struct session *perl_sess = 0;
-extern session *menu_sess;
-static int perl_restart;
 
 struct _perl_timeout_handlers
 {
@@ -98,197 +80,57 @@ struct perlscript
 
 /* external values needed to access preferences via IRC::get_prefs*/
 extern struct prefs vars[];	  /*from cfgfiles.c */
-extern struct xchatprefs prefs; /*from xchat.h */
 
-static GSList *perl_timeout_handlers;
-static GSList *perl_inbound_handlers;
-static GSList *perl_command_handlers;
-static GSList *perl_print_handlers;
-static GSList *perl_list;
+static PerlInterpreter *my_perl = NULL;
+static session *perl_sess = NULL;
 
-XS (XS_IRC_register);
-XS (XS_IRC_add_message_handler);
-XS (XS_IRC_add_command_handler);
-XS (XS_IRC_add_print_handler);
-XS (XS_IRC_add_timeout_handler);
-XS (XS_IRC_print);
-XS (XS_IRC_print_with_channel);
-XS (XS_IRC_send_raw);
-XS (XS_IRC_command);
-XS (XS_IRC_command_with_server);
-XS (XS_IRC_channel_list);
-XS (XS_IRC_server_list);
-XS (XS_IRC_user_list);
-XS (XS_IRC_user_info);
-XS (XS_IRC_ignore_list);
-XS (XS_IRC_dcc_list);
-XS (XS_IRC_get_info);
-XS (XS_IRC_get_prefs);
-XS (XS_IRC_user_list_short);
-XS (XS_IRC_perl_script_list);
+/* these must be initialized to 0 incase cmd_scpinfo is
+   called before perl is initialized */
+static GSList *perl_timeout_handlers = 0;
+static GSList *perl_inbound_handlers = 0;
+static GSList *perl_command_handlers = 0;
+static GSList *perl_print_handlers = 0;
+static GSList *perl_list = 0;
 
-/* perl module support */
-extern void xs_init _((void));
-extern void boot_DynaLoader _((CV * cv));
-void
-xs_init ()
+static XS (XS_IRC_register);
+static XS (XS_IRC_add_message_handler);
+static XS (XS_IRC_add_command_handler);
+static XS (XS_IRC_add_print_handler);
+static XS (XS_IRC_add_timeout_handler);
+static XS (XS_IRC_print);
+static XS (XS_IRC_print_with_channel);
+static XS (XS_IRC_send_raw);
+static XS (XS_IRC_command);
+static XS (XS_IRC_command_with_server);
+static XS (XS_IRC_channel_list);
+static XS (XS_IRC_server_list);
+static XS (XS_IRC_add_user_list);
+static XS (XS_IRC_sub_user_list);
+static XS (XS_IRC_clear_user_list);
+static XS (XS_IRC_user_list);
+static XS (XS_IRC_user_info);
+static XS (XS_IRC_ignore_list);
+static XS (XS_IRC_notify_list);
+static XS (XS_IRC_dcc_list);
+static XS (XS_IRC_get_info);
+static XS (XS_IRC_get_prefs);
+static XS (XS_IRC_user_list_short);
+static XS (XS_IRC_perl_script_list);
+
+extern void boot_DynaLoader (pTHX_ CV* cv);
+
+/* xs_init is the second argument perl_pars. As the name hints, it
+   initializes XS subroutines (see the perlembed manpage) */
+static void
+xs_init (pTHX)
 {
 	char *file = __FILE__;
+	
+	/* This one allows dynamic loading of perl modules in perl
+	   scripts by the 'use perlmod;' construction*/
 	newXS ("DynaLoader::boot_DynaLoader", boot_DynaLoader, file);
-}
-
-/* list some script information (handlers etc) */
-
-int
-cmd_scpinfo (struct session *sess, char *tbuf, char *word[], char *word_eol[])
-{
-	GSList *handler;
-
-	PrintText (sess, "Registered Scripts:\n");
-	handler = perl_list;
-	while (handler)
-	{
-		struct perlscript *scp = handler->data;
-		sprintf (tbuf, "  %s %s\n", scp->name, scp->version);
-		PrintText (sess, tbuf);
-		handler = handler->next;
-	}
-
-	PrintText (sess, "Inbound Handlers:\n");
-	for (handler = perl_inbound_handlers; handler != NULL;
-		  handler = handler->next)
-	{
-		struct _perl_inbound_handlers *data = handler->data;
-		sprintf (tbuf, "  %s\n", data->message_type);
-		PrintText (sess, tbuf);
-	}
-
-	PrintText (sess, "Command Handlers:\n");
-	for (handler = perl_command_handlers; handler != NULL;
-		  handler = handler->next)
-	{
-		struct _perl_command_handlers *data = handler->data;
-		sprintf (tbuf, "  %s\n", data->command_name);
-		PrintText (sess, tbuf);
-	}
-
-	PrintText (sess, "Print Handlers:\n");
-	for (handler = perl_print_handlers; handler != NULL;
-		  handler = handler->next)
-	{
-		struct _perl_print_handlers *data = handler->data;
-		sprintf (tbuf, "  %s\n", data->print_name);
-		PrintText (sess, tbuf);
-	}
-
-	return TRUE;
-}
-
-int
-cmd_unloadall (struct session *sess, char *tbuf, char *word[],
-					char *word_eol[])
-{
-	perl_end ();
-	perl_init (sess, FALSE);
-	perl_restart = TRUE;
-	return TRUE;
-}
-
-static char *
-escape_quotes (char *buf)
-{
-	static char *tmp_buf = NULL;
-	char *i, *j;
-
-	if (tmp_buf)
-		free (tmp_buf);
-	tmp_buf = malloc (strlen (buf) * 2 + 1);
-	for (i = buf, j = tmp_buf; *i; i++, j++)
-	{
-		if (*i == '\'' || *i == '\\')
-			*j++ = '\\';
-		*j = *i;
-	}
-	*j = '\0';
-
-	return (tmp_buf);
-}
-
-static SV *
-execute_perl (char *function, char *args)
-{
-	static char *perl_cmd = NULL;
-
-	if (perl_cmd)
-		free (perl_cmd);
-	perl_cmd = malloc (strlen (function) + strlen (args) * 2 + 10);
-	sprintf (perl_cmd, "&%s('%s')", function, escape_quotes (args));
-#ifndef HAVE_PERL_EVAL_PV
-	return (perl_eval_pv (perl_cmd, TRUE));
-#else
-	return (Perl_eval_pv (perl_cmd, TRUE));
-#endif
-}
-
-int
-perl_load_file (char *script_name)
-{
-	SV *return_val;
-	return_val = execute_perl ("load_file", script_name);
-	return SvNV (return_val);
-}
-
-static void
-perl_autoload_file (char *script_name)
-{
-/*	char tbuf[256];
-
-	snprintf (tbuf, sizeof tbuf, "\002Loading\00302: \017%s\00302...\017\n",
-				 file_part (script_name));
-	PrintText (0, tbuf);*/
-	perl_load_file (script_name);
-}
-
-void
-perl_init (struct session *default_sess, int autoload)
-{
-	char *perl_args[] = { "", "-e", "0", "-w" };
-	char load_file[] =
-		"sub load_file()\n"
-		"{\n" " (my $file_name) = @_;\n" "open FH, $file_name or return 2;\n"
-/*"  # save the input separator"*/
-		"my $is = $/;\n"
-		"local($/) = undef;\n" "$file = <FH>;\n" "close FH;\n"
-/*"  # and clean up"*/
-		"$/ = $is;\n"
-/*"  # hmm, workaround. (Jester told me, there is another way using newXS,"
-"  # but i found nothing in the perl docu"*/
-		"$file = \"\\@ISA = qw(Exporter DynaLoader);\\n\" . $file;\n"
-		"eval $file;\n"
-/*"  # urgs, can anybode tell me about this line? (the second eval)"
-"  # thats from the original, i commented it out"*/
-		"eval $file if $@;\n"
-		"return 1 if $@;\n" "return 0;\n" "}";
-
-	perl_sess = default_sess;
-
-	my_perl = perl_alloc ();
-	perl_construct (my_perl);
-	if (prefs.perlwarnings)
-		perl_parse (my_perl, xs_init, 4, perl_args, NULL);
-	else
-		perl_parse (my_perl, xs_init, 3, perl_args, NULL);
-#ifndef HAVE_PERL_EVAL_PV
-	perl_eval_pv (load_file, TRUE);
-#else
-	Perl_eval_pv (load_file, TRUE);
-#endif
-	perl_inbound_handlers = 0;
-	perl_command_handlers = 0;
-	perl_print_handlers = 0;
-	perl_list = 0;
-
 	/* load up all the custom IRC perl functions */
+	/* deplaced here from perl_init function (TheHobbit)*/
 	newXS ("IRC::register", XS_IRC_register, "IRC");
 	newXS ("IRC::add_message_handler", XS_IRC_add_message_handler, "IRC");
 	newXS ("IRC::add_command_handler", XS_IRC_add_command_handler, "IRC");
@@ -301,17 +143,252 @@ perl_init (struct session *default_sess, int autoload)
 	newXS ("IRC::command_with_server", XS_IRC_command_with_server, "IRC");
 	newXS ("IRC::channel_list", XS_IRC_channel_list, "IRC");
 	newXS ("IRC::server_list", XS_IRC_server_list, "IRC");
+	newXS ("IRC::add_user_list", XS_IRC_add_user_list, "IRC");
+	newXS ("IRC::sub_user_list", XS_IRC_sub_user_list, "IRC");
+	newXS ("IRC::clear_user_list", XS_IRC_clear_user_list, "IRC");
 	newXS ("IRC::user_list", XS_IRC_user_list, "IRC");
 	newXS ("IRC::user_info", XS_IRC_user_info, "IRC");
 	newXS ("IRC::ignore_list", XS_IRC_ignore_list, "IRC");
+	newXS ("IRC::notify_list", XS_IRC_notify_list, "IRC");
 	newXS ("IRC::dcc_list", XS_IRC_dcc_list, "IRC");
 	newXS ("IRC::get_info", XS_IRC_get_info, "IRC");
 	newXS ("IRC::get_prefs", XS_IRC_get_prefs, "IRC");
 	newXS ("IRC::user_list_short", XS_IRC_user_list_short, "IRC");
 	newXS ("IRC::perl_script_list", XS_IRC_perl_script_list, "IRC");
+}
 
-	if (autoload)
-		for_files (get_xdir (), "*.pl", perl_autoload_file);
+/* a session is being killed, does it affect us? */
+
+void
+perl_notify_kill (session * sess)
+{
+	struct session *s;
+	GSList *list = sess_list;
+
+	if (perl_sess == sess)		  /* need to find a new perl_sess, this one's closing */
+	{
+		while (list)
+		{
+			s = (struct session *) list->data;
+			if (s->server == perl_sess->server && s != perl_sess)
+			{
+				perl_sess = s;
+				break;
+			}
+			list = list->next;
+		}
+		if (perl_sess == sess)
+			perl_sess = 0;
+	}
+}
+
+/* list some script information (handlers etc) */
+
+int
+cmd_scpinfo (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	GSList *handler;
+
+	PrintText (sess, _("Registered Scripts:\n"));
+	handler = perl_list;
+	while (handler)
+	{
+		struct perlscript *scp = handler->data;
+		sprintf (tbuf, "  %s %s\n", scp->name, scp->version);
+		PrintText (sess, tbuf);
+		handler = handler->next;
+	}
+
+	PrintText (sess, _("Inbound Handlers:\n"));
+	for (handler = perl_inbound_handlers; handler != NULL;
+		  handler = handler->next)
+	{
+		struct _perl_inbound_handlers *data = handler->data;
+		sprintf (tbuf, "  %s\n", data->message_type);
+		PrintText (sess, tbuf);
+	}
+
+	PrintText (sess, _("Command Handlers:\n"));
+	for (handler = perl_command_handlers; handler != NULL;
+		  handler = handler->next)
+	{
+		struct _perl_command_handlers *data = handler->data;
+		sprintf (tbuf, "  %s\n", data->command_name);
+		PrintText (sess, tbuf);
+	}
+
+	PrintText (sess, _("Print Handlers:\n"));
+	for (handler = perl_print_handlers; handler != NULL;
+		  handler = handler->next)
+	{
+		struct _perl_print_handlers *data = handler->data;
+		sprintf (tbuf, "  %s\n", data->print_name);
+		PrintText (sess, tbuf);
+	}
+
+	return TRUE;
+}
+
+/* 
+   execute_perl is modified in order to avoid crashing of xchat when a
+   perl error occours. The embedded interpreter will instead print the
+   error message using IRC::print and return 1 to stop futher
+   processing of the event.
+
+   patch by TheHobbit <thehobbit@altern.org>
+
+*/
+
+/*
+  2001/06/14: execute_perl replaced by Martin Persson <mep@passagen.se>
+	      previous use of perl_eval leaked memory, replaced with
+	      a version that uses perl_call instead
+*/
+static int
+execute_perl (char *function, char *args)
+{
+	char *perl_args[2] = { args, NULL }, buf[512];
+	int count, ret_value = 1;
+	SV *sv;
+
+	dSP;
+	ENTER;
+	SAVETMPS;
+	PUSHMARK(sp);
+	count = perl_call_argv(function, G_EVAL | G_SCALAR, perl_args);
+	SPAGAIN;
+
+	sv = GvSV(gv_fetchpv("@", TRUE, SVt_PV));
+	if (SvTRUE(sv)) {
+		snprintf(buf, 512, "Perl error: %s\n", SvPV(sv, count));
+		PrintText(perl_sess, buf);
+		POPs;
+	} else if (count != 1) {
+		snprintf(buf, 512, "Perl error: expected 1 value from %s, "
+			"got: %d\n", function, count);
+		PrintText(perl_sess, buf);
+	} else {
+		ret_value = POPi;
+	}
+
+	PUTBACK;
+	FREETMPS;
+	LEAVE;
+
+	return ret_value;
+}
+
+static void
+perl_init (void)
+{
+	/*changed the name of the variable from load_file to
+	  perl_definitions since now it does much more than defining
+	  the load_file sub. Moreover, deplaced the initialisation to
+	  the xs_init function. (TheHobbit)*/
+	char *perl_args[] = { "", "-e", "0", "-w" };
+	char perl_definitions[] =
+	{
+	  /* We use to function one to load a file the other to
+	     execute the string obtained from the first and holding
+	     the file conents. This allows to have a realy local $/
+	     without introducing temp variables to hold the old
+	     value. Just a question of style:) 
+	     We also redefine the $SIG{__WARN__} handler to have XChat
+	     printing warnings in the main window. (TheHobbit)*/
+	  "sub load_file{"
+	    "my $file_name=shift;"
+	    "local $/=undef;"
+	    "open FH,$file_name or return \"__FAILED__\";"
+	    "$_=<FH>;"
+	    "close FH;"
+	    "return $_;"
+	  "}"
+	  "sub load_n_eval{"
+	    "my $file_name=shift;"
+	    "my $strin=load_file($file_name);"
+	    "return 2 if($strin eq \"__FAILED__\");"
+	    "eval $strin;"
+	    "if($@){"
+	    /*"  #something went wrong\n"*/
+	      "IRC::print \"Errors loading file $file_name:\\n\";"
+	      "IRC::print \"$@\\n\";"
+	      "return 1;"
+	    "}"
+	    "return 0;"
+	  "}"
+	  "$SIG{__WARN__}=sub{IRC::print\"$_[0]\n\";};"
+	};
+#ifdef ENABLE_NLS
+
+	/* Problem is, dynamicaly loaded modules check out the $]
+	   var. It appears that in the embedded interpreter we get
+	   5,00503 as soon as the LC_NUMERIC locale calls for a comma
+	   instead of a point in separating integer and decimal
+	   parts. I realy can't understant why... The following
+	   appears to be an awful workaround... But it'll do until I
+	   (or someone else :)) found the "right way" to solve this
+	   nasty problem. (TheHobbit <thehobbit@altern.org>)*/
+	
+ 	setlocale(LC_NUMERIC,"C"); 
+	
+#endif
+
+	my_perl = perl_alloc ();
+	perl_construct (my_perl);
+	if (prefs.perlwarnings)
+		perl_parse (my_perl, xs_init, 4, perl_args, NULL);
+	else
+		perl_parse (my_perl, xs_init, 3, perl_args, NULL);
+	/*
+	  Now initialising the perl interpreter by loading the
+	  perl_definition array.
+	*/
+#ifdef HAVE_EVAL_PV
+	eval_pv (perl_definitions, TRUE);
+#else
+	perl_eval_pv (perl_definitions, TRUE);	/* deprecated */
+#endif
+	/*perl_timeout_handlers = 0;
+	perl_inbound_handlers = 0;
+	perl_command_handlers = 0;
+	perl_print_handlers = 0;
+	perl_list = 0;*/
+}
+
+/*
+  To avoid problems, the load_n_eval sub must be executed directly
+  without going into a supplementary eval.
+
+  TheHobbit <thehobbit@altern.org>
+*/
+int
+perl_load_file (char *script_name)
+{
+	if (my_perl == NULL)
+		perl_init ();
+
+	return execute_perl ("load_n_eval", script_name);
+}
+
+static void
+perl_autoload_file (char *script_name)
+{
+	perl_load_file (script_name);
+}
+
+void
+perl_auto_load (session *sess)
+{
+	perl_sess = sess;
+	for_files (get_xdir (), "*.pl", perl_autoload_file);
+}
+
+int
+cmd_unloadall (session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	perl_sess = sess;
+	perl_end ();
+	return TRUE;
 }
 
 void
@@ -384,7 +461,7 @@ perl_inbound (struct session *sess, struct server *serv, char *buf, char *msg_ty
 {
 	GSList *handler;
 	struct _perl_inbound_handlers *data;
-	SV *handler_return;
+	int handler_return;
 
 	perl_sess = sess;
 
@@ -396,9 +473,9 @@ perl_inbound (struct session *sess, struct server *serv, char *buf, char *msg_ty
 			 || !strcmp ("INBOUND", data->message_type))
 		{
 			handler_return = execute_perl (data->handler_name, buf);
-			if (SvIV (handler_return))
+			if (handler_return)
 			{
-				return SvIV (handler_return);
+				return handler_return;
 			}
 		}
 	}
@@ -410,7 +487,7 @@ perl_dcc_chat (struct session *sess, struct server *serv, char *buf)
 {
 	GSList *handler;
 	struct _perl_inbound_handlers *data;
-	SV *handler_return;
+	int handler_return;
 
 	if (!buf)
 		return 0;
@@ -424,9 +501,9 @@ perl_dcc_chat (struct session *sess, struct server *serv, char *buf)
 			 || !strcmp ("INBOUND", data->message_type))
 		{
 			handler_return = execute_perl (data->handler_name, buf);
-			if (SvIV (handler_return))
+			if (handler_return)
 			{
-				return SvIV (handler_return);
+				return handler_return;
 			}
 		}
 	}
@@ -442,7 +519,7 @@ perl_command (char *cmd, struct session *sess)
 	char *tmp;
 	char *args;
 	char nullargs[] = "";
-	SV *handler_return;
+	int handler_return;
 	int command = FALSE;
 
 	args = NULL;
@@ -462,8 +539,6 @@ perl_command (char *cmd, struct session *sess)
 	if (!args)
 		args = nullargs;
 
-	perl_restart = FALSE;
-
 	for (handler = perl_command_handlers; handler != NULL;
 		  handler = handler->next)
 	{
@@ -476,15 +551,10 @@ perl_command (char *cmd, struct session *sess)
 				handler_return = execute_perl (data->handler_name, cmd);
 			else
 				handler_return = execute_perl (data->handler_name, args);
-			if (perl_restart)
+			if (handler_return)
 			{
 				free (command_name);
-				return TRUE;
-			}
-			if (SvIV (handler_return))
-			{
-				free (command_name);
-				return SvIV (handler_return);
+				return handler_return;
 			}
 		}
 	}
@@ -500,7 +570,7 @@ perl_print (char *cmd, struct session *sess, char *b, char *c, char *d,
 	GSList *handler;
 	struct _perl_print_handlers *data;
 	char *args;
-	SV *handler_return;
+	int handler_return;
 
 	if (!perl_print_handlers)
 		return 0;
@@ -533,9 +603,7 @@ perl_print (char *cmd, struct session *sess, char *b, char *c, char *d,
 		strcat (args, " ");
 		strcat (args, e);
 	}
-printf ("args: *%s*\n", args);
 
-	perl_restart = FALSE;
 	for (handler = perl_print_handlers; handler != NULL;
 		  handler = handler->next)
 	{
@@ -543,15 +611,10 @@ printf ("args: *%s*\n", args);
 		if (!strcasecmp (cmd, data->print_name))
 		{
 			handler_return = execute_perl (data->handler_name, args);
-			if (perl_restart)
+			if (handler_return)
 			{
 				free (args);
-				return TRUE;
-			}
-			if (SvIV (handler_return))
-			{
-				free (args);
-				return SvIV (handler_return);
+				return handler_return;
 			}
 		}
 	}
@@ -582,13 +645,12 @@ perl_timeout (struct _perl_timeout_handlers *handler)
  *
  */
 
-XS (XS_IRC_register)
+static XS (XS_IRC_register)
 {
 	char *name, *ver, *callback, *unused;
 	int junk;
 	struct perlscript *scp;
 	dXSARGS;
-	items = 0;
 
 	name = SvPV (ST (0), junk);
 	ver = SvPV (ST (1), junk);
@@ -608,7 +670,7 @@ XS (XS_IRC_register)
 
 /* print to main window */
 /* IRC::main_print(output) */
-XS (XS_IRC_print)
+static XS (XS_IRC_print)
 {
 	int junk;
 	int i;
@@ -634,7 +696,7 @@ XS (XS_IRC_print)
  *   Returns 1 on success, 0 on fail.
  */
 
-XS (XS_IRC_print_with_channel)
+static XS (XS_IRC_print_with_channel)
 {
 	int junk;
 	char *output;
@@ -642,7 +704,6 @@ XS (XS_IRC_print_with_channel)
 	GSList *list = sess_list;
 	char *channel, *server;
 	dXSARGS;
-	items = 0;
 
 	output = SvPV (ST (0), junk);
 	channel = SvPV (ST (1), junk);
@@ -659,29 +720,26 @@ XS (XS_IRC_print_with_channel)
 				if (!strcasecmp (sess->channel, channel))
 				{
 					PrintText (sess, output);
-					XSRETURN (1);
-					return;			  /* needed or not? */
+					XSRETURN_YES;
 				}
 			}
 		}
 		list = list->next;
 	}
 
-	XSRETURN (0);
+	XSRETURN_NO;
 }
 
-XS (XS_IRC_get_info)
+static XS (XS_IRC_get_info)
 {
-	int junk;
 	dXSARGS;
-	items = 0;
 
 	if (!perl_sess)
 	{
 		XST_mPV (0, "Error1");
 		XSRETURN (1);
 	}
-	switch (atoi (SvPV (ST (0), junk)))
+	switch (SvIV (ST (0)))
 	{
 	case 0:
 		XST_mPV (0, VERSION);
@@ -716,13 +774,13 @@ XS (XS_IRC_get_info)
 
 /* Added by TheHobbit <thehobbit@altern.org>*/
 /* IRC::get_prefs(var) */
-XS (XS_IRC_get_prefs)
+static XS (XS_IRC_get_prefs)
 {
 	int junk;
 	char *var;
 	int i = 0;
 	dXSARGS;
-	items = 0;
+
 	var = SvPV (ST (0), junk);
 
 	do
@@ -759,12 +817,11 @@ XS (XS_IRC_get_prefs)
 
 /* add handler for messages with message_type(ie PRIVMSG, 400, etc) */
 /* IRC::add_message_handler(message_type, handler_name) */
-XS (XS_IRC_add_message_handler)
+static XS (XS_IRC_add_message_handler)
 {
 	int junk;
 	struct _perl_inbound_handlers *handler;
 	dXSARGS;
-	items = 0;
 
 	handler = malloc (sizeof (struct _perl_inbound_handlers));
 	handler->message_type = strdup (SvPV (ST (0), junk));
@@ -775,12 +832,11 @@ XS (XS_IRC_add_message_handler)
 
 /* add handler for commands with command_name */
 /* IRC::add_command_handler(command_name, handler_name) */
-XS (XS_IRC_add_command_handler)
+static XS (XS_IRC_add_command_handler)
 {
 	int junk;
 	struct _perl_command_handlers *handler;
 	dXSARGS;
-	items = 0;
 
 	handler = malloc (sizeof (struct _perl_command_handlers));
 	handler->command_name = strdup (SvPV (ST (0), junk));
@@ -791,12 +847,11 @@ XS (XS_IRC_add_command_handler)
 
 /* add handler for commands with print_name */
 /* IRC::add_print_handler(print_name, handler_name) */
-XS (XS_IRC_add_print_handler)
+static XS (XS_IRC_add_print_handler)
 {
 	int junk;
 	struct _perl_print_handlers *handler;
 	dXSARGS;
-	items = 0;
 
 	handler = malloc (sizeof (struct _perl_print_handlers));
 	handler->print_name = strdup (SvPV (ST (0), junk));
@@ -805,30 +860,26 @@ XS (XS_IRC_add_print_handler)
 	XSRETURN_EMPTY;
 }
 
-XS (XS_IRC_add_timeout_handler)
+static XS (XS_IRC_add_timeout_handler)
 {
 	int junk;
-	long timeout;
 	struct _perl_timeout_handlers *handler;
 	dXSARGS;
-	items = 0;
 
 	handler = malloc (sizeof (struct _perl_timeout_handlers));
-	timeout = atol (SvPV (ST (0), junk));
 	handler->handler_name = strdup (SvPV (ST (1), junk));
 	perl_timeout_handlers = g_slist_prepend (perl_timeout_handlers, handler);
-	handler->iotag = fe_timeout_add (timeout, perl_timeout, handler);
+	handler->iotag = fe_timeout_add (SvIV (ST (0)), perl_timeout, handler);
 	XSRETURN_EMPTY;
 }
 
 /* send raw data to server */
 /* IRC::send_raw(data) */
-XS (XS_IRC_send_raw)
+static XS (XS_IRC_send_raw)
 {
 	char *data;
 	int junk;
 	dXSARGS;
-	items = 0;
 
 	if (perl_sess)
 	{
@@ -839,13 +890,12 @@ XS (XS_IRC_send_raw)
 	XSRETURN_EMPTY;
 }
 
-XS (XS_IRC_channel_list)
+static XS (XS_IRC_channel_list)
 {
 	struct session *sess;
 	GSList *list = sess_list;
 	int i = 0;
 	dXSARGS;
-	items = 0;
 
 	while (list)
 	{
@@ -865,18 +915,17 @@ XS (XS_IRC_channel_list)
 	XSRETURN (i);
 }
 
-XS (XS_IRC_server_list)
+static XS (XS_IRC_server_list)
 {
-	struct server *serv;
+	server *serv;
 	GSList *list = serv_list;
 	int i = 0;
 	dXSARGS;
-	items = 0;
 
 	while (list)
 	{
-		serv = (struct server *) list->data;
-		if (serv->connected && serv->servername[0])
+		serv = list->data;
+		if (serv->connected && serv->end_of_motd)
 		{
 			XST_mPV (i, serv->servername);
 			i++;
@@ -887,13 +936,12 @@ XS (XS_IRC_server_list)
 	XSRETURN (i);
 }
 
-XS (XS_IRC_ignore_list)
+static XS (XS_IRC_ignore_list)
 {
 	struct ignore *ig;
 	GSList *list = ignore_list;
 	int i = 0;
 	dXSARGS;
-	items = 0;
 
 	while (list)
 	{
@@ -921,6 +969,53 @@ XS (XS_IRC_ignore_list)
 	XSRETURN (i);
 }
 
+static XS (XS_IRC_notify_list)
+{
+	struct notify *not;
+	struct notify_per_server *notserv;
+	GSList *list = notify_list;
+	GSList *notslist;
+	int i = 0;
+	dXSARGS;
+
+	while (list)
+	{
+		not = (struct notify *) list->data;
+		notslist = not->server_list;
+
+		XST_mPV (i, not->name);
+		i++;
+		while (notslist)
+		{
+			notserv = (struct notify_per_server *)notslist->data;
+
+			XST_mPV (i, notserv->server->servername);
+			i++;
+			XST_mIV (i, notserv->laston);
+			i++;
+			XST_mIV (i, notserv->lastseen);
+			i++;
+			XST_mIV (i, notserv->lastoff);
+			i++;
+			if (notserv->ison)
+				XST_mYES (i);
+			else
+				XST_mNO (i);
+			i++;
+			XST_mPV (i, "::");
+			i++;
+			
+			notslist = notslist->next;
+		}
+		XST_mPV (i, ":");
+		i++;
+
+		list = list->next;
+	}
+	
+	XSRETURN (i);
+}
+
 
 /*
 
@@ -928,13 +1023,12 @@ XS (XS_IRC_ignore_list)
 
  */
 
-XS (XS_IRC_user_info)
+static XS (XS_IRC_user_info)
 {
 	int junk;
 	struct User *user;
 	char *nick;
 	dXSARGS;
-	items = 0;
 
 	if (perl_sess)
 	{
@@ -957,13 +1051,112 @@ XS (XS_IRC_user_info)
 	XSRETURN (0);
 }
 
+/*
+ * IRC::add_user_list(ul_channel, ul_server, nick, user_host,
+ * 		      realname, server)
+ */
+static XS (XS_IRC_add_user_list)
+{
+	int junk;
+	char *ul_channel;
+	char *ul_server;
+	char *nick;
+	char *user_host;	/* add_name() wants user and host merged */
+	char *realname;
+	char *server;
+	struct session *sess;
+	GSList *list = sess_list;
+	dXSARGS;
+
+	ul_channel = SvPV(ST(0), junk);
+	ul_server  = SvPV(ST(1), junk);
+	nick       = SvPV(ST(2), junk);
+	user_host  = SvPV(ST(3), junk);
+	realname   = SvPV(ST(4), junk);
+	server     = SvPV(ST(5), junk);
+
+	while (list) {
+		sess = (struct session *) list->data;
+		if (!server[0] || !strcasecmp(sess->server->servername, ul_server)) {
+			if (!strcasecmp(sess->channel, ul_channel)) {
+				add_name(sess, nick, NULL);
+				userlist_add_hostname(sess, nick, user_host,
+						      realname, server);
+				XSRETURN_YES;
+			}
+		}
+		list = list->next;
+	}
+
+	XSRETURN_NO;
+}
+
+/*
+ * IRC::sub_user_list(ul_channel, ul_server, nick)
+ */
+static XS (XS_IRC_sub_user_list)
+{
+	int junk;
+	char *channel;
+	char *server;
+	char *nick;
+	struct session *sess;
+	GSList *list = sess_list;
+	dXSARGS;
+
+	channel = SvPV(ST(0), junk);
+	server  = SvPV(ST(1), junk);
+	nick    = SvPV(ST(2), junk);
+
+	while (list) {
+		sess = (struct session *) list->data;
+		if (!server[0] || !strcasecmp(sess->server->servername, server)) {
+			if (!strcasecmp(sess->channel, channel)) {
+				sub_name(sess, nick);
+				XSRETURN_YES;
+			}
+		}
+		list = list->next;
+	}
+
+	XSRETURN_NO;
+}
+
+/*
+ * IRC::clear_user_list(channel, server)
+ */
+static XS (XS_IRC_clear_user_list)
+{
+	int junk;
+	char *channel, *server;
+	struct session *sess;
+	GSList *list = sess_list;
+	dXSARGS;
+
+	channel = SvPV(ST(0), junk);
+	server  = SvPV(ST(1), junk);
+
+	while (list) {
+		sess = (struct session *) list->data;
+		if (!server[0] || !strcasecmp(sess->server->servername, server)) {
+			if (!strcasecmp(sess->channel, channel)) {
+				clear_user_list(sess);
+				XSRETURN_YES;
+			}
+		}
+		list = list->next;
+	}
+
+	XSRETURN_NO;
+};
+
 /*
 
    IRC::user_list( channel, server )
 
  */
 
-XS (XS_IRC_user_list)
+static XS (XS_IRC_user_list)
 {
 	struct User *user;
 	struct session *sess;
@@ -972,8 +1165,8 @@ XS (XS_IRC_user_list)
 	int i = 0, junk;
 	int MinStk = 10;
 	dXSARGS;
+
 	EXTEND (SP, MinStk);
-	items = 0;
 
 	channel = SvPV (ST (0), junk);
 	server = SvPV (ST (1), junk);
@@ -983,8 +1176,7 @@ XS (XS_IRC_user_list)
 		sess = (struct session *) list->data;
 		if (!server[0] || !strcasecmp (sess->server->servername, server))
 		{
-			if (!strcasecmp (sess->channel, channel) && !sess->is_dialog
-				 && !sess->is_shell)
+			if (!strcasecmp (sess->channel, channel) && sess->type == SESS_CHANNEL)
 			{
 				list = sess->userlist;
 				while (list)
@@ -1016,13 +1208,12 @@ XS (XS_IRC_user_list)
 	XSRETURN (i);
 }
 
-XS (XS_IRC_dcc_list)
+static XS (XS_IRC_dcc_list)
 {
 	struct DCC *dcc;
 	GSList *list = dcc_list;
 	int i = 0;
 	dXSARGS;
-	items = 0;
 
 	while (list)
 	{
@@ -1059,12 +1250,11 @@ XS (XS_IRC_dcc_list)
 
 /* run internal xchat command */
 /* IRC::command(command) */
-XS (XS_IRC_command)
+static XS (XS_IRC_command)
 {
 	char *command;
 	int junk;
 	dXSARGS;
-	items = 0;
 
 	if (perl_sess)
 	{
@@ -1075,14 +1265,13 @@ XS (XS_IRC_command)
 	XSRETURN_EMPTY;
 }
 
-XS (XS_IRC_command_with_server)
+static XS (XS_IRC_command_with_server)
 {
 	GSList *list = serv_list;
 	struct server *serv;
 	char *server, *command;
 	int junk;
 	dXSARGS;
-	items = 0;
 
 	server = strdup (SvPV (ST (1), junk));
 
@@ -1129,7 +1318,7 @@ XS (XS_IRC_command_with_server)
    suitable for assigning to a hash.  modified IRC::user_list()
    
  */
-XS (XS_IRC_user_list_short)
+static XS (XS_IRC_user_list_short)
 {
 	struct User *user;
 	struct session *sess;
@@ -1138,8 +1327,8 @@ XS (XS_IRC_user_list_short)
 	int i = 0, junk;
 	int MinStk = 10;
 	dXSARGS;
+
 	EXTEND (SP, MinStk);
-	items = 0;
 
 	channel = SvPV (ST (0), junk);
 	server = SvPV (ST (1), junk);
@@ -1149,8 +1338,7 @@ XS (XS_IRC_user_list_short)
 		sess = (struct session *) list->data;
 		if (!server[0] || !strcasecmp (sess->server->servername, server))
 		{
-			if (!strcasecmp (sess->channel, channel) && !sess->is_dialog
-				 && !sess->is_shell)
+			if (!strcasecmp (sess->channel, channel) && sess->type == SESS_CHANNEL)
 			{
 				list = sess->userlist;
 				while (list)
@@ -1180,12 +1368,11 @@ XS (XS_IRC_user_list_short)
 
 /* MAG030600 BEGIN IRC::perl_script_list() */
 /* return a list of currently loaded perl script names and versions */
-XS (XS_IRC_perl_script_list)
+static XS (XS_IRC_perl_script_list)
 {
 	int i = 0;
 	GSList *handler;
-
-	dXSARGS; items = 0;
+	dXSARGS;
 
 	handler = perl_list;
 	while (handler)
@@ -1200,5 +1387,3 @@ XS (XS_IRC_perl_script_list)
 	XSRETURN(i);
 }
 /* MAG030600 END */
-
-#endif
