@@ -24,13 +24,13 @@
 #include <strings.h>
 #endif
 #include "xchat.h"
+#include "modes.h"
 #include "fe.h"
+#include "notify.h"
+#include "xchatc.h"
+#include "util.h"
 
-extern struct xchatprefs prefs;
-extern GSList *sess_list;
 
-void add_name (struct session *sess, char *name, char *hostname);
-int sub_name (struct session *sess, char *name);
 static void update_entry (struct session *sess, struct User *user);
 
 
@@ -55,41 +55,30 @@ userlist_add_hostname (struct session *sess, char *nick, char *hostname,
 	return 0;
 }
 
-int
+static int
 nick_cmp_az_ops (struct User *user1, struct User *user2)
 {
-	if (user1->op)
+	unsigned int access1 = user1->access;
+	unsigned int access2 = user2->access;
+	int pos;
+
+	if (access1 != access2)
 	{
-		if (!user2->op)
-			return -1;
-	} else
-	{
-		if (user2->op)
-			return +1;
-		if (user1->hop)
+		for (pos = 0; pos < USERACCESS_SIZE; pos++)
 		{
-			if (!user2->hop)
+			if ((access1&(1<<pos)) && (access2&(1<<pos)))
+				break;
+			if ((access1&(1<<pos)) && !(access2&(1<<pos)))
 				return -1;
-		} else
-		{
-			if (user2->hop)
-				return +1;
-			if (user1->voice)
-			{
-				if (!user2->voice)
-					return -1;
-			} else
-			{
-				if (user2->voice)
-					return +1;
-			}
+			if (!(access1&(1<<pos)) && (access2&(1<<pos)))
+				return 1;
 		}
 	}
 
 	return strcasecmp (user1->nick, user2->nick);
 }
 
-int
+static int
 nick_cmp (struct User *user1, struct User *user2)
 {
 	switch (prefs.userlist_sort)
@@ -108,12 +97,11 @@ nick_cmp (struct User *user1, struct User *user2)
 }
 
 void
-clear_user_list (struct session *sess)
+free_userlist (session *sess)
 {
 	struct User *user;
 	GSList *list = sess->userlist;
 
-	fe_userlist_clear (sess);
 	while (list)
 	{
 		user = (struct User *) list->data;
@@ -121,7 +109,7 @@ clear_user_list (struct session *sess)
 			free (user->realname);
 		if (user->hostname)
 			free (user->hostname);
-		if (user->servername);
+		if (user->servername)
 			free (user->servername);
 		free (user);
 		list = g_slist_remove (list, user);
@@ -133,6 +121,14 @@ clear_user_list (struct session *sess)
 	sess->total = 0;
 }
 
+void
+clear_user_list (session *sess)
+{
+	fe_userlist_clear (sess);
+	free_userlist (sess);
+	fe_userlist_numbers (sess);
+}
+
 struct User *
 find_name (struct session *sess, char *name)
 {
@@ -171,7 +167,7 @@ find_name_global (struct server *serv, char *name)
 }
 
 static int
-userlist_insertname_sorted (struct session *sess, struct User *newuser)
+userlist_insertname (struct session *sess, struct User *newuser)
 {
 	int row = 0;
 	struct User *user;
@@ -198,78 +194,78 @@ update_entry (struct session *sess, struct User *user)
 	int row;
 
 	sess->userlist = g_slist_remove (sess->userlist, user);
-	row = userlist_insertname_sorted (sess, user);
+	row = userlist_insertname (sess, user);
 
 	fe_userlist_move (sess, user, row);
 	fe_userlist_numbers (sess);
 }
 
-void
-ul_op_name (struct session *sess, char *name)
+static void
+update_counts (session *sess, struct User *user, char prefix,
+					int level, int offset)
 {
-	struct User *user = find_name (sess, name);
-	if (user && !user->op)
+	/* the only ones the userlist supports with icons */
+	switch (prefix)
 	{
-		sess->ops++;
-		user->op = TRUE;
-		update_entry (sess, user);
-	}
-}
-void
-ul_hop_name (struct session *sess, char *name)
-{
-	struct User *user = find_name (sess, name);
-	if (user && !user->hop)
-	{
-		sess->hops++;
-		user->hop = TRUE;
-		update_entry (sess, user);
-	}
-}
-void
-deop_name (struct session *sess, char *name)
-{
-	struct User *user = find_name (sess, name);
-	if (user && user->op)
-	{
-		sess->ops--;
-		user->op = FALSE;
-		update_entry (sess, user);
-	}
-}
-void
-dehop_name (struct session *sess, char *name)
-{
-	struct User *user = find_name (sess, name);
-	if (user && user->hop)
-	{
-		sess->hops--;
-		user->hop = FALSE;
-		update_entry (sess, user);
-	}
-}
-void
-voice_name (struct session *sess, char *name)
-{
-	struct User *user = find_name (sess, name);
-	if (user && !user->voice)
-	{
-		sess->voices++;
-		user->voice = TRUE;
-		update_entry (sess, user);
+	case '*':
+		user->admin = level;
+		break;
+	case '@':
+		user->op = level;
+		sess->ops += offset;
+		break;
+	case '%':
+		user->hop = level;
+		sess->hops += offset;
+		break;
+	case '+':
+		user->voice = level;
+		sess->voices += offset;
+		break;
 	}
 }
 
 void
-devoice_name (struct session *sess, char *name)
+ul_update_entry (session *sess, char *name, char mode, char sign)
 {
-	struct User *user = find_name (sess, name);
-	if (user && user->voice)
+	int access;
+	int offset = 0;
+	int level;
+	char prefix;
+	struct User *user;
+
+	user = find_name (sess, name);
+	if (!user)
+		return;
+
+	/* which bit number is affected? */
+	access = mode_access (sess->server, mode, &prefix);
+
+	if (sign == '+')
 	{
-		sess->voices--;
-		user->voice = FALSE;
-		update_entry (sess, user);
+		level = TRUE;
+		if (!(user->access & (1 << access)))
+		{
+			offset = 1;
+			user->access |= (1 << access);
+		}
+	} else
+	{
+		level = FALSE;
+		if (user->access & (1 << access))
+		{
+			offset = -1;
+			user->access &= ~(1 << access);
+		}
 	}
+
+	/* now what is this users highest prefix? e.g. @ for ops */
+	user->prefix = get_nick_prefix (sess->server, user->access);
+
+	/* update the various counts using the CHANGED prefix only */
+	update_counts (sess, user, prefix, level, offset);
+
+	update_entry (sess, user);
 }
 
 void
@@ -318,46 +314,38 @@ void
 add_name (struct session *sess, char *name, char *hostname)
 {
 	struct User *user;
-	int row;
+	int row, prefix_chars;
 
-	if (find_name (sess, name))
-		return;
+	notify_set_online (sess->server, name);
 
 	user = malloc (sizeof (struct User));
 	memset (user, 0, sizeof (struct User));
 
-	/* filter out UnrealIRCD stuff */
-	while (*name == '~' || *name == '*' || *name == '.')
-		name++;
+	user->access = nick_access (sess->server, name, &prefix_chars);
+	user->prefix = ' ';
+
+	/* assume first char is the highest level nick prefix */
+	if (prefix_chars)
+		user->prefix = name[0];
 
-	if (*name == '@')
-	{
-		name++;
-		user->op = TRUE;
-		sess->ops++;
-	}
 	sess->total++;
-	if (*name == '+')
+
+	/* most ircds don't support multiple modechars infront of the nickname
+      for /NAMES - though they should. */
+	while (prefix_chars)
 	{
+		update_counts (sess, user, name[0], TRUE, 1);
 		name++;
-		user->voice = TRUE;
-		sess->voices++;
-	}
-	if (*name == '%')
-	{
-		name++;
-		user->hop = TRUE;
-		sess->hops++;
+		prefix_chars--;
 	}
+
 	if (hostname)
 		user->hostname = strdup (hostname);
 	safe_strcpy (user->nick, name, NICKLEN);
-	row = userlist_insertname_sorted (sess, user);
+	row = userlist_insertname (sess, user);
 
 	fe_userlist_insert (sess, user, row);
 	fe_userlist_numbers (sess);
-
-	return;
 }
 
 void
