@@ -21,12 +21,19 @@
 #include <string.h>
 #include <stdlib.h>
 #include <sys/types.h>
+#ifdef WIN32
+#include <sys/timeb.h>
+#include <winsock.h>
+#include <process.h>
+#else
 #include <sys/time.h>
 #include <fcntl.h>
-#include <errno.h>
 #include <dirent.h>
-#include <ctype.h>
 #include <sys/utsname.h>
+#endif
+#include <errno.h>
+#include "xchat.h"
+#include <ctype.h>
 #include "util.h"
 #include "../../config.h"
 #ifdef USING_FREEBSD
@@ -40,26 +47,29 @@
 #define snprintf g_snprintf
 #endif
 
-extern char *g_get_home_dir (void);
-extern int g_snprintf (char *str, size_t n, const char *format, ...);
-
-
 #ifdef USE_DEBUG
 
+#undef free
+#undef malloc
+#undef realloc
+#undef strdup
+
 int current_mem_usage;
 
 struct mem_block
 {
+	char *file;
 	void *buf;
 	int size;
+	int line;
+	int total;
 	struct mem_block *next;
 };
 
 struct mem_block *mroot = NULL;
 
-
 void *
-xchat_malloc (int size)
+xchat_malloc (int size, char *file, int line)
 {
 	void *ret;
 	struct mem_block *new;
@@ -76,30 +86,32 @@ xchat_malloc (int size)
 	new->buf = ret;
 	new->size = size;
 	new->next = mroot;
+	new->line = line;
+	new->file = strdup (file);
 	mroot = new;
 
-	printf ("Malloc'ed %d bytes, now \033[35m%d\033[m\n", size,
-			  current_mem_usage);
+	printf ("%s:%d Malloc'ed %d bytes, now \033[35m%d\033[m\n", file, line,
+				size, current_mem_usage);
 
 	return ret;
 }
 
 void *
-xchat_realloc (char *old, int len)
+xchat_realloc (char *old, int len, char *file, int line)
 {
 	char *ret;
 
-	ret = xchat_malloc (len);
+	ret = xchat_malloc (len, file, line);
 	if (ret)
 	{
 		strcpy (ret, old);
-		xchat_free (old);
+		xchat_free (old, file, line);
 	}
 	return ret;
 }
 
 void *
-xchat_strdup (char *str)
+xchat_strdup (char *str, char *file, int line)
 {
 	void *ret;
 	struct mem_block *new;
@@ -119,16 +131,59 @@ xchat_strdup (char *str)
 	new->buf = ret;
 	new->size = size;
 	new->next = mroot;
+	new->line = line;
+	new->file = strdup (file);
 	mroot = new;
 
-	printf ("strdup (\"%-.40s\") size: %d, total: \033[35m%d\033[m\n", str,
-			  size, current_mem_usage);
+	printf ("%s:%d strdup (\"%-.40s\") size: %d, total: \033[35m%d\033[m\n",
+				file, line, str, size, current_mem_usage);
 
 	return ret;
 }
 
 void
-xchat_free (void *buf)
+xchat_mem_list (void)
+{
+	struct mem_block *cur, *p;
+	GSList *totals = 0;
+	GSList *list;
+
+	cur = mroot;
+	while (cur)
+	{
+		list = totals;
+		while (list)
+		{
+			p = list->data;
+			if (p->line == cur->line &&
+					strcmp (p->file, cur->file) == 0)
+			{
+				p->total += p->size;
+				break;
+			}
+			list = list->next;
+		}
+		if (!list)
+		{
+			cur->total = cur->size;
+			totals = g_slist_prepend (totals, cur);
+		}
+		cur = cur->next;
+	}
+
+	fprintf (stderr, "file              line   size    num  total\n");  
+	list = totals;
+	while (list)
+	{
+		cur = list->data;
+		fprintf (stderr, "%-15.15s %6d %6d %6d %6d\n", cur->file, cur->line,
+					cur->size, cur->total/cur->size, cur->total);
+		list = list->next;
+	}
+}
+
+void
+xchat_free (void *buf, char *file, int line)
 {
 	struct mem_block *cur, *last;
 
@@ -143,44 +198,30 @@ xchat_free (void *buf)
 	}
 	if (cur == NULL)
 	{
-		printf ("\033[31mTried to free unknown block %lx!\033[m\n",
-				  (unsigned long) buf);
+		printf ("%s:%d \033[31mTried to free unknown block %lx!\033[m\n",
+				  file, line, (unsigned long) buf);
 		/*      abort(); */
 		free (buf);
-		free (cur);
 		return;
 	}
 	current_mem_usage -= cur->size;
-	printf ("Free'ed %d bytes, usage now \033[35m%d\033[m\n", cur->size,
-			  current_mem_usage);
+	printf ("%s:%d Free'ed %d bytes, usage now \033[35m%d\033[m\n",
+				file, line, cur->size, current_mem_usage);
 	if (last)
 		last->next = cur->next;
 	else
 		mroot = cur->next;
+	free (cur->file);
 	free (cur);
 }
 
-#define malloc xchat_malloc
-#define realloc xchat_realloc
-#define free xchat_free
-#define strdup xchat_strdup
+#define malloc(n) xchat_malloc(n, __FILE__, __LINE__)
+#define realloc(n, m) xchat_realloc(n, m, __FILE__, __LINE__)
+#define free(n) xchat_free(n, __FILE__, __LINE__)
+#define strdup(n) xchat_strdup(n, __FILE__, __LINE__)
 
 #endif /* MEMORY_DEBUG */
 
-int
-is_channel (char *string)
-{
-	switch (*string)
-	{
-	case '#':						  /* normal channel */
-	case '&':						  /* normal channel */
-	case '+':						  /* modeless channel */
-	case '!':						  /* some obscure IRC RFC breaking channel */
-		return 1;
-	}
-	return 0;
-}
-
 char *
 file_part (char *file)
 {
@@ -194,6 +235,9 @@ file_part (char *file)
 		case 0:
 			return (filepart);
 		case '/':
+#ifdef WIN32
+		case '\\':
+#endif
 			filepart = file + 1;
 			break;
 		}
@@ -216,7 +260,7 @@ nocasestrstr (char *s, char *wanted)
 
 	if (len == 0)
 		return (char *)s;
-	while (toupper(*s) != toupper(*wanted) || strncasecmp (s, wanted, len))
+	while (tolower(*s) != tolower(*wanted) || strncasecmp (s, wanted, len))
 		if (*s++ == '\0')
 			return (char *)NULL;
 	return (char *)s;
@@ -231,18 +275,31 @@ errorstring (int err)
 	case -1:
 		return "";
 	case 0:
-		return "Remote host closed socket";
+		return _("Remote host closed socket");
+#ifndef WIN32
 	case ECONNREFUSED:
-		return "Connection refused";
+		return _("Connection refused");
 	case ENETUNREACH:
 	case EHOSTUNREACH:
-		return "No route to host";
+		return _("No route to host");
 	case ETIMEDOUT:
-		return "Connection timed out";
+		return _("Connection timed out");
 	case EADDRNOTAVAIL:
-		return "Cannot assign that address";
+		return _("Cannot assign that address");
 	case ECONNRESET:
-		return "Connection reset by peer";
+#else
+	case WSAECONNREFUSED:
+		return _("Connection refused");
+	case WSAENETUNREACH:
+	case WSAEHOSTUNREACH:
+		return _("No route to host");
+	case WSAETIMEDOUT:
+		return _("Connection timed out");
+	case WSAEADDRNOTAVAIL:
+		return _("Cannot assign that address");
+	case WSAECONNRESET:
+#endif
+		return _("Connection reset by peer");
 	}
 	sprintf (tbuf, "%d", err);
 	return tbuf;
@@ -257,14 +314,12 @@ waitline (int sok, char *buf, int bufsize)
 	{
 		if (read (sok, &buf[i], 1) < 1)
 			return -1;
-		if (buf[i] == '\n')
+		if (buf[i] == '\n' || bufsize == i + 1)
 		{
 			buf[i] = 0;
 			return i;
 		}
 		i++;
-		if (i == (bufsize - 1))
-			return 0;
 	}
 }
 
@@ -273,6 +328,7 @@ waitline (int sok, char *buf, int bufsize)
 char *
 expand_homedir (char *file)
 {
+#ifndef WIN32
 	char *ret;
 
 	if (*file == '~')
@@ -281,6 +337,7 @@ expand_homedir (char *file)
 		sprintf (ret, "%s%s", g_get_home_dir (), file + 1);
 		return ret;
 	}
+#endif
 	return strdup (file);
 }
 
@@ -296,15 +353,14 @@ strip_color (unsigned char *text)
 	while (len > 0)
 	{
 		if ((col && isdigit (*text) && nc < 2) ||
-			 (col && *text == ',' && nc < 3))
+			 (col && *text == ',' && isdigit (*(text+1)) && nc < 3))
 		{
 			nc++;
 			if (*text == ',')
 				nc = 0;
 		} else
 		{
-			if (col)
-				col = 0;
+			col = 0;
 			switch (*text)
 			{
 			case '\003':			  /*ATTR_COLOR: */
@@ -394,51 +450,89 @@ get_cpu_info (int *mhz, int *cpus)
 	sysctlbyname("machdep.tsc_freq", &freq, &len, NULL, 0);
 	
 	*cpus = ncpu;
-	*mhz = (freq / 996500);
+	*mhz = (freq / 1000000);
 
 #endif
 
 }
-
 #endif
 
+#ifdef WIN32
+
+static int
+get_mhz (void)
+{
+	HKEY hKey;
+	int result, data, dataSize;
+
+	if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, "Hardware\\Description\\System\\"
+		"CentralProcessor\\0", 0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
+	{
+		dataSize = sizeof (data);
+		result = RegQueryValueEx (hKey, "~MHz", 0, 0, (LPBYTE)&data, &dataSize);
+		RegCloseKey (hKey);
+		if (result == ERROR_SUCCESS)
+			return data;
+	}
+	return 0;	/* fails on Win9x */
+}
+
 char *
-get_cpu_str (int color)
+get_cpu_str (void)
+{
+	static char verbuf[64];
+	OSVERSIONINFO osvi;
+	SYSTEM_INFO si;
+	int mhz;
+
+	osvi.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
+	GetVersionEx (&osvi);
+	GetSystemInfo (&si);
+
+	mhz = get_mhz ();
+	if (mhz)
+		sprintf (verbuf, "Windoze %ld.%ld [i%1d86/%dMHz]",
+			osvi.dwMajorVersion, osvi.dwMinorVersion, si.wProcessorLevel, mhz);
+	else
+		sprintf (verbuf, "Windoze %ld.%ld [i%1d86]",
+			osvi.dwMajorVersion, osvi.dwMinorVersion, si.wProcessorLevel);
+
+	return verbuf;
+}
+
+#else
+
+char *
+get_cpu_str (void)
 {
 #if defined (USING_LINUX) || defined (USING_FREEBSD)
 	int mhz, cpus;
 #endif
 	struct utsname un;
 	static char *buf = NULL;
-	static char *color_buf;
 
 	if (buf)
-	{
-		return (color) ? color_buf : buf;
-	}
+		return buf;
 
-	color_buf = malloc (128);
+	buf = malloc (128);
 
 	uname (&un);
 
 #if defined (USING_LINUX) || defined (USING_FREEBSD)
 	get_cpu_info (&mhz, &cpus);
 	if (mhz)
-		snprintf (color_buf, 128,
-					 (cpus ==
-					  1) ? "%s \00310%s \0032[\00310%s\0032/\00310%dMHz\0032]" :
-					 "%s \00310%s \0032[\00310%s\0032/\00310%dMHz\0032/\00310SMP\0032]",
+		snprintf (buf, 128,
+					 (cpus == 1) ? "%s %s [%s/%dMHz]" : "%s %s [%s/%dMHz/SMP]",
 					 un.sysname, un.release, un.machine, mhz);
 	else
 #endif
-		snprintf (color_buf, 128, "%s \00310%s \0032[\00310%s\0032]",
-					 un.sysname, un.release, un.machine);
+		snprintf (buf, 128, "%s %s [%s]", un.sysname, un.release, un.machine);
 
-	buf = strip_color (color_buf);
-
-	return (color) ? color_buf : buf;
+	return buf;
 }
 
+#endif
+
 int
 buf_get_line (char *ibuf, char **buf, int *position, int len)
 {
@@ -571,16 +665,6 @@ for_files (char *dirname, char *mask, void callback (char *file))
 	return i;
 }
 
-/*void
-toupperStr (char *str)
-{
-   while (*str)
-   {
-      *str = toupper (*str);
-      str++;
-   }
-}*/
-
 void
 tolowerStr (char *str)
 {
@@ -603,266 +687,267 @@ country (char *hostname)
 	}
 	Domain;
 	static Domain domain[] = {
-		{"AD", "Andorra"},
-		{"AE", "United Arab Emirates"},
-		{"AF", "Afghanistan"},
-		{"AG", "Antigua and Barbuda"},
-		{"AI", "Anguilla"},
-		{"AL", "Albania"},
-		{"AM", "Armenia"},
-		{"AN", "Netherlands Antilles"},
-		{"AO", "Angola"},
-		{"AQ", "Antarctica"},
-		{"AR", "Argentina"},
-		{"AS", "American Samoa"},
-		{"AT", "Austria"},
-		{"AU", "Australia"},
-		{"AW", "Aruba"},
-		{"AZ", "Azerbaijan"},
-		{"BA", "Bosnia and Herzegovina"},
-		{"BB", "Barbados"},
-		{"BD", "Bangladesh"},
-		{"BE", "Belgium"},
-		{"BF", "Burkina Faso"},
-		{"BG", "Bulgaria"},
-		{"BH", "Bahrain"},
-		{"BI", "Burundi"},
-		{"BJ", "Benin"},
-		{"BM", "Bermuda"},
-		{"BN", "Brunei Darussalam"},
-		{"BO", "Bolivia"},
-		{"BR", "Brazil"},
-		{"BS", "Bahamas"},
-		{"BT", "Bhutan"},
-		{"BV", "Bouvet Island"},
-		{"BW", "Botswana"},
-		{"BY", "Belarus"},
-		{"BZ", "Belize"},
-		{"CA", "Canada"},
-		{"CC", "Cocos Islands"},
-		{"CF", "Central African Republic"},
-		{"CG", "Congo"},
-		{"CH", "Switzerland"},
-		{"CI", "Cote D'ivoire"},
-		{"CK", "Cook Islands"},
-		{"CL", "Chile"},
-		{"CM", "Cameroon"},
-		{"CN", "China"},
-		{"CO", "Colombia"},
-		{"CR", "Costa Rica"},
-		{"CS", "Former Czechoslovakia"},
-		{"CU", "Cuba"},
-		{"CV", "Cape Verde"},
-		{"CX", "Christmas Island"},
-		{"CY", "Cyprus"},
-		{"CZ", "Czech Republic"},
-		{"DE", "Germany"},
-		{"DJ", "Djibouti"},
-		{"DK", "Denmark"},
-		{"DM", "Dominica"},
-		{"DO", "Dominican Republic"},
-		{"DZ", "Algeria"},
-		{"EC", "Ecuador"},
-		{"EE", "Estonia"},
-		{"EG", "Egypt"},
-		{"EH", "Western Sahara"},
-		{"ER", "Eritrea"},
-		{"ES", "Spain"},
-		{"ET", "Ethiopia"},
-		{"FI", "Finland"},
-		{"FJ", "Fiji"},
-		{"FK", "Falkland Islands"},
-		{"FM", "Micronesia"},
-		{"FO", "Faroe Islands"},
-		{"FR", "France"},
-		{"FX", "France, Metropolitan"},
-		{"GA", "Gabon"},
-		{"GB", "Great Britain"},
-		{"GD", "Grenada"},
-		{"GE", "Georgia"},
-		{"GF", "French Guiana"},
-		{"GH", "Ghana"},
-		{"GI", "Gibraltar"},
-		{"GL", "Greenland"},
-		{"GM", "Gambia"},
-		{"GN", "Guinea"},
-		{"GP", "Guadeloupe"},
-		{"GQ", "Equatorial Guinea"},
-		{"GR", "Greece"},
-		{"GS", "S. Georgia and S. Sandwich Isles."},
-		{"GT", "Guatemala"},
-		{"GU", "Guam"},
-		{"GW", "Guinea-Bissau"},
-		{"GY", "Guyana"},
-		{"HK", "Hong Kong"},
-		{"HM", "Heard and McDonald Islands"},
-		{"HN", "Honduras"},
-		{"HR", "Croatia"},
-		{"HT", "Haiti"},
-		{"HU", "Hungary"},
-		{"ID", "Indonesia"},
-		{"IE", "Ireland"},
-		{"IL", "Israel"},
-		{"IN", "India"},
-		{"IO", "British Indian Ocean Territory"},
-		{"IQ", "Iraq"},
-		{"IR", "Iran"},
-		{"IS", "Iceland"},
-		{"IT", "Italy"},
-		{"JM", "Jamaica"},
-		{"JO", "Jordan"},
-		{"JP", "Japan"},
-		{"KE", "Kenya"},
-		{"KG", "Kyrgyzstan"},
-		{"KH", "Cambodia"},
-		{"KI", "Kiribati"},
-		{"KM", "Comoros"},
-		{"KN", "St. Kitts and Nevis"},
-		{"KP", "North Korea"},
-		{"KR", "South Korea"},
-		{"KW", "Kuwait"},
-		{"KY", "Cayman Islands"},
-		{"KZ", "Kazakhstan"},
-		{"LA", "Laos"},
-		{"LB", "Lebanon"},
-		{"LC", "Saint Lucia"},
-		{"LI", "Liechtenstein"},
-		{"LK", "Sri Lanka"},
-		{"LR", "Liberia"},
-		{"LS", "Lesotho"},
-		{"LT", "Lithuania"},
-		{"LU", "Luxembourg"},
-		{"LV", "Latvia"},
-		{"LY", "Libya"},
-		{"MA", "Morocco"},
-		{"MC", "Monaco"},
-		{"MD", "Moldova"},
-		{"MG", "Madagascar"},
-		{"MH", "Marshall Islands"},
-		{"MK", "Macedonia"},
-		{"ML", "Mali"},
-		{"MM", "Myanmar"},
-		{"MN", "Mongolia"},
-		{"MO", "Macau"},
-		{"MP", "Northern Mariana Islands"},
-		{"MQ", "Martinique"},
-		{"MR", "Mauritania"},
-		{"MS", "Montserrat"},
-		{"MT", "Malta"},
-		{"MU", "Mauritius"},
-		{"MV", "Maldives"},
-		{"MW", "Malawi"},
-		{"MX", "Mexico"},
-		{"MY", "Malaysia"},
-		{"MZ", "Mozambique"},
-		{"NA", "Namibia"},
-		{"NC", "New Caledonia"},
-		{"NE", "Niger"},
-		{"NF", "Norfolk Island"},
-		{"NG", "Nigeria"},
-		{"NI", "Nicaragua"},
-		{"NL", "Netherlands"},
-		{"NO", "Norway"},
-		{"NP", "Nepal"},
-		{"NR", "Nauru"},
-		{"NT", "Neutral Zone"},
-		{"NU", "Niue"},
-		{"NZ", "New Zealand"},
-		{"OM", "Oman"},
-		{"PA", "Panama"},
-		{"PE", "Peru"},
-		{"PF", "French Polynesia"},
-		{"PG", "Papua New Guinea"},
-		{"PH", "Philippines"},
-		{"PK", "Pakistan"},
-		{"PL", "Poland"},
-		{"PM", "St. Pierre and Miquelon"},
-		{"PN", "Pitcairn"},
-		{"PR", "Puerto Rico"},
-		{"PT", "Portugal"},
-		{"PW", "Palau"},
-		{"PY", "Paraguay"},
-		{"QA", "Qatar"},
-		{"RE", "Reunion"},
-		{"RO", "Romania"},
-		{"RU", "Russian Federation"},
-		{"RW", "Rwanda"},
-		{"SA", "Saudi Arabia"},
-		{"Sb", "Solomon Islands"},
-		{"SC", "Seychelles"},
-		{"SD", "Sudan"},
-		{"SE", "Sweden"},
-		{"SG", "Singapore"},
-		{"SH", "St. Helena"},
-		{"SI", "Slovenia"},
-		{"SJ", "Svalbard and Jan Mayen Islands"},
-		{"SK", "Slovak Republic"},
-		{"SL", "Sierra Leone"},
-		{"SM", "San Marino"},
-		{"SN", "Senegal"},
-		{"SO", "Somalia"},
-		{"SR", "Suriname"},
-		{"ST", "Sao Tome and Principe"},
-		{"SU", "Former USSR"},
-		{"SV", "El Salvador"},
-		{"SY", "Syria"},
-		{"SZ", "Swaziland"},
-		{"TC", "Turks and Caicos Islands"},
-		{"TD", "Chad"},
-		{"TF", "French Southern Territories"},
-		{"TG", "Togo"},
-		{"TH", "Thailand"},
-		{"TJ", "Tajikistan"},
-		{"TK", "Tokelau"},
-		{"TM", "Turkmenistan"},
-		{"TN", "Tunisia"},
-		{"TO", "Tonga"},
-		{"TP", "East Timor"},
-		{"TR", "Turkey"},
-		{"TT", "Trinidad and Tobago"},
-		{"TV", "Tuvalu"},
-		{"TW", "Taiwan"},
-		{"TZ", "Tanzania"},
-		{"UA", "Ukraine"},
-		{"UG", "Uganda"},
-		{"UK", "United Kingdom"},
-		{"UM", "US Minor Outlying Islands"},
-		{"US", "United States of America"},
-		{"UY", "Uruguay"},
-		{"UZ", "Uzbekistan"},
-		{"VA", "Vatican City State"},
-		{"VC", "St. Vincent and the grenadines"},
-		{"VE", "Venezuela"},
-		{"VG", "British Virgin Islands"},
-		{"VI", "US Virgin Islands"},
-		{"VN", "Vietnam"},
-		{"VU", "Vanuatu"},
-		{"WF", "Wallis and Futuna Islands"},
-		{"WS", "Samoa"},
-		{"YE", "Yemen"},
-		{"YT", "Mayotte"},
-		{"YU", "Yugoslavia"},
-		{"ZA", "South Africa"},
-		{"ZM", "Zambia"},
-		{"ZR", "Zaire"},
-		{"ZW", "Zimbabwe"},
-		{"COM", "Internic Commercial"},
-		{"EDU", "Educational Institution"},
-		{"GOV", "Government"},
-		{"INT", "International"},
-		{"MIL", "Military"},
-		{"NET", "Internic Network"},
-		{"ORG", "Internic Non-Profit Organization"},
-		{"RPA", "Old School ARPAnet"},
-		{"ATO", "Nato Fiel"},
-		{"MED", "United States Medical"},
-		{"ARPA", "Reverse DNS"},
+		{"AD", N_("Andorra") },
+		{"AE", N_("United Arab Emirates") },
+		{"AF", N_("Afghanistan") },
+		{"AG", N_("Antigua and Barbuda") },
+		{"AI", N_("Anguilla") },
+		{"AL", N_("Albania") },
+		{"AM", N_("Armenia") },
+		{"AN", N_("Netherlands Antilles") },
+		{"AO", N_("Angola") },
+		{"AQ", N_("Antarctica") },
+		{"AR", N_("Argentina") },
+		{"AS", N_("American Samoa") },
+		{"AT", N_("Austria") },
+		{"AU", N_("Australia") },
+		{"AW", N_("Aruba") },
+		{"AZ", N_("Azerbaijan") },
+		{"BA", N_("Bosnia and Herzegovina") },
+		{"BB", N_("Barbados") },
+		{"BD", N_("Bangladesh") },
+		{"BE", N_("Belgium") },
+		{"BF", N_("Burkina Faso") },
+		{"BG", N_("Bulgaria") },
+		{"BH", N_("Bahrain") },
+		{"BI", N_("Burundi") },
+		{"BJ", N_("Benin") },
+		{"BM", N_("Bermuda") },
+		{"BN", N_("Brunei Darussalam") },
+		{"BO", N_("Bolivia") },
+		{"BR", N_("Brazil") },
+		{"BS", N_("Bahamas") },
+		{"BT", N_("Bhutan") },
+		{"BV", N_("Bouvet Island") },
+		{"BW", N_("Botswana") },
+		{"BY", N_("Belarus") },
+		{"BZ", N_("Belize") },
+		{"CA", N_("Canada") },
+		{"CC", N_("Cocos Islands") },
+		{"CF", N_("Central African Republic") },
+		{"CG", N_("Congo") },
+		{"CH", N_("Switzerland") },
+		{"CI", N_("Cote D'ivoire") },
+		{"CK", N_("Cook Islands") },
+		{"CL", N_("Chile") },
+		{"CM", N_("Cameroon") },
+		{"CN", N_("China") },
+		{"CO", N_("Colombia") },
+		{"CR", N_("Costa Rica") },
+		{"CS", N_("Former Czechoslovakia") },
+		{"CU", N_("Cuba") },
+		{"CV", N_("Cape Verde") },
+		{"CX", N_("Christmas Island") },
+		{"CY", N_("Cyprus") },
+		{"CZ", N_("Czech Republic") },
+		{"DE", N_("Germany") },
+		{"DJ", N_("Djibouti") },
+		{"DK", N_("Denmark") },
+		{"DM", N_("Dominica") },
+		{"DO", N_("Dominican Republic") },
+		{"DZ", N_("Algeria") },
+		{"EC", N_("Ecuador") },
+		{"EE", N_("Estonia") },
+		{"EG", N_("Egypt") },
+		{"EH", N_("Western Sahara") },
+		{"ER", N_("Eritrea") },
+		{"ES", N_("Spain") },
+		{"ET", N_("Ethiopia") },
+		{"FI", N_("Finland") },
+		{"FJ", N_("Fiji") },
+		{"FK", N_("Falkland Islands") },
+		{"FM", N_("Micronesia") },
+		{"FO", N_("Faroe Islands") },
+		{"FR", N_("France") },
+		{"FX", N_("France, Metropolitan") },
+		{"GA", N_("Gabon") },
+		{"GB", N_("Great Britain") },
+		{"GD", N_("Grenada") },
+		{"GE", N_("Georgia") },
+		{"GF", N_("French Guiana") },
+		{"GG", N_("British Channel Isles") },
+		{"GH", N_("Ghana") },
+		{"GI", N_("Gibraltar") },
+		{"GL", N_("Greenland") },
+		{"GM", N_("Gambia") },
+		{"GN", N_("Guinea") },
+		{"GP", N_("Guadeloupe") },
+		{"GQ", N_("Equatorial Guinea") },
+		{"GR", N_("Greece") },
+		{"GS", N_("S. Georgia and S. Sandwich Isles.") },
+		{"GT", N_("Guatemala") },
+		{"GU", N_("Guam") },
+		{"GW", N_("Guinea-Bissau") },
+		{"GY", N_("Guyana") },
+		{"HK", N_("Hong Kong") },
+		{"HM", N_("Heard and McDonald Islands") },
+		{"HN", N_("Honduras") },
+		{"HR", N_("Croatia") },
+		{"HT", N_("Haiti") },
+		{"HU", N_("Hungary") },
+		{"ID", N_("Indonesia") },
+		{"IE", N_("Ireland") },
+		{"IL", N_("Israel") },
+		{"IN", N_("India") },
+		{"IO", N_("British Indian Ocean Territory") },
+		{"IQ", N_("Iraq") },
+		{"IR", N_("Iran") },
+		{"IS", N_("Iceland") },
+		{"IT", N_("Italy") },
+		{"JM", N_("Jamaica") },
+		{"JO", N_("Jordan") },
+		{"JP", N_("Japan") },
+		{"KE", N_("Kenya") },
+		{"KG", N_("Kyrgyzstan") },
+		{"KH", N_("Cambodia") },
+		{"KI", N_("Kiribati") },
+		{"KM", N_("Comoros") },
+		{"KN", N_("St. Kitts and Nevis") },
+		{"KP", N_("North Korea") },
+		{"KR", N_("South Korea") },
+		{"KW", N_("Kuwait") },
+		{"KY", N_("Cayman Islands") },
+		{"KZ", N_("Kazakhstan") },
+		{"LA", N_("Laos") },
+		{"LB", N_("Lebanon") },
+		{"LC", N_("Saint Lucia") },
+		{"LI", N_("Liechtenstein") },
+		{"LK", N_("Sri Lanka") },
+		{"LR", N_("Liberia") },
+		{"LS", N_("Lesotho") },
+		{"LT", N_("Lithuania") },
+		{"LU", N_("Luxembourg") },
+		{"LV", N_("Latvia") },
+		{"LY", N_("Libya") },
+		{"MA", N_("Morocco") },
+		{"MC", N_("Monaco") },
+		{"MD", N_("Moldova") },
+		{"MG", N_("Madagascar") },
+		{"MH", N_("Marshall Islands") },
+		{"MK", N_("Macedonia") },
+		{"ML", N_("Mali") },
+		{"MM", N_("Myanmar") },
+		{"MN", N_("Mongolia") },
+		{"MO", N_("Macau") },
+		{"MP", N_("Northern Mariana Islands") },
+		{"MQ", N_("Martinique") },
+		{"MR", N_("Mauritania") },
+		{"MS", N_("Montserrat") },
+		{"MT", N_("Malta") },
+		{"MU", N_("Mauritius") },
+		{"MV", N_("Maldives") },
+		{"MW", N_("Malawi") },
+		{"MX", N_("Mexico") },
+		{"MY", N_("Malaysia") },
+		{"MZ", N_("Mozambique") },
+		{"NA", N_("Namibia") },
+		{"NC", N_("New Caledonia") },
+		{"NE", N_("Niger") },
+		{"NF", N_("Norfolk Island") },
+		{"NG", N_("Nigeria") },
+		{"NI", N_("Nicaragua") },
+		{"NL", N_("Netherlands") },
+		{"NO", N_("Norway") },
+		{"NP", N_("Nepal") },
+		{"NR", N_("Nauru") },
+		{"NT", N_("Neutral Zone") },
+		{"NU", N_("Niue") },
+		{"NZ", N_("New Zealand") },
+		{"OM", N_("Oman") },
+		{"PA", N_("Panama") },
+		{"PE", N_("Peru") },
+		{"PF", N_("French Polynesia") },
+		{"PG", N_("Papua New Guinea") },
+		{"PH", N_("Philippines") },
+		{"PK", N_("Pakistan") },
+		{"PL", N_("Poland") },
+		{"PM", N_("St. Pierre and Miquelon") },
+		{"PN", N_("Pitcairn") },
+		{"PR", N_("Puerto Rico") },
+		{"PT", N_("Portugal") },
+		{"PW", N_("Palau") },
+		{"PY", N_("Paraguay") },
+		{"QA", N_("Qatar") },
+		{"RE", N_("Reunion") },
+		{"RO", N_("Romania") },
+		{"RU", N_("Russian Federation") },
+		{"RW", N_("Rwanda") },
+		{"SA", N_("Saudi Arabia") },
+		{"Sb", N_("Solomon Islands") },
+		{"SC", N_("Seychelles") },
+		{"SD", N_("Sudan") },
+		{"SE", N_("Sweden") },
+		{"SG", N_("Singapore") },
+		{"SH", N_("St. Helena") },
+		{"SI", N_("Slovenia") },
+		{"SJ", N_("Svalbard and Jan Mayen Islands") },
+		{"SK", N_("Slovak Republic") },
+		{"SL", N_("Sierra Leone") },
+		{"SM", N_("San Marino") },
+		{"SN", N_("Senegal") },
+		{"SO", N_("Somalia") },
+		{"SR", N_("Suriname") },
+		{"ST", N_("Sao Tome and Principe") },
+		{"SU", N_("Former USSR") },
+		{"SV", N_("El Salvador") },
+		{"SY", N_("Syria") },
+		{"SZ", N_("Swaziland") },
+		{"TC", N_("Turks and Caicos Islands") },
+		{"TD", N_("Chad") },
+		{"TF", N_("French Southern Territories") },
+		{"TG", N_("Togo") },
+		{"TH", N_("Thailand") },
+		{"TJ", N_("Tajikistan") },
+		{"TK", N_("Tokelau") },
+		{"TM", N_("Turkmenistan") },
+		{"TN", N_("Tunisia") },
+		{"TO", N_("Tonga") },
+		{"TP", N_("East Timor") },
+		{"TR", N_("Turkey") },
+		{"TT", N_("Trinidad and Tobago") },
+		{"TV", N_("Tuvalu") },
+		{"TW", N_("Taiwan") },
+		{"TZ", N_("Tanzania") },
+		{"UA", N_("Ukraine") },
+		{"UG", N_("Uganda") },
+		{"UK", N_("United Kingdom") },
+		{"UM", N_("US Minor Outlying Islands") },
+		{"US", N_("United States of America") },
+		{"UY", N_("Uruguay") },
+		{"UZ", N_("Uzbekistan") },
+		{"VA", N_("Vatican City State") },
+		{"VC", N_("St. Vincent and the grenadines") },
+		{"VE", N_("Venezuela") },
+		{"VG", N_("British Virgin Islands") },
+		{"VI", N_("US Virgin Islands") },
+		{"VN", N_("Vietnam") },
+		{"VU", N_("Vanuatu") },
+		{"WF", N_("Wallis and Futuna Islands") },
+		{"WS", N_("Samoa") },
+		{"YE", N_("Yemen") },
+		{"YT", N_("Mayotte") },
+		{"YU", N_("Yugoslavia") },
+		{"ZA", N_("South Africa") },
+		{"ZM", N_("Zambia") },
+		{"ZR", N_("Zaire") },
+		{"ZW", N_("Zimbabwe") },
+		{"COM", N_("Internic Commercial") },
+		{"EDU", N_("Educational Institution") },
+		{"GOV", N_("Government") },
+		{"INT", N_("International") },
+		{"MIL", N_("Military") },
+		{"NET", N_("Internic Network") },
+		{"ORG", N_("Internic Non-Profit Organization") },
+		{"RPA", N_("Old School ARPAnet") },
+		{"ATO", N_("Nato Fiel") },
+		{"MED", N_("United States Medical") },
+		{"ARPA", N_("Reverse DNS") },
 		{NULL, NULL}
 	};
 	char *p;
-	int i = 0;
+	int i;
 	if (!hostname || !*hostname || isdigit (hostname[strlen (hostname) - 1]))
-		return "unknown";
+		return _("unknown");
 	if ((p = strrchr (hostname, '.')))
 		p++;
 	else
@@ -870,7 +955,7 @@ country (char *hostname)
 	for (i = 0; domain[i].code; i++)
 		if (!strcasecmp (p, domain[i].code))
 			return domain[i].country;
-	return "unknown";
+	return _("unknown");
 }
 
 /* I think gnome1.0.x isn't necessarily linked against popt, ah well! */
@@ -907,6 +992,7 @@ static int my_poptParseArgvString(const char * s, int * argcPtr, char *** argvPt
 		src++;
 		if (!*src) {
 		    free(argv);
+			 free(bufStart);
 		    return 1;
 		}
 		if (*src != quote) *buf++ = '\\';
@@ -930,6 +1016,7 @@ static int my_poptParseArgvString(const char * s, int * argcPtr, char *** argvPt
 	    src++;
 	    if (!*src) {
 		free(argv);
+		free(bufStart);
 		return 1;
 	    }
 	    /* fallthrough */
@@ -947,7 +1034,7 @@ static int my_poptParseArgvString(const char * s, int * argcPtr, char *** argvPt
     dst = malloc((argc + 1) * sizeof(*argv) + (buf - bufStart));
     argv2 = (void *) dst;
     dst += (argc + 1) * sizeof(*argv);
-    memcpy(argv2, argv, argc * sizeof(*argv));
+    memcpy((void *)argv2, argv, argc * sizeof(*argv));
     argv2[argc] = NULL;
     memcpy(dst, bufStart, buf - bufStart);
 
@@ -965,8 +1052,6 @@ static int my_poptParseArgvString(const char * s, int * argcPtr, char *** argvPt
     return 0;
 }
 
-/*#endif*/
-
 int
 util_exec (char *cmd)
 {
@@ -974,13 +1059,10 @@ util_exec (char *cmd)
 	char **argv;
 	int argc;
 
-/*#ifdef USE_GNOME
-	if (poptParseArgvString (cmd, &argc, &argv) != 0)
-#else*/
 	if (my_poptParseArgvString (cmd, &argc, &argv) != 0)
-/*#endif*/
 		return -1;
 
+#ifndef WIN32
 	pid = fork ();
 	if (pid == -1)
 		return -1;
@@ -993,6 +1075,24 @@ util_exec (char *cmd)
 		free (argv);
 		return pid;
 	}
+#else
+	spawnvp (_P_DETACH, argv[0], argv);
+	free (argv);
+	return 0;
+#endif
+}
+
+unsigned long
+make_ping_time (void)
+{
+#ifndef WIN32
+	struct timeval timev;
+	gettimeofday (&timev, 0);
+#else
+	GTimeVal timev;
+	g_get_current_time (&timev);
+#endif
+	return (timev.tv_sec - 50000) * 1000000 + timev.tv_usec;
 }
 
 
@@ -1009,7 +1109,6 @@ util_exec (char *cmd)
  *        --+ Dagmar d'Surreal
  */
 
-
 int
 rfc_casecmp (char *s1, char *s2)
 {
@@ -1017,7 +1116,7 @@ rfc_casecmp (char *s1, char *s2)
 	register unsigned char *str2 = (unsigned char *) s2;
 	register int res;
 
-	while ((res = toupper (*str1) - toupper (*str2)) == 0)
+	while ((res = tolower (*str1) - tolower (*str2)) == 0)
 	{
 		if (*str1 == '\0')
 			return 0;
@@ -1034,7 +1133,7 @@ rfc_ncasecmp (char *str1, char *str2, int n)
 	register unsigned char *s2 = (unsigned char *) str2;
 	register int res;
 
-	while ((res = toupper (*s1) - toupper (*s2)) == 0)
+	while ((res = tolower (*s1) - tolower (*s2)) == 0)
 	{
 		s1++;
 		s2++;
@@ -1080,7 +1179,7 @@ unsigned char tolowertab[] =
 	0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
 };
 
-unsigned char touppertab[] =
+/*static unsigned char touppertab[] =
 	{ 0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa,
 	0xb, 0xc, 0xd, 0xe, 0xf, 0x10, 0x11, 0x12, 0x13, 0x14,
 	0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d,
@@ -1113,4 +1212,4 @@ unsigned char touppertab[] =
 	0xea, 0xeb, 0xec, 0xed, 0xee, 0xef,
 	0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9,
 	0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
-};
+};*/
