@@ -17,27 +17,44 @@
  */
 
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
+#include <stdlib.h>
+#include <time.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <unistd.h>
+
+#define WANTSOCKET
+#include "inet.h"
+
+#ifndef WIN32
+#include <sys/wait.h>
+#include <signal.h>
+#endif
+
 #include "xchat.h"
 #include "fe.h"
 #include "util.h"
-#include <signal.h>
-#include <time.h>
 #include "cfgfiles.h"
 #include "ignore.h"
-#include <sys/types.h>
-#include <sys/wait.h>
-#include <sys/stat.h>
-#include <sys/time.h>
 #include "plugin.h"
+#include "notify.h"
+#include "server.h"
+#include "pythonc.h"
+#include "outbound.h"
+#include "text.h"
+#include "perlc.h"
+#include "xchatc.h"
 
 #ifdef USE_OPENSSL
 #include <openssl/ssl.h>		  /* SSL_() */
 #include "ssl.h"
 #endif
 
+#ifdef USE_JCODE
+#include "jcode.h"
+#endif
+
 GSList *popup_list = 0;
 GSList *button_list = 0;
 GSList *command_list = 0;
@@ -46,83 +63,24 @@ GSList *replace_list = 0;
 GSList *sess_list = 0;
 GSList *serv_list = 0;
 GSList *dcc_list = 0;
-GSList *url_list = 0;
-GSList *away_list = 0;
 GSList *ignore_list = 0;
 GSList *usermenu_list = 0;
 GSList *urlhandler_list = 0;
+static GSList *away_list = 0;
 
-int notify_tag = -1;
-int xchat_is_quitting = 0;
-int auto_connect = 1;
+int xchat_is_quitting = FALSE;
+int auto_connect = TRUE;
 
-#ifdef USE_PERL
-extern struct session *perl_sess;
-#endif
 struct session *current_tab;
 struct session *menu_sess = 0;
 struct xchatprefs prefs;
 
-void xchat_cleanup (void);
-void auto_reconnect (struct server *serv, int send_quit, int err);
-int is_server (server * serv);
-
 #ifdef USE_OPENSSL
 SSL_CTX *ctx = NULL;
+void ssl_cb_info (SSL * s, int where, int ret);
 #endif
 
-/* trans */
-extern void serv2user (unsigned char *);
-extern void user2serv (unsigned char *);
-extern int load_trans_table (char *full_path);
-/* ~trans */
-
-/* plugin.c */
-
-extern void module_setup ();
-extern void signal_setup ();
-
-/* server.c */
-
-extern void flush_server_queue (struct server *serv);
-extern int close_socket (int sok);
-extern void connecting_fin (struct session *);
-extern void connect_server (struct session *sess, char *server, int port,
-									 int quiet);
-extern void disconnect_server (struct session *sess, int sendquit, int err);
-extern void server_sendquit (session *sess);
-extern void server_sendpart (server *serv, char *channel, char *reason);
-
-/* notify.c */
-
-extern void notify_load (void);
-extern void notify_save (void);
-extern int notify_checklist (void);
-extern void notify_cleanup (void);
-
-#ifdef USE_PERL
-/* perl.c */
-
-void perl_init (struct session *, int);
-void perl_end (void);
-#endif
-
-/* python.c */
-
-extern void pys_init ();
-extern void pys_kill ();
-
-/* text.c */
-
-extern void end_logging (int fd);
-extern void load_text_events ();
-extern void pevent_dialog_save (char *fn);
-
-#ifdef USE_OPENSSL
-/* server.c */
-
-extern void ssl_cb_info (SSL * s, int where, int ret);
-#endif
+static void free_away_messages (server *serv);
 
 
 /* actually send to the socket. This might do a character translation or
@@ -207,6 +165,7 @@ tcp_send_queue (struct server *serv)
 		for (p = buf, i = len; i && *p != ' '; p++, i--);
 			serv->next_send += (2 + i / 120);
 		serv->sendq_len -= len;
+		fe_set_throttle (serv);
 
 		tcp_send_real (serv, buf, len);
 
@@ -218,24 +177,52 @@ tcp_send_queue (struct server *serv)
 	return 0;						  /* remove the timeout handler */
 }
 
+#ifdef USE_JCODE
+static void
+fe_add_ja_rawlog (struct server *serv, char *buf,  int outbound)
+{
+	char *jbuf =  kanji_conv_to_locale(buf);
+	if (jbuf)
+	{
+		fe_add_rawlog (serv, jbuf, TRUE);
+		free(jbuf);
+	} else
+			fe_add_rawlog (serv, buf, TRUE);
+}
+#endif
+
 int
 tcp_send_len (struct server *serv, char *buf, int len)
 {
-	char *dbuf;
-	int noqueue = !serv->outbound_queue;
+    char *dbuf;
+    int noqueue = !serv->outbound_queue;
+#ifdef USE_JCODE
+    unsigned char *jbuf = NULL;
+
+    if ( prefs.kanji_conv ) {
+        jbuf = kanji_conv_auto(buf, "ISO-2022-JP");
+        if (!jbuf)
+            return 0;
+        else {
+            buf = strdup(jbuf); free(jbuf);
+            len = strlen(buf);
+        }
+    } else
+        buf = strdup(buf); /* freed after all */
+#endif    
 
 	if (!prefs.throttle)
-		return tcp_send_real (serv, buf, len);
-
-	dbuf = malloc (len + 1);
-	memcpy (dbuf, buf, len);
-	dbuf[len] = 0;			/* tcp_send_queue uses strlen */
-	serv->outbound_queue = g_slist_append (serv->outbound_queue, dbuf);
-	serv->sendq_len += len; 
-
-	if (tcp_send_queue (serv) && noqueue)
-		fe_timeout_add (500, tcp_send_queue, serv);
+  		return tcp_send_real (serv, buf, len);
 
+    dbuf = malloc (len + 1);
+    memcpy (dbuf, buf, len);
+    dbuf[len] = 0;
+    serv->outbound_queue = g_slist_append (serv->outbound_queue, dbuf);
+    serv->sendq_len += strlen (dbuf); /* tcp_send_queue uses strlen */
+        
+    if (tcp_send_queue (serv) && noqueue)
+        fe_timeout_add (500, tcp_send_queue, serv);
+    
 	return 1;
 }
 
@@ -245,30 +232,28 @@ tcp_send (struct server *serv, char *buf)
 	return tcp_send_len (serv, buf, strlen (buf));
 }
 
+static int
+is_in_list (GSList *list, void *data)
+{
+	while (list)
+	{
+		if (list->data == data)
+			return TRUE;
+		list = list->next;
+	}
+	return FALSE;
+}
+
 int
 is_server (server * serv)
 {
-	GSList *list = serv_list;
-	while (list)
-	{
-		if (list->data == serv)
-			return 1;
-		list = list->next;
-	}
-	return 0;
+	return is_in_list (serv_list, serv);
 }
 
 int
 is_session (session * sess)
 {
-	GSList *list = sess_list;
-	while (list)
-	{
-		if (list->data == sess)
-			return 1;
-		list = list->next;
-	}
-	return 0;
+	return is_in_list (sess_list, sess);
 }
 
 struct session *
@@ -280,7 +265,7 @@ find_dialog (struct server *serv, char *nick)
 	while (list)
 	{
 		sess = (struct session *) list->data;
-		if (sess->server == serv && sess->is_dialog)
+		if (sess->server == serv && sess->type == SESS_DIALOG)
 		{
 			if (!strcasecmp (nick, sess->channel))
 				return (sess);
@@ -290,19 +275,17 @@ find_dialog (struct server *serv, char *nick)
 	return 0;
 }
 
-struct session *
-find_session_from_channel (char *chan, struct server *serv)
+session *
+find_session_from_channel (char *chan, server *serv)
 {
-	struct session *sess;
+	session *sess;
 	GSList *list = sess_list;
 	while (list)
 	{
-		sess = (struct session *) list->data;
-		if (!sess->is_shell && !strcasecmp (chan, sess->channel))
+		sess = list->data;
+		if (sess->type != SESS_SHELL && !strcasecmp (chan, sess->channel))
 		{
-			if (!serv)
-				return sess;
-			if (serv == sess->server)
+			if (!serv || serv == sess->server)
 				return sess;
 		}
 		list = list->next;
@@ -310,57 +293,196 @@ find_session_from_channel (char *chan, struct server *serv)
 	return 0;
 }
 
-struct session *
-find_session_from_nick (char *nick, struct server *serv)
+#ifndef WIN32
+
+static int
+mail_items (char *file)
 {
-	struct session *sess;
-	GSList *list = sess_list;
+	FILE *fp;
+	int items;
+	char buf[512];
 
-	sess = find_session_from_channel (nick, serv);
-	if (sess)
-		return sess;
+	fp = fopen (file, "r");
+	if (!fp)
+		return 1;
 
-	if (serv->front_session)
+	items = 0;
+	while (fgets (buf, sizeof buf, fp))
 	{
-		if (find_name (serv->front_session, nick))
-			return serv->front_session;
+		if (!strncmp (buf, "From ", 5))
+			items++;
+	}
+	fclose (fp);
+
+	return items;
+}
+
+static void
+xchat_mail_check (void)
+{
+	static int last_size = -1;
+	int size;
+	struct stat st;
+	char buf[512];
+	char *maildir;
+
+	maildir = getenv ("MAIL");
+	if (!maildir)
+	{
+		snprintf (buf, sizeof (buf), "/var/spool/mail/%s", g_get_user_name ());
+		maildir = buf;
 	}
 
-	if (menu_sess && menu_sess->server == serv)
+	if (stat (maildir, &st) < 0)
+		return;
+
+	size = st.st_size;
+
+	if (last_size == -1)
 	{
-		if (find_name (menu_sess, nick))
-			return menu_sess;
+		last_size = size;
+		return;
 	}
 
+	if (size > last_size)
+	{
+		sprintf (buf, "%d", mail_items (maildir));
+		sprintf (buf + 16, "%d", size);
+		if (menu_sess && menu_sess->type != SESS_SHELL)
+			EMIT_SIGNAL (XP_TE_NEWMAIL, menu_sess, buf, buf + 16, NULL, NULL, 0);
+	}
+
+	last_size = size;
+}
+
+#endif
+
+static int
+lagcheck_update (void)
+{
+	server *serv;
+	GSList *list = serv_list;
+	
+	if (!prefs.lagometer)
+		return 1;
+
 	while (list)
 	{
-		sess = (struct session *) list->data;
-		if (sess->server == serv)
+		serv = list->data;
+		if (serv->lag_sent)
+			fe_set_lag (serv, -1);
+
+		list = list->next;
+	}
+	return 1;
+}
+
+void
+lag_check (void)
+{
+	server *serv;
+	GSList *list = serv_list;
+	unsigned long tim;
+	char tbuf[256];
+	time_t now = time (0);
+	int lag;
+
+	tim = make_ping_time ();
+
+	while (list)
+	{
+		serv = list->data;
+		if (serv->connected && serv->end_of_motd)
 		{
-			if (find_name (sess, nick))
-				return sess;
+			lag = now - serv->ping_recv;
+			if (prefs.pingtimeout && lag > prefs.pingtimeout && lag > 0)
+			{
+				sprintf (tbuf, "%d", lag);
+				EMIT_SIGNAL (XP_TE_PINGTIMEOUT, serv->front_session, tbuf, NULL,
+								 NULL, NULL, 0);
+				auto_reconnect (serv, FALSE, -1);
+			} else
+			{
+				/*sprintf (tbuf, "PING LAG%lu :%s\r\n", tim, serv->servername);*/
+				sprintf (tbuf, "PING LAG%lu\r\n", tim);
+				tcp_send (serv, tbuf);
+				serv->lag_sent = tim;
+				fe_set_lag (serv, -1);
+			}
 		}
 		list = list->next;
 	}
-	return 0;
 }
 
-struct session *
-new_session (struct server *serv, char *from)
+static int
+xchat_misc_checks (void)		  /* this gets called every 2 seconds */
 {
-	struct session *sess;
+	static int count = 0;
+
+	dcc_check_timeouts ();
+
+	count++;
+
+	if (count == 7 && prefs.lagometer)	/* every 30 seconds */
+		lag_check ();
+
+	if (count == 15)							/* every 30 seconds */
+	{
+		count = 0;
+#ifndef WIN32
+		if (prefs.mail_check)
+			xchat_mail_check ();
+#endif
+	}
+
+	return 1;
+}
+
+/* executed when the first irc window opens */
+
+static void
+irc_init (session *sess)
+{
+	static int done_init = FALSE;
+
+	if (done_init)
+		return;
+
+	done_init = TRUE;
+
+#ifdef USE_PYTHON
+	pys_init ();
+#endif
+#ifdef USE_PLUGIN
+	module_setup ();
+#endif
+#ifdef USE_PERL
+	perl_auto_load (sess);
+#endif
+
+	if (prefs.notify_timeout)
+		notify_tag = fe_timeout_add (prefs.notify_timeout * 1000,
+											  notify_checklist, 0);
+
+	fe_timeout_add (2000, xchat_misc_checks, 0);
+	fe_timeout_add (500, lagcheck_update, 0);
+}
+
+static session *
+new_session (server *serv, char *from, int type)
+{
+	session *sess;
 
 	sess = malloc (sizeof (struct session));
 	memset (sess, 0, sizeof (struct session));
 
 	sess->server = serv;
 	sess->logfd = -1;
+	sess->type = type;
+	sess->current_modes = strdup ("");
 
-	if (from)
-	{
-		sess->is_dialog = TRUE;
-		strcpy (sess->channel, from);
-	}
+	if (from != NULL)
+		safe_strcpy (sess->channel, from, CHANLEN);
 
 	sess_list = g_slist_prepend (sess_list, sess);
 
@@ -369,70 +491,108 @@ new_session (struct server *serv, char *from)
 	return sess;
 }
 
-struct server *
+void
+set_server_defaults (server *serv)
+{
+	if (serv->chantypes)
+		free (serv->chantypes);
+	if (serv->chanmodes)
+		free (serv->chanmodes);
+	if (serv->nick_prefixes)
+		free (serv->nick_prefixes);
+	if (serv->nick_modes)
+		free (serv->nick_modes);
+
+	serv->chantypes = strdup ("#&!+");
+	serv->chanmodes = strdup ("beI,k,l");
+	serv->nick_prefixes = strdup ("@%+");
+	serv->nick_modes = strdup ("ohv");
+
+	serv->nickcount = 1;
+	serv->end_of_motd = FALSE;
+	serv->is_away = FALSE;
+	serv->supports_watch = FALSE;
+	serv->bad_prefix = FALSE;
+}
+
+static server *
 new_server (void)
 {
-	struct server *serv;
+	server *serv;
 
 	serv = malloc (sizeof (struct server));
 	memset (serv, 0, sizeof (struct server));
 
 	serv->sok = -1;
-	serv->iotag = -1;
-	serv->recondelay_tag = -1;
-#ifdef USE_OPENSSL
-	serv->ssl_do_connect_tag = -1;
-#endif
 	strcpy (serv->nick, prefs.nick1);
+	set_server_defaults (serv);
+
 	serv_list = g_slist_prepend (serv_list, serv);
 
 	fe_new_server (serv);
 
-	if (prefs.use_server_tab)
-	{
-		register unsigned int oldh = prefs.hideuserlist;
-		prefs.hideuserlist = 1;
-		serv->front_session = new_session (serv, 0);
-		prefs.hideuserlist = oldh;
-		serv->front_session->is_server = TRUE;
-	}
 	return serv;
 }
 
+session *
+new_ircwindow (server *serv, char *name, int type)
+{
+	session *sess;
+
+	switch (type)
+	{
+	case SESS_SHELL:
+		sess = new_session (new_server (), name, SESS_SHELL);
+		break;
+	case SESS_SERVER:
+		serv = new_server ();
+		if (prefs.use_server_tab)
+		{
+			register unsigned int oldh = prefs.hideuserlist;
+			prefs.hideuserlist = 1;
+			sess = new_session (serv, name, SESS_SERVER);
+			prefs.hideuserlist = oldh;
+			serv->front_session = sess;
+		} else
+		{
+			sess = new_session (serv, name, SESS_CHANNEL);
+		}
+		break;
+	default:
+/*	case SESS_CHANNEL:
+	case SESS_DIALOG:
+	case SESS_NOTICES:
+	case SESS_SNOTICES:*/
+		sess = new_session (serv, name, type);
+		break;
+	}
+
+	irc_init (sess);
+
+	return sess;
+}
+
 static void
 kill_server_callback (server * serv)
 {
-	if (serv->connected)
-	{
-		if (serv->iotag != -1)
-			fe_input_remove (serv->iotag);
-		/*fe_timeout_add (5000, close_socket, (void *) serv->sok);*/
-		close (serv->sok);
-	}
-	if (serv->connecting)
-	{
-		kill (serv->childpid, SIGKILL);
-		waitpid (serv->childpid, NULL, 0 /*WNOHANG*/);
-		if (serv->iotag != -1)
-			fe_input_remove (serv->iotag);
-#ifdef USE_OPENSSL
-		if (serv->ssl_do_connect_tag != -1)
-		{
-			fe_timeout_remove (serv->ssl_do_connect_tag);
-			serv->ssl_do_connect_tag = -1;
-		}
-#endif
-		close (serv->childread);
-		close (serv->childwrite);
-		close (serv->sok);
-	}
+	server_cleanup (serv);
+
 /*	fe_server_callback (serv);*/
 
 	serv_list = g_slist_remove (serv_list, serv);
 
 	dcc_notify_kill (serv);
 	flush_server_queue (serv);
+	free_away_messages (serv);
 
+	free (serv->nick_modes);
+	free (serv->nick_prefixes);
+	free (serv->chanmodes);
+	free (serv->chantypes);
+	if (serv->bad_nick_prefixes)
+		free (serv->bad_nick_prefixes);
+	if (serv->last_away_reason)
+		free (serv->last_away_reason);
 	free (serv->gui);
 	free (serv);
 
@@ -449,44 +609,22 @@ log_notify_kill (session * sess)
 static void
 exec_notify_kill (session * sess)
 {
+#ifndef WIN32
 	struct nbexec *re;
-
 	if (sess->running_exec != NULL)
 	{
 		re = sess->running_exec;
 		sess->running_exec = NULL;
 		kill (re->childpid, SIGKILL);
+		waitpid (re->childpid, NULL, WNOHANG);
 		fe_input_remove (re->iotag);
 		close (re->myfd);
-		close (re->childfd);
+		if (re->linebuf)
+			free(re->linebuf);
 		free (re);
 	}
-}
-
-#ifdef USE_PERL
-static void
-perl_notify_kill (session * sess)
-{
-	struct session *s;
-	GSList *list = sess_list;
-
-	if (perl_sess == sess)		  /* need to find a new perl_sess, this one's closing */
-	{
-		while (list)
-		{
-			s = (struct session *) list->data;
-			if (s->server == perl_sess->server && s != perl_sess)
-			{
-				perl_sess = s;
-				break;
-			}
-			list = list->next;
-		}
-		if (perl_sess == sess)
-			perl_sess = 0;
-	}
-}
 #endif
+}
 
 static void
 send_quit_or_part (session * killsess)
@@ -524,8 +662,7 @@ send_quit_or_part (session * killsess)
 			}
 		} else
 		{
-			if (!killsess->is_dialog && !killsess->is_server
-				 && killsess->channel[0])
+			if (killsess->type == SESS_CHANNEL && killsess->channel[0])
 			{
 				server_sendpart (killserv, killsess->channel, 0);
 			}
@@ -562,6 +699,9 @@ kill_session_callback (session * killsess)
 
 	sess_list = g_slist_remove (sess_list, killsess);
 
+	if (killsess->type == SESS_CHANNEL)
+		free_userlist (killsess);
+
 	fe_session_callback (killsess);
 
 	exec_notify_kill (killsess);
@@ -576,10 +716,13 @@ kill_session_callback (session * killsess)
 
 	history_free (&killsess->history);
 	free (killsess->gui);
+	if (killsess->topic)
+		free (killsess->topic);
+	free (killsess->current_modes);
 	free (killsess);
 
 	if (!sess_list)
-		xchat_cleanup ();			  /* sess_list is empty, quit! */
+		xchat_exit ();						/* sess_list is empty, quit! */
 
 	list = sess_list;
 	while (list)
@@ -596,13 +739,11 @@ kill_session_callback (session * killsess)
 static void
 free_sessions (void)
 {
-	struct session *sess;
 	GSList *list = sess_list;
 
 	while (list)
 	{
-		sess = (struct session *) list->data;
-		fe_close_window (sess);
+		fe_close_window (list->data);
 		list = sess_list;
 	}
 }
@@ -622,6 +763,29 @@ find_away_message (struct server *serv, char *nick)
 	return 0;
 }
 
+static void
+free_away_messages (server *serv)
+{
+	GSList *list, *next;
+	struct away_msg *away;
+
+	list = away_list;
+	while (list)
+	{
+		away = list->data;
+		next = list->next;
+		if (away->server == serv)
+		{
+			away_list = g_slist_remove (away_list, away);
+			if (away->message)
+				free (away->message);
+			free (away);
+			next = away_list;
+		}
+		list = next;
+	}
+}
+
 void
 save_away_message (struct server *serv, char *nick, char *msg)
 {
@@ -646,127 +810,16 @@ save_away_message (struct server *serv, char *nick, char *msg)
 	}
 }
 
-static int
-mail_items (char *file)
-{
-	FILE *fp;
-	int items;
-	char buf[512];
-
-	fp = fopen (file, "r");
-	if (!fp)
-		return 1;
-
-	items = 0;
-	while (fgets (buf, sizeof buf, fp))
-	{
-		if (!strncmp (buf, "From ", 5))
-			items++;
-	}
-	fclose (fp);
-
-	return items;
-}
-
-static void
-xchat_mail_check (void)
-{
-	static int last_size = -1;
-	int size;
-	struct stat st;
-	char buf[512];
-	char *maildir;
-
-	maildir = getenv ("MAIL");
-	if (!maildir)
-	{
-		snprintf (buf, sizeof (buf) - 1, "/var/spool/mail/%s", g_get_user_name ());
-		maildir = buf;
-	}
-
-	if (stat (maildir, &st) < 0)
-		return;
-
-	size = st.st_size;
-
-	if (last_size == -1)
-	{
-		last_size = size;
-		return;
-	}
-
-	if (size > last_size)
-	{
-		sprintf (buf, "%d", mail_items (maildir));
-		sprintf (buf + 16, "%d", size);
-		if (menu_sess && !menu_sess->is_shell)
-			EMIT_SIGNAL (XP_TE_NEWMAIL, menu_sess, buf, buf + 16, NULL, NULL, 0);
-	}
-
-	last_size = size;
-}
-
-static void
-lag_check (void)
-{
-	server *serv;
-	GSList *list = serv_list;
-	unsigned long tim;
-	struct timeval timev;
-	char tbuf[256];
-	time_t now = time (0);
-
-	gettimeofday (&timev, 0);
-	tim = (timev.tv_sec - 50000) * 1000000 + timev.tv_usec;
-
-	while (list)
-	{
-		serv = list->data;
-		if (serv->connected && serv->end_of_motd)
-		{
-			if (prefs.pingtimeout && ((now - serv->ping_recv) > prefs.pingtimeout))
-			{
-				sprintf (tbuf, "%d", (int)(now - serv->ping_recv));
-				EMIT_SIGNAL (XP_TE_PINGTIMEOUT, serv->front_session, tbuf, NULL, NULL, NULL, 0);
-				auto_reconnect (serv, 0, -1);
-			} else
-			{
-				sprintf (tbuf, "PING LAG%lu :%s\r\n", tim, serv->servername);
-				tcp_send (serv, tbuf);
-				serv->lag_sent = tim;
-			}
-		}
-		list = list->next;
-	}
-}
-
-static int
-xchat_misc_checks (void)		  /* this gets called every 2 seconds */
-{
-	static int count = 0;
-
-	dcc_check_timeouts ();
-
-	count++;
-
-	if (count == 7 && prefs.lagometer)	/* every 30 seconds */
-		lag_check ();
-
-	if (count == 15)							/* every 30 seconds */
-	{
-		count = 0;
-		if (prefs.mail_check)
-			xchat_mail_check ();
-	}
-
-	return 1;
-}
-
 #define defaultconf_ctcp \
 	"NAME TIME\n"				"CMD /nctcp %s TIME %t\n\n"\
 	"NAME PING\n"				"CMD /nctcp %s PING %d\n\n"
 
 #define defaultconf_popup \
+	"NAME SUB\n"				"CMD Direct Client-To-Client\n\n"\
+	"NAME Send File\n"		"CMD /dcc send %s\n\n"\
+	"NAME Offer Chat\n"		"CMD /dcc chat %s\n\n"\
+	"NAME Abort Chat\n"		"CMD /dcc close chat %s\n\n"\
+	"NAME ENDSUB\n"			"CMD \n\n"\
 	"NAME SUB\n"				"CMD CTCP\n\n"\
 	"NAME Version\n"			"CMD /ctcp %s VERSION\n\n"\
 	"NAME Userinfo\n"			"CMD /ctcp %s USERINFO\n\n"\
@@ -777,13 +830,8 @@ xchat_misc_checks (void)		  /* this gets called every 2 seconds */
 	"NAME XDCC List\n"		"CMD /ctcp %s XDCC LIST\n\n"\
 	"NAME CDCC List\n"		"CMD /ctcp %s CDCC LIST\n\n"\
 	"NAME ENDSUB\n"			"CMD \n\n"\
-	"NAME SUB\n"				"CMD DCC\n\n"\
-	"NAME Send File\n"		"CMD /dcc send %s\n\n"\
-	"NAME Offer Chat\n"		"CMD /dcc chat %s\n\n"\
-	"NAME Abort Chat\n"		"CMD /dcc close chat %s\n\n"\
-	"NAME ENDSUB\n"			"CMD \n\n"\
 	"NAME SUB\n"				"CMD Oper\n\n"\
-	"NAME Kill\n"				"CMD /quote KILL %s :die!\n\n"\
+	"NAME Kill this user\n"	"CMD /quote KILL %s :die!\n\n"\
 	"NAME ENDSUB\n"			"CMD \n\n"\
 	"NAME SUB\n"				"CMD Mode\n\n"\
 	"NAME Give Voice\n"		"CMD /voice %s\n\n"\
@@ -829,10 +877,8 @@ xchat_misc_checks (void)		  /* this gets called every 2 seconds */
 	"NAME DeOp\n"				"CMD /deop %a\n\n"\
 	"NAME Ban\n"				"CMD /ban %s\n\n"\
 	"NAME Kick\n"				"CMD /kick %s\n\n"\
-	"NAME Send\n"				"CMD /dcc send %s\n\n"\
-	"NAME Dialog\n"			"CMD /query %s\n\n"\
-	"NAME Lookup\n"			"CMD /dns %s\n\n"\
-	"NAME Whois\n"				"CMD /whois %s\n"
+	"NAME Sendfile\n"			"CMD /dcc send %s\n\n"\
+	"NAME Dialog\n"			"CMD /query %s\n\n"
 
 #define defaultconf_replace \
 	"NAME teh\n"				"CMD the\n\n"\
@@ -859,7 +905,6 @@ xchat_misc_checks (void)		  /* this gets called every 2 seconds */
 	"NAME SPING\n"			"CMD /ping\n\n"\
    "NAME SV\n"				"CMD /echo xchat %v %m\n\n"\
    "NAME UMODE\n"			"CMD /mode %n &2\n\n"\
-	"NAME UNBAN\n"			"CMD /quote MODE %c -b &2\n\n"\
    "NAME UPTIME\n"		"CMD /quote STATS u\n\n"\
    "NAME VER\n"			"CMD /ctcp %2 VERSION\n\n"\
    "NAME VERSION\n"		"CMD /ctcp %2 VERSION\n\n"\
@@ -895,6 +940,11 @@ xchat_misc_checks (void)		  /* this gets called every 2 seconds */
 	"NAME Run RXVT\n"				"CMD !rxvt\n\n"\
    "NAME ENDSUB\n"				"CMD \n\n"
 
+#ifdef WIN32
+#define defaultconf_urlhandlers \
+	"NAME Open URL\n"							"CMD !start %s\n\n"\
+	"NAME Connect as IRC server\n"		"CMD /newserver %s\n\n"
+#else
 #define defaultconf_urlhandlers \
 	"NAME SUB\n"								"CMD Netscape...\n\n"\
    	"NAME Open in existing\n"			"CMD !netscape -remote 'openURL(%s)'\n\n"\
@@ -908,12 +958,20 @@ xchat_misc_checks (void)		  /* this gets called every 2 seconds */
 	"NAME ENDSUB\n"							"CMD \n\n"\
 	"NAME SUB\n"								"CMD Galeon...\n\n"\
 		"NAME Open in existing\n"			"CMD !galeon -x '%s'\n\n"\
+   	"NAME Open in new window\n"		"CMD !galeon -w '%s'\n\n"\
+   	"NAME Open in new tab\n"			"CMD !galeon -n '%s'\n\n"\
 		"NAME Run new Galeon\n"				"CMD !galeon '%s'\n\n"\
 	"NAME ENDSUB\n"							"CMD \n\n"\
+	"NAME SUB\n"								"CMD Opera...\n\n"\
+   	"NAME Open in existing\n"			"CMD !opera -remote 'openURL(%s)'\n\n"\
+   	"NAME Open in new window\n"		"CMD !opera -remote 'openURL(%s,new-window)'\n\n"\
+   	"NAME Run new Opera\n"				"CMD !opera %s\n\n"\
+	"NAME ENDSUB\n"							"CMD \n\n"\
 	"NAME SUB\n"								"CMD Send URL to...\n\n"\
    	"NAME Gnome URL Handler\n"			"CMD !gnome-moz-remote %s\n\n"\
    	"NAME Lynx\n"							"CMD !rxvt -e lynx %s\n\n"\
    	"NAME Links\n"							"CMD !rxvt -e links %s\n\n"\
+   	"NAME w3m\n"							"CMD !rxvt -e w3m %s\n\n"\
    	"NAME NcFTP\n" 						"CMD !rxvt -e ncftp %s\n\n"\
    	"NAME gFTP\n"							"CMD !gftp %s\n\n"\
 		"NAME KFM\n"							"CMD !kfmclient openURL %s\n\n"\
@@ -921,12 +979,16 @@ xchat_misc_checks (void)		  /* this gets called every 2 seconds */
 		"NAME Ping\n"							"CMD !rxvt -e ping -c 4 %s\n\n"\
 	"NAME ENDSUB\n"							"CMD \n\n"\
 	"NAME Connect as IRC server\n"		"CMD /newserver %s\n\n"
+#endif
 
 static void
 xchat_init (void)
 {
-	struct session *sess;
-	struct server *serv;
+#ifdef WIN32
+	WSADATA wsadata;
+
+	WSAStartup(0x0101, &wsadata);
+#else
 	struct sigaction act;
 
 	/* ignore SIGPIPE's */
@@ -934,6 +996,7 @@ xchat_init (void)
 	act.sa_flags = 0;
 	sigemptyset (&act.sa_mask);
 	sigaction (SIGPIPE, &act, NULL);
+#endif
 
 	signal_setup ();
 	load_text_events ();
@@ -956,41 +1019,31 @@ xchat_init (void)
 	}
 #endif
 
-	serv = new_server ();
-	if (prefs.use_server_tab)
-		sess = serv->front_session;
-	else
-		sess = new_session (serv, 0);
-
-#ifdef USE_PYTHON
-	pys_init ();
-#endif
-#ifdef USE_PLUGIN
-	module_setup ();
-#endif
-#ifdef USE_PERL
-	perl_init (sess, TRUE);
-#endif
-
-	if (prefs.notify_timeout)
-		notify_tag =
-			fe_timeout_add (prefs.notify_timeout * 1000, notify_checklist, 0);
-
-	fe_timeout_add (2000, xchat_misc_checks, 0);
+	/* if serverlist didnt open any irc windows .... */
+	if (!fe_open_serverlist (NULL, auto_connect, prefs.skipserverlist))
+	{
+		/* and serverlist isn't open .... */
+		if (prefs.skipserverlist)
+			/* we'll have to open one. */
+			new_ircwindow (NULL, NULL, SESS_SERVER);
+	}
 }
 
 void
-xchat_cleanup (void)
+xchat_exit (void)
 {
 	xchat_is_quitting = TRUE;
 #ifdef USE_PERL
 	perl_end ();
+#endif
+#ifdef USE_PYTHON
+	pys_kill ();
 #endif
 	fe_cleanup ();
 	if (prefs.autosave)
 	{
 		save_config ();
-		pevent_dialog_save (NULL);
+		pevent_save (NULL);
 	}
 	notify_save ();
 	ignore_save ();
@@ -998,6 +1051,8 @@ xchat_cleanup (void)
 	fe_exit ();
 }
 
+#ifndef WIN32
+
 static int
 child_handler (int pid)
 {
@@ -1006,21 +1061,26 @@ child_handler (int pid)
 	return 1;						  /* keep the timeout handler */
 }
 
+#endif
+
 void
 xchat_exec (char *cmd)
 {
-	int pid;
-
-	pid = util_exec (cmd);
+#ifdef WIN32
+	util_exec (cmd);
+#else
+	int pid = util_exec (cmd);
 	if (pid != -1)
 	/* zombie avoiding system. Don't ask! it has to be like this to work
       with zvt (which overrides the default handler) */
 		fe_timeout_add (1000, child_handler, (void *)pid);
+#endif
 }
 
 int
 main (int argc, char *argv[])
 {
+
 #ifdef SOCKS
 	SOCKSinit (argv[0]);
 #endif
@@ -1043,13 +1103,13 @@ main (int argc, char *argv[])
 
 	fe_main ();
 
+#ifdef USE_DEBUG
+	xchat_mem_list ();
+#endif
+
 #ifdef USE_OPENSSL
 	_SSL_context_free (ctx);
 #endif
 
-#ifdef USE_PYTHON
-	pys_kill ();					  /* this never returns! */
-#endif
-
 	return 0;
 }
