@@ -1,20 +1,11 @@
 #include "../../config.h"
 
-#ifndef VERSION	/* fixes prob compiling on RH5.2, *shrug* */
-#define PACKAGE "xchat"
-#define VERSION "1.6.3"
-#endif
-
 #ifdef USE_MYGLIB
 #include "../fe-text/myglib.h"
 #else
 #include <glib.h>
 #endif
-#include <time.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
+#include <time.h>			/* need time_t */
 
 #include "history.h"
 
@@ -23,10 +14,14 @@
 #endif
 
 #ifdef USE_DEBUG
-#define malloc xchat_malloc
-#define realloc xchat_realloc
-#define free xchat_free
-#define strdup xchat_strdup
+#define malloc(n) xchat_malloc(n, __FILE__, __LINE__)
+#define realloc(n, m) xchat_realloc(n, m, __FILE__, __LINE__)
+#define free(n) xchat_free(n, __FILE__, __LINE__)
+#define strdup(n) xchat_strdup(n, __FILE__, __LINE__)
+void *xchat_malloc (int size, char *file, int line);
+void *xchat_strdup (char *str, char *file, int line);
+void xchat_free (void *buf, char *file, int line);
+void *xchat_realloc (char *old, int len, char *file, int line);
 #endif
 
 #ifdef SOCKS
@@ -48,23 +43,54 @@
 #define PATH_MAX MAXPATHLEN
 #define FILEPATH_LEN_MAX MAXPATHLEN
 #else
+#ifdef WIN32
+#define OFLAGS O_BINARY
+#define sleep(t) _sleep(t*1000)
+#include <direct.h>
+#define	F_OK	0
+#define	X_OK	1
+#define	W_OK	2
+#define	R_OK	4
+#ifndef S_ISDIR
+#define	S_ISDIR(m)	((m) & _S_IFDIR)
+#endif
+#define NETWORK_PRIVATE
+#else
 #define OFLAGS 0
 #endif
+#endif
 
-#define FONTNAMELEN		127
-#define PATHLEN			255
-#define NICKLEN			64				/* including the NULL, so 63 really */
-#define CHANLEN			202
+#define FONTNAMELEN	127
+#define PATHLEN		255
+#define NICKLEN		64				/* including the NULL, so 63 really */
+#define CHANLEN		202
+#define PDIWORDS		32
 
 #define safe_strcpy(dest,src,len)	strncpy(dest,src,len); \
 												dest[len-1] = 0;
 
+#if defined(ENABLE_NLS) && !defined(_)
+#  include <libintl.h>
+#  define _(x) gettext(x)
+#  ifdef gettext_noop
+#    define N_(String) gettext_noop (String)
+#  else
+#    define N_(String) (String)
+#  endif
+#endif
+#if !defined(_)
+#  define N_(String) (String)
+#  define _(x) (x)
+#endif
+
 struct nbexec
 {
-	int myfd, childfd;
+	int myfd;
 	int childpid;
 	int tochannel;					  /* making this int keeps the struct 4-byte aligned */
 	int iotag;
+	char *linebuf;
+	int buffill;
 	struct session *sess;
 };
 
@@ -81,6 +107,7 @@ struct xchatprefs
 	char partreason[256];
 	char font_normal[FONTNAMELEN + 1];
 	char dialog_font_normal[FONTNAMELEN + 1];
+	char font_shell[FONTNAMELEN + 1];
 	char doubleclickuser[256];
 	char sounddir[PATHLEN + 1];
 	char soundcmd[PATHLEN + 1];
@@ -93,10 +120,11 @@ struct xchatprefs
 	char cmdchar[4];
 	char trans_file[256];
 	char logmask[256];
+	char stamp_format[64];
 
 	char proxy_host[64];
 	int proxy_port;
-	int proxy_type;		/* 0 = disabled, 1 = socks4, 3 = wingate */
+	int proxy_type; /* 0=disabled, 1=wingate 2=socks4, 3=socks5, 4=http */
 
 	int first_dcc_send_port;
 	int last_dcc_send_port;
@@ -129,7 +157,9 @@ struct xchatprefs
 	int nu_color;
 	unsigned long local_ip;
 	unsigned long dcc_ip;
+	char dcc_ip_str[16];
 
+	unsigned int perc_color;
 	unsigned int perc_ascii;
 	unsigned int hilightnick;
 	unsigned int use_trans;
@@ -147,7 +177,8 @@ struct xchatprefs
 	unsigned int colorednicks;
 	unsigned int chanmodebuttons;
 	unsigned int userlistbuttons;
-   unsigned int showhostname_in_userlist;
+	unsigned int userlist_icons;
+	unsigned int showhostname_in_userlist;
 	unsigned int nickcompletion;
 	unsigned int old_nickcompletion;
 	unsigned int tabchannels;
@@ -162,12 +193,16 @@ struct xchatprefs
 	unsigned int tint;
 	unsigned int dialog_transparent;
 	unsigned int dialog_tint;
+	unsigned int dialog_width;
+	unsigned int dialog_height;
 	unsigned int stripcolor;
 	unsigned int timestamp;
 	unsigned int fastdccsend;
+	unsigned int dcc_send_fillspaces;
 	unsigned int skipserverlist;
 	unsigned int filterbeep;
 	unsigned int beepmsg;
+	unsigned int beepchans;
 	unsigned int privmsgtab;
 	unsigned int logging;
 	unsigned int timestamp_logs;
@@ -183,9 +218,13 @@ struct xchatprefs
 	unsigned int show_away_message;
 	unsigned int userhost;
 	unsigned int use_server_tab;
+	unsigned int notices_tabs;
 	unsigned int style_namelistgad;
 	unsigned int style_inputbox;
 	unsigned int windows_as_tabs;
+#ifdef USE_JCODE
+	unsigned int kanji_conv;
+#endif
 	unsigned int use_fontset;
 	unsigned int indent_nicks;
 	unsigned int show_separator;
@@ -204,6 +243,7 @@ struct xchatprefs
 	unsigned int hidemenu;
 	unsigned int perlwarnings;
 	unsigned int lagometer;
+	unsigned int throttlemeter;
 	unsigned int hebrew;
 	unsigned int limitedtabhighlight;
 	unsigned int pingtimeout;
@@ -213,15 +253,23 @@ struct xchatprefs
 	unsigned int inputgad_superfocus;
 	unsigned int nickgad;
 	unsigned int channelbox;
+	unsigned int persist_chans;
 
 	unsigned int ctcp_number_limit;	/*flood */
 	unsigned int ctcp_time_limit;	/*seconds of floods */
 
 	unsigned int msg_number_limit;	/*same deal */
 	unsigned int msg_time_limit;
-
 };
 
+/* Session types */
+#define SESS_SERVER	1
+#define SESS_CHANNEL	2
+#define SESS_DIALOG	3
+#define SESS_SHELL	4
+#define SESS_NOTICES	5
+#define SESS_SNOTICES	6
+
 struct session
 {
 	struct server *server;
@@ -231,7 +279,6 @@ struct session
 	char willjoinchannel[CHANLEN];	  /* /join done for this channel */
 	char channelkey[64];			  /* XXX correct max length? */
 	int limit;						  /* channel user limit */
-	int channelflags;				  /* channel flags (bitmap) */
 	int logfd;
 
 	char lastnick[NICKLEN];			  /* last nick you /msg'ed */
@@ -244,6 +291,10 @@ struct session
 	int total;							/* num. of users in channel */
 
 	char *quitreason;
+	char *topic;
+	char *current_modes;					/* free() me */
+
+	int mode_timeout_tag;
 
 	struct session *lastlog_sess;
 	struct setup *setup;
@@ -252,12 +303,10 @@ struct session
 	struct session_gui *gui;		/* initialized by fe_new_window */
 
 	int userlisthidden;
-	int is_shell:1;
+	int type;
 	int is_tab:1;			/* is this a tab or toplevel window? */
-	int is_dialog:1;
-	int is_server:1;		/* for use_server_tab feature */
 	int new_data:1;		/* new data avail? (red tab) */
-	int nick_said:1;		/* your nick mentioned? (green tab) */
+	int nick_said:1;		/* your nick mentioned? (blue tab) */
 	int ignore_date:1;
 	int ignore_mode:1;
 	int ignore_names:1;
@@ -272,6 +321,8 @@ struct server
 {
 	int port;
 	int sok;
+	int sok4;
+	int sok6;
 #ifdef USE_OPENSSL
 	SSL *ssl;
 	int ssl_do_connect_tag;
@@ -289,9 +340,16 @@ struct server
 	char password[86];
 	char nick[NICKLEN];
 	char linebuf[522];				/* RFC says 512 including \r\n */
-	int pos;
+	char *last_away_reason;
+	int pos;								/* current position in linebuf */
 	int nickcount;
 
+	char *chantypes;					/* for 005 numeric - free me */
+	char *chanmodes;					/* for 005 numeric - free me */
+	char *nick_prefixes;				/* e.g. "*@%+" */
+	char *nick_modes;					/* e.g. "aohv" */
+	char *bad_nick_prefixes;		/* for ircd that doesn't give the modes */
+
 	GSList *outbound_queue;
 	int next_send;						/* cptr->since in ircu */
 	int sendq_len;						/* queue size */
@@ -300,10 +358,10 @@ struct server
 
 	struct server_gui *gui;		  /* initialized by fe_new_server */
 
-	int ctcp_counter;				  /*flood */
+	unsigned int ctcp_counter;	  /*flood */
 	time_t ctcp_last_time;
 
-	int msg_counter;				  /*counts the msg tab opened in a certain time */
+	unsigned int msg_counter;	  /*counts the msg tab opened in a certain time */
 	time_t msg_last_time;
 
 	/*time_t connect_time;*/				/* when did it connect? */
@@ -323,6 +381,10 @@ struct server
 	int is_newtype:1;		/* undernet and dalnet need /list >0,<10000 */
 	int six_modes:1;		/* undernet allows 6 modes per line */
 	int is_away:1;
+	int reconnect_away:1;		/* whether to reconnect in is_away state */
+	int dont_use_proxy:1;		/* to proxy or not to proxy */
+	int supports_watch:1;		/* supports the WATCH command */
+	int bad_prefix:1;				/* gave us a bad PREFIX= 005 number */
 };
 
 typedef struct server server;
@@ -345,3 +407,12 @@ struct away_msg
 	char nick[NICKLEN];
 	char *message;
 };
+
+/* not just for popups, but used for usercommands, ctcp replies,
+   userlist buttons etc */
+
+struct popup
+{
+	char *cmd;
+	char *name;
+};
