@@ -24,25 +24,27 @@
 #include <strings.h>
 #endif
 #include <unistd.h>
-#include "../common/xchat.h"
-#include "fe-gtk.h"
-#include "gtkutil.h"
 #include <fcntl.h>
 #include <time.h>
+
+#include "fe-gtk.h"
+#include "../common/xchat.h"
+#include "../common/xchatc.h"
+#include "../common/outbound.h"
+#include "../common/util.h"
 #include <gdk/gdkkeysyms.h>
+#include "gtkutil.h"
+#include "wins.h"
 
 
-extern unsigned char *strip_color (unsigned char *text);
-extern int tcp_send_len (struct server *serv, char *buf, int len);
-extern int tcp_send (struct server *serv, char *buf);
-extern int handle_command (char *cmd, struct session *sess, int history, int);
-
 /**
  * Accepts a regex_t pointer and string to test it with 
  * Returns 0 if no match, 1 if a match.
  */
+#ifndef WIN32
+
 static int
-match (regex_t * regexptr, const char *str)
+reg_match (regex_t * regexptr, const char *str)
 {
 	int m;
 
@@ -52,6 +54,16 @@ match (regex_t * regexptr, const char *str)
 	return m == 0 ? 1 : 0;
 }
 
+#else
+
+static int
+reg_match (char ** regexptr, const char *str)
+{
+	return match (*regexptr, str);
+}
+
+#endif
+
 /**
  * Sorts the channel list based upon the user field.
  */
@@ -93,11 +105,11 @@ static void
 chanlist_update_caption (struct server *serv)
 {
 	static gchar *title =
-		"User and Channel Statistics: %d/%d Users on %d/%d Channels";
+		N_("User and Channel Statistics: %d/%d Users on %d/%d Channels");
 
 	gchar tbuf[256];
 
-	snprintf (tbuf, sizeof tbuf, title,
+	snprintf (tbuf, sizeof tbuf, _(title),
 				 serv->gui->chanlist_users_shown_count,
 				 serv->gui->chanlist_users_found_count,
 				 serv->gui->chanlist_channels_shown_count,
@@ -205,8 +217,8 @@ chanlist_place_row_in_gui (struct server *serv, gchar ** next_row)
 			 (!serv->gui->chanlist_match_wants_channel
 			  && !serv->gui->chanlist_match_wants_topic))
 		{
-			if (!match (&serv->gui->chanlist_match_regex, next_row[0])
-				 && !match (&serv->gui->chanlist_match_regex, next_row[2]))
+			if (!reg_match (&serv->gui->chanlist_match_regex, next_row[0])
+				 && !reg_match (&serv->gui->chanlist_match_regex, next_row[2]))
 			{
 				chanlist_update_caption (serv);
 				return;
@@ -215,7 +227,7 @@ chanlist_place_row_in_gui (struct server *serv, gchar ** next_row)
 
 		else if (serv->gui->chanlist_match_wants_channel)
 		{
-			if (!match (&serv->gui->chanlist_match_regex, next_row[0]))
+			if (!reg_match (&serv->gui->chanlist_match_regex, next_row[0]))
 			{
 				chanlist_update_caption (serv);
 				return;
@@ -224,7 +236,7 @@ chanlist_place_row_in_gui (struct server *serv, gchar ** next_row)
 
 		else if (serv->gui->chanlist_match_wants_topic)
 		{
-			if (!match (&serv->gui->chanlist_match_regex, next_row[2]))
+			if (!reg_match (&serv->gui->chanlist_match_regex, next_row[2]))
 			{
 				chanlist_update_caption (serv);
 				return;
@@ -365,11 +377,16 @@ chanlist_wild (GtkWidget * wid, struct server *serv)
 	strncpy (serv->gui->chanlist_wild_text,
 				gtk_entry_get_text (GTK_ENTRY (wid)), 255);
 
+#ifdef WIN32
+	serv->gui->chanlist_match_regex =
+		strdup (gtk_entry_get_text (GTK_ENTRY (wid)));
+#else
 	/* recompile the regular expression. */
 	regfree (&serv->gui->chanlist_match_regex);
 	regcomp (&serv->gui->chanlist_match_regex,
 				gtk_entry_get_text (GTK_ENTRY (wid)),
 				REG_ICASE | REG_EXTENDED | REG_NOSUB);
+#endif
 }
 
 static void
@@ -490,10 +507,10 @@ chanlist_save (GtkWidget * wid, struct server *serv)
 	if (!gtk_clist_get_text
 		 (GTK_CLIST (serv->gui->chanlist_list), 0, 0, &temp))
 	{
-		gtkutil_simpledialog ("I can't save an empty list!");
+		gtkutil_simpledialog (_("I can't save an empty list!"));
 		return;
 	}
-	gtkutil_file_req ("Select an output filename", chanlist_filereq_done,
+	gtkutil_file_req (_("Select an output filename"), chanlist_filereq_done,
 							serv, 0, TRUE);
 }
 
@@ -526,20 +543,24 @@ static void
 chanlist_destroy_widget (GtkObject * object, struct server *serv)
 {
 	chanlist_data_free (serv);
+#ifndef WIN32
 	regfree (&serv->gui->chanlist_match_regex);
+#else
+	free (serv->gui->chanlist_match_regex);
+#endif
 }
 
-static int
-chanlist_closegui (GtkWidget * wid, struct server *serv)
+static void
+chanlist_closegui (server *serv)
 {
-	serv->gui->chanlist_window = 0;
-	return 0;
+	if (is_server (serv))
+		serv->gui->chanlist_window = 0;
 }
 
 void
 chanlist_opengui (struct server *serv)
 {
-	static gchar *titles[] = { "Channel", "Users", "Topic" };
+	gchar *titles[] = { _("Channel"), _("Users"), _("Topic") };
 	GtkWidget *frame, *vbox, *hbox, *sortvbox, *sorthbox, *numtable,
 		*table, *wid, *real_wid;
 	char tbuf[256];
@@ -550,7 +571,7 @@ chanlist_opengui (struct server *serv)
 		return;
 	}
 
-	snprintf (tbuf, sizeof tbuf, "X-Chat: Channel List (%s)",
+	snprintf (tbuf, sizeof tbuf, _("X-Chat: Channel List (%s)"),
 				 serv->servername);
 
 	serv->gui->chanlist_data_stored_rows = NULL;
@@ -561,16 +582,13 @@ chanlist_opengui (struct server *serv)
 	if (!serv->gui->chanlist_maxusers)
 		serv->gui->chanlist_maxusers = 0;
 
+
 	serv->gui->chanlist_window =
-		gtkutil_window_new (tbuf, "chanlist", 450, 300, chanlist_closegui, serv,
-								  TRUE);
+		maingui_window ("chanlist", tbuf, FALSE, TRUE, chanlist_closegui, serv,
+								450, 300, &serv->gui->chanlist_window);
+	vbox = wins_get_vbox (serv->gui->chanlist_window);
 
-	vbox = gtk_vbox_new (FALSE, 2);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
-	gtk_container_add (GTK_CONTAINER (serv->gui->chanlist_window), vbox);
-	gtk_widget_show (vbox);
-
-	frame = gtk_frame_new ("List display options:");
+	frame = gtk_frame_new (_("List display options:"));
 	gtk_container_set_border_width (GTK_CONTAINER (frame), 2);
 	gtk_widget_show (frame);
 
@@ -588,7 +606,7 @@ chanlist_opengui (struct server *serv)
 	gtk_widget_show (numtable);
 
 	wid = gtk_alignment_new (0.0, 0.0, 0.0, 0.0);
-	real_wid = gtk_label_new ("Minimum Users: ");
+	real_wid = gtk_label_new (_("Minimum Users: "));
 	gtk_container_add (GTK_CONTAINER (wid), real_wid);
 	gtk_table_attach_defaults (GTK_TABLE (numtable), wid, 0, 1, 0, 1);
 	gtk_widget_show (wid);
@@ -607,7 +625,7 @@ chanlist_opengui (struct server *serv)
 	gtk_widget_show (wid);
 
 	wid = gtk_alignment_new (0.0, 0.0, 0.0, 0.0);
-	real_wid = gtk_label_new ("Maximum Users: ");
+	real_wid = gtk_label_new (_("Maximum Users: "));
 	gtk_container_add (GTK_CONTAINER (wid), real_wid);
 	gtk_table_attach_defaults (GTK_TABLE (numtable), wid, 0, 1, 1, 2);
 	gtk_widget_show (real_wid);
@@ -637,7 +655,7 @@ chanlist_opengui (struct server *serv)
 	gtk_widget_show (wid);
 
 	wid = gtk_alignment_new (1.0, 0.0, 1.0, 0.0);
-	real_wid = gtk_label_new ("Regex Match: ");
+	real_wid = gtk_label_new (_("Regex Match: "));
 	gtk_container_add (GTK_CONTAINER (wid), real_wid);
 	gtk_box_pack_start (GTK_BOX (sorthbox), wid, 0, 0, 0);
 	gtk_widget_show (real_wid);
@@ -665,7 +683,7 @@ chanlist_opengui (struct server *serv)
 	gtk_box_pack_start (GTK_BOX (sortvbox), sorthbox, 0, 0, 0);
 	gtk_widget_show (sorthbox);
 
-	wid = gtk_check_button_new_with_label ("Channel");
+	wid = gtk_check_button_new_with_label (_("Channel"));
 	gtk_box_pack_start (GTK_BOX (sorthbox), wid, 0, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (wid), "toggled",
 							  GTK_SIGNAL_FUNC
@@ -673,7 +691,7 @@ chanlist_opengui (struct server *serv)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid), TRUE);
 	gtk_widget_show (wid);
 
-	wid = gtk_check_button_new_with_label ("Topic");
+	wid = gtk_check_button_new_with_label (_("Topic"));
 	gtk_signal_connect (GTK_OBJECT (wid), "toggled",
 							  GTK_SIGNAL_FUNC (chanlist_match_topic_button_toggled),
 							  serv);
@@ -682,7 +700,7 @@ chanlist_opengui (struct server *serv)
 	gtk_widget_show (wid);
 
 	wid = gtk_alignment_new (0.5, 0.0, 0.0, 0.0);
-	real_wid = gtk_button_new_with_label (" Apply ");
+	real_wid = gtk_button_new_with_label (_(" Apply "));
 	gtk_container_add (GTK_CONTAINER (wid), real_wid);
 	gtk_table_attach_defaults (GTK_TABLE (table), wid, 0, 3, 1, 2);
 	gtk_signal_connect (GTK_OBJECT (real_wid), "pressed",
@@ -707,6 +725,9 @@ chanlist_opengui (struct server *serv)
 										 chanlist_compare_text_ignore_case);
 	gtk_clist_set_sort_column (GTK_CLIST (serv->gui->chanlist_list), 0);
 	gtk_clist_set_auto_sort (GTK_CLIST (serv->gui->chanlist_list), 1);
+	/* makes the horiz. scrollbar appear when needed */
+	gtk_clist_set_column_auto_resize (GTK_CLIST (serv->gui->chanlist_list),
+											2, TRUE);
 
 	/* make a label to store the user/channel info */
 	wid = gtk_label_new ("");
@@ -721,11 +742,11 @@ chanlist_opengui (struct server *serv)
 
 	serv->gui->chanlist_refresh =
 		gtkutil_button (serv->gui->chanlist_window, GNOME_STOCK_PIXMAP_REFRESH,
-							 "Refresh the list", chanlist_refresh, serv, hbox);
+							 _("Refresh the list"), chanlist_refresh, serv, hbox);
 	gtkutil_button (serv->gui->chanlist_window, GNOME_STOCK_PIXMAP_SAVE,
-						 "Save the list", chanlist_save, serv, hbox);
+						 _("Save the list"), chanlist_save, serv, hbox);
 	gtkutil_button (serv->gui->chanlist_window, GNOME_STOCK_PIXMAP_JUMP_TO,
-						 "Join Channel", chanlist_join, serv, hbox);
+						 _("Join Channel"), chanlist_join, serv, hbox);
 
 	/* connect a destroy event handler to this window so that the dynamic
 	   memory can be freed */
