@@ -20,24 +20,22 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <netdb.h>
 #include <fcntl.h>
-#include <errno.h>
 #include <time.h>
+
+#define WANTSOCKET
+#define WANTARPA
+#include "../common/inet.h"
+#include "fe-gtk.h"
 #include "../common/xchat.h"
+#include "../common/xchatc.h"
 #include "../common/plugin.h"
 #include "../common/fe.h"
 #include "../common/util.h"
-#include "fe-gtk.h"
+#include "../common/network.h"
 #include "gtkutil.h"
-
-extern GdkColor colors[];
-extern struct dccstat_info dccstat[];	/* defined in common/dcc.c */
-extern char *dcctypes[];
+#include "wins.h"
+#include "palette.h"
 
 struct dccwindow
 {
@@ -56,13 +54,6 @@ static struct dccwindow dccrwin;	/* recv */
 static struct dccwindow dccswin;	/* send */
 static struct dccwindow dcccwin;	/* chat */
 
-extern struct xchatprefs prefs;
-extern GSList *dcc_list;
-
-extern int child_handler (int pid);
-extern void dcc_resume (struct DCC *dcc);
-extern void xchat_exec (char *cmd);
-
 
 static void
 dcc_send_filereq_done (struct session *sess, char *nick, char *file)
@@ -83,7 +74,7 @@ fe_dcc_send_filereq (struct session *sess, char *nick)
 	char tbuf[128];
 
 	nick = strdup (nick);
-	snprintf (tbuf, sizeof tbuf, "Send file to %s", nick);
+	snprintf (tbuf, sizeof tbuf, _("Send file to %s"), nick);
 	gtkutil_file_req (tbuf, dcc_send_filereq_done, sess, nick, FALSE);
 }
 
@@ -110,7 +101,7 @@ fe_dcc_update_recv (struct DCC *dcc)
 
 	gtk_clist_freeze (GTK_CLIST (dccrwin.list));
 	gtk_clist_set_text (GTK_CLIST (dccrwin.list), row, 0,
-							  dccstat[(int) dcc->dccstat].name);
+							  _(dccstat[(int) dcc->dccstat].name));
 	gtk_clist_set_text (GTK_CLIST (dccrwin.list), row, 3, pos);
 	gtk_clist_set_text (GTK_CLIST (dccrwin.list), row, 4, perc);
 	gtk_clist_set_text (GTK_CLIST (dccrwin.list), row, 5, cps);
@@ -164,7 +155,7 @@ fe_dcc_update_send (struct DCC *dcc)
 		strcpy (eta, "--:--:--");
 	gtk_clist_freeze (GTK_CLIST (dccswin.list));
 	gtk_clist_set_text (GTK_CLIST (dccswin.list), row, 0,
-							  dccstat[(int) dcc->dccstat].name);
+							  _(dccstat[(int) dcc->dccstat].name));
 	if (dccstat[(int) dcc->dccstat].color != 1)
 		gtk_clist_set_foreground
 			(GTK_CLIST (dccswin.list), row,
@@ -199,11 +190,10 @@ dcc_abort (struct DCC *dcc)
 	}
 }
 
-static int
+static void
 close_dcc_recv_window (void)
 {
 	dccrwin.window = 0;
-	return 0;
 }
 
 void
@@ -238,7 +228,7 @@ fe_dcc_update_recv_win (void)
 		dcc = (struct DCC *) list->data;
 		if (dcc->type == TYPE_RECV)
 		{
-			nnew[0] = dccstat[(int) dcc->dccstat].name;
+			nnew[0] = _(dccstat[(int) dcc->dccstat].name);
 			nnew[1] = dcc->file;
 			nnew[7] = dcc->nick;
 #ifdef USE_GNOME
@@ -281,17 +271,17 @@ static void
 dcc_info (struct DCC *dcc)
 {
 	char tbuf[256];
-	snprintf (tbuf, 255, "      File: %s\n"
+	snprintf (tbuf, 255, _("      File: %s\n"
 				 "   To/From: %s\n"
 				 "      Size: %d\n"
 				 "      Port: %d\n"
 				 " IP Number: %s\n"
-				 "Start Time: %s",
+				 "Start Time: %s"),
 				 dcc->file,
 				 dcc->nick,
 				 dcc->size,
 				 dcc->port,
-				 inet_ntoa (dcc->SAddr.sin_addr), ctime (&dcc->starttime));
+				 net_ip (dcc->addr), ctime (&dcc->starttime));
 	gtkutil_simpledialog (tbuf);
 }
 
@@ -405,31 +395,29 @@ recv_row_selected (GtkWidget * clist, gint row, gint column,
 }
 
 void
-fe_dcc_open_recv_win (void)
+fe_dcc_open_recv_win (int passive)
 {
 	GtkWidget *vbox, *bbox;
 #ifdef USE_GNOME
-	static gchar *titles[] =
-		{ "Status", "File", "Size", "Position", "%", "CPS", "ETA", "From",
-			"MIME Type" };
+	gchar *titles[] =
+		{ _("Status"), _("File"), _("Size"), _("Position"), "%", _("CPS"), _("ETA"), _("From"),
+			_("MIME Type") };
 #else
-	static gchar *titles[] =
-		{ "Status", "File", "Size", "Position", "%", "CPS", "ETA", "From" };
+	gchar *titles[] =
+		{ _("Status"), _("File"), _("Size"), _("Position"), "%", _("CPS"), _("ETA"), _("From") };
 #endif
 
 	if (dccrwin.window)
 	{
 		fe_dcc_update_recv_win ();
+		if (!passive)
+			wins_bring_tofront (dccrwin.window);
 		return;
 	}
-	dccrwin.window = gtkutil_window_new ("X-Chat: DCC Receive List",
-													 "dccrecv", 0, 0,
-													 close_dcc_recv_window, 0, TRUE);
-
-	vbox = gtk_vbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
-	gtk_container_add (GTK_CONTAINER (dccrwin.window), vbox);
-	gtk_widget_show (vbox);
+	dccrwin.window = maingui_window ("dccrecv", _("X-Chat: File Receive List"),
+								FALSE, TRUE, close_dcc_recv_window, NULL, 601, 0,
+								&dccrwin.window);
+	vbox = wins_get_vbox (dccrwin.window);
 
 #ifdef USE_GNOME
 	dccrwin.list = gtkutil_clist_new (9, titles, vbox, GTK_POLICY_ALWAYS,
@@ -440,8 +428,6 @@ fe_dcc_open_recv_win (void)
 												 recv_row_selected, 0,
 												 0, 0, GTK_SELECTION_SINGLE);
 #endif
-	if (!prefs.windows_as_tabs)
-		gtk_widget_set_usize (dccrwin.list, 601, 0);
 	gtk_clist_set_column_width (GTK_CLIST (dccrwin.list), 0, 65);
 	gtk_clist_set_column_width (GTK_CLIST (dccrwin.list), 1, 100);
 	gtk_clist_set_column_width (GTK_CLIST (dccrwin.list), 2, 50);
@@ -457,25 +443,24 @@ fe_dcc_open_recv_win (void)
 	gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 1);
 	gtk_widget_show (bbox);
 
-	gtkutil_button (dccrwin.window, 0, "Accept", accept_clicked, 0, bbox);
+	gtkutil_button (dccrwin.window, 0, _("Accept"), accept_clicked, 0, bbox);
 
-	gtkutil_button (dccrwin.window, 0, "Resume", resume_clicked, 0, bbox);
+	gtkutil_button (dccrwin.window, 0, _("Resume"), resume_clicked, 0, bbox);
 
-	gtkutil_button (dccrwin.window, 0, "Abort", abort_clicked, 0, bbox);
+	gtkutil_button (dccrwin.window, 0, _("Abort"), abort_clicked, 0, bbox);
 
-	gtkutil_button (dccrwin.window, 0, "Info", info_clicked, 0, bbox);
+	gtkutil_button (dccrwin.window, 0, _("Info"), info_clicked, 0, bbox);
 #ifdef USE_GNOME
-	gtkutil_button (dccrwin.window, 0, "Open", open_clicked, 0, bbox);
+	gtkutil_button (dccrwin.window, 0, _("Open"), open_clicked, 0, bbox);
 #endif
 	gtk_widget_show (dccrwin.window);
 	fe_dcc_update_recv_win ();
 }
 
-static int
+static void
 close_dcc_send_window (void)
 {
 	dccswin.window = 0;
-	return 0;
 }
 
 void
@@ -512,7 +497,7 @@ fe_dcc_update_send_win (void)
 		dcc = (struct DCC *) list->data;
 		if (dcc->type == TYPE_SEND)
 		{
-			nnew[0] = dccstat[(int) dcc->dccstat].name;
+			nnew[0] = _(dccstat[(int) dcc->dccstat].name);
 			nnew[1] = file_part (dcc->file);
 			nnew[8] = dcc->nick;
 			/* percentage ack'ed */
@@ -595,32 +580,29 @@ abort_send_clicked (GtkWidget * wid, gpointer none)
 }
 
 void
-fe_dcc_open_send_win (void)
+fe_dcc_open_send_win (int passive)
 {
 	GtkWidget *vbox, *bbox;
-	static gchar *titles[] =
-		{ "Status", "File", "Size", "Position", "Ack", "%", "CPS", "ETA",
-			"To" };
+	gchar *titles[] =
+		{ _("Status"), _("File"), _("Size"), _("Position"), _("Ack"), "%", _("CPS"), _("ETA"),
+			_("To") };
 
 	if (dccswin.window)
 	{
 		fe_dcc_update_send_win ();
+		if (!passive)
+			wins_bring_tofront (dccswin.window);
 		return;
 	}
-	dccswin.window =
-		gtkutil_window_new ("X-Chat: DCC Send List", "dccsend", 0, 0,
-								  close_dcc_send_window, 0, TRUE);
 
-	vbox = gtk_vbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
-	gtk_container_add (GTK_CONTAINER (dccswin.window), vbox);
-	gtk_widget_show (vbox);
+	dccswin.window = maingui_window ("dccsend", _("X-Chat: File Send List"),
+								FALSE, TRUE, close_dcc_send_window, NULL, 595, 0,
+								&dccswin.window);
+	vbox = wins_get_vbox (dccswin.window);
 
 	dccswin.list = gtkutil_clist_new (9, titles, vbox, GTK_POLICY_ALWAYS,
 												 send_row_selected, 0,
 												 0, 0, GTK_SELECTION_SINGLE);
-	if (!prefs.windows_as_tabs)
-		gtk_widget_set_usize (dccswin.list, 595, 0);
 	gtk_clist_set_column_width (GTK_CLIST (dccswin.list), 0, 65);
 	gtk_clist_set_column_width (GTK_CLIST (dccswin.list), 1, 100);
 	gtk_clist_set_column_width (GTK_CLIST (dccswin.list), 2, 50);
@@ -636,8 +618,8 @@ fe_dcc_open_send_win (void)
 	gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 1);
 	gtk_widget_show (bbox);
 
-	gtkutil_button (dccswin.window, 0, "Abort", abort_send_clicked, 0, bbox);
-	gtkutil_button (dccswin.window, 0, "Info", info_send_clicked, 0, bbox);
+	gtkutil_button (dccswin.window, 0, _("Abort"), abort_send_clicked, 0, bbox);
+	gtkutil_button (dccswin.window, 0, _("Info"), info_send_clicked, 0, bbox);
 
 	gtk_widget_show (dccswin.window);
 	fe_dcc_update_send_win ();
@@ -707,7 +689,7 @@ fe_dcc_update_chat_win (void)
 		dcc = (struct DCC *) list->data;
 		if ((dcc->type == TYPE_CHATSEND || dcc->type == TYPE_CHATRECV))
 		{
-			nnew[0] = dccstat[(int) dcc->dccstat].name;
+			nnew[0] = _(dccstat[(int) dcc->dccstat].name);
 			nnew[1] = dcc->nick;
 			sprintf (pos, "%d", dcc->pos);
 			sprintf (siz, "%d", dcc->size);
@@ -722,39 +704,35 @@ fe_dcc_update_chat_win (void)
 		gtk_clist_select_row ((GtkCList *) dcccwin.list, selrow, 0);
 }
 
-static int
+static void
 close_dcc_chat_window (void)
 {
 	dcccwin.window = 0;
-	return 0;
 }
 
 void
-fe_dcc_open_chat_win (void)
+fe_dcc_open_chat_win (int passive)
 {
 	GtkWidget *vbox, *bbox;
-	static gchar *titles[] =
-		{ "Status", "To/From", "Recv", "Sent", "StartTime" };
+	gchar *titles[] =
+		{ _("Status"), _("To/From"), _("Recv"), _("Sent"), _("StartTime") };
 
 	if (dcccwin.window)
 	{
 		fe_dcc_update_chat_win ();
+		if (!passive)
+			wins_bring_tofront (dcccwin.window);
 		return;
 	}
-	dcccwin.window =
-		gtkutil_window_new ("X-Chat: DCC Chat List", "dccchat", 0, 0,
-								  close_dcc_chat_window, 0, TRUE);
 
-	vbox = gtk_vbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
-	gtk_container_add (GTK_CONTAINER (dcccwin.window), vbox);
-	gtk_widget_show (vbox);
+	dcccwin.window = maingui_window ("dccchat", _("X-Chat: DCC Chat List"),
+								FALSE, TRUE, close_dcc_chat_window, NULL, 550, 0,
+								&dcccwin.window);
+	vbox = wins_get_vbox (dcccwin.window);
 
 	dcccwin.list = gtkutil_clist_new (5, titles, vbox, GTK_POLICY_ALWAYS,
 												 chat_row_selected, 0,
 												 0, 0, GTK_SELECTION_BROWSE);
-	if (!prefs.windows_as_tabs)
-		gtk_widget_set_usize (dcccwin.list, 550, 0);
 	gtk_clist_set_column_width (GTK_CLIST (dcccwin.list), 0, 65);
 	gtk_clist_set_column_width (GTK_CLIST (dcccwin.list), 1, 100);
 	gtk_clist_set_column_width (GTK_CLIST (dcccwin.list), 2, 65);
@@ -764,8 +742,8 @@ fe_dcc_open_chat_win (void)
 	gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 1);
 	gtk_widget_show (bbox);
 
-	gtkutil_button (dcccwin.window, 0, "Accept", accept_chat_clicked, 0, bbox);
-	gtkutil_button (dcccwin.window, 0, "Abort", abort_chat_clicked, 0, bbox);
+	gtkutil_button (dcccwin.window, 0, _("Accept"), accept_chat_clicked, 0, bbox);
+	gtkutil_button (dcccwin.window, 0, _("Abort"), abort_chat_clicked, 0, bbox);
 
 	gtk_widget_show (dcccwin.window);
 	fe_dcc_update_chat_win ();
