@@ -16,54 +16,28 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-#include <stdio.h>
-#include <stdlib.h>
+#include <string.h>
 #include <time.h>
-#include <sys/time.h>
-#include <fcntl.h>
+#include "fe-gtk.h"
 #include "../common/xchat.h"
 #include "../common/fe.h"
-#include "fe-gtk.h"
+#include "../common/text.h"
+#include "../common/util.h"
+#include "../common/outbound.h"
+#include "../common/xchatc.h"
 #include "gtkutil.h"
 #include "menu.h"
-#include "../common/util.h"
 #include "xtext.h"
+#include "wins.h"
+#include "fkeys.h"
+#include "maingui.h"
+#include "palette.h"
+#include "dialog.h"
 
-extern GSList *sess_list;
-extern struct xchatprefs prefs;
-extern GdkFont *dialog_font_normal;
-extern GdkColor colors[];
-extern GdkPixmap *dialogwin_pix;
-extern GtkStyle *inputgad_style;
-extern GtkWidget *main_window;
-extern GtkWidget *main_book;
-extern struct session *current_tab;
-extern struct session *menu_sess;
-extern GtkStyle *normaltab_style;
 #ifdef USE_GNOME
 extern GtkTargetEntry dnd_targets[];
 #endif
 
-extern void gui_create_standard_buttons (session *sess, GtkWidget *win, GtkWidget *tbox);
-extern void gui_create_toolbox (session *sess, GtkWidget *box);
-extern int auto_insert (char *dest, char *src, char *word[], char *word_eol[], char *a, char *c, char *d, char *h, char *n, char *s);
-extern void maingui_configure (session *sess);
-extern int maingui_word_check (GtkXText * xtext, char *word);
-extern void maingui_word_clicked (GtkXText * xtext, char *word,
-											 GdkEventButton * even, session * sess);
-extern struct session *find_dialog (struct server *serv, char *nick);
-extern int maingui_pagetofront (int page);
-extern void show_and_unfocus (GtkWidget * wid);
-extern void add_tip (GtkWidget * wid, char *text);
-extern int handle_command (char *cmd, struct session *sess, int history, int);
-extern int open_log_file (char *servname, char *channame);
-struct session *new_dialog (struct session *sess);
-extern void dcc_send (struct session *sess, char *tbuf, char *to, char *file);
-extern void handle_inputgad (GtkWidget * igad, struct session *sess);
-extern gint gtk_kill_session_callback (GtkWidget * win, struct session *sess);
-extern void focus_in (GtkWindow * win, GtkWidget * wid, struct session *sess);
-int key_handle_key_press (GtkWidget, GdkEventKey, gpointer);
-
 
 #ifdef USE_GNOME
 static void
@@ -94,9 +68,8 @@ fe_change_nick (struct server *serv, char *nick, char *newnick)
 	struct session *sess = find_dialog (serv, nick);
 	if (sess)
 	{
-		strcpy (sess->channel, newnick);
-		if (!sess->is_tab)
-			gtk_window_set_title (GTK_WINDOW (sess->gui->window), newnick);
+		safe_strcpy (sess->channel, newnick, CHANLEN);
+		fe_set_title (sess);
 	}
 }
 
@@ -136,64 +109,40 @@ open_dialog_window (struct session *sess)
 	int page = prefs.privmsgtab;
 	struct User *user;
 
-	if (!main_book)
+	if (!main_window)
 		page = 0;
 	if (!page)
 	{
-#ifdef USE_GNOME
-		sess->gui->window = gnome_app_new ("X-Chat", sess->channel);
-		gtk_signal_connect ((GtkObject *) sess->gui->window, "destroy",
-								  GTK_SIGNAL_FUNC (gtk_kill_session_callback), sess);
-#else
-		sess->gui->window =
-			gtkutil_window_new (sess->channel, "X-Chat", 300, 100,
-									  gtk_kill_session_callback, sess, FALSE);
-#endif
+		sess->gui->window = wins_new (sess->channel, "",
+												kill_session_callback, sess, link_cb,
+												sess, FALSE, FALSE, NULL);
+		gtk_widget_set_usize (sess->gui->window, prefs.dialog_width,
+									 prefs.dialog_height);
 		gtk_signal_connect ((GtkObject *) sess->gui->window, "focus_in_event",
 								  GTK_SIGNAL_FUNC (focus_in), sess);
-		gtk_window_set_policy ((GtkWindow *) sess->gui->window, TRUE, TRUE,
-									  FALSE);
 		sess->is_tab = FALSE;
 	} else
 	{
-		sess->gui->window = gtk_hbox_new (0, 0);
+		sess->gui->window = wins_new (sess->channel, "",
+												kill_session_callback, sess, link_cb,
+												sess, TRUE, FALSE, NULL);
 		sess->is_tab = TRUE;
 	}
-	vbox = gtk_vbox_new (0, 0);
-	sess->gui->vbox = vbox;
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 0);
+
+	sess->gui->vbox = vbox = wins_get_vbox (sess->gui->window);
 
 	if (!page)
-	{
-#ifdef USE_GNOME
-		gnome_app_set_contents (GNOME_APP (sess->gui->window), vbox);
-#else
-		gtk_container_add (GTK_CONTAINER (sess->gui->window), vbox);
-#endif
-	} else
-		gtk_container_add (GTK_CONTAINER (sess->gui->window), vbox);
-	if (page)
-	{
-		sess->gui->changad = gtk_label_new (sess->channel);
-		gtk_notebook_append_page (GTK_NOTEBOOK (main_book), sess->gui->window,
-										  sess->gui->changad);
-	} else
 		sess->gui->changad = NULL;
-	gtk_widget_show (vbox);
+	else
+		sess->gui->changad = wins_get_notebooklabel (sess->gui->window);
 
-	if (page)
-	{
-		gtk_signal_connect ((GtkObject *) sess->gui->window, "destroy",
-								  GTK_SIGNAL_FUNC (gtk_kill_session_callback), sess);
-	}
 	sess->gui->tbox = hbox = gtk_hbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 2);
 	if (prefs.topicbar)
 		gtk_widget_show (hbox);
 
 	/* create the X ^ < > buttons */
-	gui_create_standard_buttons (sess, sess->gui->window, hbox);
+	wins_create_linkbuttons (sess->gui->window, hbox);
 
 	sess->gui->topicgad = gtk_entry_new ();
 	gtk_entry_set_editable ((GtkEntry *) sess->gui->topicgad, FALSE);
@@ -203,11 +152,11 @@ open_dialog_window (struct session *sess)
 	else
 		gtk_widget_show (sess->gui->topicgad);
 
-	dialog_button (hbox, "WhoIs", "/whois %s");
-	dialog_button (hbox, "Send", "/dcc send %s");
-	dialog_button (hbox, "Chat", "/dcc chat %s");
-	dialog_button (hbox, "Ping", "/ping %s");
-	dialog_button (hbox, "Clear", "/clear");
+	dialog_button (hbox, _("WhoIs"), "/whois %s");
+	dialog_button (hbox, _("Send"), "/dcc send %s");
+	dialog_button (hbox, _("Chat"), "/dcc chat %s");
+	dialog_button (hbox, _("Ping"), "/ping %s");
+	dialog_button (hbox, _("Clear"), "/clear");
 
 	hbox = gtk_hbox_new (FALSE, 0);
 	sess->gui->leftpane = hbox;
@@ -273,8 +222,6 @@ open_dialog_window (struct session *sess)
 
 	sess->gui->inputgad = gtk_entry_new_with_max_length (2048);
 	gtk_container_add (GTK_CONTAINER (bbox), sess->gui->inputgad);
-	/*gtk_signal_connect (GTK_OBJECT (sess->gui->inputgad), "activate",
-							  GTK_SIGNAL_FUNC (handle_inputgad), sess);*/
 	gtk_signal_connect (GTK_OBJECT (sess->gui->inputgad), "key_press_event",
 							  GTK_SIGNAL_FUNC (key_handle_key_press), sess);
 	if (prefs.style_inputbox)
@@ -286,26 +233,8 @@ open_dialog_window (struct session *sess)
 
 	gtk_widget_show (sess->gui->window);
 
-	if (page)
-	{
-		if (prefs.newtabstofront)
-		{
-			page =
-				gtk_notebook_page_num (GTK_NOTEBOOK (main_book),
-											  sess->gui->window);
-			maingui_pagetofront (page);
-
-			gtk_widget_grab_focus (sess->gui->inputgad);
-			current_tab = sess;
-			menu_sess = sess;
-			if (!prefs.use_server_tab)
-				sess->server->front_session = sess;
-			sess->nick_said = FALSE;
-			sess->new_data = FALSE;
-			gtk_widget_set_style (sess->gui->changad, normaltab_style);
-		} else
-			sess->new_data = TRUE;
-	}
+	if (page && prefs.newtabstofront)
+		wins_bring_tofront (sess->gui->window);
 
 	fe_set_title (sess);
 
@@ -317,15 +246,12 @@ open_dialog_window (struct session *sess)
 	}
 }
 
-struct session *
-new_dialog (struct session *sess)
+void
+new_dialog (session *sess)
 {
 	if (prefs.logging)
-		sess->logfd = open_log_file (sess->server->servername, sess->channel);
-	else
-		sess->logfd = -1;
+		setup_logging (sess);
+
 	open_dialog_window (sess);
 	sess->new_data = FALSE;
-
-	return (sess);
 }
