@@ -18,30 +18,22 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include "fe-gtk.h"
 #include "../common/xchat.h"
 #include "../common/cfgfiles.h"
-#include "fe-gtk.h"
+#include "../common/xchatc.h"
+#include "../common/fe.h"
 #include "menu.h"
 #include "gtkutil.h"
 #include "usermenu.h"
+#include "wins.h"
+#include "editlist.h"
 
-extern void maingui_updatebuttons (struct session *sess);
-extern int buf_get_line (char *, char **, int *, int);
-extern void list_loadconf (char *, GSList **, char *);
-extern void fe_buttons_update (struct session *sess);
-
-extern GSList *sess_list;
-extern GSList *ctcp_list;
-extern GSList *popup_list;
-extern GSList *button_list;
-extern GSList *command_list;
-extern GSList *replace_list;
-extern GSList *usermenu_list;
-extern GSList *urlhandler_list;
 
 static GtkWidget *editlist_gui_entry_name;
 static GtkWidget *editlist_gui_entry_cmd;
@@ -130,7 +122,7 @@ static void
 editlist_gui_addnew (GtkWidget * igad)
 {
 	int i;
-	static gchar *nnew[2] = { "*NEW*", "EDIT ME" };
+	gchar *nnew[2] = { _("*NEW*"), _("EDIT ME") };
 
 	i = gtk_clist_append (GTK_CLIST (editlist_gui_list), nnew);
 	gtk_clist_select_row (GTK_CLIST (editlist_gui_list), i, 0);
@@ -180,47 +172,35 @@ editlist_gui_save (GtkWidget * igad)
 		{
 			list_free (&popup_list);
 			list_loadconf (editlist_file, &popup_list, 0);
+		} else if (editlist_list == button_list)
+		{
+			GSList *list = sess_list;
+			struct session *sess;
+			list_free (&button_list);
+			list_loadconf (editlist_file, &button_list, 0);
+			while (list)
+			{
+				sess = (struct session *) list->data;
+				fe_buttons_update (sess);
+				list = list->next;
+			}
+		} else if (editlist_list == ctcp_list)
+		{
+			list_free (&ctcp_list);
+			list_loadconf (editlist_file, &ctcp_list, 0);
+		} else if (editlist_list == command_list)
+		{
+			list_free (&command_list);
+			list_loadconf (editlist_file, &command_list, 0);
+		} else if (editlist_list == usermenu_list)
+		{
+			list_free (&usermenu_list);
+			list_loadconf (editlist_file, &usermenu_list, 0);
+			usermenu_update ();
 		} else
 		{
-			if (editlist_list == button_list)
-			{
-				GSList *list = sess_list;
-				struct session *sess;
-				list_free (&button_list);
-				list_loadconf (editlist_file, &button_list, 0);
-				while (list)
-				{
-					sess = (struct session *) list->data;
-					fe_buttons_update (sess);
-					list = list->next;
-				}
-			} else
-			{
-				if (editlist_list == ctcp_list)
-				{
-					list_free (&ctcp_list);
-					list_loadconf (editlist_file, &ctcp_list, 0);
-				} else
-				{
-					if (editlist_list == command_list)
-					{
-						list_free (&command_list);
-						list_loadconf (editlist_file, &command_list, 0);
-					} else
-					{
-						if (editlist_list == usermenu_list)
-						{
-							list_free (&usermenu_list);
-							list_loadconf (editlist_file, &usermenu_list, 0);
-							usermenu_update ();
-						} else
-						{
-							list_free (&urlhandler_list);
-							list_loadconf (editlist_file, &urlhandler_list, 0);
-						}
-					}
-				}
-			}
+			list_free (&urlhandler_list);
+			list_loadconf (editlist_file, &urlhandler_list, 0);
 		}
 	}
 }
@@ -282,36 +262,33 @@ editlist_gui_moveup (GtkWidget * igad)
 	}
 }
 
-static int
+static void
 editlist_gui_close (void)
 {
 	editlist_gui_window = 0;
-	return 0;
 }
 
 void
 editlist_gui_open (GSList * list, char *title, char *wmclass, char *file,
 						 char *help)
 {
-	static gchar *titles[] = { "Name", "Command" };
+	gchar *titles[] = { _("Name"), _("Command") };
 	GtkWidget *vbox, *hbox, *button;
 
 	if (editlist_gui_window)
+	{
+		wins_bring_tofront (editlist_gui_window);
 		return;
+	}
 
 	editlist_list = list;
 	editlist_file = file;
 	editlist_help = help;
 
-	editlist_gui_window = gtkutil_window_new (title, wmclass, 450, 250,
-															editlist_gui_close, 0, FALSE);
-	gtk_window_set_policy (GTK_WINDOW (editlist_gui_window), TRUE, TRUE,
-								  FALSE);
-
-	vbox = gtk_vbox_new (0, 2);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
-	gtk_container_add (GTK_CONTAINER (editlist_gui_window), vbox);
-	gtk_widget_show (vbox);
+	editlist_gui_window = maingui_window (wmclass, title, TRUE, FALSE,
+														editlist_gui_close, NULL,
+														450, 250, NULL);
+	vbox = wins_get_vbox (editlist_gui_window);
 
 	editlist_gui_list = gtkutil_clist_new (2, titles, vbox, GTK_POLICY_ALWAYS,
 														editlist_gui_row_selected, 0,
@@ -324,12 +301,12 @@ editlist_gui_open (GSList * list, char *title, char *wmclass, char *file,
 	gtk_widget_show (hbox);
 
 	button =
-		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_UP, "Move Up",
+		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_UP, _("Move Up"),
 							 editlist_gui_moveup, 0, hbox);
 	gtk_widget_set_usize (button, 100, 0);
 
 	button =
-		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_DOWN, "Move Dn",
+		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_DOWN, _("Move Dn"),
 							 editlist_gui_movedown, 0, hbox);
 	gtk_widget_set_usize (button, 100, 0);
 
@@ -339,11 +316,11 @@ editlist_gui_open (GSList * list, char *title, char *wmclass, char *file,
 
 	button =
 		gtkutil_button (editlist_gui_window, GNOME_STOCK_BUTTON_CANCEL,
-							 "Cancel", gtkutil_destroy, editlist_gui_window, hbox);
+							 _("Cancel"), gtkutil_destroy, editlist_gui_window, hbox);
 	gtk_widget_set_usize (button, 100, 0);
 
 	button =
-		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_SAVE, "Save",
+		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_SAVE, _("Save"),
 							 editlist_gui_save, 0, hbox);
 	gtk_widget_set_usize (button, 100, 0);
 
@@ -352,12 +329,12 @@ editlist_gui_open (GSList * list, char *title, char *wmclass, char *file,
 	gtk_widget_show (hbox);
 
 	button =
-		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_NEW, "Add New",
+		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_NEW, _("Add New"),
 							 editlist_gui_addnew, 0, hbox);
 	gtk_widget_set_usize (button, 100, 0);
 
 	button =
-		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_CUT, "Delete",
+		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_CUT, _("Delete"),
 							 editlist_gui_delete, 0, hbox);
 	gtk_widget_set_usize (button, 100, 0);
 
@@ -367,11 +344,11 @@ editlist_gui_open (GSList * list, char *title, char *wmclass, char *file,
 
 	button =
 		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_SPELLCHECK,
-							 "Sort", editlist_gui_sort, 0, hbox);
+							 _("Sort"), editlist_gui_sort, 0, hbox);
 	gtk_widget_set_usize (button, 100, 0);
 
 	button =
-		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_HELP, "Help",
+		gtkutil_button (editlist_gui_window, GNOME_STOCK_PIXMAP_HELP, _("Help"),
 							 editlist_gui_help, 0, hbox);
 	gtk_widget_set_usize (button, 100, 0);
 
