@@ -28,41 +28,30 @@
 #endif
 #include <fcntl.h>
 #include <ctype.h>
+
+#include "fe-gtk.h"
 #include "../common/xchat.h"
+#include "../common/xchatc.h"
 #include "../common/cfgfiles.h"
 #include "../common/userlist.h"
-#include "fe-gtk.h"
+#include "../common/outbound.h"
+#include "../common/util.h"
+#include "../common/text.h"
 #include <gdk/gdkkeysyms.h>
 #include "gtkutil.h"
 #include "menu.h"
 #include "xtext.h"
+#include "wins.h"
+#include "palette.h"
+#include "maingui.h"
+#include "textgui.h"
+#include "fkeys.h"
 
-extern GSList *fkey_list;
-extern GtkWidget *main_window, *main_book;
-extern int handle_multiline (struct session *sess, char *cmd, int history,
-									  int nocommand);
-extern void handle_inputgad (GtkWidget * igad, struct session *sess);
 
-extern GSList *sess_list;
-extern GSList *command_list;
-extern GSList *replace_list;
-extern GdkColor colors[];
-extern GdkFont *font_normal;
-extern GdkFont *dialog_font_normal;
-extern GdkPixmap *channelwin_pix;
-extern struct session *current_tab;
-extern struct commands cmds[1];
-extern struct xchatprefs prefs;
-
-extern void PrintText (struct session *sess, unsigned char *text);
-extern int buf_get_line (char *, char **, int *, int len);
 static int tab_nick_comp (GtkWidget * t, int shift);
 static void nick_comp_chng (GtkWidget * t, int updown);
 static void replace_handle (GtkWidget * wid);
 
-void maingui_moveleft (GtkWidget * wid);
-void maingui_moveright (GtkWidget * wid);
-
 
 /***************** Key Binding Code ******************/
 
@@ -79,7 +68,7 @@ void maingui_moveright (GtkWidget * wid);
  */
 
 /* Remember that the *number* of actions is this *plus* 1 --AGL */
-#define KEY_MAX_ACTIONS 14
+#define KEY_MAX_ACTIONS 12
 /* These are cp'ed from history.c --AGL */
 #define STATE_SHIFT     GDK_SHIFT_MASK
 #define	STATE_ALT	GDK_MOD1_MASK
@@ -103,15 +92,9 @@ struct key_action
 	char *help;
 };
 
-int key_load_kbs (char *);
+static int key_load_kbs (char *);
 static void key_load_defaults ();
-void key_save_kbs (char *);
-void PrintTextRaw (GtkWidget * textwidget, unsigned char *text, int indent);
-
-static int key_action_delete_backward_word(GtkWidget * wid, GdkEventKey * evt, char *d1,
-														 char *d2, struct session *sess);
-static int key_action_delete_forward_word(GtkWidget * wid, GdkEventKey * evt, char *d1,
-														char *d2, struct session *sess);
+static void key_save_kbs (char *);
 static int key_action_handle_command (GtkWidget * wid, GdkEventKey * evt,
 												  char *d1, char *d2,
 												  struct session *sess);
@@ -148,36 +131,32 @@ static struct key_binding *keys_root = NULL;
 
 static struct key_action key_actions[KEY_MAX_ACTIONS + 1] = {
 
-	{key_action_delete_backward_word, "Delete Word",
-	 "The \002Delete Word\002 action deletes the word to the left of the cursor."},
-	{key_action_delete_forward_word, "Delete Word Forward",
-	 "The \002Delete Word Forward\002 action deletes the word to the right of the cursor."},
 	{key_action_handle_command, "Run Command",
-	 "The \002Run Command\002 action runs the data in Data 1 as if it has been typed into the entry box where you pressed the key sequence. Thus it can contain text (which will be sent to the channel/person), commands or user commands. When run all \002\\n\002 charactors in Data 1 are used to deliminate seperate commands so it is possible to run more than one command. If you want a \002\\\002 in the actual text run then enter \002\\\\\002"},
+	 N_("The \002Run Command\002 action runs the data in Data 1 as if it has been typed into the entry box where you pressed the key sequence. Thus it can contain text (which will be sent to the channel/person), commands or user commands. When run all \002\\n\002 characters in Data 1 are used to deliminate seperate commands so it is possible to run more than one command. If you want a \002\\\002 in the actual text run then enter \002\\\\\002")},
 	{key_action_page_switch, "Change Page",
-	 "The \002Change Page\002 command switchs between pages in the notebook. Set Data 1 to the page you want to switch to. If Data 2 is set to anything then the switch will be relative to the current position"},
+	 N_("The \002Change Page\002 command switches between pages in the notebook. Set Data 1 to the page you want to switch to. If Data 2 is set to anything then the switch will be relative to the current position")},
 	{key_action_insert, "Insert in Buffer",
-	 "The \002Insert in Buffer\002 command will insert the contents of Data 1 into the entry where the key sequence was pressed at the current cursor position"},
+	 N_("The \002Insert in Buffer\002 command will insert the contents of Data 1 into the entry where the key sequence was pressed at the current cursor position")},
 	{key_action_scroll_page, "Scroll Page",
-	 "The \002Scroll Page\002 command scrolls the text widget up or down one page. If Data 1 is set to anything the page scrolls up, else it scrolls down"},
+	 N_("The \002Scroll Page\002 command scrolls the text widget up or down one page. If Data 1 is set to anything the page scrolls up, else it scrolls down")},
 	{key_action_set_buffer, "Set Buffer",
-	 "The \002Set Buffer\002 command sets the entry where the key sequence was entered to the contents of Data 1"},
+	 N_("The \002Set Buffer\002 command sets the entry where the key sequence was entered to the contents of Data 1")},
 	{key_action_history_up, "Last Command",
-	 "The \002Last Command\002 command sets the entry to contain the last command entered - the same as pressing up in a shell"},
+	 N_("The \002Last Command\002 command sets the entry to contain the last command entered - the same as pressing up in a shell")},
 	{key_action_history_down, "Next Command",
-	 "The \002Next Command\002 command sets the entry to contain the next command entered - the same as pressing down in a shell"},
+	 N_("The \002Next Command\002 command sets the entry to contain the next command entered - the same as pressing down in a shell")},
 	{key_action_tab_comp, "Complete nick/command",
-	 "This command changes the text in the entry to finish an incomplete nickname or command. If Data 1 is set then double-tabbing in a string will select the last nick, not the next"},
+	 N_("This command changes the text in the entry to finish an incomplete nickname or command. If Data 1 is set then double-tabbing in a string will select the last nick, not the next")},
 	{key_action_comp_chng, "Change Selected Nick",
-	 "This command scrolls up and down through the list of nicks. If Data 1 is set to anything then if scrolls up else if scrolls down"},
+	 N_("This command scrolls up and down through the list of nicks. If Data 1 is set to anything it will scroll up, else it scrolls down")},
 	{key_action_replace, "Check For Replace",
-	 "This command checks the last work entered in the entry against the replace list and replaces it if it finds a match"},
+	 N_("This command checks the last work entered in the entry against the replace list and replaces it if it finds a match")},
 	{key_action_move_tab_left, "Move front tab left",
-	 "This command move the front tab left by one"},
+	 N_("This command move the front tab left by one")},
 	{key_action_move_tab_right, "Move front tab right",
-	 "This command move the front tab right by one"},
+	 N_("This command move the front tab right by one")},
 	{key_action_put_history, "Push input line into history",
-	 "Push input line into history but doesn't send to server"},
+	 N_("Push input line into history but doesn't send to server")},
 };
 
 void
@@ -189,7 +168,7 @@ key_init ()
 		key_load_defaults ();
 		if (key_load_kbs (NULL) == 1)
 			gtkutil_simpledialog
-				("There was an error loading key bindings configuration");
+				(_("There was an error loading key bindings configuration"));
 	}
 }
 
@@ -396,7 +375,7 @@ key_dialog_close ()
 static void
 key_dialog_add_new (GtkWidget * button, GtkCList * list)
 {
-	gchar *strs[] = { "", "<none>", "<none>", "<none>", "<none>" };
+	gchar *strs[] = { "", _("<none>"), _("<none>"), _("<none>"), _("<none>") };
 	struct key_binding *kb;
 
 	kb = malloc (sizeof (struct key_binding));
@@ -446,7 +425,7 @@ key_dialog_delete (GtkWidget * button, GtkCList * list)
 			last = cur;
 			cur = cur->next;
 		}
-		printf ("*** key_dialog_delete: couldn't find kb in list!\n");
+		printf (_("*** key_dialog_delete: couldn't find kb in list!\n"));
 		/*if (getenv ("XCHAT_DEBUG"))
 			abort ();*/
 	}
@@ -463,11 +442,11 @@ key_dialog_sel_act (GtkWidget * un, int num)
 		kb = gtk_clist_get_row_data (GTK_CLIST (key_dialog_kb_clist), row);
 		kb->action = num;
 		gtk_clist_set_text (GTK_CLIST (key_dialog_kb_clist), row, 2,
-								  key_actions[num].name);
+								  _(key_actions[num].name));
 		if (key_actions[num].help)
 		{
 			gtk_xtext_clear (GTK_XTEXT (key_dialog_text));
-			PrintTextRaw (key_dialog_text, key_actions[num].help, 0);
+			PrintTextRaw (key_dialog_text, _(key_actions[num].help), 0);
 		}
 	}
 }
@@ -490,7 +469,7 @@ key_dialog_sel_row (GtkWidget * clist, gint row, gint column,
 		if (key_actions[kb->action].help)
 		{
 			gtk_xtext_clear (GTK_XTEXT (key_dialog_text));
-			PrintTextRaw (key_dialog_text, key_actions[kb->action].help, 0);
+			PrintTextRaw (key_dialog_text, _(key_actions[kb->action].help), 0);
 		}
 	}
 	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (key_dialog_tog_c),
@@ -639,22 +618,21 @@ key_dialog_show ()
 {
 	GtkWidget *vbox, *hbox, *list, *vbox2, *wid, *wid2, *wid3, *hbox2;
 	struct key_binding *kb;
-	gchar *titles[] = { "Mod", "Key", "Action", "1", "2" };
+	gchar *titles[] = { _("Mod"), _("Key"), _("Action"), "1", "2" };
 	char temp[32];
 	int i;
 
 	if (key_dialog)
 	{
-		gdk_window_show (key_dialog->window);
+		wins_bring_tofront (key_dialog);
 		return;
 	}
-	key_dialog = gtkutil_window_new ("Edit Key Bindings", "Edit Key Bindings",
-												560, 330, key_dialog_close, 0, FALSE);
-	gtk_window_set_policy (GTK_WINDOW (key_dialog), TRUE, TRUE, FALSE);
 
-	vbox = gtk_vbox_new (0, 2);
+	key_dialog = maingui_window ("editkeys", _("X-Chat: Edit Key Bindings"),
+								TRUE, FALSE, key_dialog_close, NULL, 560, 330, NULL);
+	vbox = wins_get_vbox (key_dialog);
+
 	hbox = gtk_hbox_new (0, 2);
-	gtk_container_add (GTK_CONTAINER (key_dialog), vbox);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, 1, 1, 0);
 
 	list = gtkutil_clist_new (5, titles, hbox, 0, key_dialog_sel_row, 0, NULL,
@@ -662,7 +640,6 @@ key_dialog_show ()
 	gtk_widget_set_usize (list, 400, 0);
 	key_dialog_kb_clist = list;
 
-	gtk_widget_show (vbox);
 	gtk_widget_show (hbox);
 
 	kb = keys_root;
@@ -677,18 +654,18 @@ key_dialog_show ()
 		titles[0] = key_make_mod_str (kb->mod, temp);
 		titles[1] = kb->keyname;
 		if (kb->action < 0 || kb->action > KEY_MAX_ACTIONS)
-			titles[2] = "<none>";
+			titles[2] = _("<none>");
 		else
 			titles[2] = key_actions[kb->action].name;
 		if (kb->data1)
 			titles[3] = kb->data1;
 		else
-			titles[3] = "<none>";
+			titles[3] = _("<none>");
 
 		if (kb->data2)
 			titles[4] = kb->data2;
 		else
-			titles[4] = "<none>";
+			titles[4] = _("<none>");
 
 		gtk_clist_set_row_data (GTK_CLIST (list),
 										gtk_clist_append (GTK_CLIST (list), titles),
@@ -699,12 +676,12 @@ key_dialog_show ()
 
 	vbox2 = gtk_vbox_new (0, 2);
 	gtk_box_pack_end (GTK_BOX (hbox), vbox2, 1, 1, 0);
-	wid = gtk_button_new_with_label ("Add new");
+	wid = gtk_button_new_with_label (_("Add new"));
 	gtk_box_pack_start (GTK_BOX (vbox2), wid, 0, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (wid), "clicked",
 							  GTK_SIGNAL_FUNC (key_dialog_add_new), list);
 	gtk_widget_show (wid);
-	wid = gtk_button_new_with_label ("Delete");
+	wid = gtk_button_new_with_label (_("Delete"));
 	gtk_box_pack_start (GTK_BOX (vbox2), wid, 0, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (wid), "clicked",
 							  GTK_SIGNAL_FUNC (key_dialog_delete), list);
@@ -716,7 +693,7 @@ key_dialog_show ()
 
 	for (i = 0; i <= KEY_MAX_ACTIONS; i++)
 	{
-		wid3 = gtk_menu_item_new_with_label (key_actions[i].name);
+		wid3 = gtk_menu_item_new_with_label (_(key_actions[i].name));
 		gtk_widget_show (wid3);
 		gtk_menu_append (GTK_MENU (wid2), wid3);
 		gtk_signal_connect (GTK_OBJECT (wid3), "activate",
@@ -729,21 +706,21 @@ key_dialog_show ()
 	gtk_widget_show (wid);
 	key_dialog_act_menu = wid;
 
-	key_dialog_tog_s = key_dialog_make_toggle ("Shift", key_dialog_tog_key,
+	key_dialog_tog_s = key_dialog_make_toggle (_("Shift"), key_dialog_tog_key,
 															 (void *) STATE_SHIFT, vbox2);
-	key_dialog_tog_a = key_dialog_make_toggle ("Alt", key_dialog_tog_key,
+	key_dialog_tog_a = key_dialog_make_toggle (_("Alt"), key_dialog_tog_key,
 															 (void *) STATE_ALT, vbox2);
-	key_dialog_tog_c = key_dialog_make_toggle ("Ctrl", key_dialog_tog_key,
+	key_dialog_tog_c = key_dialog_make_toggle (_("Ctrl"), key_dialog_tog_key,
 															 (void *) STATE_CTRL, vbox2);
 
-	key_dialog_ent_key = key_dialog_make_entry ("Key", "key_press_event",
+	key_dialog_ent_key = key_dialog_make_entry (_("Key"), "key_press_event",
 															  key_dialog_set_key, NULL,
 															  vbox2);
 
-	key_dialog_ent_d1 = key_dialog_make_entry ("Data 1", "activate",
+	key_dialog_ent_d1 = key_dialog_make_entry (_("Data 1"), "activate",
 															 key_dialog_set_data, NULL,
 															 vbox2);
-	key_dialog_ent_d2 = key_dialog_make_entry ("Data 2", "activate",
+	key_dialog_ent_d2 = key_dialog_make_entry (_("Data 2"), "activate",
 															 key_dialog_set_data,
 															 (void *) 1, vbox2);
 
@@ -770,7 +747,7 @@ key_dialog_show ()
 	gtk_widget_show_all (key_dialog);
 }
 
-void
+static void
 key_save_kbs (char *fn)
 {
 	int fd, i;
@@ -784,7 +761,7 @@ key_save_kbs (char *fn)
 	fd = open (buf, O_CREAT | O_TRUNC | O_WRONLY | OFLAGS, 0x180);
 	if (fd < 0)
 	{
-		gtkutil_simpledialog ("Error opening keys config file\n");
+		gtkutil_simpledialog (_("Error opening keys config file\n"));
 		return;
 	}
 	write (fd, buf,
@@ -844,7 +821,7 @@ key_save_kbs (char *fn)
 /* I just know this is going to be a nasty parse, if you think it's bugged
    it almost certainly is so contact the XChat dev team --AGL */
 
-inline int
+static inline int
 key_load_kbs_helper_mod (char *in, int *out)
 {
 	int n, len, mod = 0;
@@ -896,7 +873,7 @@ key_load_kbs_helper_mod (char *in, int *out)
 
 /* *** Warning, Warning! - massive function ahead! --AGL */
 
-int
+static int
 key_load_kbs (char *filename)
 {
 	char *buf, *ibuf;
@@ -955,7 +932,7 @@ key_load_kbs (char *filename)
 				free (ibuf);
 				ibuf = malloc (1024);
 				snprintf (ibuf, 1024,
-							 "Unknown keyname %s in key bindings config file\nLoad aborted, please fix ~/.xchat/keybindings.conf\n",
+							 _("Unknown keyname %s in key bindings config file\nLoad aborted, please fix ~/.xchat/keybindings.conf\n"),
 							 buf);
 				gtkutil_simpledialog (ibuf);
 				free (ibuf);
@@ -993,7 +970,7 @@ key_load_kbs (char *filename)
 				free (ibuf);
 				ibuf = malloc (1024);
 				snprintf (ibuf, 1024,
-							 "Unknown action %s in key bindings config file\nLoad aborted, Please fix ~/.xchat/keybindings\n",
+							 _("Unknown action %s in key bindings config file\nLoad aborted, Please fix ~/.xchat/keybindings\n"),
 							 buf);
 				gtkutil_simpledialog (ibuf);
 				free (ibuf);
@@ -1014,7 +991,7 @@ key_load_kbs (char *filename)
 				free (ibuf);
 				ibuf = malloc (1024);
 				snprintf (ibuf, 1024,
-							 "Expecting Data line (beginning Dx{:|!}) but got:\n%s\n\nLoad aborted, Please fix ~/.xchat/keybindings\n",
+							 _("Expecting Data line (beginning Dx{:|!}) but got:\n%s\n\nLoad aborted, Please fix ~/.xchat/keybindings\n"),
 							 buf);
 				gtkutil_simpledialog (ibuf);
 				free (ibuf);
@@ -1083,11 +1060,8 @@ key_load_kbs (char *filename)
 	/*if (getenv ("XCHAT_DEBUG"))
 		abort ();*/
 	free (ibuf);
-	buf = malloc (1024);
-	snprintf (buf, 1024,
-				 "Key bindings config file is corrupt, load aborted\nPlease fix ~/.xchat/keybindings.conf\n");
-	gtkutil_simpledialog (buf);
-	free (buf);
+	gtkutil_simpledialog ("Key bindings config file is corrupt, load aborted\n"
+								 "Please fix ~/.xchat/keybindings.conf\n");
 	return 5;
 }
 
@@ -1095,86 +1069,6 @@ key_load_kbs (char *filename)
 
 /* See the NOTES above --AGL */
 
-int
-key_action_delete_forward_word(GtkWidget * wid, GdkEventKey * ent, char *d1, char *d2,
-										 struct session *sess)
-{
-	char * text, * cp;
-	int len, curpos;
-	char buf[16384];
-
-	curpos = GTK_EDITABLE(wid)->current_pos;
-
-	text = gtk_entry_get_text (GTK_ENTRY (wid));
-	len = strlen(text);
-
-	if (curpos >= len)
-		return 2;
-
-	if (len == 0 || len >= 16384)
-		/* Not worth handling this case. */
-		return 2;
-
-	memmove(buf, text, curpos);
-	buf[curpos] = 0;
-	cp = &text[curpos];
-
-	/* Skip leading whitespace. */
-	while (isspace(*cp))
-		++cp;
-
-	/* Skip non-whitespace (i.e., a word). */
-	while (*cp && !isspace(*cp))
-		++cp;
-
-	if (*cp)
-		strcat(buf, cp);
-
-	gtk_entry_set_text (GTK_ENTRY (wid), buf);
-	GTK_EDITABLE(wid)->current_pos = curpos;
-	return 2;
-}
-
-int
-key_action_delete_backward_word(GtkWidget * wid, GdkEventKey * ent, char *d1, char *d2,
-										  struct session *sess)
-{
-	char * text, * cp;
-	int len, curpos;
-	char buf[16384];
-
-	curpos = GTK_EDITABLE(wid)->current_pos;
-
-	if (curpos == 0)
-		return 2;
-
-	text = gtk_entry_get_text (GTK_ENTRY (wid));
-	len = strlen(text);
-
-	if (len == 0 || len >= 16384)
-		/* Not worth handling this case. */
-		return 2;
-
-	memmove(buf, text, curpos);
-	buf[curpos] = 0;
-	cp = &buf[curpos - 1];
-
-	/* Delete trailing whitespace. */
-	while (cp >= buf && isspace(*cp))
-		*cp-- = 0;
-
-	/* Delete trailing non-whitespace (i.e., a word). */
-	while (cp >= buf && !isspace(*cp))
-		*cp-- = 0;
-
-	if (curpos < len)
-		strcat(buf, &text[curpos]);
-
-	gtk_entry_set_text (GTK_ENTRY (wid), buf);
-	GTK_EDITABLE(wid)->current_pos = curpos - (len - strlen(buf));
-	return 2;
-}
-
 /* "Run command" */
 static int
 key_action_handle_command (GtkWidget * wid, GdkEventKey * evt, char *d1,
@@ -1388,7 +1282,7 @@ static int
 key_action_move_tab_left (GtkWidget * wid, GdkEventKey * ent, char *d1,
 								  char *d2, struct session *sess)
 {
-	maingui_moveleft (NULL);
+	wins_move_leftorright (sess->gui->window, TRUE);
 	return 2;						  /* don't allow default action */
 }
 
@@ -1396,7 +1290,7 @@ static int
 key_action_move_tab_right (GtkWidget * wid, GdkEventKey * ent, char *d1,
 									char *d2, struct session *sess)
 {
-	maingui_moveright (NULL);
+	wins_move_leftorright (sess->gui->window, FALSE);
 	return 2;						  /* -''- */
 }
 
@@ -1649,16 +1543,11 @@ tab_comp_cmd (GtkWidget * t)
 			{
 				snprintf (buf, sizeof (buf), "%s %s ", last, pop->name);
 				PrintText (sess, buf);
-				/*PrintText (sess, last);
-				   PrintText (sess, "\t");
-				   PrintText (sess, pop->name);
-				   PrintText (sess, "\t"); */
 				last = (void *) 1;
 				tab_comp_find_common (lcmd, pop->name);
 			} else if (last == (void *) 1)
 			{
 				PrintText (sess, pop->name);
-				/*PrintText (sess, "\t"); */
 				tab_comp_find_common (lcmd, pop->name);
 			}
 		}
@@ -1666,9 +1555,9 @@ tab_comp_cmd (GtkWidget * t)
 	}
 
 	i = 0;
-	while (cmds[i].name != NULL)
+	while (xc_cmds[i].name != NULL)
 	{
-		cmd = cmds[i].name;
+		cmd = xc_cmds[i].name;
 		slen = strlen (cmd);
 		if (len > slen)
 		{
@@ -1685,17 +1574,11 @@ tab_comp_cmd (GtkWidget * t)
 			{
 				snprintf (buf, sizeof (buf), "%s %s ", last, cmd);
 				PrintText (sess, buf);
-				/*PrintText (sess, last);
-				   PrintText (sess, "\t");
-				   PrintText (sess, cmd);
-				   PrintText (sess, "\t"); */
 				last = (void *) 1;
 				tab_comp_find_common (lcmd, cmd);
 			} else if (last == (void *) 1)
 			{
 				PrintText (sess, cmd);
-				/*PrintText (sess, cmd);
-				   PrintText (sess, "\t"); */
 				tab_comp_find_common (lcmd, cmd);
 			}
 		}
@@ -1789,7 +1672,7 @@ tab_nick_comp (GtkWidget *t, int shift)
 	sess = find_session_from_inputgad (t);
 	if (sess == NULL)
 		return 0;
-	if (sess->is_dialog)
+	if (sess->type == SESS_DIALOG)
 		return 0;
 	
 	len = strlen (text);
