@@ -20,82 +20,42 @@
 #include <stdlib.h>
 #include <fcntl.h>
 #include <string.h>
-#include <sys/utsname.h>
 #include <unistd.h>
-#include "../common/xchat.h"
+
 #include "fe-gtk.h"
-#include "menu.h"
-#include "usermenu.h"
+#include "../common/xchat.h"
+#include "../common/xchatc.h"
 #include "../common/cfgfiles.h"
-#include "gtkutil.h"
+#include "../common/outbound.h"
 #include "../common/ignore.h"
 #include "../common/fe.h"
 #include "../common/util.h"
+#include "../common/plugin.h"
+#include "../common/perlc.h"
 #include "xtext.h"
+#include "about.h"
+#include "banlist.h"
+#include "chanlist.h"
+#include "editlist.h"
+#include "fkeys.h"
+#include "gtkutil.h"
+#include "maingui.h"
+#include "notifygui.h"
+#include "rawlog.h"
+#include "palette.h"
+#include "search.h"
+#include "settings.h"
+#include "textgui.h"
+#include "usermenu.h"
+#include "urlgrab.h"
+#include "menu.h"
+
 #ifdef USE_ZVT
 #include <zvt/zvtterm.h>
 #endif
 
-GtkWidget *createmenus (void *app, struct session *sess, int bar);
-
-extern struct session *current_tab;
-extern struct xchatprefs prefs;
-extern GdkColor colors[];
-extern GSList *sess_list;
-extern GdkFont *font_normal;
-extern GtkWidget *main_window, *main_book;
-extern GtkWidget *main_menu_bar, *main_menu;
-
-extern int cfg_get_bool (char *var);
-extern void maingui_showhide_topic (GtkWidget * button, session * sess);
-extern int auto_insert (char *dest, char *src, char *word[], char *word_eol[],
-								char *a, char *c, char *d, char *h, char *n, char *s);
-extern void palette_save (void);
-extern void search_open (session * sess);
-extern void xchat_exec (char *cmd);
-extern void show_and_unfocus (GtkWidget * wid);
-extern char *default_file (void);
-extern void menu_about (GtkWidget * wid, gpointer none);
-extern void userlist_hide (GtkWidget * igad, struct session *sess);
-extern void palette_edit (void);
-extern GdkFont *my_font_load (char *fontname);
-extern int tcp_send_len (struct server *serv, char *buf, int len);
-extern int tcp_send (struct server *serv, char *buf);
-extern void fkeys_edit (void);
-extern void ctcp_gui_load (GtkWidget * list);
-extern void ctcp_gui_save (GtkWidget * igad);
-extern void editlist_gui_open (GSList * list, char *, char *, char *,
-										 char *help);
-extern void ctcp_gui_open (void);
-extern void notify_opengui (void);
-extern void chanlist_opengui (struct server *serv);
-extern void url_opengui (GtkWidget *, gpointer);
-extern void cmd_help (struct session *sess, char *tbuf, char *word[],
-							 char *word_eol[]);
-extern void xchat_cleanup (GtkWidget * wid, gpointer sess);
-extern struct server *new_server (void);
-extern struct session *new_session (struct server *, char *from);
-extern void PrintText (struct session *, char *);
-extern void open_rawlog (struct server *);
-extern void open_server_list (GtkWidget *wid, gpointer sess, int dontshow);
-extern void settings_opengui (struct session *sess);
-extern char *find_selected_nick (struct session *sess);
-extern int handle_command (char *cmd, struct session *sess, int, int);
-extern int cmd_unloadall (struct session *sess, char *tbuf, char *word[],
-								  char *word_eol[]);
-extern int module_unload (char *name, struct session *sess);
 extern void module_glist (struct session *sess);
-extern void pevent_dialog_show ();
-extern void key_dialog_show ();
 
-extern struct session *menu_sess;
-extern GSList *popup_list;
-extern GSList *button_list;
-extern GSList *command_list;
-extern GSList *ctcp_list;
-extern GSList *replace_list;
-extern GSList *usermenu_list;
-extern GSList *urlhandler_list;
 static GSList *submenu_list;
 
 
@@ -105,8 +65,12 @@ goto_url (char *url)
 #ifdef USE_GNOME
 	gnome_url_show (url);
 #else
-	char tbuf[256];
+	char tbuf[512];
+#ifdef WIN32
+	snprintf (tbuf, sizeof (tbuf), "start %s", url);
+#else
 	snprintf (tbuf, sizeof (tbuf), "netscape -remote 'openURL(%s)'", url);
+#endif
 	xchat_exec (tbuf);
 #endif
 }
@@ -128,14 +92,21 @@ void
 nick_command_parse (session *sess, char *cmd, char *nick, char *allnick)
 {
 	char *buf;
-	char *host;
+	char *host = _("Host unknown");
 	struct User *user;
 
-	user = find_name (sess, nick);
-	if (user && user->hostname)
-		host = strchr (user->hostname, '@') + 1;
-	else
-		host = "Host unknown";
+	if (sess->type == SESS_DIALOG)
+	{
+		buf = gtk_entry_get_text (GTK_ENTRY (sess->gui->topicgad));
+		buf = strchr (buf, '@');
+		if (buf)
+			host = buf + 1;
+	} else
+	{
+		user = find_name (sess, nick);
+		if (user && user->hostname)
+			host = strchr (user->hostname, '@') + 1;
+	}
 
 	/* this can't overflow, since popup->cmd is only 256 */
 	buf = malloc (strlen (cmd) + strlen (nick) + strlen (allnick) + 512);
@@ -359,12 +330,19 @@ menu_create (GtkWidget *menu, GSList *list, char *target)
 		submenu_list = g_slist_remove (submenu_list, submenu_list->data);
 }
 
+static void
+menu_destroy (GtkObject *object, gpointer unused)
+{
+	gtk_widget_destroy (GTK_WIDGET (object));
+}
+
 static void
 menu_popup (GtkWidget *menu, GdkEventButton *event)
 {
+	gtk_signal_connect (GTK_OBJECT (menu), "selection-done",
+							  GTK_SIGNAL_FUNC (menu_destroy), NULL);
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
 						 event->button, event->time);
-	gtk_widget_show (menu);
 }
 
 static char *str_copy = 0;		/* for all pop-up menus */
@@ -387,24 +365,24 @@ menu_nickmenu (session *sess, GdkEventButton *event, char *nick)
 	{
 		submenu = menu_quick_sub (str_copy, menu);
 
-		sprintf (buf, "User: %s",
-					user->hostname ? user->hostname : "Unknown");
+		sprintf (buf, _("User: %s"),
+					user->hostname ? user->hostname : _("Unknown"));
 		wid = menu_quick_item (0, buf, submenu, 0, 0);
 
-		sprintf (buf, "Country: %s",
-					user->hostname ? country(user->hostname) : "Unknown");
+		sprintf (buf, _("Country: %s"),
+					user->hostname ? country(user->hostname) : _("Unknown"));
 		wid = menu_quick_item (0, buf, submenu, 0, 0);
 
-		sprintf (buf, "Realname: %s",
-					user->realname ? user->realname : "Unknown");
+		sprintf (buf, _("Realname: %s"),
+					user->realname ? user->realname : _("Unknown"));
 		wid = menu_quick_item (0, buf, submenu, 0, 0);
 
-		sprintf (buf, "Server: %s",
-					user->servername ? user->servername : "Unknown");
+		sprintf (buf, _("Server: %s"),
+					user->servername ? user->servername : _("Unknown"));
 		wid = menu_quick_item (0, buf, submenu, 0, 0);
 
-		sprintf (buf, "Last Msg: %s",
-					user->lasttalk ? ctime (&(user->lasttalk)) : "Unknown");
+		sprintf (buf, _("Last Msg: %s"),
+					user->lasttalk ? ctime (&(user->lasttalk)) : _("Unknown"));
 		if (user->lasttalk)
 			buf[strlen (buf) - 1] = 0;
 		wid = menu_quick_item (0, buf, submenu, 0, 0);
@@ -484,7 +462,7 @@ menu_middle_cb (GtkWidget *item, int n)
 		}
 		break;
 	case 1:
-		maingui_showhide_topic (0, menu_sess);
+		maingui_showhide_topic (menu_sess);
 		break;
 	case 2:
 		userlist_hide (0, menu_sess);
@@ -499,15 +477,15 @@ menu_middlemenu (session *sess, GdkEventButton *event)
 
 	menu = createmenus (0, sess, 0);
 
-	menu_toggle_item ("Menu Bar", menu, (void*)menu_middle_cb, 0, !prefs.hidemenu);
-	menu_toggle_item ("Topic Bar", menu, (void*)menu_middle_cb, (void*)1, prefs.topicbar);
-	menu_toggle_item ("User List", menu, (void*)menu_middle_cb, (void*)2, !prefs.hideuserlist);
+	menu_toggle_item (_("Menu Bar"), menu, (void*)menu_middle_cb, 0, !prefs.hidemenu);
+	menu_toggle_item (_("Topic Bar"), menu, (void*)menu_middle_cb, (void*)1, prefs.topicbar);
+	menu_toggle_item (_("User List"), menu, (void*)menu_middle_cb, (void*)2, !prefs.hideuserlist);
 
 	menu_popup (menu, event);
 }
 
 void
-menu_urlmenu (session *sess, GdkEventButton *event, char *url)
+menu_urlmenu (GdkEventButton *event, char *url)
 {
 	GtkWidget *menu;
 	int i;
@@ -526,6 +504,12 @@ menu_urlmenu (session *sess, GdkEventButton *event, char *url)
 			str_copy[i++] = '%';
 			str_copy[i++] = '2';
 			str_copy[i] = 'c';
+#ifdef WIN32
+		} else if (*url == '\\')
+		{
+			str_copy[i++] = '\\';
+			str_copy[i] = '\\';
+#endif
 		} else
 			str_copy[i] = *url;
 		i++;
@@ -619,24 +603,23 @@ menu_chanmenu (struct session *sess, GdkEventButton * event, char *chan)
 	menu_quick_item (0, 0, menu, 1, str_copy);
 
 	if (!is_joined)
-		menu_quick_item_with_callback (menu_chan_join, "Join Channel", menu,
+		menu_quick_item_with_callback (menu_chan_join, _("Join Channel"), menu,
 												 str_copy);
 	else
 	{
-		menu_quick_item_with_callback (menu_chan_part, "Part Channel", menu,
+		menu_quick_item_with_callback (menu_chan_part, _("Part Channel"), menu,
 												 str_copy);
-		menu_quick_item_with_callback (menu_chan_cycle, "Cycle Channel", menu,
+		menu_quick_item_with_callback (menu_chan_cycle, _("Cycle Channel"), menu,
 												 str_copy);
 	}
 
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
-						 event->button, event->time);
+	menu_popup (menu, event);
 }
 
 static void
 menu_open_server_list (GtkWidget *wid, gpointer none)
 {
-	open_server_list (0, menu_sess, FALSE);
+	fe_open_serverlist (menu_sess, FALSE, FALSE);
 }
 
 static void
@@ -695,74 +678,78 @@ menu_newshell_set_palette (session *sess)
 										zvt_red, zvt_grn, zvt_blu);
 }
 
-#ifndef USE_GNOME
-#define gnome_util_user_shell() g_strdup(getenv("SHELL"))
-#endif
-
 static void
 menu_newshell_tab (GtkWidget * wid, struct session *sess)
 {
 	int oldb = prefs.chanmodebuttons;
 	int oldp = prefs.paned_userlist;
-	struct server *serv;
+	int oldu = prefs.use_server_tab;
 	char *shell, *name;
 	char buf[16];
+	ZvtTerm *zvt;
 
-	prefs.chanmodebuttons = prefs.paned_userlist = 0;
-	serv = new_server ();
-	if (!serv)
-		return;
-	if (prefs.use_server_tab)
-		sess = serv->front_session;
-	else
-		sess = new_session (serv, 0);
-	sess->is_shell = 1;
+	prefs.chanmodebuttons = prefs.paned_userlist = prefs.use_server_tab = 0;
+	sess = new_ircwindow (NULL, NULL, SESS_SHELL);
 	prefs.chanmodebuttons = oldb;
 	prefs.paned_userlist = oldp;
+	prefs.use_server_tab = oldu;
 
-	gtk_widget_destroy (sess->gui->textgad);
-	gtk_widget_destroy (sess->gui->vscrollbar);
-
-	sess->gui->textgad = zvt_term_new ();
-	zvt_term_set_blink ((ZvtTerm *) sess->gui->textgad, FALSE);
-	zvt_term_set_fonts ((ZvtTerm *) sess->gui->textgad, font_normal, NULL);
-	zvt_term_set_scrollback ((ZvtTerm *) sess->gui->textgad, 720);
-	zvt_term_set_scroll_on_keystroke ((ZvtTerm *) sess->gui->textgad, TRUE);
-	zvt_term_set_background ((ZvtTerm *) sess->gui->textgad,
-									 prefs.background, prefs.transparent, 0);
-	gtk_container_add (GTK_CONTAINER (sess->gui->leftpane),
-							 sess->gui->textgad);
-	gtk_widget_show (sess->gui->textgad);
-	menu_newshell_set_palette (sess);
-
-	sess->gui->vscrollbar =
-		gtk_vscrollbar_new (((ZvtTerm *) sess->gui->textgad)->adjustment);
-	gtk_box_pack_start (GTK_BOX (sess->gui->leftpane), sess->gui->vscrollbar,
-							  0, 0, 1);
-	show_and_unfocus (sess->gui->vscrollbar);
-
-	GTK_WIDGET_UNSET_FLAGS (sess->gui->topicgad, GTK_CAN_FOCUS);
-	gtk_widget_destroy (sess->gui->userlistbox);
-	sess->gui->userlistbox = 0;
-	gtk_signal_connect (GTK_OBJECT (sess->gui->textgad), "child_died",
-							  GTK_SIGNAL_FUNC (shell_exit), sess);
-	gtk_signal_connect (GTK_OBJECT (sess->gui->textgad), "title_changed",
-							  GTK_SIGNAL_FUNC (menu_shell_title), sess);
-	gtk_widget_destroy (sess->gui->inputgad->parent);
-	sess->gui->inputgad = 0;
-	sess->gui->toolbox = 0;
-	gtk_widget_grab_focus (sess->gui->textgad);
+	fe_set_title (sess);
 
+#ifdef USE_GNOME
 	shell = gnome_util_user_shell ();
+#else
+	shell = strdup (getenv ("SHELL"));
+#endif
 	name = strrchr (shell, '/');
 	if (!name)
 		name = shell;
 	else
 		name++;
-
 	strcpy (sess->channel, name);
 	fe_set_channel (sess);
 
+	gtk_widget_destroy (sess->gui->textgad);
+	gtk_widget_destroy (sess->gui->vscrollbar);
+
+	sess->gui->textgad = zvt_term_new ();
+	zvt = (ZvtTerm *) sess->gui->textgad;
+
+	zvt_term_set_blink (zvt, FALSE);
+	if (prefs.font_shell[0] == 0)
+		zvt_term_set_fonts (zvt, font_normal, NULL);
+	else
+		zvt_term_set_font_name (zvt, prefs.font_shell);
+	zvt_term_set_scrollback (zvt, 720);
+	zvt_term_set_scroll_on_keystroke (zvt, TRUE);
+	zvt_term_set_background (zvt, prefs.background, prefs.transparent, 0);
+	gtk_container_add (GTK_CONTAINER (sess->gui->leftpane), GTK_WIDGET (zvt));
+	gtk_widget_show (GTK_WIDGET (zvt));
+	menu_newshell_set_palette (sess);
+
+	sess->gui->vscrollbar = gtk_vscrollbar_new (zvt->adjustment);
+	gtk_box_pack_start (GTK_BOX (sess->gui->leftpane), sess->gui->vscrollbar,
+							  0, 0, 1);
+	show_and_unfocus (sess->gui->vscrollbar);
+
+	GTK_WIDGET_UNSET_FLAGS (sess->gui->topicgad, GTK_CAN_FOCUS);
+
+	if (sess->gui->userlistbox)
+	{
+		gtk_widget_destroy (sess->gui->userlistbox);
+		sess->gui->userlistbox = 0;
+	}
+
+	gtk_signal_connect (GTK_OBJECT (sess->gui->textgad), "child_died",
+							  GTK_SIGNAL_FUNC (shell_exit), sess);
+	gtk_signal_connect (GTK_OBJECT (sess->gui->textgad), "title_changed",
+							  GTK_SIGNAL_FUNC (menu_shell_title), sess);
+	gtk_widget_destroy (sess->gui->inputgad->parent);
+
+	sess->gui->inputgad = 0;
+	sess->gui->toolbox = 0;
+	gtk_widget_grab_focus (sess->gui->textgad);
+
 	switch (zvt_term_forkpty ((ZvtTerm *) sess->gui->textgad, 0))
 	{
 	case -1:
@@ -782,10 +769,14 @@ menu_newshell_tab (GtkWidget * wid, struct session *sess)
 		_exit (127);
 	}
 
-	if (main_book)
+	if (main_window)
 		gdk_window_set_back_pixmap (main_book->window, 0, 0);
 
+#ifdef USE_GNOME
 	g_free (shell);				/* FIXME: is this a race condition ???? */
+#else
+	free (shell);
+#endif
 }
 #endif
 
@@ -793,12 +784,9 @@ static void
 menu_newserver_window (GtkWidget * wid, gpointer none)
 {
 	int old = prefs.tabchannels;
-	struct server *serv;
 
 	prefs.tabchannels = 0;
-	serv = new_server ();
-	if (serv && !prefs.use_server_tab)
-		new_session (serv, 0);
+	new_ircwindow (NULL, NULL, SESS_SERVER);
 	prefs.tabchannels = old;
 }
 
@@ -808,7 +796,7 @@ menu_newchannel_window (GtkWidget * wid, gpointer none)
 	int old = prefs.tabchannels;
 
 	prefs.tabchannels = 0;
-	new_session (menu_sess->server, 0);
+	new_ircwindow (menu_sess->server, NULL, SESS_CHANNEL);
 	prefs.tabchannels = old;
 }
 
@@ -816,12 +804,9 @@ static void
 menu_newserver_tab (GtkWidget * wid, gpointer none)
 {
 	int old = prefs.tabchannels;
-	struct server *serv;
 
 	prefs.tabchannels = 1;
-	serv = new_server ();
-	if (serv && !prefs.use_server_tab)
-		new_session (serv, 0);
+	new_ircwindow (NULL, NULL, SESS_SERVER);
 	prefs.tabchannels = old;
 }
 
@@ -831,7 +816,7 @@ menu_newchannel_tab (GtkWidget * wid, gpointer none)
 	int old = prefs.tabchannels;
 
 	prefs.tabchannels = 1;
-	new_session (menu_sess->server, 0);
+	new_ircwindow (menu_sess->server, NULL, SESS_CHANNEL);
 	prefs.tabchannels = old;
 }
 
@@ -845,16 +830,18 @@ static void
 menu_autodccsend (GtkWidget * wid, gpointer none)
 {
 	prefs.autodccsend = !prefs.autodccsend;
+#ifndef WIN32
 	if (prefs.autodccsend)
 	{
 		if (!strcasecmp (g_get_home_dir (), prefs.dccdir))
 		{
-			gtkutil_simpledialog ("*WARNING*\n"
+			gtkutil_simpledialog (_("*WARNING*\n"
 										 "Auto accepting DCC to your home directory\n"
 										 "can be dangerous and is exploitable. Eg:\n"
-										 "Someone could send you a .bash_profile");
+										 "Someone could send you a .bash_profile"));
 		}
 	}
+#endif
 }
 
 static void
@@ -872,8 +859,7 @@ menu_search ()
 static void
 menu_flushbuffer (GtkWidget * wid, gpointer none)
 {
-	if (!menu_sess->is_shell)
-		fe_text_clear (menu_sess);
+	fe_text_clear (menu_sess);
 }
 
 static void
@@ -893,7 +879,7 @@ savebuffer_req_done (session *sess, void *arg2, char *file)
 		while (ent)
 		{
 			buf = gtk_xtext_strip_color (ent->str, ent->str_len, NULL,
-								&newlen, GTK_XTEXT(sess->gui->textgad)->fonttype);
+							&newlen, GTK_XTEXT(sess->gui->textgad)->fonttype, NULL);
 			write (fh, buf, strlen (buf));
 			write (fh, "\n", 1);
 			free (buf);
@@ -907,7 +893,7 @@ savebuffer_req_done (session *sess, void *arg2, char *file)
 static void
 menu_savebuffer (GtkWidget * wid, gpointer none)
 {
-	gtkutil_file_req ("Select an output filename", savebuffer_req_done,
+	gtkutil_file_req (_("Select an output filename"), savebuffer_req_done,
 							menu_sess, 0, TRUE);
 }
 
@@ -962,18 +948,12 @@ menu_invisible (GtkWidget * wid, gpointer none)
 	}
 }
 
-static void
-menu_help (GtkWidget * wid, gpointer none)
-{
-	cmd_help (menu_sess, 0, 0, 0);
-}
-
 static void
 menu_savedefault (GtkWidget * wid, gpointer none)
 {
 	palette_save ();
 	if (save_config ())
-		gtkutil_simpledialog ("Settings saved.");
+		gtkutil_simpledialog (_("Settings saved."));
 }
 
 static void
@@ -982,6 +962,13 @@ menu_chanlist (GtkWidget * wid, gpointer none)
 	chanlist_opengui (menu_sess->server);
 }
 
+static void
+menu_banlist (GtkWidget * wid, gpointer none)
+{
+	banlist_opengui (menu_sess);
+}
+
+
 #ifdef USE_PERL
 
 static void
@@ -1001,7 +988,7 @@ menu_loadperl_callback (struct session *sess, void *data2, char *file)
 static void
 menu_loadperl (void)
 {
-	gtkutil_file_req ("Select a Perl script to load", menu_loadperl_callback,
+	gtkutil_file_req (_("Select a Perl script to load"), menu_loadperl_callback,
 							menu_sess, 0, FALSE);
 }
 
@@ -1045,7 +1032,7 @@ menu_loadplugin_callback (struct session *sess, void *data2, char *file)
 static void
 menu_loadplugin (void)
 {
-	gtkutil_file_req ("Select a Plugin to load", menu_loadplugin_callback,
+	gtkutil_file_req (_("Select a Plugin to load"), menu_loadplugin_callback,
 							menu_sess, 0, FALSE);
 }
 
@@ -1089,7 +1076,7 @@ menu_loadpython_callback (struct session *sess, void *data2, char *file)
 static void
 menu_loadpython (void)
 {
-	gtkutil_file_req ("Select a Python script to load",
+	gtkutil_file_req (_("Select a Python script to load"),
 							menu_loadpython_callback, menu_sess, 0, FALSE);
 }
 
@@ -1107,7 +1094,7 @@ menu_pythonlist (void)
 #endif
 
 
-#define usercommands_help  "User Commands - Special codes:\n\n"\
+#define usercommands_help  _("User Commands - Special codes:\n\n"\
                            "%c  =  current channel\n"\
 									"%m  =  machine info\n"\
                            "%n  =  your nick\n"\
@@ -1120,26 +1107,26 @@ menu_pythonlist (void)
                            "eg:\n"\
                            "/cmd john hello\n\n"\
                            "%2 would be \042john\042\n"\
-                           "&2 would be \042john hello\042."
+                           "&2 would be \042john hello\042.")
 
-#define ulpopup_help       "Userlist Popup - Special codes:\n\n"\
+#define ulpopup_help       _("Userlist Popup - Special codes:\n\n"\
                            "%c  =  current channel\n"\
                            "%h  =  selected nick's hostname\n"\
 									"%m  =  machine info\n"\
                            "%n  =  your nick\n"\
                            "%s  =  selected nick\n"\
-									"%t  =  time/date\n"
+									"%t  =  time/date\n")
 
-#define ulbutton_help       "Userlist Buttons - Special codes:\n\n"\
+#define ulbutton_help       _("Userlist Buttons - Special codes:\n\n"\
                            "%a  =  all selected nicks\n"\
                            "%c  =  current channel\n"\
                            "%h  =  selected nick's hostname\n"\
 									"%m  =  machine info\n"\
                            "%n  =  your nick\n"\
                            "%s  =  selected nick\n"\
-									"%t  =  time/date\n"
+									"%t  =  time/date\n")
 
-#define ctcp_help          "CTCP Replies - Special codes:\n\n"\
+#define ctcp_help          _("CTCP Replies - Special codes:\n\n"\
                            "%d  =  data (the whole ctcp)\n"\
 									"%m  =  machine info\n"\
                            "%s  =  nick who sent the ctcp\n"\
@@ -1147,46 +1134,46 @@ menu_pythonlist (void)
                            "%2  =  word 2\n"\
                            "%3  =  word 3\n"\
                            "&2  =  word 2 to the end of line\n"\
-                           "&3  =  word 3 to the end of line\n\n"
+                           "&3  =  word 3 to the end of line\n\n")
 
-#define url_help           "URL Handlers - Special codes:\n\n"\
+#define url_help           _("URL Handlers - Special codes:\n\n"\
                            "%s  =  the URL string\n\n"\
                            "Putting a ! infront of the command\n"\
                            "indicates it should be sent to a\n"\
-                           "shell instead of X-Chat"
+                           "shell instead of X-Chat")
 
 static void
 menu_usercommands (void)
 {
-	editlist_gui_open (command_list, "X-Chat: User Defined Commands",
+	editlist_gui_open (command_list, _("X-Chat: User Defined Commands"),
 							 "commands", "commands.conf", usercommands_help);
 }
 
 static void
 menu_ulpopup (void)
 {
-	editlist_gui_open (popup_list, "X-Chat: Userlist Popup menu", "popup",
+	editlist_gui_open (popup_list, _("X-Chat: Userlist Popup menu"), "popup",
 							 "popup.conf", ulpopup_help);
 }
 
 static void
 menu_rpopup (void)
 {
-	editlist_gui_open (replace_list, "X-Chat: Replace menu", "replace",
+	editlist_gui_open (replace_list, _("X-Chat: Replace"), "replace",
 							 "replace.conf", 0);
 }
 
 static void
 menu_usermenu (void)
 {
-	editlist_gui_open (usermenu_list, "X-Chat: User menu", "usermenu",
+	editlist_gui_open (usermenu_list, _("X-Chat: User menu"), "usermenu",
 							 "usermenu.conf", 0);
 }
 
 static void
 menu_urlhandlers (void)
 {
-	editlist_gui_open (urlhandler_list, "X-Chat: URL Handlers", "urlhandlers",
+	editlist_gui_open (urlhandler_list, _("X-Chat: URL Handlers"), "urlhandlers",
 							 "urlhandlers.conf", url_help);
 }
 
@@ -1205,14 +1192,14 @@ menu_keypopup (void)
 static void
 menu_ulbuttons (void)
 {
-	editlist_gui_open (button_list, "X-Chat: Userlist buttons", "buttons",
+	editlist_gui_open (button_list, _("X-Chat: Userlist buttons"), "buttons",
 							 "buttons.conf", ulbutton_help);
 }
 
 static void
 menu_ctcpguiopen (void)
 {
-	editlist_gui_open (ctcp_list, "X-Chat: CTCP Replies", "ctcpreply",
+	editlist_gui_open (ctcp_list, _("X-Chat: CTCP Replies"), "ctcpreply",
 							 "ctcpreply.conf", ctcp_help);
 }
 
@@ -1247,7 +1234,10 @@ menu_autoreconnectonfail (GtkWidget *wid, gpointer none)
 static void
 menu_autodialog (GtkWidget *wid, gpointer none)
 {
-	prefs.autodialog = !prefs.autodialog;
+	if (GTK_CHECK_MENU_ITEM (wid)->active)
+		prefs.autodialog = 1;
+	else
+		prefs.autodialog = 0;
 }
 
 static void
@@ -1274,6 +1264,24 @@ menu_webpage (GtkWidget *wid, gpointer none)
 	goto_url ("http://xchat.org");
 }
 
+static void
+menu_dcc_recv_win (GtkWidget *wid, gpointer none)
+{
+	fe_dcc_open_recv_win (FALSE);
+}
+
+static void
+menu_dcc_send_win (GtkWidget *wid, gpointer none)
+{
+	fe_dcc_open_send_win (FALSE);
+}
+
+static void
+menu_dcc_chat_win (GtkWidget *wid, gpointer none)
+{
+	fe_dcc_open_chat_win (FALSE);
+}
+
 #ifdef USE_GNOME
 
 #include "../pixmaps/win.xpm"
@@ -1337,7 +1345,7 @@ static GnomeUIInfo xchatmenu[] = {
 	{
 	 GNOME_APP_UI_ITEM,
 	 N_("Quit"), 0,
-	 xchat_cleanup, 0, 0,
+	 xchat_exit, 0, 0,
 	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_QUIT,
 	 'Q', GDK_MOD1_MASK, 0},
 	GNOMEUIINFO_END
@@ -1351,18 +1359,18 @@ static GnomeUIInfo windowsmenu[] = {
 	 },
 	{
 	 GNOME_APP_UI_ITEM,
-	 N_("DCC Send Window.."), 0,
-	 fe_dcc_open_send_win, 0, 0, GNOME_APP_PIXMAP_DATA, win_xpm,
+	 N_("File Send Window.."), 0,
+	 menu_dcc_send_win, 0, 0, GNOME_APP_PIXMAP_DATA, win_xpm,
 	 },
 	{
 	 GNOME_APP_UI_ITEM,
-	 N_("DCC Receive Window.."), 0,
-	 fe_dcc_open_recv_win, 0, 0, GNOME_APP_PIXMAP_DATA, win_xpm,
+	 N_("File Receive Window.."), 0,
+	 menu_dcc_recv_win, 0, 0, GNOME_APP_PIXMAP_DATA, win_xpm,
 	 },
 	{
 	 GNOME_APP_UI_ITEM,
 	 N_("DCC Chat Window.."), 0,
-	 fe_dcc_open_chat_win, 0, 0, GNOME_APP_PIXMAP_DATA, win_xpm,
+	 menu_dcc_chat_win, 0, 0, GNOME_APP_PIXMAP_DATA, win_xpm,
 	 },
 	{
 	 GNOME_APP_UI_ITEM,
@@ -1379,6 +1387,11 @@ static GnomeUIInfo windowsmenu[] = {
 	 N_("Notify List Window.."), 0,
 	 (menucallback) notify_opengui, 0, 0, GNOME_APP_PIXMAP_DATA, win_xpm,
 	 },
+	{
+	 GNOME_APP_UI_ITEM,
+	 N_("Ban List Window.."), 0,
+	 (menucallback) menu_banlist, 0, 0, GNOME_APP_PIXMAP_DATA, win_xpm,
+	},
 	{
 	 GNOME_APP_UI_ITEM,
 	 N_("Ignore Window.."), 0,
@@ -1537,13 +1550,6 @@ static GnomeUIInfo settingsmenu[] = {
 };
 
 static GnomeUIInfo helpmenu[] = {
-	{
-	 GNOME_APP_UI_ITEM,
-	 N_("Help.."), 0,
-	 menu_help, 0, 0,
-	 GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_HELP,
-	 0, 0, 0},
-	GNOMEUIINFO_SEPARATOR,
 	{
 	 GNOME_APP_UI_ITEM,
 	 N_("X-Chat Homepage.."), 0,
@@ -1725,21 +1731,22 @@ static struct mymenu mymenu[] = {
 #endif
 	{M_MENU, N_("Close"), (menucallback) menu_close, 0, 1},
 	{M_SEP, 0, 0, 0, 0},
-	{M_MENU, N_("Quit"), (menucallback) xchat_cleanup, 0, 1},	/* 12 */
+	{M_MENU, N_("Quit"), (menucallback) xchat_exit, 0, 1},	/* 12 */
 
 	{M_NEWMENU, N_("Windows"), 0, 0, 1},
 	{M_MENU, N_("Channel List Window.."), (menucallback) menu_chanlist, 0, 1},
-	{M_MENU, N_("DCC Send Window.."), (menucallback) fe_dcc_open_send_win, 0, 1},
-	{M_MENU, N_("DCC Receive Window.."), (menucallback) fe_dcc_open_recv_win, 0, 1},
-	{M_MENU, N_("DCC Chat Window.."), (menucallback) fe_dcc_open_chat_win, 0, 1},
+	{M_MENU, N_("File Send Window.."), (menucallback) menu_dcc_send_win, 0, 1},
+	{M_MENU, N_("File Receive Window.."), (menucallback) menu_dcc_recv_win, 0, 1},
+	{M_MENU, N_("DCC Chat Window.."), (menucallback) menu_dcc_chat_win, 0, 1},
 	{M_MENU, N_("Raw Log Window.."), (menucallback) menu_rawlog, 0, 1},
 	{M_MENU, N_("URL Grabber Window.."), (menucallback) url_opengui, 0, 1},
 	{M_MENU, N_("Notify List Window.."), (menucallback) notify_opengui, 0, 1},
 	{M_MENU, N_("Ignore Window.."), (menucallback) ignore_gui_open, 0, 1},
+	{M_MENU, N_("Ban List Window.."), (menucallback) menu_banlist, 0, 1},
 	{M_SEP, 0, 0, 0, 0},
 	{M_MENU, N_("Flush Buffer"), (menucallback) menu_flushbuffer, 0, 1},
 	{M_MENU, N_("Search Buffer.."), (menucallback) menu_search, 0, 1},
-	{M_MENU, N_("Save Buffer.."), (menucallback) menu_savebuffer, 0, 1},	/* 25 */
+	{M_MENU, N_("Save Buffer.."), (menucallback) menu_savebuffer, 0, 1},	/* 26 */
 
 	{M_NEWMENU, N_("User Modes"), 0, 0, 1},
 	{M_MENUTOG, N_("Invisible"), (menucallback) menu_invisible, 1, 1},
@@ -1747,16 +1754,16 @@ static struct mymenu mymenu[] = {
 	{M_MENUTOG, N_("Receive Server Notices"), (menucallback) menu_servernotice, 1, 1},
 	{M_SEP, 0, 0, 0, 0},
 	{M_MENUTOG, N_("Marked Away"), (menucallback) menu_away, 0, 1},
-	{M_SEP, 0, 0, 0},															/* 32 */
+	{M_SEP, 0, 0, 0, 0},														/* 33 */
 	{M_MENUTOG, N_("Auto ReJoin on Kick"), (menucallback) menu_autorejoin, 0, 1},
 	{M_MENUTOG, N_("Auto ReConnect to Server"), (menucallback) menu_autoreconnect, 0, 1},
 	{M_MENUTOG, N_("Never-give-up ReConnect"), (menucallback) menu_autoreconnectonfail, 0, 1},
-	{M_SEP, 0, 0, 0},															/* 36 */
+	{M_SEP, 0, 0, 0, 0},														/* 37 */
 	{M_MENUTOG, N_("Auto Open Dialog Windows"), (menucallback) menu_autodialog, 0, 1},
 	{M_MENUTOG, N_("Auto Accept DCC Chat"), (menucallback) menu_autodccchat, 0, 1},
 	{M_MENUTOG, N_("Auto Accept DCC Send"), (menucallback) menu_autodccsend, 0, 1},
 
-	{M_NEWMENU, N_("Settings"), 0, 0, 1},	/* 40 */
+	{M_NEWMENU, N_("Settings"), 0, 0, 1},	/* 41 */
 	{M_MENU, N_("Setup.."), (menucallback) menu_settings, 0, 1},
 	{M_MENU, N_("Palette.."), (menucallback) palette_edit, 0, 1},
 	{M_MENU, N_("User Commands.."), (menucallback) menu_usercommands, 0, 1},
@@ -1765,14 +1772,14 @@ static struct mymenu mymenu[] = {
 	{M_MENU, N_("Userlist Popup.."), (menucallback) menu_ulpopup, 0, 1},
 	{M_MENU, N_("Replace.."), (menucallback) menu_rpopup, 0, 1},
 	{M_MENU, N_("URL Handlers.."), (menucallback) menu_urlhandlers, 0, 1},
-	{M_MENU, N_("Edit Event Texts.."), (menucallback) menu_evtpopup, 0, 1},
+	{M_MENU, N_("Edit Event Texts.."), (menucallback) menu_evtpopup, 0, 1},	/* 50 */
 	{M_MENU, N_("Edit Key Bindings.."), (menucallback) menu_keypopup, 0, 1},
 	{M_SEP, 0, 0, 0, 0},
 	{M_MENU, N_("Reload Settings"), (menucallback) menu_reload, 0, 1},
 	{M_SEP, 0, 0, 0, 0},
 	{M_MENU, N_("Save Settings now"), (menucallback) menu_savedefault, 0, 1},
 	{M_MENUTOG, N_("Save Settings on exit"), (menucallback) menu_saveexit, 1, 1},
-	{M_NEWMENU, N_("Scripts & Plugins"), 0, 0, 1},	/* 55 */
+	{M_NEWMENU, N_("Scripts & Plugins"), 0, 0, 1},	/* 57 */
 #ifdef USE_PERL
 	{M_MENU, N_("Load Perl Script.."), (menucallback) menu_loadperl, 0, 1},
 	{M_MENU, N_("Unload All Scripts"), (menucallback) menu_unloadall, 0, 1},
@@ -1806,8 +1813,6 @@ static struct mymenu mymenu[] = {
 	{M_SEP, 0, 0, 0, 0},
 
 	{M_NEWMENURIGHT, N_("Help"), 0, 0, 1},
-	{M_MENU, N_("Help.."), (menucallback) menu_help, 0, 1},
-	{M_SEP, 0, 0, 0, 0},
 	{M_MENU, N_("X-Chat Homepage.."), (menucallback) menu_webpage, 0, 1},
 	{M_MENU, N_("Online Docs.."), (menucallback) menu_docs, 0, 1},
 	{M_SEP, 0, 0, 0, 0},
@@ -1835,17 +1840,17 @@ createmenus (void *app, struct session *sess, int bar)
 	if (!menu_sess)
 		menu_sess = sess;
 
-	mymenu[27-menuoffset].state = prefs.invisible;
-	mymenu[28-menuoffset].state = prefs.wallops;
-	mymenu[29-menuoffset].state = prefs.servernotice;
-	mymenu[31-menuoffset].state = sess->server->is_away;
-	mymenu[33-menuoffset].state = prefs.autorejoin;
-	mymenu[34-menuoffset].state = prefs.autoreconnect;
-	mymenu[35-menuoffset].state = prefs.autoreconnectonfail;
-	mymenu[37-menuoffset].state = prefs.autodialog;
-	mymenu[38-menuoffset].state = prefs.autodccchat;
-	mymenu[39-menuoffset].state = prefs.autodccsend;
-	mymenu[54-menuoffset].state = prefs.autosave;
+	mymenu[28-menuoffset].state = prefs.invisible;
+	mymenu[29-menuoffset].state = prefs.wallops;
+	mymenu[30-menuoffset].state = prefs.servernotice;
+	mymenu[32-menuoffset].state = sess->server->is_away;
+	mymenu[34-menuoffset].state = prefs.autorejoin;
+	mymenu[35-menuoffset].state = prefs.autoreconnect;
+	mymenu[36-menuoffset].state = prefs.autoreconnectonfail;
+	mymenu[38-menuoffset].state = prefs.autodialog;
+	mymenu[39-menuoffset].state = prefs.autodccchat;
+	mymenu[40-menuoffset].state = prefs.autodccsend;
+	mymenu[56-menuoffset].state = prefs.autosave;
 
 	while (1)
 	{
@@ -1885,13 +1890,13 @@ createmenus (void *app, struct session *sess, int bar)
 			gtk_check_menu_item_set_state (GTK_CHECK_MENU_ITEM (item),
 													 mymenu[i].state);
 			if (mymenu[i].callback)
-				gtk_signal_connect_object (GTK_OBJECT (item), "toggled",
-													GTK_SIGNAL_FUNC (mymenu[i].callback),
-													(gpointer)sess);
+				gtk_signal_connect (GTK_OBJECT (item), "toggled",
+										  GTK_SIGNAL_FUNC (mymenu[i].callback),
+										  (gpointer)sess);
 			gtk_menu_append (GTK_MENU (menu), item);
 			gtk_widget_show (item);
 			gtk_widget_set_sensitive (item, mymenu[i].activate);
-			if (bar && i == 31 - menuoffset)
+			if (bar && i == 32 - menuoffset)
 				sess->gui->awaymenuitem = item;
 			break;
 		case M_SEP:
