@@ -18,33 +18,31 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <fcntl.h>
 #include <time.h>
-#include "../common/xchat.h"
+
 #include "fe-gtk.h"
-#include "gtkutil.h"
+#include "../common/xchat.h"
 #include "../common/notify.h"
 #include "../common/cfgfiles.h"
 #include "../common/plugin.h"
 #include "../common/util.h"
+#include "../common/userlist.h"
+#include "gtkutil.h"
+#include "wins.h"
+#include "palette.h"
+#include "notifygui.h"
 
-extern void update_all_of (char *name);
-extern GSList *serv_list;
-extern GdkColor colors[];
 
-extern GSList *notify_list;
 static GtkWidget *notify_window = 0;
 static GtkWidget *notify_guilist;
 
-void notify_adduser (char *name);
-int notify_deluser (char *name);
 
-
-static int
+static void
 notify_closegui (void)
 {
 	notify_window = 0;
-	return 0;
 }
 
 void
@@ -87,10 +85,10 @@ notify_gui_update (void)
 
 		if (!online)				  /* Offline on all servers */
 		{
-			nnew[1] = "Offline";
+			nnew[1] = _("Offline");
 			nnew[2] = "";
 			if (!lastseen)
-				nnew[3] = "Never";
+				nnew[3] = _("Never");
 			else
 				nnew[3] = ctime (&lastseen);
 			crow = gtk_clist_append ((GtkCList *) notify_guilist, nnew);
@@ -109,7 +107,7 @@ notify_gui_update (void)
 						nnew[0] = notify->name;
 					else
 						nnew[0] = "";
-					nnew[1] = "Online";
+					nnew[1] = _("Online");
 					nnew[2] = servnot->server->servername;
 					nnew[3] = ctime (&servnot->laston);
 					crow = gtk_clist_append ((GtkCList *) notify_guilist, nnew);
@@ -144,28 +142,25 @@ static void
 notify_add_clicked (GtkWidget * igad)
 {
 	gtkutil_simpledialog
-		("Use the /NOTIFY command to add people to your notify list.");
+		(_("Use the /NOTIFY command to add people to your notify list."));
 }
 
 void
 notify_opengui (void)
 {
-	GtkWidget *vbox, *bbox, *Remove, *Add;
-	static gchar *titles[] = { "User", "Status", "Server", "Last Seen" };
+	GtkWidget *vbox, *bbox;
+	gchar *titles[] = { _("User"), _("Status"), _("Server"), _("Last Seen") };
 
 	if (notify_window)
 	{
-		gdk_window_show (notify_window->window);
+		wins_bring_tofront (notify_window);
 		return;
 	}
-	notify_window =
-		gtkutil_window_new ("X-Chat: Notify List", "notify", 400, 120,
-								  notify_closegui, 0, TRUE);
 
-	vbox = gtk_vbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
-	gtk_container_add (GTK_CONTAINER (notify_window), vbox);
-	gtk_widget_show (vbox);
+	notify_window =
+		maingui_window ("notify", _("X-Chat: Notify List"), FALSE, TRUE,
+							 notify_closegui, NULL, 400, 120, &notify_window);
+	vbox = wins_get_vbox (notify_window);
 
 	notify_guilist = gtkutil_clist_new (4, titles, vbox, GTK_POLICY_ALWAYS,
 													0, 0, 0, 0, GTK_SELECTION_BROWSE);
@@ -178,9 +173,9 @@ notify_opengui (void)
 	gtk_box_pack_start (GTK_BOX (vbox), bbox, FALSE, FALSE, 0);
 	gtk_widget_show (bbox);
 
-	Add = gtkutil_button (notify_window, GNOME_STOCK_PIXMAP_NEW, "Add",
+	gtkutil_button (notify_window, GNOME_STOCK_PIXMAP_NEW, _("Add"),
 								 notify_add_clicked, 0, bbox);
-	Remove = gtkutil_button (notify_window, GNOME_STOCK_PIXMAP_CLOSE, "Remove",
+	gtkutil_button (notify_window, GNOME_STOCK_PIXMAP_CLOSE, _("Remove"),
 									 notify_remove_clicked, 0, bbox);
 
 	notify_gui_update ();
