@@ -18,26 +18,23 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <pwd.h>
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <string.h>
 #include <unistd.h>
+
+#include "fe-gtk.h"
 #include "../common/xchat.h"
 #include "../common/cfgfiles.h"
 #include "../common/fe.h"
-#include "fe-gtk.h"
+#include "../common/server.h"
+#include "../common/xchatc.h"
 #include "serverlist.h"
 #include "gtkutil.h"
 #include "pixmaps.h"
+#include "wins.h"
 
-extern struct xchatprefs prefs;
-
-extern void connect_server (struct session *sess, char *server, int port,
-									 int quiet);
-extern struct session *new_session (struct server *serv, char *from);
-extern struct server *new_server (void);
 
 static GtkWidget *slwin = 0;
 static GtkWidget *sleditwin = 0;
@@ -49,6 +46,7 @@ static GtkWidget *entry_comment;
 static GtkWidget *entry_password;
 static GtkWidget *entry_nick;
 static GtkWidget *entry_autoconnect;
+static GtkWidget *entry_use_proxy;
 static GtkWidget *entry_use_ssl;
 static GtkWidget *entry_accept_invalid_cert;
 static GtkWidget *nick1gad;
@@ -71,6 +69,7 @@ static int closing_list = 0;
 /*static gboolean channelbox = 0;*/
 static gboolean first_autoconnect = 0;
 static gboolean new_entry = 0;
+static gboolean some_are_auto = 0;
 
 /**************************
  * Public functions:      *
@@ -160,8 +159,11 @@ add_server_entry (struct slentry *serv)
 		gtk_ctree_node_set_row_data (GTK_CTREE (sltree), node, new);
 		server_malloc_list = g_slist_prepend (server_malloc_list, new);
 		if (new->flags & AUTOCONNECT)
+		{
 			gtk_ctree_node_set_foreground (GTK_CTREE (sltree), node,
 													 &autoconnect_color);
+			some_are_auto = TRUE;
+		}
 	}
 }
 
@@ -239,7 +241,7 @@ load_serverentrys (int select_row)
 	char file[256];
 
 	snprintf (file, sizeof file, "%s/serverlist.conf", get_xdir ());
-	fh = open (file, O_RDONLY);
+	fh = open (file, O_RDONLY | OFLAGS);
 	if (fh != -1)
 	{
 		fstat (fh, &st);
@@ -256,6 +258,7 @@ load_serverentrys (int select_row)
 			do
 			{
 				serv.nick[0] = 0;
+				serv.flags = 0;
 				my_cfg = read_next_server_entry (my_cfg, &serv);
 				if (my_cfg)
 					add_server_entry (&serv);
@@ -309,7 +312,7 @@ parse_serverentrys (GtkCTree * ctree, GtkCTreeNode * node, int *fh)
 {
 	struct slentry *temporary =
 		gtk_ctree_node_get_row_data (GTK_CTREE (sltree), node);
-	struct slentry *buf = malloc (sizeof (struct slentry));
+	struct slentry buf;
 	GtkCTreeNode *tempus = node;
 	int diff = 0, depth = 0;
 
@@ -325,40 +328,39 @@ parse_serverentrys (GtkCTree * ctree, GtkCTreeNode * node, int *fh)
 
 	while (diff < 0)
 	{
-		buf->port = 0;
-		buf->flags = 0;
-		*(buf->channel) = 0;
-		*(buf->password) = 0;
-		*(buf->nick) = 0;
-		*(buf->comment) = 0;
-		strcpy (buf->server, "ENDSUB");
-		write_serverentry (buf, *fh);
+		buf.port = 0;
+		buf.flags = 0;
+		buf.channel[0] = 0;
+		buf.password[0] = 0;
+		buf.nick[0] = 0;
+		buf.comment[0] = 0;
+		strcpy (buf.server, "ENDSUB");
+		write_serverentry (&buf, *fh);
 		diff++;
 	}
 
-	strcpy (buf->channel, temporary->channel);
-	strcpy (buf->server, temporary->server);
-	strcpy (buf->password, temporary->password);
-	strcpy (buf->nick, temporary->nick);
-	strcpy (buf->comment, temporary->comment);
-	buf->port = temporary->port;
-	buf->flags = temporary->flags;
-	write_serverentry (buf, *fh);
+	strcpy (buf.channel, temporary->channel);
+	strcpy (buf.server, temporary->server);
+	strcpy (buf.password, temporary->password);
+	strcpy (buf.nick, temporary->nick);
+	strcpy (buf.comment, temporary->comment);
+	buf.port = temporary->port;
+	buf.flags = temporary->flags;
+	write_serverentry (&buf, *fh);
 
-	if (!strcmp (buf->server, "SUB")
+	if (!strcmp (buf.server, "SUB")
 		 && (((GTK_CTREE_ROW (node))->children)) == 0)
 	{
 		/* If we got a group without any children we need an ENDSUB at least */
-		buf->port = 0;
-		buf->flags = 0;
-		*(buf->channel) = 0;
-		*(buf->password) = 0;
-		*(buf->nick) = 0;
-		*(buf->comment) = 0;
-		strcpy (buf->server, "ENDSUB");
-		write_serverentry (buf, *fh);
+		buf.port = 0;
+		buf.flags = 0;
+		buf.channel[0] = 0;
+		buf.password[0] = 0;
+		buf.nick[0] = 0;
+		buf.comment[0] = 0;
+		strcpy (buf.server, "ENDSUB");
+		write_serverentry (&buf, *fh);
 	}
-	free (buf);
 }
 
 /*
@@ -375,7 +377,7 @@ save_serverentrys (void)
 	check_prefs_dir ();
 
 	snprintf (file, sizeof file, "%s/serverlist.conf", get_xdir ());
-	fh = open (file, O_TRUNC | O_WRONLY | O_CREAT, 0600);
+	fh = open (file, O_TRUNC | O_WRONLY | O_CREAT | OFLAGS, 0600);
 	if (fh != -1)
 	{
 		previous_depth = 0;
@@ -494,6 +496,11 @@ done_edit_entry (void)
 		else
 			editentry->flags |= AUTOCONNECT;
 
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (entry_use_proxy)))
+			editentry->flags &= ~DONT_USE_PROXY;
+		else
+			editentry->flags |= DONT_USE_PROXY;
+
 		if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (entry_use_ssl)))
 			editentry->flags &= ~USE_SSL;
 		else
@@ -548,7 +555,7 @@ edit_entry_clicked (GtkWidget * wid, GtkCTreeNode * temp)
 		return;
 
 	sleditwin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_title (GTK_WINDOW (sleditwin), "X-Chat: Edit entry");
+	gtk_window_set_title (GTK_WINDOW (sleditwin), _("X-Chat: Edit entry"));
 	gtk_signal_connect (GTK_OBJECT (sleditwin), "delete_event",
 							  GTK_SIGNAL_FUNC (close_edit_entry), 0);
 	editentry =
@@ -569,7 +576,7 @@ edit_entry_clicked (GtkWidget * wid, GtkCTreeNode * temp)
 	gtk_box_pack_start (GTK_BOX (hbox), wid1, FALSE, FALSE, 0);
 	gtk_widget_show (wid1);
 
-	frame = gtk_frame_new ("Edit entry:");
+	frame = gtk_frame_new (_("Edit entry:"));
 	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
 	if (!is_group (editnode))
 		gtk_widget_set_usize (frame, 400, 150);
@@ -589,26 +596,26 @@ edit_entry_clicked (GtkWidget * wid, GtkCTreeNode * temp)
 	gtk_box_pack_start (GTK_BOX (hbox), box2, TRUE, TRUE, 0);
 	gtk_widget_show (box2);
 
-	gtkutil_label_new ("Name:", box1);
+	gtkutil_label_new (_("Name:"), box1);
 	entry_comment = gtkutil_entry_new (99, box2, NULL, NULL);
 	gtk_entry_set_text (GTK_ENTRY (entry_comment), editentry->comment);
 
 	if (!is_group (editnode))
 	{
-		gtkutil_label_new ("Server:", box1);
+		gtkutil_label_new (_("Server:"), box1);
 		entry_server = gtkutil_entry_new (131, box2, NULL, NULL);
 		gtk_entry_set_text (GTK_ENTRY (entry_server), editentry->server);
 
 		sprintf (portbuffer, "%d", editentry->port);
-		gtkutil_label_new ("Port:", box1);
+		gtkutil_label_new (_("Port:"), box1);
 		entry_port = gtkutil_entry_new (5, box2, NULL, NULL);
 		gtk_entry_set_text (GTK_ENTRY (entry_port), portbuffer);
 
-		gtkutil_label_new ("Password:", box1);
+		gtkutil_label_new (_("Password:"), box1);
 		entry_password = gtkutil_entry_new (85, box2, NULL, NULL);
 		gtk_entry_set_text (GTK_ENTRY (entry_password), editentry->password);
 
-		gtkutil_label_new ("Channels:", box1);
+		gtkutil_label_new (_("Channels:"), box1);
 		entry_channel = gtkutil_entry_new (201, box2, NULL, NULL);
 		gtk_entry_set_text (GTK_ENTRY (entry_channel), editentry->channel);
 		gtk_widget_show (entry_channel);
@@ -619,11 +626,16 @@ edit_entry_clicked (GtkWidget * wid, GtkCTreeNode * temp)
 		gtk_widget_show (entry_nick);
 
 		box1 = gtk_hbox_new (FALSE, 0);
-		entry_autoconnect = gtk_check_button_new_with_label ("Autoconnect");
+		entry_autoconnect = gtk_check_button_new_with_label (_("Autoconnect"));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (entry_autoconnect),
 												(editentry->flags & AUTOCONNECT));
 
-		entry_use_ssl = gtk_check_button_new_with_label ("Use SSL");
+		entry_use_proxy = gtk_check_button_new_with_label (_("Use Proxy"));
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (entry_use_proxy),
+												!(editentry->flags & DONT_USE_PROXY));
+		add_tip (entry_use_proxy, _("Use proxy server if one is set in Setup->Proxy Server"));
+
+		entry_use_ssl = gtk_check_button_new_with_label (_("Use SSL"));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (entry_use_ssl),
 												(editentry->flags & USE_SSL));
 #ifndef USE_OPENSSL
@@ -631,7 +643,7 @@ edit_entry_clicked (GtkWidget * wid, GtkCTreeNode * temp)
 #endif
 
 		entry_accept_invalid_cert =
-			gtk_check_button_new_with_label ("Accept invalid cert.");
+			gtk_check_button_new_with_label (_("Accept invalid cert."));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
 												(entry_accept_invalid_cert),
 												(editentry->flags & ACCEPT_INVALID_CERT));
@@ -647,20 +659,22 @@ edit_entry_clicked (GtkWidget * wid, GtkCTreeNode * temp)
 #endif
 
 		gtk_box_pack_start (GTK_BOX (box1), entry_autoconnect, TRUE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (box1), entry_use_proxy, TRUE, FALSE, 0);
 		gtk_box_pack_start (GTK_BOX (box1), entry_use_ssl, TRUE, FALSE, 0);
 		gtk_box_pack_start (GTK_BOX (box1), entry_accept_invalid_cert, TRUE,
 								  FALSE, 0);
 		gtk_box_pack_start (GTK_BOX (vbox), box1, TRUE, TRUE, 0);
 		gtk_widget_show (entry_autoconnect);
+		gtk_widget_show (entry_use_proxy);
 		gtk_widget_show (entry_use_ssl);
 		gtk_widget_show (entry_accept_invalid_cert);
 		gtk_widget_show (box1);
 	}
 	box1 = gtk_hbox_new (TRUE, 2);
 
-	gtkutil_button (slwin, GNOME_STOCK_BUTTON_OK, "Ok",
+	gtkutil_button (slwin, GNOME_STOCK_BUTTON_OK, _("Ok"),
 						 done_edit_entry, 0, box1);
-	gtkutil_button (slwin, GNOME_STOCK_BUTTON_CANCEL, "Cancel",
+	gtkutil_button (slwin, GNOME_STOCK_BUTTON_CANCEL, _("Cancel"),
 						 close_edit_entry, 0, box1);
 
 	gtk_box_pack_start (GTK_BOX (vbox), box1, TRUE, TRUE, 0);
@@ -687,7 +701,7 @@ new_server_clicked (GtkWidget * wid, struct session *sess)
 	new = malloc (sizeof (struct slentry));
 	new->port = 6667;
 	new->flags = 0;
-	strcpy (new->comment, "New Server");
+	strcpy (new->comment, _("New Server"));
 	new->channel[0] = 0;
 	new->server[0] = 0;
 	new->password[0] = 0;
@@ -740,7 +754,7 @@ new_group_clicked (GtkWidget * wid, struct session *sess)
 	new = malloc (sizeof (struct slentry));
 	new->port = 0;
 	new->flags = 0;
-	strcpy (new->comment, "New Group");
+	strcpy (new->comment, _("New Group"));
 	new->channel[0] = 0;
 	strcpy (new->server, "SUB");
 	new->password[0] = 0;
@@ -819,30 +833,34 @@ channelbox_toggled (GtkWidget * igad, gpointer serv)
  * close_server_list:
  * self explainatory?
  */
-static int
-close_server_list (GtkWidget * win, int destroy)
+static void
+close_server_list (int dont_save)
 {
-	strcpy (prefs.nick1, gtk_entry_get_text ((GtkEntry *) nick1gad));
-	strcpy (prefs.nick2, gtk_entry_get_text ((GtkEntry *) nick2gad));
-	strcpy (prefs.nick3, gtk_entry_get_text ((GtkEntry *) nick3gad));
-	strcpy (prefs.username, gtk_entry_get_text ((GtkEntry *) usernamegad));
-	strcpy (prefs.realname, gtk_entry_get_text ((GtkEntry *) realnamegad));
-	save_serverentrys ();
-	if (destroy && slwin)
+	if (!dont_save)
+	{
+		strcpy (prefs.nick1, gtk_entry_get_text ((GtkEntry *) nick1gad));
+		strcpy (prefs.nick2, gtk_entry_get_text ((GtkEntry *) nick2gad));
+		strcpy (prefs.nick3, gtk_entry_get_text ((GtkEntry *) nick3gad));
+		strcpy (prefs.username, gtk_entry_get_text ((GtkEntry *) usernamegad));
+		strcpy (prefs.realname, gtk_entry_get_text ((GtkEntry *) realnamegad));
+		save_serverentrys ();
+	}
+
+	if (slwin)
 	{
 		gtk_widget_destroy (slwin);
 		if (sleditwin)
 			gtk_widget_destroy (sleditwin);
 		slwin = 0;
 		sleditwin = 0;
-		while (server_malloc_list)
-		{
-			free (server_malloc_list->data);
-			server_malloc_list = g_slist_remove (server_malloc_list, server_malloc_list->data);
-		}
 		sltree = 0;
 	}
-	return 0;
+
+	while (server_malloc_list)
+	{
+		free (server_malloc_list->data);
+		server_malloc_list = g_slist_remove (server_malloc_list, server_malloc_list->data);
+	}
 }
 
 /*
@@ -865,20 +883,17 @@ connect_auto (struct slentry *conn, struct session *sess)
 	safe_strcpy (nick, conn->nick, NICKLEN);
 	safe_strcpy (channel, conn->channel, CHANLEN);
 	port = conn->port;
+
 	if (!sess)
-	{
-		struct server *serv = new_server ();
-		if (prefs.use_server_tab)
-			sess = serv->front_session;
-		else
-			sess = new_session (serv, 0);
-	}
+		sess = new_ircwindow (NULL, NULL, SESS_SERVER);
+
 	strcpy (sess->willjoinchannel, channel);
 	strcpy (sess->server->password, password);
 	if (nick[0] == 0)
 		strcpy (sess->server->nick, prefs.nick1);
 	else
 		strcpy (sess->server->nick, nick);
+	sess->server->dont_use_proxy = (conn->flags & DONT_USE_PROXY) ? TRUE : FALSE;
 #ifdef USE_OPENSSL
 	sess->server->use_ssl = (conn->flags & USE_SSL) ? TRUE : FALSE;
 	sess->server->accept_invalid_cert =
@@ -917,14 +932,12 @@ autoconnect_parser (GtkCTree * ctree, GtkCTreeNode * node,
  * serverlist_autoconnect: [PUBLIC]
  * called from xchat.c if the user has selected that he/she wants autoconnect
  */
-void
-serverlist_autoconnect (struct session *sess, int close)
+static void
+serverlist_autoconnect (struct session *sess)
 {
 	first_autoconnect = 1;
 	gtk_ctree_pre_recursive (GTK_CTREE (sltree), NULL,
 									 GTK_CTREE_FUNC (autoconnect_parser), sess);
-	if (close)
-		close_server_list (0, TRUE);
 }
 
 /*
@@ -937,12 +950,7 @@ connectnew_clicked (GtkWidget * wid, struct session *sess)
 {
 	GtkCTreeNode *selected;
 	struct slentry *conn;
-	struct server *serv;
-	char channel[CHANLEN];
-	char server[132];
-	char password[86];
-	char nick[NICKLEN];
-	int port;
+	struct slentry myconn;
 
 	selected = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (sltree)->row_list,
 														GTK_CLIST (sltree)->focus_row));
@@ -952,35 +960,24 @@ connectnew_clicked (GtkWidget * wid, struct session *sess)
 	conn = gtk_ctree_node_get_row_data (GTK_CTREE (sltree), selected);
 	if (!(strlen (conn->server)))
 		return;
-	safe_strcpy (server, conn->server, sizeof (server));
-	safe_strcpy (password, conn->password, sizeof (password));
-	safe_strcpy (nick, conn->nick, NICKLEN);
-	safe_strcpy (channel, conn->channel, CHANLEN);
-	port = conn->port;
-	serv = new_server ();
-	if (serv)
-	{
-		close_server_list (0, TRUE);
-		if (prefs.use_server_tab)
-			sess = serv->front_session;
-		else
-			sess = new_session (serv, 0);
-		if (sess)
-		{
-			strcpy (sess->server->password, password);
-			strcpy (sess->willjoinchannel, channel);
-			if (nick[0] == 0)
-				strcpy (sess->server->nick, prefs.nick1);
-			else
-				strcpy (sess->server->nick, nick);
+	memcpy(&myconn, conn, sizeof(myconn));
+
+	sess = new_ircwindow (NULL, NULL, SESS_SERVER);
+
+	close_server_list (FALSE);
+	strcpy (sess->server->password, myconn.password);
+	strcpy (sess->willjoinchannel, myconn.channel);
+	if (myconn.nick[0] == 0)
+		strcpy (sess->server->nick, prefs.nick1);
+	else
+		strcpy (sess->server->nick, myconn.nick);
+	sess->server->dont_use_proxy = (myconn.flags & DONT_USE_PROXY) ? TRUE : FALSE;
 #ifdef USE_OPENSSL
-			sess->server->use_ssl = (conn->flags & USE_SSL) ? TRUE : FALSE;
-			sess->server->accept_invalid_cert =
-				(conn->flags & ACCEPT_INVALID_CERT) ? TRUE : FALSE;
+	sess->server->use_ssl = (myconn.flags & USE_SSL) ? TRUE : FALSE;
+	sess->server->accept_invalid_cert =
+				(myconn.flags & ACCEPT_INVALID_CERT) ? TRUE : FALSE;
 #endif
-			connect_server (sess, server, port, FALSE);
-		}
-	}
+	connect_server (sess, myconn.server, myconn.port, FALSE);
 }
 
 /*
@@ -993,25 +990,22 @@ connect_clicked (GtkWidget * wid, struct session *sess)
 {
 	GtkCTreeNode *selected;
 	struct slentry *conn;
-	char server[132];
-	char nick[NICKLEN];
-	int port;
-
-	if (sess->is_shell)
-	{
-		gtkutil_simpledialog ("You want to connect in a shell-tab eh? "
-										"Well I don't think so!\n\n"
-										"Try the CONNECT NEW button instead.");
-		return;
-	}
+	struct slentry myconn;
 
 	/* clueless user filter! */
 	if (!strcmp ("root", gtk_entry_get_text ((GtkEntry *) nick1gad)))
 	{
-		gtkutil_simpledialog ("Cannot use \"root\" as a nickname,\n"
-									 "please change that first.");
+		gtkutil_simpledialog (_("Cannot use \"root\" as a nickname,\n"
+									 "please change that first."));
 		return;
 	}
+
+	if (sess == NULL)
+	{
+		connectnew_clicked (wid, NULL);
+		return;
+	}
+
 	selected = GTK_CTREE_NODE (g_list_nth (GTK_CLIST (sltree)->row_list,
 														GTK_CLIST (sltree)->focus_row));
 	if (!selected)
@@ -1020,15 +1014,13 @@ connect_clicked (GtkWidget * wid, struct session *sess)
 	conn = gtk_ctree_node_get_row_data (GTK_CTREE (sltree), selected);
 	if (conn->server[0] == 0)
 		return;
-	safe_strcpy (server, conn->server, sizeof (server));
-	safe_strcpy (nick, conn->nick, NICKLEN);
+	memcpy(&myconn, conn, sizeof(myconn));
 	safe_strcpy (sess->server->password, conn->password, 85);
 	safe_strcpy (sess->willjoinchannel, conn->channel, CHANLEN);
-	port = conn->port;
 
-	close_server_list (0, TRUE);
+	close_server_list (FALSE);
 
-	if (nick[0] == 0)
+	if (myconn.nick[0] == 0)
 	{
 		if (strcmp (sess->server->nick, prefs.nick1))
 		{
@@ -1037,19 +1029,20 @@ connect_clicked (GtkWidget * wid, struct session *sess)
 		}
 	} else
 	{
-		if (strcmp (sess->server->nick, nick))
+		if (strcmp (sess->server->nick, myconn.nick))
 		{
-			fe_set_nick (sess->server, nick);
-			strcpy (sess->server->nick, nick);
+			fe_set_nick (sess->server, myconn.nick);
+			strcpy (sess->server->nick, myconn.nick);
 		}
 	}
 
+	sess->server->dont_use_proxy = (myconn.flags & DONT_USE_PROXY) ? TRUE : FALSE;
 #ifdef USE_OPENSSL
-	sess->server->use_ssl = (conn->flags & USE_SSL) ? TRUE : FALSE;
+	sess->server->use_ssl = (myconn.flags & USE_SSL) ? TRUE : FALSE;
 	sess->server->accept_invalid_cert =
-		(conn->flags & ACCEPT_INVALID_CERT) ? TRUE : FALSE;
+		(myconn.flags & ACCEPT_INVALID_CERT) ? TRUE : FALSE;
 #endif
-	connect_server (sess, server, port, FALSE);
+	connect_server (sess, myconn.server, myconn.port, FALSE);
 }
 
 /*
@@ -1124,32 +1117,45 @@ group_expand (GtkCTree * tree, GtkCTreeNode * node)
 	}
 }
 
+static int
+serverlist_close_cb (GtkWidget *wid, gpointer unused)
+{
+	close_server_list (FALSE);
+
+	if (sess_list == NULL)
+		xchat_exit ();
+
+	return 0;
+}
+
 /*
  * open_server_list [PUBLIC]
  * 
  * called by xchat.c to open the server list GUI
  *
  */
-void
-open_server_list (GtkWidget *widd, struct session *sess, int dontshow)
+int
+fe_open_serverlist (session *sess, int auto_connect, int dont_show)
 {
 	GtkWidget *scroll;
 	GtkWidget *vbox;
 	GtkWidget *hbox;
 	GtkWidget *wid1, *wid2, *wid3, *wid4;
+	int ret;
 
 	if (slwin)
 	{
 		gdk_window_show (slwin->window);
-		return;
+		return FALSE;
 	}
+
 	slwin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_title (GTK_WINDOW (slwin), "X-Chat: Server List");
+	gtk_window_set_title (GTK_WINDOW (slwin), _("X-Chat: Server List"));
 	gtk_signal_connect (GTK_OBJECT (slwin), "delete_event",
-							  GTK_SIGNAL_FUNC (close_server_list), 0);
+							  GTK_SIGNAL_FUNC (serverlist_close_cb), 0);
 	gtk_window_set_policy (GTK_WINDOW (slwin), TRUE, TRUE, FALSE);
 	gtk_window_set_wmclass (GTK_WINDOW (slwin), "servlist", "X-Chat");
-	gtk_widget_realize (slwin);
+	wins_set_icon (slwin);
 
 	gtk_container_border_width (GTK_CONTAINER (slwin), 5);
 	vbox = gtk_vbox_new (FALSE, 1);
@@ -1157,7 +1163,7 @@ open_server_list (GtkWidget *widd, struct session *sess, int dontshow)
 
 	/* NICKNAME ETC. */
 	/* boxes */
-	wid1 = gtk_frame_new ("User info:");
+	wid1 = gtk_frame_new (_("User info:"));
 
 	wid3 = gtk_hbox_new (0, 0);
 	gtk_widget_set_usize (wid3, 602, 0);
@@ -1175,7 +1181,7 @@ open_server_list (GtkWidget *widd, struct session *sess, int dontshow)
 	gtk_box_pack_start (GTK_BOX (wid2), hbox, FALSE, TRUE, 3);
 	gtk_widget_show (hbox);
 	/* nickboxes */
-	gtkutil_label_new ("Nicknames:", hbox);
+	gtkutil_label_new (_("Nicknames:"), hbox);
 	nick1gad = gtkutil_entry_new (63, hbox, 0, 0);
 	gtk_entry_set_text (GTK_ENTRY (nick1gad), prefs.nick1);
 	nick2gad = gtkutil_entry_new (63, hbox, 0, 0);
@@ -1186,10 +1192,10 @@ open_server_list (GtkWidget *widd, struct session *sess, int dontshow)
 	gtk_box_pack_start (GTK_BOX (wid2), hbox, FALSE, TRUE, 3);
 	gtk_widget_show (hbox);
 	/* nameboxes */
-	gtkutil_label_new ("Realname:", hbox);
+	gtkutil_label_new (_("Realname:"), hbox);
 	realnamegad = gtkutil_entry_new (63, hbox, 0, 0);
 	gtk_entry_set_text (GTK_ENTRY (realnamegad), prefs.realname);
-	gtkutil_label_new ("Username:", hbox);
+	gtkutil_label_new (_("Username:"), hbox);
 	usernamegad = gtkutil_entry_new (63, hbox, 0, 0);
 	gtk_entry_set_text (GTK_ENTRY (usernamegad), prefs.username);
 
@@ -1202,7 +1208,7 @@ open_server_list (GtkWidget *widd, struct session *sess, int dontshow)
 	scroll = gtk_scrolled_window_new (0, 0);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
 											  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-	gtk_widget_set_usize (GTK_WIDGET (scroll), 450, 250);
+	gtk_widget_set_usize (GTK_WIDGET (scroll), 450, 240);
 	gtk_box_pack_start (GTK_BOX (vbox), scroll, TRUE, TRUE, 0);
 	gtk_widget_show (scroll);
 	sltree = gtk_ctree_new (1, 0);
@@ -1241,13 +1247,20 @@ open_server_list (GtkWidget *widd, struct session *sess, int dontshow)
 
 	/* FIRST "NORMAL" BUTTON */
 	button_conn =
-		gtkutil_button (slwin, GNOME_STOCK_PIXMAP_REFRESH, "Connect",
+		gtkutil_button (slwin, GNOME_STOCK_PIXMAP_REFRESH, _("Connect"),
 							 connect_clicked, sess, wid1);
 
 	button_connnew =
-		gtkutil_button (slwin, GNOME_STOCK_PIXMAP_REFRESH, "Connect New",
+		gtkutil_button (slwin, GNOME_STOCK_PIXMAP_REFRESH, _("Connect New"),
 							 connectnew_clicked, sess, wid1);
 
+	wid4 = gtk_vseparator_new ();
+	gtk_box_pack_start (GTK_BOX (wid1), wid4, 0, 0, 2);
+	gtk_widget_show (wid4);
+
+	if (sess == NULL)
+		gtk_widget_hide (button_connnew);
+
 #ifdef USE_GNOME
 	/* VERY NICE GRAPHICAL BUTTON 1 */
 	wid4 = gtk_hbox_new (0, 0);
@@ -1259,7 +1272,7 @@ open_server_list (GtkWidget *widd, struct session *sess, int dontshow)
 	scroll = gtk_pixmap_new (pix_server, mask_server);
 	gtk_container_add (GTK_CONTAINER (wid4), scroll);
 	gtk_widget_show (scroll);
-	scroll = gtk_label_new ("New Server");
+	scroll = gtk_label_new (_("New Server"));
 	gtk_widget_show (scroll);
 	gtk_container_add (GTK_CONTAINER (wid4), scroll);
 	gtk_container_add (GTK_CONTAINER (wid3), wid4);
@@ -1275,7 +1288,7 @@ open_server_list (GtkWidget *widd, struct session *sess, int dontshow)
 	scroll = gtk_pixmap_new (pix_globe, mask_globe);
 	gtk_container_add (GTK_CONTAINER (wid4), scroll);
 	gtk_widget_show (scroll);
-	scroll = gtk_label_new ("New Group");
+	scroll = gtk_label_new (_("New Group"));
 	gtk_widget_show (scroll);
 	gtk_container_add (GTK_CONTAINER (wid4), scroll);
 	gtk_container_add (GTK_CONTAINER (wid3), wid4);
@@ -1283,15 +1296,15 @@ open_server_list (GtkWidget *widd, struct session *sess, int dontshow)
 
 #else
 
-	gtkutil_button (slwin, 0, "New Server", new_server_clicked, 0, wid1);
-	gtkutil_button (slwin, 0, "New Group", new_group_clicked, 0, wid1);
+	gtkutil_button (slwin, 0, _("New Server"), new_server_clicked, 0, wid1);
+	gtkutil_button (slwin, 0, _("New Group"), new_group_clicked, 0, wid1);
 
 #endif
 
 	/* 2 MORE "NORMAL" BUTTONS */
-	gtkutil_button (slwin, GNOME_STOCK_PIXMAP_CUT, "Delete",
+	gtkutil_button (slwin, GNOME_STOCK_PIXMAP_CUT, _("Delete"),
 						 delete_server_clicked, 0, wid1);
-	gtkutil_button (slwin, GNOME_STOCK_PIXMAP_PREFERENCES, "Edit",
+	gtkutil_button (slwin, GNOME_STOCK_PIXMAP_PREFERENCES, _("Edit"),
 						 edit_entry_clicked, 0, wid1);
 
 	gtk_widget_set_sensitive (button_conn, FALSE);
@@ -1301,20 +1314,21 @@ open_server_list (GtkWidget *widd, struct session *sess, int dontshow)
 	wid1 = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (wid2), wid1, FALSE, FALSE, 0);
 
-	wid3 = gtk_check_button_new_with_label ("Skip MOTD");
+	wid3 = gtk_check_button_new_with_label (_("Skip MOTD"));
 	gtk_signal_connect (GTK_OBJECT (wid3), "toggled",
 							  GTK_SIGNAL_FUNC (skip_motd), 0);
 	gtk_box_pack_start (GTK_BOX (wid1), wid3, TRUE, FALSE, 0);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid3), (prefs.skipmotd));
 	gtk_widget_show (wid3);
+	add_tip (wid3, _("Don't display the message-of-the-day when logging in"));
 
-	button_chan_new = gtk_check_button_new_with_label ("Show channels");
+	button_chan_new = gtk_check_button_new_with_label (_("Show channels"));
 	gtk_box_pack_start (GTK_BOX (wid1), button_chan_new, TRUE, FALSE, 0);
 	gtk_widget_show (button_chan_new);
 	gtk_signal_connect (GTK_OBJECT (button_chan_new), "toggled",
 							  GTK_SIGNAL_FUNC (channelbox_toggled), 0);
 
-	wid3 = gtk_check_button_new_with_label ("No ServerList on startup");
+	wid3 = gtk_check_button_new_with_label (_("No ServerList on startup"));
 	gtk_signal_connect (GTK_OBJECT (wid3), "toggled",
 							  GTK_SIGNAL_FUNC (no_servlist), 0);
 	gtk_box_pack_start (GTK_BOX (wid1), wid3, TRUE, FALSE, 0);
@@ -1336,6 +1350,17 @@ open_server_list (GtkWidget *widd, struct session *sess, int dontshow)
 	gtk_widget_show (hbox);
 	gtk_widget_show (vbox);
 
-	if (!dontshow)
+	ret = FALSE;
+	if (auto_connect && some_are_auto)
+	{
+		serverlist_autoconnect (sess);
+		ret = TRUE;
+	}
+
+	if (dont_show)
+		close_server_list (TRUE);
+	else
 		gtk_widget_show (slwin);
+
+	return ret;
 }
