@@ -1,7 +1,8 @@
 /* flags */
-#define	AUTOCONNECT		1
-#define	USE_SSL			2
-#define	ACCEPT_INVALID_CERT	4
+#define AUTOCONNECT				1
+#define USE_SSL					2
+#define ACCEPT_INVALID_CERT	4
+#define DONT_USE_PROXY			8
 
 struct slentry
 {
@@ -35,6 +36,7 @@ static struct defaultserv dserv[] =
 	{"", "eu.ircnet.org", "IRCNet Europe", 6667},
 	{"", "irc.ircd.it", "IRCNet Italy", 6667},
 	{"", "au.ircnet.org", "IRCNet Australia", 6667},
+	{"", "irc.stealth.net", "irc.stealth.net", 6660},
 	{"", "ENDSUB", "", 0},
 
 	{"", "SUB", "DALNet", 0},
@@ -47,6 +49,10 @@ static struct defaultserv dserv[] =
 	{"", "ca.rr.efnet.net", "EFNet Canada", 6667},
 	{"", "eu.rr.efnet.net", "EFNet Europe", 6667},
 	{"", "au.rr.efnet.net", "EFNet Australia", 6667},
+	{"", "irc.efnet.org", "irc.efnet.org", 6667},
+	{"", "irc.light.se", "irc.light.se", 6667},
+	{"", "irc.stanford.edu", "irc.stanford.edu", 6667},
+	{"", "irc.solidstreaming.net", "irc.solidstreaming.net", 6667},
 	{"", "ENDSUB", "", 0},
 
 	{"", "SUB", "GalaxyNet", 0},
@@ -85,6 +91,11 @@ static struct defaultserv dserv[] =
 
 	{"", "SUB", "FoxChat", 0},
 	{"", "irc.FoxChat.net", "irc.FoxChat.net", 6667},
+	{"", "irc.ac6.org", "irc.ac6.org", 6667},
+	{"", "beastie.ac6.org", "beastie.ac6.org", 6667},
+	{"", "wild.FoxChat.net", "wild.FoxChat.net", 6667},
+	{"", "roadkill.FoxChat.net", "roadkill.FoxChat.net", 6667},
+	{"", "slick.FoxChat.net", "slick.FoxChat.net", 6667},
 	{"", "ENDSUB", "", 0},
 
 	{"", "SUB", "IrcLink", 0},
@@ -96,11 +107,13 @@ static struct defaultserv dserv[] =
 	{"", "ENDSUB", "", 0},
 
 	{"", "SUB", "XWorld", 0},
-	{"#chatzone", "Minneapolis.MN.US.Xworld.Org", "Minneapolis.MN.US.Xworld.Org", 6667},
 	{"#chatzone", "Buffalo.NY.US.XWorld.org", "Buffalo.NY.US.XWorld.org", 6667},
-	{"#chatzone", "Oslo.NO.EU.XWorld.org", "Oslo.NO.EU.XWorld.org", 6667},
+	{"#chatzone", "Minneapolis.MN.US.Xworld.Org", "Minneapolis.MN.US.Xworld.Org", 6667},
+	{"#chatzone", "PalmSprings.CA.US.XWorld.Org", "PalmSprings.CA.US.XWorld.Org", 6667},
+	{"#chatzone", "Quebec.QC.CA.XWorld.Org", "Quebec.QC.CA.XWorld.Org", 6667},
+	{"#chatzone", "Rochester.NY.US.XWorld.org", "Rochester.NY.US.XWorld.org", 6667},
 	{"#chatzone", "Bayern.DE.EU.XWorld.Org", "Bayern.DE.EU.XWorld.Org", 6667},
-	{"#chatzone", "Berkeley.CA.US.XWorld.Org", "Berkeley.CA.US.XWorld.Org", 6667},
+	{"#chatzone", "Chicago.IL.US.XWorld.Org", "Chicago.IL.US.XWorld.Org", 6667},
 	{"", "ENDSUB", "", 0},
 
 	{"", "SUB", "ChatNet", 0},
@@ -122,10 +135,6 @@ static struct defaultserv dserv[] =
 	{"", "irc.magicstar.net", "irc.magicstar.net", 6667},
 	{"", "ENDSUB", "", 0},
 
-	{"", "SUB", "edu.ar", 0},
-	{"#linux", "galadriel.ubp.edu.ar", "galadriel.ubp.edu.ar", 6667},
-	{"", "ENDSUB", "", 0},
-
 	{"", "SUB", "SceneNet", 0},
 	{"", "irc.scene.org", "irc.scene.org", 6667},
 	{"", "irc.eu.scene.org", "irc.eu.scene.org", 6667},
@@ -151,9 +160,15 @@ static struct defaultserv dserv[] =
 	{"", "ENDSUB", "", 0},
 
 	{"", "SUB", "EUIrc", 0},
-	{"", "irc.hes.de.euirc.net", "irc.hes.de.euirc.net", 6667},
+	{"", "irc.euirc.net", "irc.euirc.net", 6667},
+	{"", "irc.ham.de.euirc.net", "irc.ham.de.euirc.net", 6667},
 	{"", "irc.ber.de.euirc.net", "irc.ber.de.euirc.net", 6667},
-	{"", "irc.bay.de.euirc.net", "irc.bay.de.euirc.net", 6667},
+	{"", "irc.ffm.de.euirc.net", "irc.ffm.de.euirc.net", 6667},
+	{"", "irc.bre.de.euirc.net", "irc.bre.de.euirc.net", 6667},
+	{"", "irc.hes.de.euirc.net", "irc.hes.de.euirc.net", 6667},
+	{"", "irc.vie.at.euirc.net", "irc.vie.at.euirc.net", 6667},
+	{"", "irc.inn.at.euirc.net", "irc.inn.at.euirc.net", 6667},
+	{"", "irc.bas.ch.euirc.net", "irc.bas.ch.euirc.net", 6667},
 	{"", "ENDSUB", "", 0},
 
 	{"", "SUB", "HabberNet", 0},
@@ -286,22 +301,6 @@ static struct defaultserv dserv[] =
 	{"", "irc.astrolink.org", "irc.astrolink.org", 6667}, 
 	{"", "ENDSUB", "", 0},
 
-	{"", "SUB", "Appleslice.net", 0},
-	{"", "flowers.appleslice.net", "flowers.appleslice.net", 6667}, 
-	{"", "koreen.appleslice.net", "koreen.appleslice.net", 6667}, 
-	{"", "irc.appleslice.net", "irc.appleslice.net", 6667}, 
-	{"", "ENDSUB", "", 0},
-
-	{"", "SUB", "NebulaNet", 0},
-	{"", "irc.NebulaNet.Org", "irc.NebulaNet.Org", 6667},
-	{"", "London.UK.EU.NebulaNet.Org", "London.UK.EU.NebulaNet.Org", 6667},
-	{"", "Boring.OR.US.NebulaNet.Org", "Boring.OR.US.NebulaNet.Org", 6667},
-	{"", "Denver.NebulaNet.Org", "Denver.NebulaNet.Org", 6667},
-	{"", "Stardust.NebulaNet.Org", "Stardust.NebulaNet.Org", 6667},
-	{"", "Xfusion.MA.US.NebulaNet.Org", "Xfusion.MA.US.NebulaNet.Org", 6667},
-	{"", "Anaheim.CA.US.NebulaNet.Org", "Anaheim.CA.US.NebulaNet.Org", 6667},
-	{"", "ENDSUB", "", 0},
-
 	{"", "SUB", "AxeNet", 0},
 	{"", "irc.axenet.org", "irc.axenet.org", 6667},
 	{"", "angel.axenet.org", "angel.axenet.org", 6667},
@@ -357,5 +356,46 @@ static struct defaultserv dserv[] =
 	{"", "bigcove.rebelchat.org", "bigcove.rebelchat.org", 6667},
 	{"", "ENDSUB", "", 0},
 
+	{"", "SUB", "DragonLynk", 0},
+	{"", "irc.dragonlynk.net", "DragonLynk", 6667},
+	{"", "ENDSUB", "", 0},
+
+	{"", "SUB", "Lunarnet IRC", 0},
+	{"", "isarania.lunarnetirc.org", "isarania.lunarnetirc.org", 6667},
+	{"", "ayerie.lunarnetirc.org", "ayerie.lunarnetirc.org", 6667},
+	{"", "ENDSUB", "", 0},
+
+	{"", "SUB", "CoolChat", 0},
+	{"", "irc.coolchat.net", "CoolChat: Random Server", 6667},
+	{"", "unix.coolchat.net", "CoolChat: Los Angeles, CA, US", 6667},
+	{"", "south.coolchat.net", "CoolChat: Atlanta, GA, US", 6667},
+	{"", "toronto.coolchat.net", "CoolChat: Toronto, ON, CA", 6667},
+	{"", "ENDSUB", "", 0},
+
+	{"", "SUB", "unsecurity.org", 0},
+	{"", "irc.unsecurity.org", "irc.unsecurity.org", 6667},
+	{"", "wc.unsecurity.org", "wc.unsecurity.org", 6667},
+	{"", "thegift.unsecurity.org", "thegift.unsecurity.org", 6667},
+	{"", "sysgate.unsecurity.org", "sysgate.unsecurity.org", 6667},
+	{"", "ENDSUB", "", 0},
+
+	{"", "SUB", "QChat.net", 0},
+	{"", "irc.qchat.net", "irc.qchat.net", 6667},
+	{"", "ENDSUB", "", 0},
+
+	{"", "SUB", "SubCultNet", 0},
+	{"", "irc.subcult.ch", "irc.subcult.ch", 6667},
+	{"", "irc.phuncrew.ch", "irc.phuncrew.ch", 6668},
+	{"", "irc.mgz.ch", "irc.mgz.ch", 6667},
+	{"", "ENDSUB", "", 0},
+
+	{"", "SUB", "AbleNET", 0},
+	{"", "california.ablenet.org", "AbleNET California", 6667},
+	{"", "amazon.ablenet.org", "AbleNET Tulsa, OK", 6667},
+	{"", "agora.ablenet.org", "AbleNET Portland, Oregon", 6667},
+	{"", "extreme.ablenet.org", "AbleNET Texus", 6667},
+	{"", "irc.ablenet.org", "AbleNET Random Server", 6667},
+	{"", "ENDSUB", "", 0},
+
 	{0, 0, 0, 0}
 };
