@@ -20,40 +20,96 @@
 #include <stdlib.h>
 #include <fcntl.h>
 #include <string.h>
+
+#include "fe-gtk.h"
 #include "../common/xchat.h"
 #include "../common/cfgfiles.h"
 #include "../common/fe.h"
 #include "../common/util.h"
-#include "fe-gtk.h"
+#include "../common/outbound.h"
+#include "../common/notify.h"
+#include "../common/xchatc.h"
 #include "settings.h"
 #include "gtkutil.h"
 #include "xtext.h"
+#include "palette.h"
+#include "maingui.h"
+#include "menu.h"
 #include "pixmaps.h"
 
-extern struct xchatprefs prefs;
-extern GdkColor colors[];
-extern GdkFont *font_normal;
-extern GdkFont *dialog_font_normal;
-extern GSList *sess_list;
-extern gint notify_tag;
-extern GdkPixmap *channelwin_pix;
-extern GdkPixmap *dialogwin_pix;
-extern GtkWidget *main_window, *main_book;
 
-extern void maingui_set_tab_pos (int);
-extern int load_trans_table (char *full_path);
-extern void add_tip (GtkWidget *, char *);
-extern GdkFont *my_font_load (char *fontname);
-extern void maingui_create_textlist (struct session *sess,
-												 GtkWidget * leftpane);
-extern void size_request_textgad (GtkWidget * win, GtkRequisition * req);
-extern void notify_checklist (void);
-extern void path_part (char *file, char *path);
-extern GtkWidget *menu_quick_item_with_callback (void *callback, char *label,
-																 GtkWidget * menu, void *arg);
+struct setup
+{
+	GtkWidget *settings_window;
+	GtkWidget *nu_color;
+	GtkWidget *bt_color;
+
+	GtkWidget *font_normal;
+	GtkWidget *font_bold;
+
+	GtkWidget *dialog_font_normal;
+	GtkWidget *dialog_font_bold;
+
+	GtkWidget *entry_stamp_format;
+	GtkWidget *entry_bluestring;
+	GtkWidget *entry_doubleclickuser;
+	GtkWidget *entry_max_lines;
+	GtkWidget *entry_nick_suffix;
+	GtkWidget *entry_quit;
+	GtkWidget *entry_part;
+	GtkWidget *entry_away;
+	GtkWidget *entry_timeout;
+	GtkWidget *entry_dccdir;
+	GtkWidget *entry_dcctimeout;
+	GtkWidget *entry_dccstalltimeout;
+	GtkWidget *entry_permissions;
+	GtkWidget *entry_dcc_blocksize;
+	GtkWidget *entry_dnsprogram;
+	GtkWidget *entry_recon_delay;
+
+	GtkWidget *entry_proxy_host;
+	GtkWidget *entry_proxy_port;
+
+	GtkWidget *entry_dcc_send_port_first;
+	GtkWidget *entry_dcc_send_port_last;
+
+	GtkWidget *entry_sounddir;
+	GtkWidget *entry_soundcmd;
+
+	GtkWidget *entry_mainw_left;
+	GtkWidget *entry_mainw_top;
+	GtkWidget *entry_mainw_width;
+	GtkWidget *entry_mainw_height;
+
+	GtkWidget *entry_hostname;
+	GtkWidget *entry_dcc_ip_str;
+
+	GtkWidget *background;
+	GtkWidget *background_dialog;
+
+	GtkWidget *check_transparent;
+	GtkWidget *check_tint;
+	GtkWidget *check_detecthost;
+	GtkWidget *check_detectip;
+	GtkWidget *check_ip;
+
+	GtkWidget *entry_trans_file;
+
+	GtkWidget *dialog_check_transparent;
+	GtkWidget *dialog_check_tint;
+
+	GtkWidget *cancel_button;
+
+	GtkWidget *logmask_entry;
+
+	struct xchatprefs prefs;
+};
+
 
 static GtkWidget *fontdialog = 0;
 
+extern GtkStyle *create_inputgad_style (void);
+
 
 /* Generic interface creation functions */
 
@@ -418,29 +474,38 @@ settings_fontok (GtkWidget * ok_button, GtkWidget * entry)
 			gtk_entry_set_text (GTK_ENTRY (entry), fontname);
 	}
 	gtk_widget_destroy (fontdialog);
-	fontdialog = 0;
+}
+
+static void
+settings_fontdialogdestroy (GtkWidget * dialog, gpointer unused)
+{
+	fontdialog = NULL;
 }
 
 static void
 settings_openfontdialog (GtkWidget * button, GtkWidget * entry)
 {
-	GtkWidget *dialog;
+	GtkFontSelectionDialog *dialog;
 
 	if (fontdialog)
-		gtk_widget_destroy (fontdialog);
+	{
+		gdk_window_show (GTK_WIDGET (fontdialog)->window);
+		return;
+	}
 
-	dialog = gtk_font_selection_dialog_new (_("Select Font"));
-	gtk_signal_connect
-		(GTK_OBJECT (GTK_FONT_SELECTION_DIALOG (dialog)->ok_button), "clicked",
-		 GTK_SIGNAL_FUNC (settings_fontok), entry);
-	gtk_signal_connect
-		(GTK_OBJECT (GTK_FONT_SELECTION_DIALOG (dialog)->cancel_button),
-		 "clicked", GTK_SIGNAL_FUNC (gtkutil_destroy), dialog);
-	gtk_font_selection_dialog_set_font_name
-		(GTK_FONT_SELECTION_DIALOG (dialog),
-		 gtk_entry_get_text (GTK_ENTRY (entry)));
-	fontdialog = dialog;
-	gtk_widget_show (dialog);
+	dialog = (GtkFontSelectionDialog *)
+				gtk_font_selection_dialog_new (_("Select Font"));
+	gtk_signal_connect (GTK_OBJECT (dialog->ok_button), "clicked",
+							  GTK_SIGNAL_FUNC (settings_fontok), entry);
+	gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
+							  GTK_SIGNAL_FUNC (settings_fontdialogdestroy), 0);
+	gtk_signal_connect (GTK_OBJECT (dialog->cancel_button), "clicked",
+							  GTK_SIGNAL_FUNC (gtkutil_destroy), dialog);
+	gtk_font_selection_dialog_set_font_name (dialog,
+							  gtk_entry_get_text (GTK_ENTRY (entry)));
+	gtk_widget_show (GTK_WIDGET (dialog));
+
+	fontdialog = (GtkWidget *)dialog;
 }
 
 static void
@@ -462,10 +527,12 @@ settings_ok_clicked (GtkWidget * wid, struct session *sess)
 	int fontdialogchange = FALSE;
 	struct session *s;
 	GdkPixmap *old_pix;
+	GtkStyle *new_style;
 	GSList *list;
 
 	/* get all the spinbutton and GtkEntry values into our structure */
 
+	GETSTR (sess->setup->prefs.dcc_ip_str, entry_dcc_ip_str);
 	GETSTR (sess->setup->prefs.hostname, entry_hostname);
 	GETSTR (sess->setup->prefs.proxy_host, entry_proxy_host);
 	sess->setup->prefs.proxy_port = GETINT (entry_proxy_port);
@@ -488,6 +555,7 @@ settings_ok_clicked (GtkWidget * wid, struct session *sess)
 	GETSTR (sess->setup->prefs.partreason, entry_part);
 	GETSTR (sess->setup->prefs.logmask, logmask_entry);
 	GETSTR (sess->setup->prefs.nick_suffix, entry_nick_suffix);
+	GETSTR (sess->setup->prefs.stamp_format, entry_stamp_format);
 
 	sess->setup->prefs.max_lines = GETINT (entry_max_lines);
 	sess->setup->prefs.notify_timeout = GETINT (entry_timeout);
@@ -516,7 +584,7 @@ settings_ok_clicked (GtkWidget * wid, struct session *sess)
 	{
 		if (load_trans_table (sess->setup->prefs.trans_file) == 0)
 		{
-			gtkutil_simpledialog ("Failed to load translation table.");
+			gtkutil_simpledialog (_("Failed to load translation table."));
 			sess->setup->prefs.use_trans = 0;
 		}
 	}
@@ -524,13 +592,25 @@ settings_ok_clicked (GtkWidget * wid, struct session *sess)
 
 	if (sess->setup->prefs.notify_timeout != prefs.notify_timeout)
 	{
-		if (notify_tag != -1)
-			gtk_timeout_remove (notify_tag);
+		if (notify_tag)
+		{
+			fe_timeout_remove (notify_tag);
+			notify_tag = 0;
+		}
 		if (sess->setup->prefs.notify_timeout)
 			notify_tag =
-				gtk_timeout_add (sess->setup->prefs.notify_timeout * 1000,
-									  (GtkFunction) notify_checklist, 0);
+				fe_timeout_add (sess->setup->prefs.notify_timeout * 1000,
+									 notify_checklist, 0);
 	}
+
+	if (prefs.use_fontset != sess->setup->prefs.use_fontset)
+	{
+		/* force font reloading */
+		prefs.font_normal[0] = 0;
+		prefs.dialog_font_normal[0] = 0;
+		prefs.use_fontset = sess->setup->prefs.use_fontset;
+	}
+
 	if (strcmp (prefs.font_normal, sess->setup->prefs.font_normal) != 0)
 	{
 		gdk_font_unref (font_normal);
@@ -574,11 +654,11 @@ settings_ok_clicked (GtkWidget * wid, struct session *sess)
 
 	old_pix = channelwin_pix;
 	list = sess_list;
-	channelwin_pix = pixmap_load_from_file_with_msg (prefs.background);
+	channelwin_pix = pixmap_load_from_file (prefs.background);
 	while (list)
 	{
 		s = (struct session *) list->data;
-		if (!s->is_dialog && !s->is_shell)
+		if (s->type != SESS_DIALOG && s->type != SESS_SHELL)
 		{
 			GTK_XTEXT (s->gui->textgad)->tint_red = prefs.tint_red;
 			GTK_XTEXT (s->gui->textgad)->tint_green = prefs.tint_green;
@@ -617,11 +697,11 @@ settings_ok_clicked (GtkWidget * wid, struct session *sess)
 
 	old_pix = dialogwin_pix;
 	list = sess_list;
-	dialogwin_pix = pixmap_load_from_file_with_msg (prefs.background_dialog);
+	dialogwin_pix = pixmap_load_from_file (prefs.background_dialog);
 	while (list)
 	{
 		s = (struct session *) list->data;
-		if (s->is_dialog)
+		if (s->type == SESS_DIALOG)
 		{
 			GTK_XTEXT (s->gui->textgad)->tint_red = prefs.dialog_tint_red;
 			GTK_XTEXT (s->gui->textgad)->tint_green = prefs.dialog_tint_green;
@@ -658,13 +738,34 @@ settings_ok_clicked (GtkWidget * wid, struct session *sess)
 	if (old_pix)
 		gdk_pixmap_unref (old_pix);
 
+	/* update the font in inputgad_style */
+	if (fontchange && prefs.style_inputbox)
+	{
+		new_style = create_inputgad_style ();
+		list = sess_list;
+		while (list)
+		{
+			s = list->data;
+			switch (s->type)
+			{
+			case SESS_CHANNEL:
+			case SESS_DIALOG:
+			case SESS_SERVER:
+				gtk_widget_set_style (s->gui->inputgad, new_style);
+			}
+			list = list->next;
+		}
+		gtk_style_unref (inputgad_style);
+		inputgad_style = new_style;
+	}
+
 	if (wid)
 		gtk_widget_destroy (sess->setup->settings_window);
 	else
 		gtk_widget_set_sensitive (sess->setup->cancel_button, FALSE);
 
 	if (noapply)
-		gtkutil_simpledialog ("The following prefs do not take effect\n"
+		gtkutil_simpledialog (_("The following prefs do not take effect\n"
 									 "immediately, you will have to close the\n"
 									 "window and re-open it:\n\n"
 									 " - Channel Tabs\n"
@@ -673,7 +774,7 @@ settings_ok_clicked (GtkWidget * wid, struct session *sess)
 									 " - Disable Paned Userlist\n"
 									 " - Notify User color\n"
 									 " - Layout for a vertical panel\n"
-									 " - Auto Indent");
+									 " - Auto Indent"));
 }
 
 static void
@@ -833,7 +934,6 @@ static void
 settings_create_window_page (GtkWidget * vvbox,
 									  GtkWidget ** font_normal_wid,
 									  char *font_normal_str,
-									  GdkFont * font_nn,
 									  GtkWidget ** backgroundwid,
 									  char *background,
 									  int transparent,
@@ -858,6 +958,9 @@ settings_create_window_page (GtkWidget * vvbox,
 	settings_create_entry (_("Font:"), FONTNAMELEN, table, font_normal_str,
 								  font_normal_wid, 0, settings_openfontdialog,
 								  row_index++);
+#ifndef PATH_MAX
+#define PATH_MAX 4096
+#endif
 #ifdef USE_GDK_PIXBUF
 	settings_create_entry (_("Background Picture:"), PATH_MAX, table,
 								  background, backgroundwid, 0,
@@ -894,10 +997,14 @@ settings_create_window_page (GtkWidget * vvbox,
 	*tintwid = settings_create_toggle (_("Tint Transparency"), right_box, tint,
 												  settings_ptoggle_check, tint_pref);
 	add_tip (*tintwid, _("Tint the see-through text box to make it darker"));
-	settings_create_toggle (_("Use gdk_fontset_load"),
+	settings_create_toggle (_("Use a font set"),
 									left_box, prefs.use_fontset, settings_ptoggle_check,
 									&(sess->setup->prefs.use_fontset));
 
+#ifndef USE_XLIB
+	gtk_widget_set_sensitive (*transwid, FALSE);
+#endif
+
 #ifdef USE_GDK_PIXBUF
 	if (!transparent)
 #endif
@@ -971,6 +1078,9 @@ settings_page_interface (struct session *sess, GtkWidget * vbox)
 											prefs.paned_userlist, settings_ptoggle_check,
 											&(sess->setup->prefs.paned_userlist));
 	add_tip (tog, _("Use paned user list instead of a fixed width one"));
+	tog = settings_create_toggle (_("Userlist icons"), wid,
+											prefs.userlist_icons, settings_ptoggle_check,
+											&(sess->setup->prefs.userlist_icons));
 	tab = settings_create_table (wid);
 	row_index = 0;
 	settings_create_entry (_("Double Click Command:"),
@@ -1030,10 +1140,10 @@ settings_page_interface_inout (struct session *sess, GtkWidget * vbox)
 
 	wid = settings_create_group (vbox, _("Output Box"));
 	settings_create_tworows (wid, &left_box, &right_box);
-	tog = settings_create_toggle (_("Time Stamp All Text"), right_box,
-											prefs.timestamp, settings_ptoggle_check,
-											&(sess->setup->prefs.timestamp));
-	add_tip (tog, _("Prefix all text with the current time stamp"));
+	tog = settings_create_toggle (_("Strip MIRC Color"), right_box,
+											prefs.stripcolor, settings_ptoggle_check,
+											&(sess->setup->prefs.stripcolor));
+	add_tip (tog, _("Strip MIRC color codes from text before displaying"));
 	tog = settings_create_toggle (_("Colored Nicks"), left_box,
 											prefs.colorednicks, settings_ptoggle_check,
 											&(sess->setup->prefs.colorednicks));
@@ -1043,21 +1153,34 @@ settings_page_interface_inout (struct session *sess, GtkWidget * vbox)
 											settings_ptoggle_check,
 											&(sess->setup->
 											  prefs.show_invite_in_front_session));
-
-	tog = settings_create_toggle (_("Strip MIRC Color"), left_box,
-											prefs.stripcolor, settings_ptoggle_check,
-											&(sess->setup->prefs.stripcolor));
-	add_tip (tog, _("Strip MIRC color codes from text before displaying"));
+	tog = settings_create_toggle (_("Time Stamp All Text"), left_box,
+											prefs.timestamp, settings_ptoggle_check,
+											&(sess->setup->prefs.timestamp));
+	add_tip (tog, _("Prefix all text with the current time stamp"));
 	tog = settings_create_toggle (_("Filter out BEEPs"), right_box,
 											prefs.filterbeep, settings_ptoggle_check,
 											&(sess->setup->prefs.filterbeep));
 	add_tip (tog, _("Remove ^G BEEP codes from text before displaying"));
 
+	tab = settings_create_table (wid);
+	entry = settings_create_entry (_("Time Stamp Format:"), 63, tab,
+											 prefs.stamp_format,
+											 &sess->setup->entry_stamp_format,
+											 0, 0, 0);
+
 	wid = settings_create_group (vbox, _("Buffer Settings"));
 	tab = settings_create_table (wid);
 	sess->setup->entry_max_lines =
 		settings_create_numberentry (_("Text Buffer Size:"), 0, 10000, 1, tab,
 								prefs.max_lines, 0, _("lines (0=Unlimited)."));
+#ifdef USE_JCODE
+	wid = settings_create_group (vbox, _("Other Settings"));
+    tog = settings_create_toggle (_("Kanji locale <=> JIS translation"),
+                                  wid,
+                                  prefs.kanji_conv, settings_ptoggle_check,
+                                  &(sess->setup->prefs.kanji_conv));
+        
+#endif
 }
 
 static void
@@ -1067,11 +1190,26 @@ settings_tabsset (GtkWidget * item, int num)
 	sess->setup->prefs.tabs_position = num;
 }
 
+static void
+settings_lagmeterset (GtkWidget * item, int num)
+{
+	session *sess = gtk_object_get_user_data (GTK_OBJECT (item->parent));
+	sess->setup->prefs.lagometer = num;
+}
+
+static void
+settings_throttlemeterset (GtkWidget * item, int num)
+{
+	session *sess = gtk_object_get_user_data (GTK_OBJECT (item->parent));
+	sess->setup->prefs.throttlemeter = num;
+}
+
 static void
 settings_page_interface_layout (struct session *sess, GtkWidget * vbox)
 {
 	GtkWidget *wid;
 	GtkWidget *tog, *tab;
+	GtkWidget *left_box, *right_box;
 	static char *tabmenutext[] = 
 	{
 		N_("Bottom"),
@@ -1081,34 +1219,59 @@ settings_page_interface_layout (struct session *sess, GtkWidget * vbox)
 		N_("Hidden"),
 		NULL
 	};
+	static char *metermenutext[] = 
+	{
+		N_("Off"),
+		N_("Progress bar"),
+		N_("Info text"),
+		N_("Both"),
+		NULL
+	};
 
 	wid = settings_create_group (vbox, _("Buttons"));
-	tog = settings_create_toggle (_("Channel Mode Buttons"), wid,
+	settings_create_tworows (wid, &left_box, &right_box);
+	tog = settings_create_toggle (_("Channel Mode Buttons"), right_box,
 											prefs.chanmodebuttons,
 											settings_ptoggle_check,
 											&(sess->setup->prefs.chanmodebuttons));
 	add_tip (tog, _("Show the TNSIPMLK buttons"));
-	tog = settings_create_toggle (_("User List Buttons"), wid,
+	tog = settings_create_toggle (_("User List Buttons"), left_box,
 											prefs.userlistbuttons,
 											settings_ptoggle_check,
 											&(sess->setup->prefs.userlistbuttons));
 	add_tip (tog, _("Show the buttons below the user list"));
 
+	wid = settings_create_group (vbox, _("Meters"));
+	settings_create_tworows (wid, &left_box, &right_box);
+	tab = settings_create_table (left_box);
+	settings_create_optmenu (tab, _("Lag meter:"), metermenutext,
+								settings_lagmeterset, sess, 0, prefs.lagometer);
+	tab = settings_create_table (right_box);
+	settings_create_optmenu (tab, _("Throttle meter:"), metermenutext,
+								settings_throttlemeterset, sess, 0, prefs.throttlemeter);
+
+
 	wid = settings_create_group (vbox, _("Tabs"));
 	tab = settings_create_table (wid);
 
 	settings_create_optmenu (tab, _("Tabs Located at:"), tabmenutext,
 								settings_tabsset, sess, 0, prefs.tabs_position);
 
-	tog = settings_create_toggle (_("New Tabs to front"), wid,
-											prefs.newtabstofront, settings_ptoggle_check,
-											&(sess->setup->prefs.newtabstofront));
-	add_tip (tog, _("Bring new query/channel tabs to front"));
-	tog = settings_create_toggle (_("Channel Tabs"), wid,
+	settings_create_tworows (wid, &left_box, &right_box);
+
+	tog = settings_create_toggle (_("Channel Tabs"), right_box,
 											prefs.tabchannels, settings_ptoggle_check,
 											&(sess->setup->prefs.tabchannels));
 	add_tip (tog, _("Use tabs for channels instead of separate windows"));
-	tog = settings_create_toggle (_("Private Message Tabs"), wid,
+	tog = settings_create_toggle (_("Limited Tab Highlighting"), left_box,
+											prefs.limitedtabhighlight, settings_ptoggle_check,
+											&(sess->setup->prefs.limitedtabhighlight));
+	add_tip (tog, _("Only highlight channel tabs for channel messages and actions"));
+	tog = settings_create_toggle (_("New Tabs to front"), left_box,
+											prefs.newtabstofront, settings_ptoggle_check,
+											&(sess->setup->prefs.newtabstofront));
+	add_tip (tog, _("Bring new query/channel tabs to front"));
+	tog = settings_create_toggle (_("Private Message Tabs"), right_box,
 											prefs.privmsgtab, settings_ptoggle_check,
 											&(sess->setup->prefs.privmsgtab));
 	add_tip (tog, _("Use tabs for /query instead of separate windows"));
@@ -1117,16 +1280,13 @@ settings_page_interface_layout (struct session *sess, GtkWidget * vbox)
 										("Use a separate tab/window for server messages"),
 wid, prefs.use_server_tab, settings_ptoggle_check,
 &(sess->setup->prefs.use_server_tab));
+	tog = settings_create_toggle (_("Use a separate tab/window for notices and server notices"),
+		wid, prefs.notices_tabs, settings_ptoggle_check, &(sess->setup->prefs.notices_tabs));
 	tog =
 		settings_create_toggle (_
 										("Use tabs for DCC, Ignore, Notify etc windows."),
 wid, prefs.windows_as_tabs, settings_ptoggle_check,
 &(sess->setup->prefs.windows_as_tabs));
-
-	tog = settings_create_toggle (_("Limited Tab Highlighting"), wid,
-											prefs.limitedtabhighlight, settings_ptoggle_check,
-											&(sess->setup->prefs.limitedtabhighlight));
-	add_tip (tog, _("Only highlight channel tabs for channel messages and actions"));
 }
 
 static void
@@ -1168,7 +1328,6 @@ settings_page_interface_channelwindow (struct session *sess, GtkWidget * vbox)
 	settings_create_window_page (vbox,
 										  &sess->setup->font_normal,
 										  prefs.font_normal,
-										  font_normal,
 										  &sess->setup->background,
 										  prefs.background,
 										  prefs.transparent,
@@ -1192,7 +1351,6 @@ settings_page_interface_dialogwindow (struct session *sess, GtkWidget * vbox)
 	settings_create_window_page (vbox,
 										  &sess->setup->dialog_font_normal,
 										  prefs.dialog_font_normal,
-										  dialog_font_normal,
 										  &sess->setup->background_dialog,
 										  prefs.background_dialog,
 										  prefs.dialog_transparent,
@@ -1250,6 +1408,10 @@ settings_page_irc (struct session *sess, GtkWidget * vbox)
 											prefs.beepmsg, settings_ptoggle_check,
 											&(sess->setup->prefs.beepmsg));
 	add_tip (tog, _("Beep when a private message for you is received"));
+	tog = settings_create_toggle (_("Beep on Channel Messages"), left_box,
+											prefs.beepchans, settings_ptoggle_check,
+											&(sess->setup->prefs.beepchans));
+	add_tip (tog, _("Beep when a channel message is received"));
 	tog = settings_create_toggle (_("Send /who #chan on join."), right_box,
 											prefs.userhost, settings_ptoggle_check,
 											&(sess->setup->prefs.userhost));
@@ -1306,6 +1468,18 @@ settings_page_irc_ipaddress (struct session *sess, GtkWidget * vbox)
 								"usefull for machines with multiple addresses."),
 							 wid);
 
+	wid = settings_create_group (vbox, _("Public IP Address"));
+	tab = settings_create_table (wid);
+	row_index = 0;
+
+	settings_create_entry (_("DCC IP Address:"),
+								  sizeof (prefs.hostname) - 1, tab, prefs.dcc_ip_str,
+								  &sess->setup->entry_dcc_ip_str, 0, 0, row_index++);
+
+	gtkutil_label_new (_("You may need to set the DCC IP Address if\n"
+			     "you're behind a NAT router or proxy firewall."),
+							 wid);
+
 	tog = sess->setup->check_ip = settings_create_toggle
 		(_("Get my IP from Server (for use in DCC Send only)"), wid,
 		 prefs.ip_from_server, settings_ptoggle_check,
@@ -1332,6 +1506,7 @@ settings_page_proxy (struct session *sess, GtkWidget * vbox)
 		N_("Wingate"),
 		N_("Socks4"),
 		N_("Socks5"),
+		N_("HTTP"),
 		NULL
 	};
 
@@ -1513,8 +1688,9 @@ static void
 settings_page_dcc_filetransfer (struct session *sess, GtkWidget * vbox)
 {
 	GtkWidget *wid;
-	GtkWidget *tab;
-	GtkWidget *tog;
+	GtkWidget *tab, *tog;
+	GtkWidget *left_box, *right_box;
+
 	gint row_index;
 	char buf[127];
 
@@ -1547,16 +1723,24 @@ settings_page_dcc_filetransfer (struct session *sess, GtkWidget * vbox)
 
 	wid = settings_create_group (vbox, _("DCC Send Options"));
 	tab = settings_create_table (wid);
-	tog = settings_create_toggle (_("Fast DCC Send"), wid,
+
+	settings_create_tworows (wid, &left_box, &right_box);
+
+	tog = settings_create_toggle (_("Fast DCC Send"), left_box,
 											prefs.fastdccsend, settings_ptoggle_check,
 											&(sess->setup->prefs.fastdccsend));
 	add_tip (tog, _("Don\'t wait for ACKs to send more data"));
 
+	tog = settings_create_toggle (_("Fill Spaces"), right_box,
+						 					prefs.dcc_send_fillspaces, settings_ptoggle_check,
+											&(sess->setup->prefs.dcc_send_fillspaces));
+	add_tip (tog, _("File names with spaces will be filled with underscore"));	
+
 	sess->setup->entry_dcc_send_port_first =
-		settings_create_numberentry (_("First DCC Send Port:"), 0, 10000, 1, tab,
+		settings_create_numberentry (_("First DCC Send Port:"), 0, 65535, 1, tab,
 								prefs.first_dcc_send_port, row_index++, _("(0=Disabled)"));
 	sess->setup->entry_dcc_send_port_last =
-		settings_create_numberentry (_("Last DCC Send Port:"), 0, 10000, 1, tab,
+		settings_create_numberentry (_("Last DCC Send Port:"), 0, 65535, 1, tab,
 								prefs.last_dcc_send_port, row_index++, 0);
 
 	sess->setup->entry_dcc_blocksize =
@@ -1683,7 +1867,7 @@ settings_opengui (struct session *sess)
 	ctree = gtk_ctree_new_with_titles (1, 0, titles);
 	gtk_clist_set_selection_mode (GTK_CLIST (ctree), GTK_SELECTION_BROWSE);
 	gtk_ctree_set_indent (GTK_CTREE (ctree), 15);
-	gtk_widget_set_usize (ctree, 140, 0);
+	gtk_widget_set_usize (ctree, 164, 0);
 	gtk_box_pack_start (GTK_BOX (hbox), ctree, FALSE, FALSE, 0);
 	gtk_widget_show (ctree);
 
