@@ -24,29 +24,19 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+
+#include "fe-gtk.h"
 #include "../common/xchat.h"
+#include "../common/xchatc.h"
 #include "../common/cfgfiles.h"
 #include "../common/plugin.h"
-#include "fe-gtk.h"
+#include "../common/outbound.h"
+#include "../common/text.h"
 #include "gtkutil.h"
 #include "xtext.h"
-
-extern GdkColor colors[];
-extern GdkFont *font_normal;
-extern GdkPixmap *channelwin_pix;
-extern struct xchatprefs prefs;
-
-extern int text_event (int i);
-extern void pevent_make_pntevts ();
-extern int pevent_load (char *filename);
-extern void pevent_dialog_save (char *fn);
-extern void check_special_chars (char *, int perc_ascii);
-extern int buf_get_line (char *, char **, int *, int len);
-extern void show_and_unfocus (GtkWidget *);
-#ifdef USE_PERL
-extern int perl_print (char *cmd, struct session *sess, char *b, char *c,
-							  char *d, char *e);
-#endif
+#include "wins.h"
+#include "palette.h"
+#include "textgui.h"
 
 struct text_event
 {
@@ -65,17 +55,10 @@ static GtkWidget *pevent_dialog = NULL, *pevent_dialog_twid,
 	*pevent_dialog_entry, *pevent_dialog_sound_entry,
 	*pevent_dialog_list, *pevent_dialog_hlist;
 
-int pevt_build_string (char *input, char **output, int *max_arg);
-
-static void
-timecpy (char *buf)
+int
+get_stamp_str (time_t tim, char *dest, int size)
 {
-	time_t timval = time (0);
-	char *tim = ctime (&timval) + 10;
-	memcpy (buf, tim, 9);
-	buf[0] = '[';
-	buf[9] = ']';
-	buf[10] = ' ';
+	return strftime (dest, size, prefs.stamp_format, localtime (&tim));
 }
 
 static void
@@ -84,9 +67,6 @@ PrintTextLine (GtkWidget * textwidget, unsigned char *text, int len, int indent)
 	char *tab, *new_text;
 	int leftlen;
 
-	if (len == -1)
-		len = strlen (text);
-
 	if (len == 0)
 		len = 1;
 
@@ -94,10 +74,14 @@ PrintTextLine (GtkWidget * textwidget, unsigned char *text, int len, int indent)
 	{
 		if (prefs.timestamp)
 		{
-			new_text = malloc (len + 12);
-			timecpy (new_text);
-			memcpy (new_text + 11, text, len);
-			gtk_xtext_append (GTK_XTEXT (textwidget), new_text, len + 11);
+			char buf[64];
+			int stamp_size;
+
+			stamp_size = get_stamp_str (time (0), buf, sizeof (buf));
+			new_text = malloc (len + stamp_size + 1);
+			memcpy (new_text, buf, stamp_size);
+			memcpy (new_text + stamp_size, text, len);
+			gtk_xtext_append (GTK_XTEXT (textwidget), new_text, len + stamp_size);
 			free (new_text);
 		} else
 			gtk_xtext_append (GTK_XTEXT (textwidget), text, len);
@@ -117,39 +101,41 @@ PrintTextLine (GtkWidget * textwidget, unsigned char *text, int len, int indent)
 void
 PrintTextRaw (GtkWidget * textwidget, unsigned char *text, int indent)
 {
-	char *cr;
+	char *last_text = text;
+	int len = 0;
 
-	if (!prefs.filterbeep && strchr (text, 7))
-		gdk_beep ();
-
-	cr = strchr (text, '\n');
-	if (cr)
+	/* split the text into separate lines */
+	while (1)
 	{
-		while (1)
+		switch (*text)
 		{
-			PrintTextLine (textwidget, text,
-								(unsigned long) cr - (unsigned long) text, indent);
-			text = cr + 1;
+		case 0:
+			PrintTextLine (textwidget, last_text, len, indent);
+			return;
+		case '\n':
+			PrintTextLine (textwidget, last_text, len, indent);
+			text++;
 			if (*text == 0)
-				break;
-			cr = strchr (text, '\n');
-			if (!cr)
-			{
-				PrintTextLine (textwidget, text, -1, indent);
-				break;
-			}
+				return;
+			last_text = text;
+			len = 0;
+			break;
+		case ATTR_BEEP:
+			*text = ' ';
+			if (!prefs.filterbeep)
+				gdk_beep ();
+		default:
+			text++;
+			len++;
 		}
-	} else
-	{
-		PrintTextLine (textwidget, text, -1, indent);
 	}
 }
 
 static void
-pevent_dialog_close (GtkWidget * wid, gpointer * arg)
+pevent_dialog_close (gpointer * arg)
 {
 	pevent_dialog = NULL;
-	pevent_dialog_save (NULL);
+	pevent_save (NULL);
 }
 
 static void
@@ -190,7 +176,7 @@ pevent_dialog_update (GtkWidget * wid, GtkWidget * twid)
 
 	if (pevt_build_string (text, &out, &m) != 0)
 	{
-		gtkutil_simpledialog ("There was an error parsing the string");
+		gtkutil_simpledialog (_("There was an error parsing the string"));
 		return;
 	}
 	if (m > te[sig].num_args)
@@ -198,7 +184,7 @@ pevent_dialog_update (GtkWidget * wid, GtkWidget * twid)
 		free (out);
 		out = malloc (4096);
 		snprintf (out, 4096,
-					 "This signal is only passed %d args, $%d is invalid",
+					 _("This signal is only passed %d args, $%d is invalid"),
 					 te[sig].num_args, m);
 		gtkutil_simpledialog (out);
 		free (out);
@@ -244,7 +230,7 @@ pevent_dialog_hfill (GtkWidget * list, int e)
 	if (!text_event (e))
 		return;
 
-	text = te[e].help[i];
+	text = _(te[e].help[i]);
 
 	gtk_clist_clear (GTK_CLIST (list));
 	while (text)
@@ -257,7 +243,7 @@ pevent_dialog_hfill (GtkWidget * list, int e)
 		gtk_clist_append (GTK_CLIST (list), nnew);
 
 		i++;
-		text = te[e].help[i];
+		text = _(te[e].help[i]);
 	}
 }
 
@@ -311,7 +297,7 @@ pevent_save_req_cb (void *arg1, void *arg2, char *file)
 {
 	if (file)
 	{
-		pevent_dialog_save (file);
+		pevent_save (file);
 		free (file);
 	}
 }
@@ -321,11 +307,11 @@ pevent_save_cb (GtkWidget * wid, void *data)
 {
 	if (data)
 	{
-		gtkutil_file_req ("Print Texts File", pevent_save_req_cb, NULL, NULL,
+		gtkutil_file_req (_("Print Texts File"), pevent_save_req_cb, NULL, NULL,
 								TRUE);
 		return;
 	}
-	pevent_dialog_save (NULL);
+	pevent_save (NULL);
 }
 
 static void
@@ -344,7 +330,7 @@ pevent_load_req_cb (void *arg1, void *arg2, char *file)
 static void
 pevent_load_cb (GtkWidget * wid, void *data)
 {
-	gtkutil_file_req ("Print Texts File", pevent_load_req_cb, NULL, NULL,
+	gtkutil_file_req (_("Print Texts File"), pevent_load_req_cb, NULL, NULL,
 							FALSE);
 }
 
@@ -364,7 +350,7 @@ pevent_test_cb (GtkWidget * wid, GtkWidget * twid)
 	{
 		if (!text_event (n))
 			continue;
-		text = pntevts_text[n];
+		text = _(pntevts_text[n]);
 		len = strlen (text);
 
 		out = malloc (len + 2);
@@ -400,24 +386,22 @@ void
 pevent_dialog_show ()
 {
 	GtkWidget *vbox, *vbox2, *hbox, *tbox, *wid, *bh, *th;
-	gchar *titles[] = { "Event", "Text", "Sound" };
-	gchar *help_titles[] = { "$ Number", "Description" };
+	gchar *titles[] = { _("Event"), _("Text"), _("Sound") };
+	gchar *help_titles[] = { _("$ Number"), _("Description") };
 
 	if (pevent_dialog)
 	{
-		gdk_window_show (pevent_dialog->window);
+		wins_bring_tofront (pevent_dialog);
 		return;
 	}
-	pevent_dialog = gtkutil_window_new ("Edit Events", "Edit Events", 600, 455,
-													pevent_dialog_close, 0, FALSE);
-	if (GTK_IS_WINDOW (pevent_dialog))
-		gtk_window_set_policy (GTK_WINDOW (pevent_dialog), TRUE, TRUE, FALSE);
+
+	pevent_dialog = maingui_window ("edit events", _("Edit Events"),
+											  TRUE, FALSE, pevent_dialog_close, NULL,
+											  600, 455, NULL);
 
 	vbox2 = gtk_vbox_new (0, 2);
-	vbox = gtk_vbox_new (0, 2);
+	vbox = wins_get_vbox (pevent_dialog);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
-	gtk_container_add (GTK_CONTAINER (pevent_dialog), vbox);
-	gtk_widget_show (vbox);
 
 	wid = gtk_vpaned_new ();
 	th = gtk_vbox_new (0, 2);
@@ -452,7 +436,7 @@ pevent_dialog_show ()
 	gtk_box_pack_start (GTK_BOX (bh), pevent_dialog_entry, 0, 0, 0);
 	gtk_widget_show (pevent_dialog_entry);
 
-	gui_entry ("Sound file: ", 64, bh, &pevent_dialog_sound_entry);
+	gui_entry (_("Sound file: "), 64, bh, &pevent_dialog_sound_entry);
 	gtk_signal_connect (GTK_OBJECT (pevent_dialog_sound_entry), "activate",
 							  GTK_SIGNAL_FUNC (pevent_dialog_update_sound),
 							  pevent_dialog_list);
@@ -474,29 +458,29 @@ pevent_dialog_show ()
 														  GTK_POLICY_ALWAYS,
 														  NULL, 0, NULL, 0,
 														  GTK_SELECTION_BROWSE);
-	gtk_clist_set_column_width (GTK_CLIST (pevent_dialog_list), 0, 65);
+	gtk_clist_set_column_width (GTK_CLIST (pevent_dialog_list), 0, 120);
 	gtk_widget_show (pevent_dialog_hlist);
 
 	pevent_dialog_fill (pevent_dialog_list);
 
 	hbox = gtk_hbox_new (0, 2);
 	gtk_box_pack_end (GTK_BOX (vbox), hbox, 0, 0, 0);
-	wid = gtk_button_new_with_label ("Save");
+	wid = gtk_button_new_with_label (_("Save"));
 	gtk_box_pack_end (GTK_BOX (hbox), wid, 0, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (wid), "clicked",
 							  GTK_SIGNAL_FUNC (pevent_save_cb), NULL);
 	gtk_widget_show (wid);
-	wid = gtk_button_new_with_label ("Save As");
+	wid = gtk_button_new_with_label (_("Save As"));
 	gtk_box_pack_end (GTK_BOX (hbox), wid, 0, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (wid), "clicked",
 							  GTK_SIGNAL_FUNC (pevent_save_cb), (void *) 1);
 	gtk_widget_show (wid);
-	wid = gtk_button_new_with_label ("Load From");
+	wid = gtk_button_new_with_label (_("Load From"));
 	gtk_box_pack_end (GTK_BOX (hbox), wid, 0, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (wid), "clicked",
 							  GTK_SIGNAL_FUNC (pevent_load_cb), (void *) 0);
 	gtk_widget_show (wid);
-	wid = gtk_button_new_with_label ("Test All");
+	wid = gtk_button_new_with_label (_("Test All"));
 	gtk_box_pack_end (GTK_BOX (hbox), wid, 0, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (wid), "clicked",
 							  GTK_SIGNAL_FUNC (pevent_test_cb), pevent_dialog_twid);
@@ -504,7 +488,7 @@ pevent_dialog_show ()
 #ifdef USE_GNOME
 	wid = gnome_stock_button (GNOME_STOCK_BUTTON_OK);
 #else
-	wid = gtk_button_new_with_label ("Ok");
+	wid = gtk_button_new_with_label (_("Ok"));
 #endif
 	gtk_box_pack_start (GTK_BOX (hbox), wid, 0, 0, 0);
 	gtk_signal_connect (GTK_OBJECT (wid), "clicked",
