@@ -24,30 +24,28 @@
 #include <strings.h>
 #endif
 #include <stdlib.h>
-#include "../common/xchat.h"
+
 #include "fe-gtk.h"
-#include "gtkutil.h"
+#include "../common/xchat.h"
 #include "../common/cfgfiles.h"
-#include "../common/popup.h"
+#include "../common/util.h"
+#include "../common/xchatc.h"
+#include "gtkutil.h"
+#include "menu.h"
+#include "wins.h"
+#include "urlgrab.h"
 
 
-extern GSList *url_list;
-extern GSList *urlhandler_list;
-
-extern char *nocasestrstr (char *, char *);
-extern void menu_urlmenu (struct session *sess, GdkEventButton * event,
-								  char *url);
-
+static GSList *url_list = 0;
 static GtkWidget *urlgrabberwindow = 0;
 static GtkWidget *urlgrabberlist;
 static void url_addurlgui (char *urltext);
 
 
-static int
-url_closegui (void)
+static void
+url_closegui (gpointer userdata)
 {
 	urlgrabberwindow = 0;
-	return 0;
 }
 
 static void
@@ -112,7 +110,7 @@ url_save_callback (void *arg1, void *arg2, char *file)
 static void
 url_button_save (void)
 {
-	gtkutil_file_req ("Select a file to save to",
+	gtkutil_file_req (_("Select a file to save to"),
 							url_save_callback, 0, 0, TRUE);
 }
 
@@ -133,7 +131,7 @@ url_clicklist (GtkWidget * widget, GdkEventButton * event)
 		{
 			if (text && text[0])
 			{
-				menu_urlmenu (0, event, text);
+				menu_urlmenu (event, text);
 			}
 		}
 	}
@@ -147,17 +145,14 @@ url_opengui ()
 
 	if (urlgrabberwindow)
 	{
-		gdk_window_show (urlgrabberwindow->window);
+		wins_bring_tofront (urlgrabberwindow);
 		return;
 	}
-	urlgrabberwindow =
-		gtkutil_window_new ("X-Chat: URL Grabber", "urlgrabber", 350, 100,
-								  url_closegui, 0, TRUE);
 
-	vbox = gtk_vbox_new (FALSE, 1);
-	gtk_container_add (GTK_CONTAINER (urlgrabberwindow), vbox);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
-	gtk_widget_show (vbox);
+	urlgrabberwindow =
+		maingui_window ("urlgrabber", _("X-Chat: URL Grabber"), FALSE, TRUE,
+							 url_closegui, NULL, 350, 100, &urlgrabberwindow);
+	vbox = wins_get_vbox (urlgrabberwindow);
 
 	urlgrabberlist = gtkutil_clist_new (1, 0, vbox, GTK_POLICY_AUTOMATIC,
 													0, 0, 0, 0, GTK_SELECTION_BROWSE);
@@ -171,9 +166,9 @@ url_opengui ()
 	gtk_widget_show (hbox);
 
 	gtkutil_button (urlgrabberwindow, GNOME_STOCK_PIXMAP_CLEAR,
-						 "Clear", url_button_clear, 0, hbox);
+						 _("Clear"), url_button_clear, 0, hbox);
 	gtkutil_button (urlgrabberwindow, GNOME_STOCK_PIXMAP_SAVE,
-						 "Save", url_button_save, 0, hbox);
+						 _("Save"), url_button_save, 0, hbox);
 
 	gtk_widget_show (urlgrabberwindow);
 
@@ -216,7 +211,10 @@ url_addurl (char *urltext)
 		data[strlen (data) - 1] = 0;
 
 	if (url_findurl (data))
+	{
+		free (data);
 		return;
+	}
 
 	url_list = g_slist_prepend (url_list, data);
 	url_addurlgui (data);
