@@ -5,10 +5,7 @@
 #include <gtk/gtkadjustment.h>
 #include <gtk/gtkwidget.h>
 
-/*#define GTK_XTEXT(obj)          GTK_CHECK_CAST (obj, gtk_xtext_get_type (), GtkXText)*/
 #define GTK_XTEXT(obj) ((GtkXText*)obj)
-#define GTK_XTEXT_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gtk_xtext_get_type (), GtkXTextClass)
-#define GTK_IS_XTEXT(obj)       GTK_CHECK_TYPE (obj, gtk_xtext_get_type ())
 
 #define FONT_1BYTE 0
 #define FONT_2BYTE 1
@@ -28,7 +25,7 @@ typedef struct _GtkXTextClass GtkXTextClass;
 typedef struct textentry
 {
 	struct textentry *next;
-	char *str;
+	unsigned char *str;
 	int str_width;
 	time_t stamp;
 	short str_len;
@@ -37,6 +34,7 @@ typedef struct textentry
 	short indent;
 	short lines_taken;
 	short left_len;
+	unsigned int mb:1;	/* is multibyte? */
 }
 textentry;
 
@@ -98,6 +96,7 @@ struct _GtkXText
 	int num_lines;
 	int max_lines;
 
+	int pagetop_line;
 	int pagetop_subline;
 	textentry *pagetop_ent;			/* what's at xtext->adj->value */
 
@@ -118,11 +117,10 @@ struct _GtkXText
 	short grid_offset[256];
 
 	GtkWidget *(*error_function) (char *text);
-	int (*urlcheck_function) (GtkXText * xtext, char *word);
+	int (*urlcheck_function) (GtkWidget * xtext, char *word);
 
 	unsigned char scratch_buffer[4096];
 
-	unsigned int fixed_width_font:1;
 	unsigned int auto_indent:1;
 	unsigned int moving_separator:1;
 	unsigned int time_stamp:1;
@@ -147,6 +145,8 @@ struct _GtkXText
 	unsigned int skip_border_fills:1;
 	unsigned int skip_stamp:1;
 	unsigned int do_underline_fills_only:1;
+	unsigned int skip_exposure:1;
+	unsigned int jump_out_early:1;
 };
 
 struct _GtkXTextClass
@@ -156,11 +156,10 @@ struct _GtkXTextClass
 };
 
 GtkWidget *gtk_xtext_new (int indent, int separator);
-guint gtk_xtext_get_type (void);
-void gtk_xtext_append (GtkXText * xtext, char *text, int len);
+void gtk_xtext_append (GtkXText * xtext, unsigned char *text, int len);
 void gtk_xtext_append_indent (GtkXText * xtext,
-										char *left_text, int left_len,
-										char *right_text, int right_len);
+										unsigned char *left_text, int left_len,
+										unsigned char *right_text, int right_len);
 void gtk_xtext_set_font (GtkXText * xtext, GdkFont * font, char *name);
 void gtk_xtext_set_background (GtkXText * xtext, GdkPixmap * pixmap,
 										 int trans, int shaded);
@@ -170,7 +169,7 @@ gchar *gtk_xtext_get_chars (GtkXText * xtext);
 void gtk_xtext_refresh (GtkXText * xtext, int do_trans);
 void gtk_xtext_thaw (GtkXText * xtext);
 void gtk_xtext_freeze (GtkXText * xtext);
-void *gtk_xtext_search (GtkXText * xtext, char *text, void *start);
-char *gtk_xtext_strip_color (unsigned char *text, int len, char *outbuf, int *newlen, int fonttype);
+void *gtk_xtext_search (GtkXText * xtext, unsigned char *text, void *start);
+unsigned char *gtk_xtext_strip_color (unsigned char *text, int len, unsigned char *outbuf, int *newlen, int fonttype, int *mb_ret);
 
 #endif
