@@ -27,18 +27,12 @@
 #include <unistd.h>
 #include <ctype.h>
 #include "../common/xchat.h"
+#include "../common/xchatc.h"
+#include "../common/outbound.h"
+#include "../common/util.h"
 #include "../common/fe.h"
 #include "fe-text.h"
 
-extern char *get_cpu_str (int color);
-extern int handle_command (char *cmd, struct session *sess, int history,
-									int nocommand);
-extern int get_mhz (void);
-extern void kill_session_callback (struct session *sess);
-
-extern struct xchatprefs prefs;
-extern struct session *current_tab;
-extern GSList *sess_list;
 
 static GSList *tmr_list;		  /* timer list */
 static int tmr_list_count;
@@ -103,22 +97,16 @@ fe_new_window (struct session *sess)
 		return;
 	done_intro = 1;
 
-	snprintf (buf, sizeof (buf),
-   	"\n"
-		"\00314-\00315-\00301-\00310-\00311 xchat "VERSION""
-		" \00310-\00301-\00315-\00314-\017\n"
-		"\n"
-		"\00315[ \0036 system  \00315] \017 %s\017\n"
-		"\00315[ \0036  lib    \00315] \017 GLIB %d.%d.%d\n"
-		"\00315[ \0036nickname \00315] \017 %s\n"
-		"\00315[ \0036compiled \00315] \017 "__DATE__"\n",
-				get_cpu_str (1),
-				glib_major_version, glib_minor_version, glib_micro_version,
-				prefs.nick1
-				);
+	snprintf (buf, sizeof (buf),	
+				"\n"
+				" \017xchat \00310"VERSION"\n"
+				" \017Running on \00310%s \017glib \00310%d.%d.%d\n"
+				" \017This binary compiled \00310"__DATE__"\017\n",
+				get_cpu_str(), 
+				glib_major_version, glib_minor_version, glib_micro_version);
 	fe_print_text (sess, buf);
 
-	strcpy (buf, "\00315[ \0036features \00315] \017 ");
+	strcpy (buf, "\n\nCompiled in Features\0032:\017 ");
 #ifdef USE_PERL
 	strcat (buf, "Perl ");
 #endif
@@ -142,25 +130,33 @@ fe_new_window (struct session *sess)
 #endif
 #ifdef SOCKS
 	strcat (buf, "Socks5 ");
+#endif
+#ifdef HAVE_ICONV
+	strcat (buf, "JCode ");
+#endif
+#ifdef USE_IPV6
+	strcat (buf, "IPv6 ");
 #endif
 	strcat (buf, "\n\n");
 	fe_print_text (sess, buf);
-	fe_print_text (sess, "\n\0033*\00315 Type \017/help\00315 for a list of commands\017\n\n");
 	fflush (stdout);
 	fflush (stdin);
 }
 
+static int
+get_stamp_str (time_t tim, char *dest, int size)
+{
+	return strftime (dest, size, prefs.stamp_format, localtime (&tim));
+}
+
 static int
 timecat (char *buf)
 {
-	time_t timval = time (0);
-	char *tim = ctime (&timval) + 10;
-	tim[0] = '[';
-	tim[9] = ']';
-	tim[10] = ' ';
-	tim[11] = 0;
-	strcat (buf, tim);
-	return 11;
+	char stampbuf[64];
+
+	get_stamp_str (time (0), stampbuf, sizeof (stampbuf));
+	strcat (buf, stampbuf);
+	return strlen (stampbuf);
 }
 
 /*                       0  1  2  3  4  5  6  7   8   9   10 11  12  13  14 15 */
@@ -353,6 +349,8 @@ fe_timeout_add (int interval, void *callback, void *userdata)
 	struct timeval now;
 	timerevent *te = malloc (sizeof (timerevent));
 
+	tmr_list_count++;				  /* this overflows at 2.2Billion, who cares!! */
+
 	te->tag = tmr_list_count;
 	te->interval = interval;
 	te->callback = callback;
@@ -362,7 +360,6 @@ fe_timeout_add (int interval, void *callback, void *userdata)
 	te->next_call = now.tv_sec * 1000 + (now.tv_usec / 1000) + te->interval;
 
 	tmr_list = g_slist_prepend (tmr_list, te);
-	tmr_list_count++;				  /* this overflows at 2.2Billion, who cares!! */
 
 	return te->tag;
 }
@@ -393,6 +390,8 @@ fe_input_add (int sok, int rread, int wwrite, int eexcept, void *func,
 {
 	socketevent *se = malloc (sizeof (socketevent));
 
+	se_list_count++;				  /* this overflows at 2.2Billion, who cares!! */
+
 	se->tag = se_list_count;
 	se->sok = sok;
 	se->rread = rread;
@@ -402,8 +401,6 @@ fe_input_add (int sok, int rread, int wwrite, int eexcept, void *func,
 	se->userdata = data;
 	se_list = g_slist_prepend (se_list, se);
 
-	se_list_count++;				  /* this overflows at 2.2Billion, who cares!! */
-
 	return se->tag;
 }
 
@@ -432,6 +429,7 @@ fe_init (void)
 	prefs.use_server_tab = 0;
 	prefs.autodialog = 0;
 	prefs.lagometer = 0;
+	prefs.skipserverlist = 1;
 }
 
 void
@@ -444,6 +442,11 @@ fe_main (void)
 	GSList *list;
 	guint64 shortest, delay;
 
+#ifdef ENABLE_NLS
+	bindtextdomain (PACKAGE, LOCALEDIR);
+	textdomain (PACKAGE);
+#endif
+
 	while (!done)
 	{
 		FD_ZERO (&rd);
@@ -502,13 +505,13 @@ fe_main (void)
 			se->checked = 1;
 			if (se->rread && FD_ISSET (se->sok, &rd))
 			{
-				se->callback (se->userdata, se->sok);
+				se->callback (NULL, 0, se->userdata);
 			} else if (se->wwrite && FD_ISSET (se->sok, &wd))
 			{
-				se->callback (se->userdata, se->sok);
+				se->callback (NULL, 0, se->userdata);
 			} else if (se->eexcept && FD_ISSET (se->sok, &ex))
 			{
-				se->callback (se->userdata, se->sok);
+				se->callback (NULL, 0, se->userdata);
 			}
 			list = se_list;
 			if (list)
@@ -620,6 +623,19 @@ void
 fe_chan_list_end (struct server *serv)
 {
 }
+int
+fe_is_banwindow (struct session *sess)
+{
+	return 0;
+}
+void
+fe_add_ban_list (struct session *sess, char *chan, char *users, char *topic)
+{
+}               
+void
+fe_ban_list_end (struct session *sess)
+{
+}
 void
 fe_notify_update (char *name)
 {
@@ -729,15 +745,15 @@ fe_ignore_update (int level)
 {
 }
 void
-fe_dcc_open_recv_win (void)
+fe_dcc_open_recv_win (int passive)
 {
 }
 void
-fe_dcc_open_send_win (void)
+fe_dcc_open_send_win (int passive)
 {
 }
 void
-fe_dcc_open_chat_win (void)
+fe_dcc_open_chat_win (int passive)
 {
 }
 int
@@ -745,6 +761,11 @@ fe_is_confmode (struct session *sess)
 {
 	return 0;
 }
+int
+fe_is_beep (struct session *sess)
+{
+	return 0;
+}
 
 void
 fe_userlist_hide (session * sess)
@@ -759,6 +780,15 @@ fe_set_lag (server * serv, int lag)
 {
 }
 void
+fe_set_throttle (server * serv)
+{
+}
+void
 fe_set_away (server *serv)
 {
 }
+int
+fe_open_serverlist (session *sess, int auto_connect, int dont_show)
+{
+	return FALSE;
+}
